# -*- coding: utf-8 -*-

# Copyright 2017 Xiaomi, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import logging
import os
import re

import requests
import sys
import yaml
try:
  from urllib.request import urlopen, Request # Python 3
except ImportError:
  from urllib2 import urlopen, Request # Python 2

import fds
import pkg_resources
from cloud_ml_common.auth.signature import Signer

from cloud_ml_sdk.command import constant

from cloud_ml_sdk.command.cloudml_init import read_config_yaml
from cloud_ml_sdk.command.cloudml_init import get_config_file
from cloud_ml_sdk.command.constant import GlobalArgsStore

sys.path.append("../../cloud_ml_common/")

# TODO(xychu): maybe we should consider make this a item in config file
# so if user has any complain about this dir, they could change it by their own
CLOUDML_UPLOAD_PACKAGE_DIR = "user_packages/"
CLOUDML_MODEL_URL = "/cloud_ml/v1/model"

class CloudMlClient(object):
  """The client to auth and operate to cloud-ml.

  A CloudMlClient instance authentic the user identity as it is created.
  And a client can handle cloud-ml job)
   methods.
  """

  def __init__(self, access_key=None, secret_key=None, endpoint=None, **kwargs):
    """Create a new CloudMlClient with given definition.

    The `access_key` and `secret_key` must be provided.

    Args:
      access_key: Access key for authentic.
      secret_key: Secret key for authentic.
    """
    config_data = self.get_config_data_v2()

    self._secret_key = None
    self._access_key = None
    self._headers = {constant.HEADER_X_XIAOMI_SDK_VERSION: pkg_resources.require(constant.SDK_NAME)[0].version}
    if access_key == None or secret_key == None:
      # TODO: check and catch exception
      # Get keys from environment variables
      if "XIAOMI_ACCESS_KEY_ID" in os.environ and "XIAOMI_SECRET_ACCESS_KEY" in os.environ:
        access_key = os.environ["XIAOMI_ACCESS_KEY_ID"]
        secret_key = os.environ["XIAOMI_SECRET_ACCESS_KEY"]
      else:
        if config_data:
          access_key = config_data["xiaomi_access_key_id"]
          secret_key = config_data["xiaomi_secret_access_key"]
        else:
          raise Exception(
            "Can't find access key and secret key, please run cloudml config init")

    self.access_key = access_key
    self.secret_key = secret_key

    # Declare
    self._endpoint = None
    if endpoint == None:
      if "XIAOMI_CLOUDML_ENDPOINT" in os.environ:
        self.endpoint = os.environ["XIAOMI_CLOUDML_ENDPOINT"]
      else:
        if config_data:
          self.endpoint = config_data["xiaomi_cloudml_endpoint"]
        else:
          raise Exception(
            "Can't find cloudml endpoint, please run cloudml config init")
    else:
      self.endpoint = endpoint

    if "XIAOMI_FDS_ENDPOINT" in os.environ and "CLOUDML_DEFAULT_FDS_BUCKET" in os.environ:
      self._fds_endpoint = os.environ["XIAOMI_FDS_ENDPOINT"]
      self._fds_bucket = os.environ["CLOUDML_DEFAULT_FDS_BUCKET"]
    elif config_data:
      self._fds_endpoint = config_data.get("xiaomi_fds_endpoint")
      self._fds_bucket = config_data.get("cloudml_default_fds_bucket")

    # for hdfs
    if "XIAOMI_HDFS_KRB_ACCOUNT" in os.environ and "XIAOMI_HDFS_KRB_PASSWORD" in os.environ:
      self._hdfs_krb_account = os.environ["XIAOMI_HDFS_KRB_ACCOUNT"]
      self._hdfs_krb_password = os.environ["XIAOMI_HDFS_KRB_PASSWORD"]
    elif config_data:
      self._hdfs_krb_account = config_data.get("xiaomi_hdfs_krb_account")
      self._hdfs_krb_password = config_data.get("xiaomi_hdfs_krb_password")

    if "XIAOMI_HDFS_ENDPOINT" in os.environ:
      self._hdfs_endpoint = os.environ["XIAOMI_HDFS_ENDPOINT"]
    elif config_data:
      self._hdfs_endpoint = config_data.get("xiaomi_hdfs_endpoint")

    self._auth = Signer(access_key, secret_key)

    if "xiaomi_org_mail" in kwargs and kwargs["xiaomi_org_mail"]:
      self._org_mail = kwargs["xiaomi_org_mail"]
    elif "XIAOMI_ORG_MAIL" in os.environ:
      self._org_mail = os.environ["XIAOMI_ORG_MAIL"]
    elif config_data and "xiaomi_org_mail" in config_data:
      self._org_mail = config_data.get("xiaomi_org_mail")
    else:
      raise Exception(
        "Can't find org mail, please set one by re-run 'cloudml config init'")
    self.org_id = self.get_org_id().get("org_id")

  def get_config_data(self):
    # Read keys from configuration file
    data = {}
    try:
      config_filename = os.path.join(
        os.path.expanduser("~"), constant.CONFIG_PATH, constant.CONFIG_FILENAME)
      if os.path.exists(config_filename):
        with open(config_filename) as f:
          data = yaml.safe_load(f)
    except Exception as e:
      raise ValueError(
        "Failed to load config data, is the json data in right format? Exception content: {}".format(
          e))
    try:
      if constant.CLOUDML_CONFIG_KEY in data:
        # NOTE(xychu): XIAOMI_CLOUDML_CONFIG_CONTEXT Env will be set by cloudml -k/--context arg
        if "XIAOMI_CLOUDML_CONFIG_CONTEXT" in os.environ:
          config_context_name = os.environ["XIAOMI_CLOUDML_CONFIG_CONTEXT"]
          if config_context_name in data[constant.CLOUDML_CONFIG_KEY]:
            # load user demand context config
            context_config = data[constant.CLOUDML_CONFIG_KEY][config_context_name]
            data.update(context_config)
          else:
            # user demand config context not found
            raise ValueError(
              "Config context with name {} not found.\n".format(config_context_name) +
              "Please set a valid one or init it first.")
        else:
          # load default context specific config
          if constant.DEFAULT_CONFIG_CONTEXT in data[constant.CLOUDML_CONFIG_KEY]:
            default_cluster = data[constant.CLOUDML_CONFIG_KEY][constant.DEFAULT_CONFIG_CONTEXT]
            context_config = data[constant.CLOUDML_CONFIG_KEY][default_cluster]
            data.update(context_config)
          elif len(data[constant.CLOUDML_CONFIG_KEY]) == 1:
            # if there's only one config context, use it as default
            for default_cluster in data[constant.CLOUDML_CONFIG_KEY]:
              context_config = data[constant.CLOUDML_CONFIG_KEY][default_cluster]
              data.update(context_config)
          else:
            raise ValueError(
              "No default config context found. \n" +
              "Please set your default context by: cloudml config set_default <name_of_your_config_context>")
    except Exception as e:
      raise ValueError(
        "Failed to load context config. Exception content: {}".format(
          e))
    return data

  def get_config_data_v2(self):
    if os.path.exists(os.path.join(
      os.path.expanduser("~"), constant.CONFIG_PATH, constant.CONFIG_FILENAME)):
      return self.get_config_data()

    config_data = {}
    if os.path.exists(os.path.join(
      os.path.expanduser("~"), constant.CONFIG_PATH, constant.CONFIG_FILENAME_V2)):
      config_data = read_config_yaml( get_config_file() )

    real_data = {}

    try:
      if constant.CLOUDML_CONFIG_KEY in config_data:
        # NOTE(xychu): XIAOMI_CLOUDML_CONFIG_CONTEXT Env will be set by cloudml -k/--context arg
        if "XIAOMI_CLOUDML_CONFIG_CONTEXT" in os.environ:
          config_context_name = os.environ["XIAOMI_CLOUDML_CONFIG_CONTEXT"]
          if config_context_name in config_data[constant.CLOUDML_CONFIG_KEY]:
            # load user demand context config
            context_config = config_data[constant.CLOUDML_CONFIG_KEY][config_context_name]
            real_data.update(context_config)
          else:
            # user demand config context not found
            raise ValueError(
              "Config context with name {} not found.\n".format(config_context_name) +
              "Please set a valid one or init it first.")
        else:
          # load default context specific config
          if constant.DEFAULT_CONFIG_CONTEXT in config_data[constant.CLOUDML_CONFIG_KEY]:
            default_cluster = config_data[constant.CLOUDML_CONFIG_KEY][constant.DEFAULT_CONFIG_CONTEXT]
            context_config = config_data[constant.CLOUDML_CONFIG_KEY][default_cluster]
            real_data.update(context_config)
          elif len(config_data[constant.CLOUDML_CONFIG_KEY]) == 1:
            # if there's only one config context, use it as default
            default_cluster = list(config_data[constant.CLOUDML_CONFIG_KEY].keys())[0]
            context_config = config_data[constant.CLOUDML_CONFIG_KEY][default_cluster]
            real_data.update(context_config)
          else:
            raise ValueError(
              "No default config context found. \n" +
              "Please set your default context by: cloudml config set_default <name_of_your_config_context>")

          if 'fds' in config_data:
            # aksk will be overwrite by fds's config.
            real_data.update(config_data['fds'])

          if 'hdfs' in config_data:
            real_data.update(config_data['hdfs'])

    except Exception as e:
      raise ValueError(
        "Failed to load context config. Exception content: {}".format(e))

    return real_data

  @property
  def access_key(self):
    return self._access_key

  @access_key.setter
  def access_key(self, value):
    """Function for setting access_key.
    """
    if not isinstance(value, str):
      raise ValueError("access_key must be a string!")
    self._access_key = value

  @property
  def secret_key(self):
    return self._secret_key

  @secret_key.setter
  def secret_key(self, value):
    """Function for setting secret_key.
    """
    if not isinstance(value, str):
      raise ValueError("secret_key must be a string!")
    self._secret_key = value

  @property
  def endpoint(self):
    return self._endpoint

  @endpoint.setter
  def endpoint(self, value):
    """Function for setting endpoint.

    Args:
      value: String value that is going to be set to endpoint.
    """
    if not isinstance(value, str):
      raise ValueError("endpoint must be a string!")
    if not (value.startswith("http://") or value.startswith("https://")):
      value = "https://" + value
    self._endpoint = value
    self._hp_url = self._endpoint + "/cloud_ml/v1/hp"
    self._deploy_url = self._endpoint + "/cloud_ml/v1/deploy"
    self._train_url = self._endpoint + "/cloud_ml/v1/train"
    self._model_url = self._endpoint + "/cloud_ml/v1/model"
    self._model_url_v2 = self._endpoint + "/cloud_ml/v2/model"
    self._model_testcase_url_v1 = self._endpoint + "/cloud_ml/v1/model/testcase"
    self._autoscaler_url_v2 = self._endpoint + "/cloud_ml/v2/model/autoscaler/rule"
    self._dev_url = self._endpoint + "/cloud_ml/v1/dev"
    self._dev_url_v2 = self._endpoint + "/cloud_ml/v2/dev"
    self._tensorboard_url = self._endpoint + "/cloud_ml/v1/tensorboard"
    self._quota_url = self._endpoint + "/cloud_ml/v1/quota"
    self._quota_application_url = self._endpoint + "/cloud_ml/v1/application"
    self._framework_url = self._endpoint + "/cloud_ml/v1/framework"
    self._authentication_url = self._endpoint + "/cloud_ml/v1/authentication"
    self._org_id_url = self._endpoint + "/cloud_ml/v1/org_ids"
    self._dev_server_url = self._endpoint + "/dev_server/v1/dev_servers"
    self._pipeline_url = self._endpoint + "/cloud_ml/v1/pipeline"
    self._schedule_url = self._endpoint + "/cloud_ml/v1/schedule"
    self._resources_url = self._endpoint + "/cloud_ml/v1/resources"
    self._ceph_url = self._endpoint + "/cloud_ml/v1/ceph"
    self._env_url = self._endpoint + "/cloud_ml/v1/env"
    self._secret_url = self._endpoint + "/cloud_ml/v1/secret"
    self._hdfs_secret_url = self._endpoint + "/cloud_ml/v1/secret/hdfs"
    self._fds_secret_url = self._endpoint + "/cloud_ml/v1/secret/fds"
    self._git_secret_url = self._endpoint + "/cloud_ml/v1/secret/git"
    self._namespace_url = self._endpoint + "/cloud_ml/v1/namespace"
    self._priority_class_url = self._endpoint + "/cloud_ml/v1/priority_class"
    self._namespace_quota_url = self._endpoint + "/cloud_ml/v1/namespace_quota"
    self._namespace_user_url = self._endpoint + "/cloud_ml/v1/namespace_user"
    self._project_url = self._endpoint + "/cloud_ml/v2/projects"
    self._train_v2_url = self._endpoint + "/cloud_ml/v2/trains"
    self._quota_v2_url = self._endpoint + "/cloud_ml/v2/quota"
    self._quota_v2_application_url = self._endpoint + "/cloud_ml/v2/application"
    self._dataset_url = self._endpoint + "/automl/v1/dataset"

  @property
  def fds_endpoint(self):
    return self._fds_endpoint

  @fds_endpoint.setter
  def fds_endpoint(self, value):
    """Function for setting xiaomi_fds_endpoint.
    """
    if not isinstance(value, str):
      raise ValueError("fds_endpoint must be a string!")
    self._fds_endpoint = value

  @property
  def org_mail(self):
    return self._org_mail

  @org_mail.setter
  def org_mail(self, value):
    """Function for setting xiaomi_org_mail.
    """
    if not isinstance(value, str):
      raise ValueError("org_mail must be a string!")
    self._org_mail = value

  @property
  def fds_bucket(self):
    return self._fds_bucket

  @fds_bucket.setter
  def fds_bucket(self, value):
    """Function for setting cloudml_default_fds_bucket.
    """
    if not isinstance(value, str):
      raise ValueError("fds_bucket must be a string!")
    self._fds_bucket = value

  # for hdfs
  @property
  def hdfs_krb_account(self):
    return self._hdfs_krb_account

  @hdfs_krb_account.setter
  def hdfs_krb_account(self, value):
    if not isinstance(value, str):
      raise ValueError("hdfs_krb_account must be a string!")
    self._hdfs_krb_account = value

  @property
  def hdfs_krb_password(self):
    return self._hdfs_krb_password

  @hdfs_krb_password.setter
  def hdfs_krb_password(self, value):
    if not isinstance(value, str):
      raise ValueError("hdfs_krb_password must be a string!")
    self._hdfs_krb_password = value

  @property
  def hdfs_endpoint(self):
    return self._hdfs_endpoint

  @hdfs_endpoint.setter
  def hdfs_endpoint(self, value):
    if not isinstance(value, str) or value.startswith("hdfs://"):
      raise ValueError("hdfs_endpoint must start with hdfs://")
    self._hdfs_endpoint = value

  def get_fds_client(self):
    """Init FDS client with correct configuration including: ak, sk, fds_endpoint.

    :return: GalaxyFDSClient or ValueError
    """
    fds_client = None
    try:
      fds_config = fds.FDSClientConfiguration()
      fds_config.set_endpoint(self.fds_endpoint)
      fds_client = fds.GalaxyFDSClient(self.access_key, self.secret_key, fds_config)
    except Exception as e:
      raise ValueError("Init FDS client got error: {}".format(e))
    else:
      return fds_client

  # For hptuning
  def submit_hptuning(self, json_data):
    response = self.process_request('post', self._hp_url, auth=self._auth, data=json_data)
    return self.process_response(response)

  def delete_hp_job(self, hp_name, org_id=None):
    """Delete the train job.

    Args:
      job_name: The name of the train job.

    Returns:
      The response.
    """
    if org_id:
      url = self._hp_url + "/" + hp_name + "?org_id=" + org_id
    else:
      url = self._hp_url + "/" + hp_name
    response = self.process_request('delete', url, auth=self._auth)
    return self.process_response(response)

  def list_hp_jobs(self, org_id=None, priority_class=None, page_number=None, page_size=None):
    """List hp jobs.

    Args:
      The org_id whose hp_jobs to list.

    Returns:
      The list of dictionary of hp jobs.
    """
    url = self._hp_url
    params = []
    if org_id:
      params.append("org_id=" + org_id)
    if priority_class:
      params.append("priority_class=" + priority_class)
    if page_number:
      params.append("page_number=" + str(page_number))
    if page_size:
      params.append("page_size=" + str(page_size))
    if params:
      url = url + "?" + "&".join(params)
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response, 'data')

  def list_hp_trials(self, hp_name, org_id=None):
    """List hp jobs'trials.

    Args:
      The org_id whose hp jobs' trials to list.

    Returns:
      The list of dictionary of trials.
    """
    if org_id:
      url = self._hp_url + "/" + hp_name + "/trials" + "?org_id=" + org_id
    else:
      url = self._hp_url + "/" + hp_name + "/trials"
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)

  def get_hp_trial_logs(self, hp_name, trial_id, org_id=None, follow=False, lines=None):
    """Get logs of the hp trial.

        Args:
          hp_name: The name of the hp job.
          trial_id: The id of the trial(pod)

        Returns:
          The logs of trial.
        """
    params = {}
    if org_id:
      params['org_id'] = org_id
    if follow:
      params['follow'] = follow
    if lines:
      params['lines'] = lines

    url = self._hp_url + "/" + hp_name + "/" + trial_id + "/logs"
    return self.process_request('get', url, params=params, auth=self._auth, stream=follow)

  def describe_hp_job(self, hp_name, org_id=None):
    """Describe and get information of the hp job.

    Args:
      Hp_name: The name of the hpjob.

    Returns:
      The dictionary of hp job.
    """
    if org_id:
      url = self._hp_url + "/" + hp_name + "?org_id=" + org_id
    else:
      url = self._hp_url + "/" + hp_name
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)

  def get_hp_job_events(self, hp_name, org_id=None):
    """Get events of the hp job.

    Args:
      hp_name: The name of the hp job.

    Returns:
      The events of the hp job.
    """
    if org_id:
      url = self._hp_url + "/" + hp_name + "/events" + "?org_id=" + org_id
    else:
      url = self._hp_url + "/" + hp_name + "/events"
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)

  def get_hp_job_metrics(self, hp_name, org_id=None):
    """Get the metrics of the hp job.

    Args:
      hp_name: The name of the hp job.

    Returns:
      The logs of hp job.
    """
    if org_id:
      url = self._hp_url + "/" + hp_name + "/metrics" + "?org_id=" + org_id
    else:
      url = self._hp_url + "/" + hp_name + "/metrics"
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)

  def get_optimal_trial(self, hp_name, org_id=None):
    """Get the metrics of the hp job.

    Args:
      hp_name: The name of the hp job.

    Returns:
      The logs of hp job.
    """
    if org_id:
      url = self._hp_url + "/" + hp_name + "/optimal" + "?org_id=" + org_id
    else:
      url = self._hp_url + "/" + hp_name + "/optimal"
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)

  def submit_train_job(self, json_data):
    """Submit a train_job to run.

    Args:
      json_data: The json data of train job to submit.

    Returns:
      The dictionary of train job.
    """
    response = self.process_request('post', self._train_url, auth=self._auth, data=json_data)
    return self.process_response(response)

  def list_train_jobs(self, org_id=None, priority_class=None, job_name=None, job_status=None, page_number=None, page_size=None):
    """List train jobs.

    Args:
      The org_id whose train_jobs to list.

    Returns:
      The list of dictionary of train jobs.
    """

    params = []
    url = self._train_url
    if org_id:
      params.append("org_id=" + org_id)
    if priority_class:
      params.append("priority_class=" + priority_class)
    if job_name:
      params.append("job_name=" + job_name)
    if job_status:
      params.append("job_status=" + job_status)
    if page_number:
      params.append("page_number=" + str(page_number))
    if page_size:
      params.append("page_size=" + str(page_size))
    if params:
      url = url + "?" + "&".join(params)
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response, 'data')

  def describe_train_job(self, job_name, org_id=None):
    """Describe and get information of the train job.

    Args:
      job_name: The name of the train job.

    Returns:
      The dictionary of train job.
    """
    if org_id:
      url = self._train_url + "/" + job_name + "?org_id=" + org_id
    else:
      url = self._train_url + "/" + job_name
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)

  def kill_train_job(self, job_name, org_id=None):
    """Kill the train job.

    Args:
      job_name: The name of the train job.

    Returns:
      ok: true/false
      error: true/false
      message: if error.
    """
    if org_id:
      url = self._train_url + "/" + job_name + "?org_id=" + org_id
    else:
      url = self._train_url + "/" + job_name
    response = self.process_request('patch', url, auth=self._auth)
    return self.process_response(response)

  def get_train_job_logs(self, job_name, org_id=None, follow=False, lines=None):
    """Get logs of the train job.

    Args:
      job_name: The name of the train job.

    Returns:
      The logs of train job.
    """
    params = {}
    if org_id:
      params['org_id'] = org_id
    if follow:
      params['follow'] = follow
    if lines:
      params['lines'] = lines

    url = self._train_url + "/" + job_name + "/logs"
    return self.process_request('get', url, params=params, auth=self._auth, stream=follow)

  def get_train_job_pod_name(self, job_name, org_id=None):
    """Get history logs of the train job.

    Args:
      job_name: The name of the train job.

    Returns:
      The logs of train job.
    """
    params = {}
    if org_id:
      params['org_id'] = org_id

    url = self._train_url + "/" + job_name + "/pod_name"
    response = self.process_request('get', url, params=params, auth=self._auth)
    return self.process_response(response, "pod_name")

  def get_train_job_metrics(self, job_name, org_id=None):
    """Get the metrics of the train job.

    Args:
      job_name: The name of the train job.

    Returns:
      The logs of train job.
    """
    if org_id:
      url = self._train_url + "/" + job_name + "/metrics" + "?org_id=" + org_id
    else:
      url = self._train_url + "/" + job_name + "/metrics"
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)

  def get_train_job_hyperparameters_data(self, job_name):
    """Get hyperparameters data of the train job.

    Args:
      job_name: The name of the train job.

    Returns:
      The hyperparameter data of train job.
    """
    url = self._train_url + "/" + job_name + "/hyperparameters"
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)

  def delete_train_job(self, job_name, org_id=None, is_cluster_name=False):
    """Delete the train job.

    Args:
      job_name: The name of the train job.

    Returns:
      The response.
    """
    params = {}
    if is_cluster_name:
      params["is_cluster_name"] = is_cluster_name
    if org_id:
      url = self._train_url + "/" + job_name + "?org_id=" + org_id
    else:
      url = self._train_url + "/" + job_name
    response = self.process_request('delete', url, auth=self._auth, params=params)
    return self.process_response(response)

  def get_train_job_events(self, job_name, org_id=None):
    """Get events of the train job.

    Args:
      job_name: The name of the train job.

    Returns:
      The events of the train job.
    """
    if org_id:
      url = self._train_url + "/" + job_name + "/events" + "?org_id=" + org_id
    else:
      url = self._train_url + "/" + job_name + "/events"
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)

  def start_train_job(self, job_name):
    """Start the train job.

    Args:
      job_name: The name of the train job.

    Returns:
      The dictionary of train job.
    """
    url = self._train_url + "/" + job_name + "/start"
    response = self.process_request('get', url, auth=self._auth)

    return self.process_response(response)

  def rerun_train_job(self, job_name, org_id=None):
    """Rerun the train job.

    Args:
      job_name: The name of the train job.

    Returns:
      ok: true/false
      error: true/false
      message: if error.
    """
    if org_id:
      url = self._train_url + "/" + job_name + "/rerun" + "?org_id=" + org_id
    else:
      url = self._train_url + "/" + job_name + "/rerun"
    response = self.process_request('patch', url, auth=self._auth)
    return self.process_response(response)

  def create_train_v2(self, json_data):
    """Create the train v2.

    Args:
      json_data: The json_data object to create.

    Returns:
      The dictionary of train_v2.
    """
    response = self.process_request('post', self._train_v2_url,
                             auth=self._auth,
                             data=json_data)
    return self.process_response(response, 'data')

  def list_train_v2(self, org_id=None):
    """List train_v2.

    Returns:
      The list of dictionary of train_v2.
    """
    if org_id:
      url = self._train_v2_url + "?org_id=" + org_id
    else:
      url = self._train_v2_url
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response, 'data')

  def describe_train_v2(self, job_name, org_id=None):
    """Describe and get information of the train_v2.

    Args:
      job_name: The name of the train_v2.
      org_id: The org_id of the train_v2

    Returns:
      The dictionary of train_v2.
    """
    if org_id:
      url = self._train_v2_url + "/" + job_name + "?org_id=" + org_id
    else:
      url = self._train_v2_url + "/" + job_name
    response = self.process_request('get', url, auth=self._auth)
    logging.debug("response.content: {}".format(response.content))
    return self.process_response(response, 'data')

  def delete_train_v2(self, job_name, org_id=None):
    """Delete the train_v2.

    Args:
      job_name: The name of the train_v2.
      org_id: The org_id of the train_v2

    Returns:
      The response.
    """
    url = self._train_v2_url + "/" + job_name
    if org_id:
      url = url + "?org_id=" + org_id
    response = self.process_request('delete', url, auth=self._auth)
    return self.process_response(response)

  def _response_to_json(self, response):
    if response.ok:
      return json.loads(response.content.decode("utf-8"))
    else:
      print("response: {}".format(response.content))
      return str(response.content)

  def create_deploy(self, j):
    url = self._deploy_url + "/create"
    response = self.process_request('post', url, data=j, auth=self._auth, headers=self._headers)
    return self._response_to_json(response)

  def list_deploy(self):
    url = self._deploy_url + "/list"
    response = self.process_request('get', url, auth=self._auth, headers=self._headers)
    return self._response_to_json(response)

  def deploy_info(self, op, name, version):
    ul = [
      self._deploy_url,
      self.org_id,
      name,
      version
    ]
    url = '/'.join(ul)
    if op == "describe":
      response = self.process_request('get', url, auth=self._auth, headers=self._headers)
    elif op == "delete":
      response = self.process_request('delete', url, auth=self._auth, headers=self._headers)
    elif op == "status":
      url += "/status"
      response = self.process_request('get', url, auth=self._auth, headers=self._headers)
    elif op == "cmd":
      url += "/cmd"
      response = self.process_request('get', url, auth=self._auth, headers=self._headers)
    elif op == "logs":
      url += "/logs"
      response = self.process_request('get', url, auth=self._auth, headers=self._headers)
    elif op == "event":
      url += "/event"
      response = self.process_request('get', url, auth=self._auth, headers=self._headers)
    else:
      url = url + "/" + op
      response = self.process_request('get', url, auth=self._auth, headers=self._headers)
    return self._response_to_json(response)

  def deploy_cmd(self, op, name, version):
    ul = [
      self._deploy_url,
      self.org_id,
      name,
      version,
      "cmd"
    ]
    url = '/'.join(ul)
    data = {
      'cmd':op
    }
    response = self.process_request('post', url, data=json.dumps(data), auth=self._auth, headers=self._headers)
    return self._response_to_json(response)

  def create_model_service(self, model_service):
    """Create the model service.

    Args:
      model_service: The model service object to create.

    Returns:
      The dictionary of model service.
    """
    url = self._model_url_v2 + "/create"
    model_service_data = model_service.get_json_data()
    response = self.process_request('post', url, auth=self._auth, data=model_service_data, headers=self._headers)
    return self.process_response(response)

  def list_model_services(self, org_id=None, model_name=None, priority_class=None, page_number=None, page_size=None):
    """List model services.

    Returns:
      The list of dictionary of model services.
    """
    url = self._model_url
    params = []
    if org_id:
      params.append("org_id=" + org_id)
    if model_name:
      params.append("model_name=" + model_name)
    if priority_class:
      params.append("priority_class=" + priority_class)
    if page_number:
      params.append("page_number=" + str(page_number))
    if page_size:
      params.append("page_size=" + str(page_size))
    if params:
      url = url + "?" + "&".join(params)
    response = self.process_request('get', url, auth=self._auth, headers=self._headers)
    return self.process_response(response, 'data')

  def start_model_service(self, model_name, model_version):
    """Start the model service.

    Args:
      model_name: The name of the model service.
      model_version: The version of the model service.

    Returns:
      The dictionary of model service.
    """
    url = self._model_url + "/" + model_name + "/" + model_version + "/start"
    response = self.process_request('get', url, auth=self._auth, headers=self._headers)
    return self.process_response(response)

  def describe_model_service(self, model_name, model_version, org_id=None):
    """Describe and get information of the model service.

    Args:
      model_name: The name of the model service.
      model_version: The version of the model service.

    Returns:
      The dictionary of model service.
    """
    if org_id:
      url = self._model_url + "/" + model_name + "/" + model_version + "?org_id=" + org_id
    else:
      url = self._model_url + "/" + model_name + "/" + model_version
    response = self.process_request('get', url, auth=self._auth, headers=self._headers)
    return self.process_response(response)

  def update_model_service(self, model_name, model_version, update_json, org_id=None):
    """Describe and get information of the model service.

    Args:
      model_name: The name of the model service.
      model_version: The version of the model service.
      update_json: The json data to update the model service.

    Returns:
      The dictionary of model service.
    """
    if org_id:
      url = self._model_url + "/" + model_name + "/" + model_version + "?org_id=" + org_id
    else:
      url = self._model_url_v2 + "/" + model_name + "/" + model_version + "/update"
    response = self.process_request('put', url, auth=self._auth, data=update_json, headers=self._headers)
    return self._response_to_json(response)

  def rollback_model_service(self, model_name, model_version, org_id=None):
      url = self._model_url_v2 + "/history/rollback"
      param = {
        "model_name": model_name,
        "model_version": model_version,
      }
      if org_id:
        param["org_id"] = org_id
      print(url)
      response = self.process_request('get', url, auth=self._auth, headers=self._headers, params=param)
      return self.process_response(response)

  def get_model_service_logs(self, model_name, model_version, org_id=None, replica_index=None, proxy_logs=None):
    """Get logs of the model service.

    Args:
      model_name: The name of the model service.
      model_version: The version of the model service.
      org_id: the client's orgid
      replica_index: the replica's index,

    Returns:
      The logs of the model service.
    """
    if org_id:
      if replica_index:
        url = self._model_url + "/" + model_name + "/" + model_version + "/logs" + "?org_id=" + org_id + \
              "&replica=" + replica_index
      elif proxy_logs:
        url = self._model_url + "/" + model_name + "/" + model_version + "/logs" + "?org_id=" + org_id + \
              "&proxy=" + "1"
      else:
        url = self._model_url + "/" + model_name + "/" + model_version + "/logs" + "?org_id=" + org_id
    else:
      if replica_index:
        url = self._model_url + "/" + model_name + "/" + model_version + "/logs" + "?replica=" + replica_index
      elif proxy_logs:
        url = self._model_url + "/" + model_name + "/" + model_version + "/logs" + "?proxy=" + "1"
      else:
        url = self._model_url + "/" + model_name + "/" + model_version + "/logs"

    response = self.process_request('get', url, auth=self._auth, headers=self._headers)
    return self.process_response(response)

  def get_model_service_metrics(self, model_name, model_version, org_id=None):
    """Get the metrics of the model service.

    Args:
      model_name: The name of the model service.
      model_version: The version of the model service.

    Returns:
      The logs of the model service.
    """
    if org_id:
      url = self._model_url + "/" + model_name + "/" + model_version + "/metrics" + "?org_id=" + org_id
    else:
      url = self._model_url + "/" + model_name + "/" + model_version + "/metrics"
    response = self.process_request('get', url, auth=self._auth, headers=self._headers)
    return self.process_response(response)

  def delete_model_service(self, model_name, model_version, is_history=False, org_id=None):
    """Delete the model service.

    Args:
      model_name: The name of the model service.
      model_version: The version of the model service.

    Returns:
      The response.
    """
    url = self._model_url_v2 + "/" + model_name + "/" + model_version + "/delete"
    if org_id:
      url = url + "?org_id=" + org_id
    if is_history:
      url = url + "?is_history=" + str(is_history)
    response = self.process_request('delete', url, auth=self._auth, headers=self._headers)
    return self.process_response(response)

  def get_model_service_events(self, model_name, model_version, org_id=None):
    """Get events of the model service.

    Args:
      model_name: The name of the model service.
      model_version: The version of the model service.

    Returns:
      The events of the model service.
    """
    if org_id:
      url = self._model_url + "/" + model_name + "/" + model_version + "/events" + "?org_id=" + org_id
    else:
      url = self._model_url + "/" + model_name + "/" + model_version + "/events"
    response = self.process_request('get', url, auth=self._auth, headers=self._headers)
    return self.process_response(response)

  def restart_model_service_instance(self, model_name, model_version, org_id=None):
    url = self._model_url_v2 + "/" + model_name + "/" + model_version + "/restart"
    if org_id:
      url = url + "?org_id=" + org_id
    response = self.process_request('get', url, auth=self._auth, headers=self._headers)
    return self.process_response(response)

  def create_dev_env(self, dev_env_data):
    """Create the dev env.

    Args:
      dev_env: The dev env object to submit.

    Returns:
      The dictionary of dev env.
    """
    response = self.process_request('post', self._dev_url, auth=self._auth, data=dev_env_data)
    return self.process_response(response)

  def list_dev_envs(self, org_id=None, priority_class=None, page_number=None, page_size=None):
    """List the dev environments.

    Returns:
      The list of dictionary of dev env.
    """
    url = self._dev_url
    params = []
    if org_id:
      params.append("org_id=" + org_id)
    if priority_class:
      params.append("priority_class=" + priority_class)
    if page_number:
      params.append("page_number=" +  str(page_number))
    if page_size:
      params.append("page_size=" +  str(page_size))
    if params:
      url = url + "?" + "&".join(params)
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response, 'data')

  def describe_dev_env(self, dev_name, org_id=None):
    """Describe and get information of the dev environment.

    Args:
      dev_name: The name of the dev environment.

    Returns:
      The dictionary of dev environment.
    """
    url = self._dev_url + "/" + dev_name
    if org_id:
      url = url + "?org_id=" + org_id
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)

  def delete_dev_env(self, dev_name, org_id=None):
    """Delete the dev environment.

    Args:
      dev_name: The name of the dev environment.

    Returns:
      The response.
    """
    url = self._dev_url + "/" + dev_name
    if org_id:
      url = url + "?org_id=" + org_id
    response = self.process_request('delete', url, auth=self._auth)
    return self.process_response(response)

  def stop_dev_env(self, dev_name, org_id=None):
    """Stop the dev env.

    Args:
      dev_name: The name of the dev environment.

    Returns:
      The events of the dev env.
    """
    url = self._dev_url + "/" + dev_name + "/stop"
    if org_id:
      url = url + "?org_id=" + org_id
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)


  def v2_stop_dev_env(self, dev_name, org_id=None):
    """Stop the dev env.

    Args:
      dev_name: The name of the dev environment.

    Returns:
      The events of the dev env.
    """
    url = self._dev_url_v2 + "/" + dev_name + "/stop"
    if org_id:
      url = url + "?org_id=" + org_id
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)

  def save_dev_env(self, dev_name, org_id=None):
    """Save the dev env.

    Args:
      dev_name: The name of the dev environment.

    Returns:
      The events of the dev env.
    """
    url = self._dev_url + "/" + dev_name + "/save"
    if org_id:
      url = url + "?org_id=" + org_id
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)

  def v2_save_dev_env(self, dev_name, env_name, is_force=False, org_id=None):
    """Save the dev env.

    Args:
      dev_name: The name of the dev environment.

    Returns:
      The events of the dev env.
    """
    is_force = 1 if is_force else 0
    url = self._dev_url_v2 + "/" + dev_name + "/save/" + env_name + "?is_force=" + str(is_force)
    if org_id:
      url = url + "&org_id=" + org_id

    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)

  def restart_dev_env(self, dev_name, org_id=None):
    """Restart the dev env.

    Args:
      dev_name: The name of the dev environment.

    Returns:
      The events of the dev env.
    """
    url = self._dev_url + "/" + dev_name + "/restart"
    if org_id:
      url = url + "?org_id=" + org_id
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)

  def scale_dev_env(self, dev_name, dev_env_data, org_id=None):
    """Scale the dev env.

    Args:
      dev_name: The name of the dev environment.

    Returns:
      The events of the dev env.
    """
    url = self._dev_url + "/" + dev_name + "/scale"
    if org_id:
      url = url + "?org_id=" + org_id
    response = self.process_request('post', url, auth=self._auth, data=dev_env_data)
    return self.process_response(response)

  def get_dev_env_events(self, dev_name):
    """Get events of the dev env.

    Args:
      dev_name: The name of the dev environment.

    Returns:
      The events of the dev env.
    """
    url = self._dev_url + "/" + dev_name + "/events"
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)

  def get_dev_env_metrics(self, dev_name):
    """Get the metrics of the dev env.

    Args:
      dev_name: The name of the dev environment.

    Returns:
      The events of the dev env.
    """
    url = self._dev_url + "/" + dev_name + "/metrics"
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)

  def create_dev_server(self, dev_server):
    """Create the dev server.

    Args:
      dev_server: The dev server object to submit.

    Returns:
      The dictionary of dev server.
    """
    dev_server_data = dev_server.get_json_data()
    response = self.process_request('post', self._dev_server_url,
                             auth=self._auth,
                             data=dev_server_data)
    return self.process_response(response)

  def list_dev_servers(self, org_id=None):
    """List the dev servers.

    Returns:
      The list of dictionary of dev server.
    """
    if org_id:
      url = self._dev_server_url + "?org_id=" + org_id
    else:
      url = self._dev_server_url
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response, 'data')

  def describe_dev_server(self, dev_name, org_id=None):
    """Describe and get information of the dev server.

    Args:
      dev_name: The name of the dev server.

    Returns:
      The dictionary of dev server.
    """
    if org_id:
      url = self._dev_server_url + "/" + dev_name + "?org_id=" + org_id
    else:
      url = self._dev_server_url + "/" + dev_name
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)

  def delete_dev_server(self, dev_name):
    """Delete the dev server.

    Args:
      dev_name: The name of the dev server.

    Returns:
      The response.
    """
    url = self._dev_server_url + "/" + dev_name
    response = self.process_request('delete', url, auth=self._auth)
    return self.process_response(response)

  def get_dev_server_events(self, dev_name):
    """Get events of the dev server.

    Args:
      dev_name: The name of the dev server.

    Returns:
      The events of the dev server.
    """
    url = self._dev_server_url + "/" + dev_name + "/events"
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)

  def create_tensorboard_service(self, tensorboard_service):
    """Create the tensorboard_service.

    Args:
      tensorboard_service: The tensorboard_service object to create.

    Returns:
      The dictionary of tensorboard_service.
    """
    tensorboard_service_data = tensorboard_service.get_json_data()
    response = self.process_request('post', self._tensorboard_url,
                             auth=self._auth,
                             data=tensorboard_service_data)
    return self.process_response(response)

  def list_tensorboard_services(self, org_id=None, priority_class=None, page_number=None, page_size=None):
    """List tensorboard_services.

    Returns:
      The list of dictionary of tensorboard_services.
    """
    url = self._tensorboard_url
    params = []
    if org_id:
      params.append("org_id=" + org_id)
    if priority_class:
      params.append("priority_class=" + priority_class)
    if page_number:
      params.append("page_number=" + str(page_number))
    if page_size:
      params.append("page_size=" + str(page_size))
    if params:
      url = url + "?" + "&".join(params)
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response, 'data')

  def describe_tensorboard_service(self, tensorboard_name, org_id=None):
    """Describe and get information of the tensorboard_service.

    Args:
      tensorboard_name: The name of the tensorboard_service.

    Returns:
      The dictionary of tensorboard_service.
    """
    if org_id:
      url = self._tensorboard_url + "/" + tensorboard_name + "?org_id=" + org_id
    else:
      url = self._tensorboard_url + "/" + tensorboard_name
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)

  def delete_tensorboard_service(self, tensorboard_name, org_id=None):
    """Delete the tensorboard_service.

    Args:
      tensorboard_name: The name of the tensorboard_service.

    Returns:
      The response.
    """
    url = self._tensorboard_url + "/" + tensorboard_name
    if org_id:
      url = url + "?org_id=" + org_id
    response = self.process_request('delete', url, auth=self._auth)
    return self.process_response(response)

  def get_tensorboard_service_events(self, tensorboard_name):
    """Get events of the tensorboard service.

    Args:
      tensorboard_name: The name of the tensorboard service.

    Returns:
      The events of the tensorboard service.
    """
    url = self._tensorboard_url + "/" + tensorboard_name + "/events"
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)

  def create_pipeline_schedule(self, json_data, pipeline_name):
    url = self._pipeline_url + '/' + pipeline_name + "/schedule"
    response = self.process_request('post', url, auth=self._auth, data=json_data)
    return self.process_response(response)

  def list_pipeline_schedules(self, pipeline_name, org_id=None):
    if org_id:
      url = self._pipeline_url + '/' + pipeline_name + "/schedule?org_id=" + org_id
    else:
      url = self._pipeline_url + '/' + pipeline_name + "/schedule"
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response, 'data')

  def describe_pipeline_schedule(self, pipeline_schedule_id, org_id=None):
    if org_id:
      url = self._pipeline_url + '/schedule/' + pipeline_schedule_id +"?org_id=" + org_id
    else:
      url = self._pipeline_url + '/schedule/' + pipeline_schedule_id
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)

  def delete_pipeline(self, pipeline_name):
    """Delete the pipeline.

    Args:
      pipeline_name: The name of the pipeline.
    Returns:
      The response.
    """
    url = self._pipeline_url + "/" + pipeline_name

    response = self.process_request('delete', url, auth=self._auth)
    return self.process_response(response)

  def list_pipelines(self, org_id=None, priority_class=None, page_number=None, page_size=None):
    url = self._pipeline_url
    params = []
    if org_id:
      params.append("org_id=" + org_id)
    if priority_class:
      params.append("priority_class=" + priority_class)
    if page_number:
      params.append("page_number=" + str(page_number))
    if page_size:
      params.append("page_size=" + str(page_size))
    if params:
      url = url + "?" + "&".join(params)
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response, 'data')

  def delete_pipeline(self, pipeline_name):
    """Delete the pipeline.

    Args:
      pipeline_name: The name of the pipeline.
    Returns:
      The response.
    """
    url = self._pipeline_url + "/" + pipeline_name

    response = self.process_request('delete', url, auth=self._auth)
    return self.process_response(response)

  def describe_pipeline(self, pipeline_name, org_id=None):
    """Describe and get information of the pipeline.

    Args:
      pipeline_name: The name of the pipeline.

    Returns:
      The dictionary of pipeline.
    """
    if org_id:
      url = self._pipeline_url + "/" + pipeline_name + "?org_id=" + org_id
    else:
      url = self._pipeline_url + "/" + pipeline_name
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)

  def get_pipeline_logs(self, pod_name, org_id=None):
    """Get logs of the pipeline.

    Args:
      pipeline_name: The pod name of the pipeline.

    Returns:
      The logs of pipeline pod.
    """
    if org_id:
      url = self._pipeline_url + "/" + pod_name + "/logs" + "?org_id=" + org_id
    else:
      url = self._pipeline_url + "/" + pod_name + "/logs"
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)

  def start_pipeline(self, pipeline_name, org_id=None):
    if org_id:
      url = self._pipeline_url + "/" + pipeline_name + "/start" + "?org_id=" + org_id
    else:
      url = self._pipeline_url + "/" + pipeline_name + "/start"
    response = self.process_request('put', url, auth=self._auth)
    return self.process_response(response)

  def stop_pipeline(self, pipeline_name, org_id=None):
    if org_id:
      url = self._pipeline_url + "/" + pipeline_name + "/stop" + "?org_id=" + org_id
    else:
      url = self._pipeline_url + "/" + pipeline_name + "/stop"
    response = self.process_request('put', url, auth=self._auth)
    return self.process_response(response)

  def rerun_pipeline(self, pipeline_name, json_data):
    url = self._pipeline_url + "/" + pipeline_name + "/rerun"
    response = self.process_request('post', url, auth=self._auth, data=json_data)
    return self.process_response(response)

  def create_pipeline(self, json_data):
    """Submit a pipeline to run.

    Args:
      json_data: The json data of pipeline to submit.

    Returns:
      The dictionary of pipeline.
    """
    response = self.process_request('post', self._pipeline_url, auth=self._auth, data=json_data)
    return self.process_response(response)

  def update_pipeline(self, json_data, pipeline_name):
    """Submit a pipeline to run.

    Args:
      json_data: The json data of pipeline to submit.

    Returns:
      The dictionary of pipeline.
    """
    url = self._pipeline_url + "/" + pipeline_name
    response = self.process_request('put', url, auth=self._auth, data=json_data)
    return self.process_response(response)

  def list_schedules(self, org_id=None):
    if org_id:
      url = self._schedule_url + "?org_id=" + org_id
    else:
      url = self._schedule_url
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response, 'data')

  def delete_schedule(self, schedule_name):
    """Delete the schedule.

    Args:
      schedule_name: The name of the schedule.
    Returns:
      The response.
    """
    url = self._schedule_url + "/" + schedule_name

    response = self.process_request('delete', url, auth=self._auth)
    return self.process_response(response)

  def describe_schedule(self, schedule_name, org_id=None):
    """Describe and get information of the schedule.

    Args:
      schedule_name: The name of the schedule.

    Returns:
      The dictionary of schedule.
    """
    if org_id:
      url = self._schedule_url + "/" + schedule_name + "?org_id=" + org_id
    else:
      url = self._schedule_url + "/" + schedule_name
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)


  def start_schedule(self, schedule_name, org_id=None):
    if org_id:
      url = self._schedule_url + "/" + schedule_name + "/start" + "?org_id=" + org_id
    else:
      url = self._schedule_url + "/" + schedule_name + "/start"
    response = self.process_request('put', url, auth=self._auth)
    return self.process_response(response)

  def stop_schedule(self, schedule_name, org_id=None):
    if org_id:
      url = self._schedule_url + "/" + schedule_name + "/stop" + "?org_id=" + org_id
    else:
      url = self._schedule_url + "/" + schedule_name + "/stop"
    response = self.process_request('put', url, auth=self._auth)
    return self.process_response(response)

  def get_schedule_logs(self, schedule_name, org_id=None):
    if org_id:
      url = self._schedule_url + "/" + schedule_name + "/logs" + "?org_id=" + org_id
    else:
      url = self._schedule_url + "/" + schedule_name + "/logs"
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)

  def create_schedule(self, json_data):
    """Submit a schedule to run.

    Args:
      json_data: The json data of schedule to submit.

    Returns:
      The dictionary of schedule.
    """
    response = self.process_request('post', self._schedule_url, auth=self._auth, data=json_data)
    return self.process_response(response)

  def update_schedule(self, json_data, schedule_name):
    """Update a schedule to run.

    Args:
      json_data: The json data of schedule to submit.

    Returns:
      The dictionary of schedule.
    """
    url = self._schedule_url + "/" + schedule_name
    response = self.process_request('put', url, auth=self._auth, data=json_data)
    return self.process_response(response)


  def create_ceph_service(self, ceph_service):
    """Create the ceph_service.

    Args:
      ceph_service: The ceph_service object to create.

    Returns:
      The dictionary of ceph_service.
    """
    ceph_service_data = ceph_service.get_json_data()
    response = self.process_request('post', self._ceph_url,
                             auth=self._auth,
                             data=ceph_service_data)
    return self.process_response(response)

  def list_ceph_services(self, org_id=None):
    """List ceph_services.

    Returns:
      The list of dictionary of ceph_services.
    """
    if org_id:
      url = self._ceph_url + "?org_id=" + org_id
    else:
      url = self._ceph_url
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response, 'data')

  def describe_ceph_service(self, ceph_name, org_id=None):
    """Describe and get information of the ceph_service.

    Args:
      ceph_name: The name of the ceph_service.

    Returns:
      The dictionary of ceph_service.
    """
    if org_id:
      url = self._ceph_url + "/" + ceph_name + "?org_id=" + org_id
    else:
      url = self._ceph_url + "/" + ceph_name
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)

  def delete_ceph_service(self, ceph_name, org_id=None, force=False):
    """Delete the ceph_service.

    Args:
      ceph_name: The name of the ceph_service.

    Returns:
      The response.
    """
    url = self._ceph_url + "/" + ceph_name
    if org_id != None:
      url = url + "?org_id=" + org_id
    if force != None:
      url = url + "?force=" + str(force)
    response = self.process_request('delete', url, auth=self._auth)
    return self.process_response(response)

  def get_ceph_service_events(self, ceph_name):
    """Get events of the ceph service.

    Args:
      ceph_name: The name of the ceph service.

    Returns:
      The events of the ceph service.
    """
    url = self._ceph_url + "/" + ceph_name + "/events"
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)

  def sync_ceph_from_fds(self, ceph_name, fds_endpoint, fds_bucket):
    """Sync fds files to ceph volume.

    Args:
      ceph_name: The name of the ceph volume.
      fds_endpoint: The fds endpoint.
      fds_bucket: The name of fds bucket.

    Returns:
      response message
    """
    data = json.dumps({
        "fds_endpoint": fds_endpoint,
        "fds_bucket": fds_bucket
    })
    url = self._ceph_url + "/" + ceph_name + "/sync"
    response = self.process_request('post', url,
                             auth=self._auth,
                             data=data)
    return self.process_response(response)

  def list_envs(self, org_id=None):
    """List envs.

    Returns:
      The list of dictionary of envs.
    """
    if org_id:
      url = self._env_url + "?org_id=" + org_id
    else:
      url = self._env_url
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response, 'data')

  def describe_env(self, env_name, org_id=None):
    """Describe and get information of the env.

    Args:
      env_name: The name of the env.

    Returns:
      The dictionary of env.
    """
    if org_id:
      url = self._env_url + "/" + env_name + "?org_id=" + org_id
    else:
      url = self._env_url + "/" + env_name
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response)

  def delete_env(self, env_name, org_id=None):
    """Delete the env.

    Args:
      ceph_name: The name of the env.

    Returns:
      The response.
    """
    url = self._env_url + "/" + env_name
    if org_id != None:
      url = url + "?org_id=" + org_id
    response = self.process_request('delete', url, auth=self._auth)
    return self.process_response(response)



  def get_quota(self, org_id=None):
    """Get quota.
    """
    # NOTE(jishaomin): cloudml_admin command corresponds to org_id, cloudml command corresponds to no org_id
    if org_id:
      url = self._quota_url + "?org_id=" + org_id
      response = self.process_request('get', url, auth=self._auth)
    else:
      url = self._quota_url
      body = {}
      body['org_mail'] = self._org_mail
      body = json.dumps(body)
      response = self.process_request('post', url, auth=self._auth, data=body)

    return self.process_response(response, 'data')

  def do_predict(self, model_name, model_version, data_file, timeout=10.0):
    """Request generic gRPC server to predict

    Args:
      model_name: The name of the model.
      model_version: The version of the model.
      data_file: The json data file.
      timeout: The timeout of the gRPC request.
    """
    model_service = self.describe_model_service(model_name, model_version)
    if type(model_service) is dict:
      server = model_service["address"]
      return self.do_predict_server(server, model_name, data_file, timeout)
    else:
      return json.dumps({
        "error": True,
        "message": "Fail to get information of model service"
      })

  def do_predict_server(self, server, model_name, data_file, timeout=10.0):

    from .predict_client import generic_predict_client

    if os.path.isfile(data_file):
      with open(data_file) as f:
        data = json.load(f)
    else:
      return {
        "error": True,
        "message": "The data file: {} doesn't exist".format(data_file)
      }

    return generic_predict_client.predict(server, model_name, data, timeout)

  def model_review(self, review_data):
    """Get the metrics of the model service.

    Args:
      model_name: The name of the model service.
      model_version: The version of the model service.

    Returns:
      The logs of the model service.
    """
    url = self._model_url_v2 + "/review"
    body = json.dumps(review_data)
    response = self.process_request('post', url, auth=self._auth, data=body)

    return self.process_response(response)

  def model_onboard(self, model_name, model_version, org_id=None):
    url = self._model_url_v2 + "/" + model_name + "/" + model_version + "/onboard"
    if org_id:
      url = url + "?org_id=" + org_id
    response = self.process_request('post', url, auth=self._auth)
    return self.process_response(response)

  def model_daildown(self, model_name, model_version, org_id=None):
    url = self._model_url_v2 + "/" + model_name + "/" + model_version + "/daildown"
    if org_id:
      url = url + "?org_id=" + org_id
    response = self.process_request('delete', url, auth=self._auth)
    return self.process_response(response)

  def update_quota(self, quota, allow_third_level_leader=False):
    """Update quota by admin.

    Args:
      quota: The quota object with new value.
    """
    quota_data = json.loads(quota.get_json_data())
    quota_data['allow_third_level_leader'] = allow_third_level_leader

    url = self._quota_url + "/" + quota.org_id
    response = self.process_request('put', url, auth=self._auth, data=json.dumps(quota_data))
    return self.process_response(response)

  def apply_quota(self, quota_data):
    """Update quota by admin.

    Args:
      quota: The quota object with new value.
    """
    url = self._quota_application_url
    body = json.dumps(quota_data)
    response = self.process_request('post', url,
                             auth=self._auth,
                             data=body)
    return self.process_response(response)

  def create_secret_service(self, secret_service):
    """Create the secret_service.

    Args:
      secret_service: The secret_service object to create.

    Returns:
      The dictionary of secret_service.
    """
    secret_service_data = secret_service.get_json_data()
    response = self.process_request('post', self._secret_url,
                             auth=self._auth,
                             data=secret_service_data)
    return self.process_response(response, 'data')

  def list_secret_services(self, org_id=None):
    """List secret_services.

    Returns:
      The list of dictionary of secret_services.
    """
    if org_id:
      url = self._secret_url + "?org_id=" + org_id
    else:
      url = self._secret_url
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response, 'data')

  def describe_secret_service(self, secret_name, org_id=None):
    """Describe and get information of the secret_service.

    Args:
      secret_name: The name of the secret_service.
      org_id: The org_id of the secret_service

    Returns:
      The dictionary of secret_service.
    """
    if org_id:
      url = self._secret_url + "/" + secret_name + "?org_id=" + org_id
    else:
      url = self._secret_url + "/" + secret_name
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response, 'data')

  def delete_secret_service(self, secret_name, org_id=None):
    """Delete the secret_service.

    Args:
      secret_name: The name of the secret_service.
      org_id: The org_id of the secret_service

    Returns:
      The response.
    """
    url = self._secret_url + "/" + secret_name
    if org_id:
      url = url + "?org_id=" + org_id
    response = self.process_request('delete', url, auth=self._auth)
    return self.process_response(response)

  def create_hdfs_secret_service(self, secret_service):
    """Create the secret_service.

    Args:
      secret_service: The secret_service object to create.

    Returns:
      The dictionary of secret_service.
    """
    secret_service_data = secret_service.get_json_data()
    response = self.process_request('post', self._hdfs_secret_url,
                             auth=self._auth,
                             data=secret_service_data)
    return self.process_response(response, 'data')

  def list_hdfs_secret_services(self, org_id=None):
    """List secret_services.

    Returns:
      The list of dictionary of secret_services.
    """
    if org_id:
      url = self._hdfs_secret_url + "?org_id=" + org_id
    else:
      url = self._hdfs_secret_url
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response, 'data')

  def describe_hdfs_secret_service(self, secret_name, org_id=None):
    """Describe and get information of the secret_service.

    Args:
      secret_name: The name of the secret_service.
      org_id: The org_id of the secret_service

    Returns:
      The dictionary of secret_service.
    """
    if org_id:
      url = self._hdfs_secret_url + "/" + secret_name + "?org_id=" + org_id
    else:
      url = self._hdfs_secret_url + "/" + secret_name
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response, 'data')

  def delete_hdfs_secret_service(self, secret_name, org_id=None):
    """Delete the secret_service.

    Args:
      secret_name: The name of the secret_service.
      org_id: The org_id of the secret_service

    Returns:
      The response.
    """
    url = self._hdfs_secret_url + "/" + secret_name
    if org_id:
      url = url + "?org_id=" + org_id
    response = self.process_request('delete', url, auth=self._auth)
    return self.process_response(response)


  def create_fds_secret_service(self, secret_service):
    """Create the secret_service.

    Args:
      secret_service: The secret_service object to create.

    Returns:
      The dictionary of secret_service.
    """
    secret_service_data = secret_service.get_json_data()
    response = self.process_request('post', self._fds_secret_url,
                             auth=self._auth,
                             data=secret_service_data)
    return self.process_response(response, 'data')

  def list_fds_secret_services(self, org_id=None):
    """List secret_services.

    Returns:
      The list of dictionary of secret_services.
    """
    if org_id:
      url = self._fds_secret_url + "?org_id=" + org_id
    else:
      url = self._fds_secret_url
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response, 'data')

  def describe_fds_secret_service(self, secret_name, org_id=None):
    """Describe and get information of the secret_service.

    Args:
      secret_name: The name of the secret_service.
      org_id: The org_id of the secret_service

    Returns:
      The dictionary of secret_service.
    """
    if org_id:
      url = self._fds_secret_url + "/" + secret_name + "?org_id=" + org_id
    else:
      url = self._fds_secret_url + "/" + secret_name
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response, 'data')

  def delete_fds_secret_service(self, secret_name, org_id=None):
    """Delete the secret_service.

    Args:
      secret_name: The name of the secret_service.
      org_id: The org_id of the secret_service

    Returns:
      The response.
    """
    url = self._fds_secret_url + "/" + secret_name
    if org_id:
      url = url + "?org_id=" + org_id
    response = self.process_request('delete', url, auth=self._auth)
    return self.process_response(response)


  def create_git_secret_service(self, secret_service):
    """Create the secret_service.

    Args:
      secret_service: The secret_service object to create.

    Returns:
      The dictionary of secret_service.
    """
    secret_service_data = secret_service.get_json_data()
    response = self.process_request('post', self._git_secret_url,
                             auth=self._auth,
                             data=secret_service_data)
    return self.process_response(response, 'data')

  def list_git_secret_services(self, org_id=None):
    """List secret_services.

    Returns:
      The list of dictionary of secret_services.
    """
    if org_id:
      url = self._git_secret_url + "?org_id=" + org_id
    else:
      url = self._git_secret_url
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response, 'data')

  def describe_git_secret_service(self, secret_name, org_id=None):
    """Describe and get information of the secret_service.

    Args:
      secret_name: The name of the secret_service.
      org_id: The org_id of the secret_service

    Returns:
      The dictionary of secret_service.
    """
    if org_id:
      url = self._git_secret_url + "/" + secret_name + "?org_id=" + org_id
    else:
      url = self._git_secret_url + "/" + secret_name
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response, 'data')

  def delete_git_secret_service(self, secret_name, org_id=None):
    """Delete the secret_service.

    Args:
      secret_name: The name of the secret_service.
      org_id: The org_id of the secret_service

    Returns:
      The response.
    """
    url = self._git_secret_url + "/" + secret_name
    if org_id:
      url = url + "?org_id=" + org_id
    response = self.process_request('delete', url, auth=self._auth)
    return self.process_response(response)

  def create_namespace(self, namespace):
    """Create the namespace.

    Args:
      namespace: The namespace object to create.

    Returns:
      The dictionary of namespace.
    """
    namespace_json = namespace.get_json_data()
    response = self.process_request('post', self._namespace_url,
                             auth=self._auth,
                             data=namespace_json)
    return self.process_response(response, 'data')

  def update_namespace(self, namespace_json):
    """Update the namespace.

    Args:
      namespace_json: The namespace json to update with.

    Returns:
      The dictionary of namespace.
    """
    response = self.process_request('patch', self._namespace_url,
                              auth=self._auth,
                              data=namespace_json)
    return self.process_response(response, 'data')

  def list_namespaces(self):
    """List namespaces.

    Returns:
      The list of dictionary of namespace.
    """
    response = self.process_request('get', self._namespace_url, auth=self._auth)
    return self.process_response(response, 'data')

  def describe_namespace(self, name):
    """Describe and get information of the namespace.

    Args:
      name: The name of the namespace.

    Returns:
      The dictionary of namespace.
    """
    url = self._namespace_url + "/" + name
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response, 'data')

  def delete_namespace(self, name):
    """Delete the namespace.

    Args:
      name: The name of the namespace.

    Returns:
      The response.
    """
    url = self._namespace_url + "/" + name
    response = self.process_request('delete', url, auth=self._auth)
    return self.process_response(response)

  def create_priority_class(self, priority_class):
    """Create the priority_class.

    Args:
      priority_class: The priority_class object to create.

    Returns:
      The dictionary of priority_class.
    """
    priority_class_json = priority_class.get_json_data()
    response = self.process_request('post', self._priority_class_url,
                             auth=self._auth,
                             data=priority_class_json)
    return self.process_response(response, 'data')

  def update_priority_class(self, priority_class_json):
    """Update the priority_class.

    Args:
      priority_class_json: The priority_class json to update with.

    Returns:
      The dictionary of priority_class.
    """
    response = self.process_request('patch', self._priority_class_url,
                              auth=self._auth,
                              data=priority_class_json)
    return self.process_response(response, 'data')

  def list_priority_classes(self):
    """List priority_classes.

    Returns:
      The list of dictionary of priority_class.
    """
    response = self.process_request('get', self._priority_class_url, auth=self._auth)
    return self.process_response(response, 'data')

  def describe_priority_class(self, name):
    """Describe and get information of the priority_class.

    Args:
      name: The name of the priority_class.

    Returns:
      The dictionary of priority_class.
    """
    url = self._priority_class_url + "/" + name
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response, 'data')

  def delete_priority_class(self, name):
    """Delete the priority_class.

    Args:
      name: The name of the priority_class.

    Returns:
      The response.
    """
    url = self._priority_class_url + "/" + name
    response = self.process_request('delete', url, auth=self._auth)
    return self.process_response(response)

  def create_namespace_quota(self, namespace_quota):
    """Create the namespace quota.

    Args:
      namespace_quota: The namespace quota object to create/update.

    Returns:
      The dictionary of namespace quota.
    """
    namespace_quota_json = namespace_quota.get_json_data()
    response = self.process_request('post', self._namespace_quota_url,
                             auth=self._auth,
                             data=namespace_quota_json)
    return self.process_response(response, 'data')

  def update_namespace_quota(self, namespace_quota):
    """Update the namespace quota.

    Args:
      namespace_quota: The namespace quota object to create/update.

    Returns:
      The dictionary of namespace quota.
    """
    namespace_quota_json = namespace_quota.get_json_data()
    response = self.process_request('patch', self._namespace_quota_url,
                              auth=self._auth,
                              data=namespace_quota_json)
    return self.process_response(response, 'data')

  def list_namespace_quotas(self, namespace, priority_class):
    """List namespace quotas.

    Returns:
      The list of dictionary of namespace quota.
    """
    url = self._namespace_quota_url
    params = []
    if namespace:
      params.append("namespace=" + namespace)
    if priority_class:
      params.append("priority_class=" + priority_class)
    if params:
      url = url + "?" + "&".join(params)
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response, 'data')

  def describe_namespace_quota(self, namespace, priority_class):
    """Describe and get information of the namespace quota.

    Args:
      namespace: The name of the namespace.

    Returns:
      The dictionary of namespace quota.
    """
    url = self._namespace_quota_url + "/" + namespace
    if priority_class:
      url = url + "?priority_class=" + priority_class
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response, 'data')

  def delete_namespace_quota(self, namespace, priority_class):
    """Delete the namespace_quota.

    Args:
      namespace: The name of the namespace_quota.

    Returns:
      The response.
    """
    url = self._namespace_quota_url + "/" + namespace
    if priority_class:
      url = url + "?priority_class=" + priority_class
    response = self.process_request('delete', url, auth=self._auth)
    return self.process_response(response)

  def add_namespace_user(self, namespace_user):
    """Add the namespace user.

    Args:
      namespace_user: The namespace user object to add.

    Returns:
      The dictionary of namespace user.
    """
    namespace_user_json = namespace_user.get_json_data()
    response = self.process_request('post', self._namespace_user_url,
                             auth=self._auth,
                             data=namespace_user_json)
    return self.process_response(response, 'data')

  def remove_namespace_user(self, namespace_user):
    """Remove the namespace user.

    Args:
      namespace_user: The namespace user object to remove.

    Returns:
      The dictionary of namespace user.
    """
    namespace_user_json = namespace_user.get_json_data()
    response = self.process_request('delete', self._namespace_user_url,
                               auth=self._auth,
                               data=namespace_user_json)
    return self.process_response(response)

  def list_namespace_users(self, namespace):
    """List namespace users.

    Returns:
      The list of dictionary of namespace quota.
    """
    url = self._namespace_user_url
    if namespace:
      url = url + "?namespace=" + namespace
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response, 'data')

  def create_project_service(self, project_service):
    """Create the project_service.

    Args:
      project_service: The project_service object to create.

    Returns:
      The dictionary of project_service.
    """
    project_service_data = project_service.get_json_data()
    response = self.process_request('post', self._project_url,
                             auth=self._auth,
                             data=project_service_data)
    return self.process_response(response, 'data')

  def list_project_services(self, org_id=None):
    """List project_services.

    Returns:
      The list of dictionary of project_services.
    """
    if org_id:
      url = self._project_url + "?org_id=" + org_id
    else:
      url = self._project_url
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response, 'data')

  def describe_project_service(self, project_name, org_id=None):
    """Describe and get information of the project_service.

    Args:
      project_name: The name of the project_service.
      org_id: The org_id of the project_service

    Returns:
      The dictionary of project_service.
    """
    if org_id:
      url = self._project_url + "/" + project_name + "?org_id=" + org_id
    else:
      url = self._project_url + "/" + project_name
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response, 'data')

  def delete_project_service(self, project_name, org_id=None):
    """Delete the project_service.

    Args:
      project_name: The name of the project_service.
      org_id: The org_id of the project_service

    Returns:
      The response.
    """
    url = self._project_url + "/" + project_name
    if org_id:
      url = url + "?org_id=" + org_id
    response = self.process_request('delete', url, auth=self._auth)
    return self.process_response(response)

  def get_frameworks(self):
    response = self.process_request('get', self._framework_url)
    return self.process_response(response)

  def authentication(self):
    response = self.process_request('get', self._quota_url, auth=self._auth)
    if response.ok:
      return {"message": "Successfully verify aksk and Cloud-ML endpoint."}
    else:
      return {"error": True, "message": "Authentication failed."}

  def get_org_id(self):
    response = self.process_request('get', self._org_id_url, auth=self._auth)
    return self.process_response(response)

  def list_cluster_resources(self, org_id=None):
    if org_id:
      url = self._resources_url + "?org_id=" + org_id
    else:
      url = self._resources_url
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response, 'data')

  def list_autoscaler_rule(self, args):
    if "org_id" in args:
      url = self._autoscaler_url_v2 + "?org_id=" + args.org_id
    else:
      url = self._autoscaler_url_v2
    response = self.process_request('get', url, auth=self._auth, headers=self._headers)
    return self.process_response(response)

  def describe_autoscaler_rule(self, args, is_admin=False):
    if is_admin:
      url = self._autoscaler_url_v2 + "?org_id=" + args.org_id
    else:
      url = self._autoscaler_url_v2
    data = {"model_name": args.model_name, "model_version": args.model_version}
    response = self.process_request('get', url, data=data, auth=self._auth, headers=self._headers)
    return self.process_response(response)

  def create_autoscaler_rule(self, autoscaler, is_admin=False):
    if is_admin:
      url = self._autoscaler_url_v2 + "?org_id=" + autoscaler.org_id
    else:
      url = self._autoscaler_url_v2
    response = self.process_request('post', url, data=json.dumps(autoscaler.get_json_data()), auth=self._auth, headers=self._headers)
    return self.process_response(response)

  def delete_autoscaler_rule(self,args, is_admin=False):
    if is_admin:
      url = self._autoscaler_url_v2 + "?org_id=" + args.org_id
    else:
      url = self._autoscaler_url_v2
    data = {"model_name": args.model_name, "model_version": args.model_version}
    response = self.process_request('delete', url, data=data, auth=self._auth, headers=self._headers)
    return self._response_to_json(response)

  def update_autoscaler_rule(self, data, is_admin=False):
    if is_admin:
      url = self._autoscaler_url_v2 + "?org_id=" + data["org_id"]
      del data["org_id"]
    else:
      url = self._autoscaler_url_v2
    response = self.process_request('put', url, data=json.dumps(data), auth=self._auth, headers=self._headers)
    return self.process_response(response)

  def update_autoscaler_rule_run_status(self, data, is_admin=False):
    if is_admin:
      url = self._autoscaler_url_v2 + "?org_id=" + data["org_id"]
      del data["org_id"]
    else:
      url = self._autoscaler_url_v2
    response = self.process_request('put', url, data=json.dumps(data), auth=self._auth, headers=self._headers)
    return self.process_response(response)

  @staticmethod
  def process_response(response, select_col_name=None):
    if response.ok:
      if select_col_name:
        return json.loads(response.content.decode("utf-8"))[select_col_name]
      else:
        return json.loads(response.content.decode("utf-8"))
    else:
      print("response: {}".format(response.content))
      sys.exit(1)

  @staticmethod
  def process_request(method, url, **kwargs):
    method_to_call = getattr(requests, method)
    try:
      command_argv = GlobalArgsStore().argv
      white_list = ["models", "train", "dev", "deploy", "tensorboard", "quota"]
      url_key_words = re.split('\?|/', url)
      headers = kwargs.setdefault("headers", {})
      headers[constant.HEADER_X_XIAOMI_SDK_VERSION] = pkg_resources.require(constant.SDK_NAME)[0].version
      for one in white_list:
        if one in url_key_words:
          break
      else:
        return method_to_call(url, **kwargs)

      if len(command_argv) > 1 and command_argv[0].split('/')[-1] in ('cloudml_admin', 'cloudml'):
        command_argv[0] = 'cloudml_admin' if 'cloudml_admin' in command_argv[0] else 'cloudml'
        body = json.loads(kwargs.get('data', '{}'))
        body["origin_command"] = command_argv
        kwargs['data'] = json.dumps(body)
    except Exception as e:
      logging.warning("Error occurred when send requests, error is {}".format(e))
    return method_to_call(url, **kwargs)


  def create_model_test_case(self, testcase):
    """Create model test case.
    """
    url = self._model_testcase_url_v1
    response = self.process_request('post', url,
                             auth=self._auth,
                             data=testcase,
                             headers=self._headers)
    return self.process_response(response)

  def get_model_test_case(self, org_id):
    """
    get model test case by id.
    """
    url = '%s/%s/' % (self._model_testcase_url_v1 , str(org_id))
    response = self.process_request('get', url,
                            auth=self._auth,
                            headers=self._headers)

    return self.process_response(response)

  def get_model_test_cases_by_org_id(self):
    """
    get model test cases by org id.
    """
    url = self._model_testcase_url_v1
    response = self.process_request('get', url,
                            auth=self._auth,
                            headers=self._headers)

    return self.process_response(response)

  def delete_model_test_cases_by_org_id(self):
    """
    get model test cases by org id.
    """
    url = self._model_testcase_url_v1
    response = self.process_request('delete', url,
                               auth=self._auth,
                               headers=self._headers)

    return self.process_response(response)

  def apply_quota_v2(self, quota_data):
    """Update quota by admin.

    Args:
      quota: The quota object with new value.
    """
    url = self._quota_v2_application_url
    body = json.dumps(quota_data)
    response = self.process_request('post', url,
                             auth=self._auth,
                             data=body)
    return self.process_response(response)

  def get_quota_v2(self, org_id=None):
    """Get quota.
    """
    if org_id:
      url = self._quota_v2_url + "?org_id=" + org_id
    else:
      url = self._quota_v2_url
    response = self.process_request('get', url, auth=self._auth)
    return self.process_response(response, 'data')

  def create_dataset(self, dataset_data):
    """Create the dev env.

    Args:
      dev_env: The dev env object to submit.

    Returns:
      The dictionary of dev env.
    """
#    url = self._dataset_url + "/create"
#    response = self.process_request('post', url, auth=self._auth, data=dataset_data)
   # payload = {'dataset_name': 'dataset3', 'version': '2', 'filepath':'/home/wangteng5/arswt/Code/data_platform/photo2', 'userip': 'wangteng5@10.221.224.118', 'passwd': 'Mi#900227'}
   # r = self.process_request('get', 'http://localhost:8080/dataset/photo/upload', params=payload)
    dd = json.loads(dataset_data)

    permissionurl = constant.DATA_ENDPOINT + '/set/check/permission' + '?dataset_name=' + dd['dataset_name'] + '&version=' + dd['version'] + '&access_key=' + dd['access_key']
    permission = requests.get(permissionurl).text.encode("utf-8")
    print(permission)
    if permission == 'NoPermission':
      print("Create Failed!. You don't have permission to get this dataset.")
      return "You don't have permission to get this dataset."
    elif permission == 'Duplicate':
      print("Create Failed!. The dataset and version has exists.")
      return "The dataset and version has exists."

    uploadpath = 'upload-wt' + '/' + dd['dataset_name'] + '/' + dd['version']
    s = {'uploadpath': uploadpath}
    dataurl = constant.DATA_ENDPOINT + '/dataset'
    plist = []
    filepath = dd['filepath'];

    self.listdir(filepath, plist)
    for fname in plist:
      f = fname[len(filepath)+1:]
      subfilename = '@'.join(f.split(os.sep))
      files = {'file': (subfilename, open(fname, 'rb'))}
      r2 = requests.post(dataurl, data=s, files=files)
      print(r2.text)
    
    if dd['type'] == 'photo':
      requesturl = constant.DATA_ENDPOINT + '/dataset/photo/upload'
    elif dd['type'] == 'model':
      requesturl = constant.DATA_ENDPOINT + '/dataset/model/upload'
    else:
      requesturl = constant.DATA_ENDPOINT + '/dataset/text/upload'
    dd['filepath'] = uploadpath
    # dd['access_key'] = 'AKV2Z7YLRDKLRU4IVM'
    # dd['secret_key'] = '7rmGAdcw9nc1sqp0mdMrfTIeQy/0vdPxUrzecZR8'
    r = self.process_request('post', requesturl, params=dd)
    
    return(r.text)

  def download_dataset(self, dataset_data):
    dd = json.loads(dataset_data)
    requesturl = constant.DATA_ENDPOINT
    if dd['type'] == 'photo':
      requesturl = constant.DATA_ENDPOINT + '/set/downloadlocal' + '?dataset_name=' + dd['dataset_name'] + '&version=' + dd['version'] + '&access_key=' + dd['access_key'] + '&label_name=' + dd['label_name']
    elif dd['type'] == 'model':
      requesturl = constant.DATA_ENDPOINT + '/set/downloadlocal' + '?dataset_name=' + dd['dataset_name'] + '&version=' + dd['version'] + '&access_key=' + dd['access_key']
    else:
      requesturl = constant.DATA_ENDPOINT + '/set/downloadtexttolocal' + '?dataset_name=' + dd['dataset_name'] + '&version=' + dd['version'] + '&bucket=' + dd['bucket'] + '&fileDir=' + dd['filepath'] + '&access_key=' + dd['access_key']
      # requests.get(requesturl, auth=self._auth)
    relist = requests.get(requesturl, auth=self._auth).text
    result = relist

    relist = json.loads(relist)
    path = dd['filepath']
    if not os.path.exists(path):
      os.makedirs(path)
    downloadpathlen = len(constant.DATA_DOWNLOAD_DIR)
    for f in relist:
      fplist = f[downloadpathlen:].split('@')
      fname = os.sep.join(fplist)
      fname = fname.split('?')[0]
      fplist.pop()
      fdir = os.sep.join(fplist)
      finalpath = path + os.sep + fname
      if len(fdir) > 0 and not os.path.exists(path + os.sep + fdir):
        os.makedirs(path + os.sep + fdir)
      urllib.urlretrieve(f, finalpath)
    # emptyurl = constant.DATA_ENDPOINT + '/set/emptyuploaddir'
    # requests.get(emptyurl, auth=self._auth)
    return result

  def list_dataset(self, dataset_data):
    
    dd = json.loads(dataset_data)
    access_key = dd['access_key']

    photourl = constant.DATA_ENDPOINT + '/set/list?access_key=' + access_key
    
    r = self.process_request('get', photourl, auth=self._auth)
    return r.text

  def exist_dataset(self, dataset_data):
    
    dd = json.loads(dataset_data)
    access_key = dd['access_key']

    photourl = constant.DATA_ENDPOINT + '/set/check/exist' + '?dataset_name=' + dd['dataset_name'] + '&version=' + dd['version'] + '&access_key=' + dd['access_key']
    
    r = requests.get(photourl, auth=self._auth)
    return r.text

  def listdir(self, path, list_name):
    for file in os.listdir(path): 
      file_path = os.path.join(path, file) 
      if os.path.isdir(file_path): 
        self.listdir(file_path, list_name) 
      else: 
        list_name.append(file_path)

  def label_dataset(self, labelset_data):
    dd = json.loads(labelset_data)
    labelurl = constant.DATA_ENDPOINT + '/set/label'+ '?label_name=' + dd['label_name'] + '&type=' + dd['type'] + '&filemd5=' + dd['filemd5'] + '&access_key=' + dd['access_key']
    r = self.process_request('get', labelurl, auth=self._auth)
    return r.text

  def share_dataset(self, shareset_data):
    dd = json.loads(shareset_data)
    labelurl = constant.DATA_ENDPOINT + '/set/share'+ '?dataset_name=' + dd['dataset_name'] + '&type=' + dd['type'] + '&org_id=' + dd['org_id'] + '&access_key=' + dd['access_key']
    r = self.process_request('get', labelurl, auth=self._auth)
    return r.text