from datetime import datetime
import argparse
import fds
import getpass
import json
import logging
import copy
import os
import sys
import time
import optparse
import shlex
import tempfile
import requests
import yaml
import tarfile


sys.path.append("../../")

from . import color_util
from . import constant

from cloud_ml_sdk.utils import log_util
# from cloud_ml_sdk.command.util import compatibility_input


def compatibility_input(obj):
  if sys.version_info > (3, 0):
    res = input(obj)
  else:
    res = raw_input(obj)
  return res


def deprecated_init_config(args):
  """Deprecated cloudml init."""
  print("WARN: `cloudml init` has beed deprecated, please use `cloudml config init` instead.")
  print("We use a new sub-commands group for cloudml config now, including `init`, `show` and more.")
  print("You could get more details about the new config commands by `cloudml config -h`.")
  return


def init_fds(config_data=None):
  """Init fds related config.

  Returns:
    Dict of config_data.
  """
  if not config_data:
    config_data = {}
  access_key = None
  secret_key = None
  if config_data and "xiaomi_access_key_id" in config_data and "xiaomi_secret_access_key" in config_data:
    if compatibility_input("Use current AccessKey Pairs {}? [y/N]: ".format(
        config_data["xiaomi_access_key_id"])) in ["y", "Y"]:
      access_key = config_data["xiaomi_access_key_id"]
      secret_key = config_data["xiaomi_secret_access_key"]
  if not access_key and not secret_key:
    access_key = compatibility_input("\nPlease input access key: ")
    secret_key = getpass.getpass(
      "Please input secret key (will not be echoed): ")
    if access_key.find("/") >= 0 or secret_key.find("/") >= 0:
      print(color_util.colorize_warning("\nATTENSION:"))
      print(color_util.colorize_warning(
        "The access key and secret key with slashes may cause fault in getting data from fds when use tensorflow framework!\n"))

  print("\n[FDS CONFIG]")
  print("/ *")
  print("  * ----only for Xiaomi inner users----")
  print("  * {}".format(color_util.colorize_warning("Ecology cloud users can skip this steps")))
  print("  * c3 endpoint ({}): cnbj1-fds.api.xiaomi.net".format(color_util.colorize_warning("highly recommend")))
  print("  * c3-inner endpoint: cnbj1-inner-fds.api.xiaomi.net")
  print("  */")
  fds_endpoint = compatibility_input("Please input fds endpoint: ")

  if access_key:
    config_data["xiaomi_access_key_id"] = access_key
  if secret_key:
    config_data["xiaomi_secret_access_key"] = secret_key
  if fds_endpoint:
    config_data["xiaomi_fds_endpoint"] = fds_endpoint

  return config_data


def init_hdfs(config_data=None):
  """Init hdfs related config.

  Returns:
    Dict of config_data.
  """
  if not config_data:
    config_data = {}
  hdfs_krb_account = None
  hdfs_krb_password = None
  hdfs_endpoint = None
  if config_data and "xiaomi_hdfs_krb_account" in config_data and "xiaomi_hdfs_krb_password" in config_data:
    if compatibility_input("Use current krb account/password Pairs {}? [y/N]: ".format(
        config_data["xiaomi_hdfs_krb_account"])) in ["y", "Y"]:
      hdfs_krb_account = config_data["xiaomi_hdfs_krb_account"]
      hdfs_krb_password = config_data["xiaomi_hdfs_krb_password"]
  if not hdfs_krb_account and not hdfs_krb_password:
    print("\n[HDFS CONFIG]")
    print("/*")
    print("  * ----only for Xiaomi inner users----")
    print("  * If you don't want set, you can skip this steps")
    print("  * Please make sure you have quota from https://kdc.d.xiaomi.net/hdfs_quota/hadoop/")
    print("  */")
    hdfs_krb_account = compatibility_input("Please input kerberos account: ")
    hdfs_krb_password = getpass.getpass("Please input kerberos password (will not be echoed): ")

  hdfs_endpoint = compatibility_input("Please input hdfs endpoint: ")

  if hdfs_krb_account:
    config_data["xiaomi_hdfs_krb_account"] = hdfs_krb_account
  if hdfs_krb_password:
    config_data["xiaomi_hdfs_krb_password"] = hdfs_krb_password
  if hdfs_endpoint:
    config_data["xiaomi_hdfs_endpoint"] = hdfs_endpoint

  return config_data


def init_cloudml(config_data=None):
  """Init cloudml related config.

  Returns:
    Dict of cloudml_config.
  """
  cloudml_config = {}

  xiaomi_org_mail = None
  if config_data and "xiaomi_org_mail" in config_data:
    if compatibility_input("Use current org mail {}? [y/N]: ".format(
        config_data["xiaomi_org_mail"])) in ["y", "Y"]:
      xiaomi_org_mail = config_data["xiaomi_org_mail"]
  if not xiaomi_org_mail:
    xiaomi_org_mail = compatibility_input("Please input xiaomi org mail: ")

  access_key = None
  secret_key = None
  if config_data and "xiaomi_access_key_id" in config_data and "xiaomi_secret_access_key" in config_data:
    if compatibility_input("Use current AccessKey Pairs {}? [y/N]: ".format(
        config_data["xiaomi_access_key_id"])) in ["y", "Y"]:
      access_key = config_data["xiaomi_access_key_id"]
      secret_key = config_data["xiaomi_secret_access_key"]
  if not access_key and not secret_key:
    access_key = compatibility_input("\nPlease input access key: ")
    secret_key = getpass.getpass(
      "Please input secret key (will not be echoed): ")
    if access_key.find("/") >= 0 or secret_key.find("/") >= 0:
      print(color_util.colorize_warning("\nATTENSION:"))
      print(color_util.colorize_warning(
        "The access key and secret key with slashes may cause fault in getting data from fds when use tensorflow framework!\n"))

  print("\n[Cloud-ML CONFIG]")
  print("/ *")
  print("  * ----for Xiaomi inner users----")
  print("  * c5 endpoint ({}): cnbj5-cloud-ml.api.xiaomi.net".format(color_util.colorize_warning("many GPUs")))
  print("  * ----for Ecology cloud users----")
  print("  */")
  cloudml_endpoint = compatibility_input("Please input cloudml endpoint: ")

  fds_bucket = None
  if config_data and "cloudml_default_fds_bucket" in config_data:
    if compatibility_input("Use current fds bucket {}? [y/N]: ".format(
        config_data["cloudml_default_fds_bucket"])) in ["y", "Y"]:
      fds_bucket = config_data["cloudml_default_fds_bucket"]
  if not fds_bucket:
    fds_bucket = compatibility_input(
      "Please input fds bucket name (which will be mounted on path '/fds' in Cloud-ML): ")

  cloudml_config["xiaomi_cloudml_endpoint"] = cloudml_endpoint
  if xiaomi_org_mail:
    cloudml_config["xiaomi_org_mail"] = xiaomi_org_mail
  if access_key:
    cloudml_config["xiaomi_access_key_id"] = access_key
  if secret_key:
    cloudml_config["xiaomi_secret_access_key"] = secret_key
  if fds_bucket:
    cloudml_config["cloudml_default_fds_bucket"] = fds_bucket

  return cloudml_config


def init_config_data(config_context_name):
  """Init cluster config.

  Returns:
    Dict of cluster_config.
  """
  config_data = {}

  config_data.update(init_fds())
  config_data.update(init_hdfs())

  config_data[constant.CLOUDML_CONFIG_KEY] = {
    config_context_name: init_cloudml(config_data)
  }

  need_re_config = False
  while True:
    if need_re_config:
      config_data[constant.CLOUDML_CONFIG_KEY] = {
        config_context_name: init_cloudml(config_data)
      }
    print("\n[Test CloudML Access]")
    if compatibility_input("Test access with supplied credentials? [y/N]: ") in ["y", "Y"]:
      if not authentication(config_data["xiaomi_access_key_id"],
                            config_data["xiaomi_secret_access_key"],
                            config_data[constant.CLOUDML_CONFIG_KEY][config_context_name]["xiaomi_cloudml_endpoint"],
                            config_data[constant.CLOUDML_CONFIG_KEY][config_context_name]["xiaomi_org_mail"]):
        print("ERROR: Cloud-ML test failed, invalid config message.")
        need_re_config = True
      else:
        print("Test successfully!")
        break
      if not compatibility_input("\nRetry configuration? [y/N]: ") in ["y", "Y"]:
        break
    else:
      break

  need_re_config = False
  while True:
    if need_re_config:
      config_data.update(init_fds())
    print("\n[Test FDS Access]")
    if compatibility_input("Test access with supplied credentials? [y/N]: ") in ["y", "Y"]:
      fds_endpoint = None
      fds_bucket = None
      if "xiaomi_fds_endpoint" in config_data:
        fds_endpoint = config_data["xiaomi_fds_endpoint"]
      if "cloudml_default_fds_bucket" in config_data:
        fds_bucket = config_data["cloudml_default_fds_bucket"]
      if constant.CLOUDML_CONFIG_KEY in config_data and config_context_name in config_data[constant.CLOUDML_CONFIG_KEY]:
        if "cloudml_default_fds_bucket" in config_data[constant.CLOUDML_CONFIG_KEY][config_context_name]:
          fds_bucket = config_data[constant.CLOUDML_CONFIG_KEY][config_context_name]["cloudml_default_fds_bucket"]
      if fds_endpoint and fds_bucket:
        try:
          fds_config = fds.FDSClientConfiguration()
          fds_config.set_endpoint(fds_endpoint)
          fds_client = fds.GalaxyFDSClient(
              config_data["xiaomi_access_key_id"],
              config_data["xiaomi_secret_access_key"],
              fds_config)
        except Exception as e:
          print("ERROR: Init FDS client failed: {}".format(e))
        else:
          if not fds_client.does_bucket_exist(fds_bucket):
            print("ERROR: FDS test failed, does bucket '{}' already created?".format(fds_bucket))
          else:
            print("Test successfully!")
            break
      else:
        print("ERROR: Cloud-ML test failed: Both xiaomi_fds_endpoint and cloudml_default_fds_bucket are needed.")
        need_re_config = True
      if not compatibility_input("\nRetry configuration? [y/N]: ") in ["y", "Y"]:
        break
    else:
      break

  return config_data


def init_config(args):
  """Set the initial config."""

  config_dir = os.path.join(os.path.expanduser("~"), constant.CONFIG_PATH)
  config_path = os.path.join(config_dir, constant.CONFIG_FILENAME)

  config_context_name = compatibility_input("Please name your cloudml config context: (e.g. c5, GPU, CPU etc.) ")

  if config_context_name == constant.DEFAULT_CONFIG_CONTEXT:
    exit_as_error("{} is reserved for default config contest, please chose another name.".format(config_context_name))
    return

  modified = False

  config_data = {}
  if os.path.exists(config_path):
    try:
      with open(config_path) as data_file:
        config_data = json.load(data_file)
    except Exception as e:
      print(
          "{} already exists. But failed to load config file, exception: {}".format(
              config_path, e))
      exit_as_error("You need to fix the config file by hand.")
      return
    if "xiaomi_cloudml_endpoint" in config_data:
      # non-multi cluster config found, ask user to reuse it or not
      print("Non-multi cluster config found in file {} with content:".format(config_path))
      print_masked_sensitive(config_data)
      if constant.CLOUDML_CONFIG_KEY in config_data and "xiaomi_cloudml_endpoint" in config_data[constant.CLOUDML_CONFIG_KEY]:
        print(
          "Found 'xiaomi_cloudml_endpoint' and 'xiaomi_cloudml' same time.")
        print("Will ignore the 'xiaomi_cloudml_endpoint' settings.")
        del config_data["xiaomi_cloudml_endpoint"]
        # del config_data["xiaomi_org_mail"]
        # del config_data["cloudml_default_fds_bucket"]
      else:
        # non-multi cluster config found, ask user to reuse it or not
        if compatibility_input("Do you want to init {} with it? [y/N]: ".format(config_context_name)) in ["y", "Y"]:
          cluster_config = copy.deepcopy(config_data)
          del cluster_config["xiaomi_access_key_id"]
          del cluster_config["xiaomi_secret_access_key"]
          del cluster_config["xiaomi_fds_endpoint"]

          config_data[constant.CLOUDML_CONFIG_KEY] = {
            config_context_name: cluster_config
          }
          del config_data["xiaomi_cloudml_endpoint"]
          del config_data["xiaomi_org_mail"]
          del config_data["cloudml_default_fds_bucket"]
          modified = True
        else:
          # non-multi cluster config found, but not used, init new
          cluster_config = init_cloudml(config_data)
          config_data[constant.CLOUDML_CONFIG_KEY] = {
            config_context_name: cluster_config
          }
          modified = True
    else:
      if constant.CLOUDML_CONFIG_KEY in config_data:
        if config_context_name in config_data[constant.CLOUDML_CONFIG_KEY]:
          # target cluster already in cloudml_config, ask user to reset it
          print("{} cluster config found in file {}:".format(config_context_name, config_path))
          print(json.dumps(config_data[constant.CLOUDML_CONFIG_KEY][config_context_name], indent=4, sort_keys=True))
          if compatibility_input("Do you want to re-init it? [y/N]: ".format(config_context_name)) in ["y", "Y"]:
            cluster_config = init_cloudml(config_data)
            config_data[constant.CLOUDML_CONFIG_KEY][config_context_name] = cluster_config
            modified = True
        else:
          # target cluster config not found, init it with default config_data
          current_config = copy.copy(config_data)
          if constant.DEFAULT_CONFIG_CONTEXT in config_data[constant.CLOUDML_CONFIG_KEY]:
            default_cluster = config_data[constant.CLOUDML_CONFIG_KEY][constant.DEFAULT_CONFIG_CONTEXT]
            current_config.update(config_data[constant.CLOUDML_CONFIG_KEY][default_cluster])
          cluster_config = init_cloudml(current_config)
          config_data[constant.CLOUDML_CONFIG_KEY][config_context_name] = cluster_config
          modified = True
      else:
        # both 'xiaomi_cloudml_endpoint' and 'xiaomi_cloudml' not found
        # re-init whole config
        config_data = init_config_data(config_context_name)
        modified = True
  else:
    # config file not found, init whole config
    config_data = init_config_data(config_context_name)
    modified = True

  if modified:
    if constant.CLOUDML_CONFIG_KEY in config_data:
      if compatibility_input("Set current cluster as default? [y/N]: ") in ["y", "Y"]:
        config_data[constant.CLOUDML_CONFIG_KEY][constant.DEFAULT_CONFIG_CONTEXT] = config_context_name
      # check default_cluster has been set
      if (len(config_data[constant.CLOUDML_CONFIG_KEY]) > 1 and
          constant.DEFAULT_CONFIG_CONTEXT not in config_data[constant.CLOUDML_CONFIG_KEY]):
        while True:
          print("Multi cluster found but not set the default one.")
          cluster_list = [item for item in config_data[constant.CLOUDML_CONFIG_KEY]]
          default_cluster = compatibility_input("Set default cluster to {}: ".format(cluster_list))
          if default_cluster in cluster_list:
            config_data[constant.CLOUDML_CONFIG_KEY][constant.DEFAULT_CONFIG_CONTEXT] = default_cluster
            break
          else:
            print("Invalid config_context_name, please chose one from: {}".format(cluster_list))
          if compatibility_input("Skip settings? [y/N]: ") in ["y", "Y"]:
            print("Configuration aborted. Changes were NOT saved.")
            return
    else:
      exit_as_error("Invalid configuration: 'xiaomi_cloudml' not found. Please run 'cloudml config init' again.")
      return
    print("\n[Save]")
    if compatibility_input("Save settings? [y/N]: ") in ["y", "Y"]:
      if not os.path.exists(config_dir):
        os.makedirs(config_dir)
      with open(config_path, "w") as outfile:
        json.dump(config_data, outfile, indent=4)
      print("Configuration content:")
      print_masked_sensitive(config_data)
      print("Successfully initialize config file in path: {}".format(
          config_path))
    else:
      exit_as_error("Configuration aborted. Changes were NOT saved.")
      return
  else:
    print("cloudml config init operation exit without any modification.")


# def init_config_v2(args):
#   """Set the initial config."""

#   config_dir = os.path.join(os.path.expanduser("~"), constant.CONFIG_PATH)
#   # ~/.config/xiaomi/config.yaml
#   config_path = os.path.join(config_dir, constant.CONFIG_FILENAME)

#   config_context_name = compatibility_input("Please name your cloudml config context: (e.g. c5, GPU, CPU etc.) ")

#   if config_context_name == constant.DEFAULT_CONFIG_CONTEXT:
#     exit_as_error("{} is reserved for default config contest, please chose another name.".format(config_context_name))
#     return 

#   modified = False

#   config_data = {}
#   if os.path.exists(config_path):
#     config_data = read_config_yaml(config_path)

#     # i don't know when this line will work...
#     if "xiaomi_cloudml_endpoint" in config_data:
#       # non-multi cluster config found, ask user to reuse it or not
#       print("Non-multi cluster config found in file {} with content:".format(config_path))
#       print_masked_sensitive(config_data)

#       if constant.CLOUDML_CONFIG_KEY in config_data and "xiaomi_cloudml_endpoint" in config_data[constant.CLOUDML_CONFIG_KEY]:
#         print("Found 'xiaomi_cloudml_endpoint' and 'xiaomi_cloudml' same time.")
#         print("Will ignore the 'xiaomi_cloudml_endpoint' settings.")
#         del config_data["xiaomi_cloudml_endpoint"]
#       else:
#         # non-multi cluster config found, ask user to reuse it or not
#         if compatibility_input("Do you want to init {} with it? [y/N]: ".format(config_context_name)) in ["y", "Y"]:
#           cluster_config = copy.deepcopy(config_data)
#           del cluster_config["xiaomi_access_key_id"]
#           del cluster_config["xiaomi_secret_access_key"]
#           del cluster_config["xiaomi_fds_endpoint"]

#           config_data[constant.CLOUDML_CONFIG_KEY] = {
#             config_context_name: cluster_config
#           }
#           del config_data["xiaomi_cloudml_endpoint"]
#           del config_data["xiaomi_org_mail"]
#           del config_data["cloudml_default_fds_bucket"]
#           modified = True
#         else:
#           # non-multi cluster config found, but not used, init new
#           cluster_config = init_cloudml(config_data)
#           config_data[constant.CLOUDML_CONFIG_KEY] = {
#             config_context_name: cluster_config
#           }
#           modified = True
#     else:
#       if constant.CLOUDML_CONFIG_KEY in config_data:
#         if config_context_name in config_data[constant.CLOUDML_CONFIG_KEY]:
#           # target cluster already in cloudml_config, ask user to reset it
#           print("{} cluster config found in file {}:".format(config_context_name, config_path))
#           print(json.dumps(config_data[constant.CLOUDML_CONFIG_KEY][config_context_name], indent=4, sort_keys=True))
#           if compatibility_input("Do you want to re-init it? [y/N]: ".format(config_context_name)) in ["y", "Y"]:
#             cluster_config = init_cloudml(config_data)
#             config_data[constant.CLOUDML_CONFIG_KEY][config_context_name] = cluster_config
#             modified = True
#         else:
#           # target cluster config not found, init it with default config_data
#           current_config = copy.copy(config_data)
#           if constant.DEFAULT_CONFIG_CONTEXT in config_data[constant.CLOUDML_CONFIG_KEY]:
#             default_cluster = config_data[constant.CLOUDML_CONFIG_KEY][constant.DEFAULT_CONFIG_CONTEXT]
#             current_config.update(config_data[constant.CLOUDML_CONFIG_KEY][default_cluster])
#           cluster_config = init_cloudml(current_config)
#           config_data[constant.CLOUDML_CONFIG_KEY][config_context_name] = cluster_config
#           modified = True
#       else:
#         # both 'xiaomi_cloudml_endpoint' and 'xiaomi_cloudml' not found
#         # re-init whole config
#         config_data = init_config_data(config_context_name)
#         modified = True
#   else:
#     # config file not found, init whole config
#     config_data = init_config_data(config_context_name)
#     modified = True

#   if modified == False:
#     print("cloudml config init operation exit without any modification.")
#     return

#   if constant.CLOUDML_CONFIG_KEY not in config_data:
#     exit_as_error("Invalid configuration: 'xiaomi_cloudml' not found. Please run 'cloudml config init' again.")
#     return

#   if compatibility_input("Set current cluster as default? [y/N]: ") in ["y", "Y"]:
#     config_data[constant.CLOUDML_CONFIG_KEY][constant.DEFAULT_CONFIG_CONTEXT] = config_context_name
  
#   # check default_cluster has been set
#   if (len(config_data[constant.CLOUDML_CONFIG_KEY]) > 1 and
#       constant.DEFAULT_CONFIG_CONTEXT not in config_data[constant.CLOUDML_CONFIG_KEY]):
#     while True:
#       print("Multi cluster found but not set the default one.")
#       cluster_list = [item for item in config_data[constant.CLOUDML_CONFIG_KEY]]
#       default_cluster = compatibility_input("Set default cluster to {}: ".format(cluster_list))
      
#       if default_cluster in cluster_list:
#         config_data[constant.CLOUDML_CONFIG_KEY][constant.DEFAULT_CONFIG_CONTEXT] = default_cluster
#         break
#       else:
#         print("Invalid config_context_name, please chose one from: {}".format(cluster_list))
      
#       if compatibility_input("Skip settings? [y/N]: ") in ["y", "Y"]:
#         print("Configuration aborted. Changes were NOT saved.")
#         return    
  
#   print("\n[Save]")

#   if compatibility_input("Save settings? [y/N]: ") in ["y", "Y"]:
#     write_config_yaml(config_path, config_data)
#     print("Configuration content:")
#     print_masked_sensitive(config_data)
#     print("Successfully initialize config file in path: {}".format(config_path))
#   else:
#     exit_as_error("Configuration aborted. Changes were NOT saved.")
#     return
  


def fds_config(args):
  """Set the fds config."""
  config_dir = os.path.join(os.path.expanduser("~"), constant.CONFIG_PATH)
  config_path = os.path.join(config_dir, constant.CONFIG_FILENAME)

  modified = False

  config_data = {}
  if os.path.exists(config_path):
    try:
      with open(config_path) as data_file:
        config_data = json.load(data_file)
    except Exception as e:
      print("{} already exists. But failed to load config file, exception: {}"
          .format(config_path, e))
      exit_as_error("You need to fix the config file by hand.")
      return
    if "xiaomi_fds_endpoint" in config_data and "xiaomi_access_key_id" in config_data:
      # fds config found
      if compatibility_input("Use FDS endpoint/AccessKey other than {}/{}? [y/N]: ".format(
          config_data["xiaomi_fds_endpoint"], config_data["xiaomi_access_key_id"])) in ["y", "Y"]:
        config_data.update(init_fds(config_data))
        modified = True
    else:
      config_data.update(init_fds())
      modified = True
  else:
    # config file not found, need to run init
    exit_as_error("Config file not found. Please run 'cloudml config init' first.")
    return

  if modified:
    print("\n[Save]")
    if compatibility_input("Save settings? [y/N]: ") in ["y", "Y"]:
      if not os.path.exists(config_dir):
        os.makedirs(config_dir)
      with open(config_path, "w") as outfile:
        json.dump(config_data, outfile, indent=4)
      print("Configuration content:")
      print_masked_sensitive(config_data)
      print("Successfully initialize config file in path: {}".format(
          config_path))
    else:
      print("Configuration aborted. Changes were NOT saved.")
      return
  else:
    print("cloudml config fds operation exit without any modification.")


def hdfs_config(args):
  """Set the hdfs config."""
  config_dir = os.path.join(os.path.expanduser("~"), constant.CONFIG_PATH)
  config_path = os.path.join(config_dir, constant.CONFIG_FILENAME)

  modified = False

  config_data = {}
  if os.path.exists(config_path):
    try:
      with open(config_path) as data_file:
        config_data = json.load(data_file)
    except Exception as e:
      print(
          "{} already exists. But failed to load config file, exception: {}".format(
              config_path, e))
      print("You need to fix the config file by hand.")
      return
    if "xiaomi_hdfs_endpoint" in config_data and "xiaomi_hdfs_krb_account" in config_data:
      if compatibility_input("Use HDFS endpoint/Account other than {}/{}? [y/N]: ".format(
          config_data["xiaomi_hdfs_endpoint"], config_data["xiaomi_hdfs_krb_account"])) in ["y", "Y"]:
        config_data.update(init_hdfs(config_data))
        modified = True
    else:
      config_data.update(init_hdfs())
      modified = True
  else:
    # config file not found, need to run init
    exit_as_error("Config file not found. Please run 'cloudml config init' first.")
    return

  if modified:
    print("\n[Save]")
    if compatibility_input("Save settings? [y/N]: ") in ["y", "Y"]:
      if not os.path.exists(config_dir):
        os.makedirs(config_dir)
      with open(config_path, "w") as outfile:
        json.dump(config_data, outfile, indent=4)
      print("Configuration content:")
      print_masked_sensitive(config_data)
      print("Successfully initialize config file in path: {}".format(
          config_path))
    else:
      print("Configuration aborted. Changes were NOT saved.")
      return
  else:
    print("cloudml config hdfs operation exit without any modification.")


def show_config(args):
  """Show cloudml config."""

  config_dir = os.path.join(os.path.expanduser("~"), constant.CONFIG_PATH)
  config_path = os.path.join(config_dir, constant.CONFIG_FILENAME)

  config_data = {}
  if not os.path.exists(config_path):
    print("cloudml config file({}) not exists, please init one by: cloudml config init".format(config_path))
    return
  else:
    try:
      with open(config_path) as data_file:
        config_data = json.load(data_file)
    except Exception as e:
      print(
          "Failed to load config file {}, exception: {}".format(
              config_path, e))
      print("You need to fix the config file by hand.")
      return
  print_masked_sensitive(config_data)


def print_masked_sensitive(config):
  """Mask sensitive info in config data"""
  config_copy = copy.deepcopy(config)
  sensitive_keys = [
    "xiaomi_secret_access_key",
    "xiaomi_hdfs_krb_password",
  ]
  for key in sensitive_keys:
    if key in config_copy:
      config_copy[key] = "*" * 16
  if constant.CLOUDML_CONFIG_KEY in config_copy:
    for k,v in config_copy[constant.CLOUDML_CONFIG_KEY].items():
      for key in sensitive_keys:
        if key in v:
          config_copy[constant.CLOUDML_CONFIG_KEY][k][key] = "*" * 16

  print(json.dumps(config_copy, indent=4, sort_keys=True))


def set_default_config(args):
  """Set default cloudml config."""

  config_dir = os.path.join(os.path.expanduser("~"), constant.CONFIG_PATH)
  config_path = os.path.join(config_dir, constant.CONFIG_FILENAME)

  config_data = {}
  if not os.path.exists(config_path):
    print("cloudml config file({}) not exists, please init one by: cloudml config init".format(config_path))
    return
  else:
    try:
      with open(config_path) as data_file:
        config_data = json.load(data_file)
    except Exception as e:
      print(
          "Failed to load config file {}, exception: {}".format(
              config_path, e))
      print("You need to fix the config file by hand.")
      return
    if "xiaomi_cloudml_endpoint" in config_data:
      # non-multi context config found, ask user to run cloudml config init first.
      print("Non-multi context config found in file {}, please re-init by: cloudml config init".format(config_path))
      return
    else:
      if constant.CLOUDML_CONFIG_KEY in config_data:
        # verify target context key in config, if so set default_config_context else print error msg
        if args.config_context_name in config_data[constant.CLOUDML_CONFIG_KEY]:
          config_data[constant.CLOUDML_CONFIG_KEY][constant.DEFAULT_CONFIG_CONTEXT] = args.config_context_name
          with open(config_path, "w") as outfile:
            json.dump(config_data, outfile, indent=4)
          print("Successfully set default config context to: {}".format(
              args.config_context_name))
        else:
          print("Failed to set default config context to {}, it's not in the config file {}".format(
                args.config_context_name, config_path))
          print("Please chose valid config context name from the result of `cloudml config show`.")
          print("Or you can init an new config context config by: `cloudml config init`.")
      else:
        # no multi context config found, ask user to run cloudml config init first.
        print("No multi context config found in file {}, please re-init by: cloudml config init".format(config_path))
        return


def delete_config_context(args):
  """Delete cloudml config context."""

  config_dir = os.path.join(os.path.expanduser("~"), constant.CONFIG_PATH)
  config_path = os.path.join(config_dir, constant.CONFIG_FILENAME)

  config_data = {}
  if not os.path.exists(config_path):
    print("cloudml config file({}) not exists, please init one by: cloudml config init".format(config_path))
    return
  else:
    try:
      with open(config_path) as data_file:
        config_data = json.load(data_file)
    except Exception as e:
      print((
          "Failed to load config file {}, exception: {}".format(
              config_path, e)))
      print("You need to fix the config file by hand.")
      return
    if "xiaomi_cloudml_endpoint" in config_data:
      # non-multi context config found
      print("Non-multi context config found in file {} with content:".format(config_path))
      print((json.dumps(config_data, indent=4, sort_keys=True)))
      print("Please re-init it to support multi context config commands by: cloudml config init")
      return
    else:
      if constant.CLOUDML_CONFIG_KEY in config_data:
        modified = False
        for config_context_name in args.config_context_names:
          # delete default config context is not allowed
          if config_context_name == config_data[constant.CLOUDML_CONFIG_KEY][constant.DEFAULT_CONFIG_CONTEXT]:
            print("{} is your defalut config context, it's not allow to delete.".format(config_context_name))
            print("Please change your default context first by: cloudml config set_default <name_of_other_context>")
            return
          # verify target context names in config, if so delete it else print error msg
          if config_context_name in config_data[constant.CLOUDML_CONFIG_KEY]:
            del config_data[constant.CLOUDML_CONFIG_KEY][config_context_name]
            modified = True
            print("Successfully deleted config context: {}".format(config_context_name))
          else:
            print("Failed to delete config context {}, it's not in the config file {}".format(
                config_context_name, config_path))
        if modified:
          with open(config_path, "w") as outfile:
            json.dump(config_data, outfile, indent=4)
      else:
        # no multi context config found, ask user to run cloudml config init first.
        print("No multi context config found in file {}, please re-init by: cloudml config init".format(config_path))
        return

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#      ___           ___                            ___                       ___       #
#     /\__\         /\__\               ___        /\__\          ___        /\  \      #
#    /::|  |       /:/  /              /\  \      /::|  |        /\  \       \:\  \     #
#   /:|:|  |      /:/  /               \:\  \    /:|:|  |        \:\  \       \:\  \    #
#  /:/|:|__|__   /:/  /                /::\__\  /:/|:|  |__      /::\__\      /::\  \   #
# /:/ |::::\__\ /:/__/              __/:/\/__/ /:/ |:| /\__\  __/:/\/__/     /:/\:\__\  #
# \/__/~~/:/  / \:\  \             /\/:/  /    \/__|:|/:/  / /\/:/  /       /:/  \/__/  #
#       /:/  /   \:\  \            \::/__/         |:/:/  /  \::/__/       /:/  /       #
#      /:/  /     \:\  \            \:\__\         |::/  /    \:\__\       \/__/        #
#     /:/  /       \:\__\            \/__/         /:/  /      \/__/                    #
#     \/__/         \/__/                          \/__/                                #
#                                                                               V2      #
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
def read_config_yaml(path):
  try:
    with open(path) as fp:
      data = yaml.safe_load(fp)
  except Exception as e:
    print("failed to load config file, exception: {}".format(str(e)))
    return {}
  
  return data


def write_config_yaml(path, config):
  config_dir = os.path.join(os.path.expanduser("~"), constant.CONFIG_PATH)
  try:
    if not os.path.exists(config_dir):
        os.makedirs(config_dir)
    
    with open(path, 'w') as fp:
      yaml.dump(config, fp, default_flow_style = False)
  except Exception as e:
    exit_as_error(str(e))


def get_config_file():
  config_dir = os.path.join(os.path.expanduser("~"), constant.CONFIG_PATH)
  config_file = os.path.join(config_dir, constant.CONFIG_FILENAME_V2)
  return config_file


def print_current_cloudml_configs(ml_configs):
  config_copy = copy.deepcopy(ml_configs)
  
  for config_name in config_copy:
    config_copy[config_name]['xiaomi_secret_access_key'] = '*' * 16

  print(json.dumps(config_copy, sort_keys=True, indent=4, separators=(',', ':')))


def set_default_ml_cluster_config(config_dict):
  configs = config_dict['xiaomi_cloudml']

  if len(configs) == 1:
    config_dict['default_config_context'] = list(configs.keys())[0]
    return config_dict
  
  for k in configs:
    print('  * %s' % k)
  
  default_ml_config = compatibility_input("Multi xiaomi cloudml config found, choose one as the default: ")

  config_dict['default_config_context'] = default_ml_config

  return config_dict


def show_config_v2(args):
  # for compatibility
  if os.path.exists(os.path.join(
    os.path.expanduser("~"), constant.CONFIG_PATH, constant.CONFIG_FILENAME)):
    return show_config(args)

  config_path = get_config_file()

  if not os.path.exists(config_path):
    print("cloudml config file({}) not exists, please init one by: cloudml config init".format(config_path))
    return

  try:
    with open(config_path, 'r') as fp:
      data = fp.read()
      print(data)
  except Exception as e:
    print(str(e))
  return


def set_default_config_v2(args):
  # for compatibility
  if os.path.exists(os.path.join(
    os.path.expanduser("~"), constant.CONFIG_PATH, constant.CONFIG_FILENAME)):
    return set_default_config(args)

  config_path = get_config_file()

  config_data = {}
  if not os.path.exists(config_path):
    print("cloudml config file({}) not exists, please init one by: cloudml config init".format(config_path))
    return

  config_data = read_config_yaml(config_path)

  if constant.CLOUDML_CONFIG_KEY not in config_data:
    print("No xiaomi cloudml config found in file {}, please re-init by: cloudml config init".format(config_path))
    return

  configs = list(config_data[constant.CLOUDML_CONFIG_KEY].keys())

  if len(configs) < 1:
    print("No xiaomi cloudml context config found in file {}, please re-init by: cloudml config init".format(config_path))
    return

  # verify target context key in config, if so set default_config_context else print error msg
  if args.config_context_name not in configs:
    print("Failed to set default config context to {}, it's not in the config file {}".format(
        args.config_context_name, config_path))
    print("Please chose valid config context name from the result of `cloudml config show`.")
    print("Or you can init an new config context config by: `cloudml config init`.")
    return
  
  config_data[constant.DEFAULT_CONFIG_CONTEXT] = args.config_context_name
  write_config_yaml(config_path, config_data)
  print("Successfully set default config context to: {}".format(args.config_context_name))
  return


def delete_config_context_v2(args):
  # for compatibility
  if os.path.exists(os.path.join(
    os.path.expanduser("~"), constant.CONFIG_PATH, constant.CONFIG_FILENAME)):
    return delete_config_context(args)

  config_path = get_config_file()

  if not os.path.exists(config_path):
    print("cloudml config file({}) not exists, please init one by: cloudml config init".format(config_path))
    return

  config_data = read_config_yaml(config_path)

  if constant.CLOUDML_CONFIG_KEY not in config_data:
    print("No multi context config found in file {}, please re-init by: cloudml config init".format(config_path))
    return

  modified = False
  for name in args.config_context_names:
    # delete default config context is not allowed
    if name == config_data[constant.DEFAULT_CONFIG_CONTEXT]:
      print("{} is your defalut config context, it's not allow to delete.".format(name))
      print("Please change your default context first by: cloudml config set_default <name_of_other_context>")
      return

    # verify target context names in config, if so delete it else print error msg
    if name not in config_data[constant.CLOUDML_CONFIG_KEY]:
      print("Failed to delete config context {}, it's not in the config file {}".format(name, config_path))
      continue

    del config_data[constant.CLOUDML_CONFIG_KEY][name]
    modified = True
    print("Successfully deleted config context: {}".format(name))

  if modified:
    write_config_yaml(config_path, config_data)

  return


def fds_config_v2(args):
  # for compatibility
  if os.path.exists(os.path.join(
    os.path.expanduser("~"), constant.CONFIG_PATH, constant.CONFIG_FILENAME)):
    return fds_config(args)

  config_path = get_config_file()
  modified = False

  if not os.path.exists(config_path):
    print("cloudml config file({}) not exists, please init one by: cloudml config init".format(config_path))
    return

  config_data = read_config_yaml(config_path)
  if 'fds' not in config_data:
    config_data['fds'] = {}

  fds = config_data['fds']
  
  if "xiaomi_fds_endpoint" in fds and "xiaomi_access_key_id" in fds:
    # fds config found
    if compatibility_input("Use FDS endpoint/AccessKey other than {}/{}? [y/N]: ".format(
        fds["xiaomi_fds_endpoint"], fds["xiaomi_access_key_id"])) in ["y", "Y"]:
      config_data = config_fds(config_data)
      modified = True
  else:
    config_data = config_fds(config_data)
    modified = True

  if modified:
    write_config_yaml(config_path, config_data)
    print("Successfully initialize config file in path: {}".format(config_path))
    return

  print("cloudml config fds operation exit without any modification.")
  return

def hdfs_config_v2(args):
  # for compatibility
  if os.path.exists(os.path.join(
    os.path.expanduser("~"), constant.CONFIG_PATH, constant.CONFIG_FILENAME)):
    return hdfs_config(args)

  config_path = get_config_file()
  modified = False

  if not os.path.exists(config_path):
    print("cloudml config file({}) not exists, please init one by: cloudml config init".format(config_path))
    return

  config_data = read_config_yaml(config_path)
  if 'hdfs' not in config_data:
    config_data['hdfs'] = {}
  
  hdfs = config_data['hdfs']

  if "xiaomi_hdfs_endpoint" in hdfs and "xiaomi_hdfs_krb_account" in hdfs:
    if compatibility_input("Use HDFS endpoint/Account other than {}/{}? [y/N]: ".format(
        hdfs["xiaomi_hdfs_endpoint"], hdfs["xiaomi_hdfs_krb_account"])) in ["y", "Y"]:
      config_data = config_hdfs(config_data)
      modified = True
  else:
    config_data = config_hdfs(config_data)
    modified = True

  if modified:
    write_config_yaml(config_path, config_data)
    print("Successfully write config file in path: {}".format(config_path))
    return
  
  print("cloudml config hdfs operation exit without any modification.")
  return


def config_xiaomi_cloudml(config_dict):
  # first init
  if 'xiaomi_cloudml' not in config_dict:
    config_dict['xiaomi_cloudml'] = {}
  
  if len(config_dict['xiaomi_cloudml']) > 0:
    print('\nCurrent xiaomi cloudml configs:')
    print_current_cloudml_configs(config_dict['xiaomi_cloudml'])

  config_context_name = compatibility_input("Please name your cloudml config context: (e.g. c5, GPU, CPU etc.) ")

  if config_context_name == constant.DEFAULT_CONFIG_CONTEXT:
    exit_as_error("{} is reserved for default config contest, please chose another name.".format(config_context_name))
    return 
  
  print("\n[Cloud-ML CONFIG]")
  print("/ *")
  print("  * ----for Xiaomi inner users----")
  print("  * c5 endpoint ({}): cnbj5-cloud-ml.api.xiaomi.net".format(color_util.colorize_warning("many GPUs")))
  print("  * ----for Ecology cloud users----")
  print("  */")
  cloudml_endpoint = compatibility_input("Please input cloudml endpoint: ")
  xiaomi_org_mail = compatibility_input("Please input xiaomi org mail: ")
  fds_bucket = compatibility_input("Please input fds bucket name (which will be mounted on path '/fds' in Cloud-ML): ")
  access_key = compatibility_input("Please input access key: ")
  secret_key = getpass.getpass("Please input secret key (will not be echoed): ")
  if access_key.find("/") >= 0 or secret_key.find("/") >= 0:
    print(color_util.colorize_warning("\nATTENSION:"))
    print(color_util.colorize_warning("The access key and secret key with\
      slashes may cause fault in getting data from fds when use tensorflow framework!\n"))

  config_dict['xiaomi_cloudml'][config_context_name] = {}
  cluster = config_dict['xiaomi_cloudml'][config_context_name]
  cluster['xiaomi_cloudml_endpoint'] = cloudml_endpoint
  cluster['xiaomi_org_mail'] = xiaomi_org_mail
  cluster['cloudml_default_fds_bucket'] = fds_bucket
  cluster['xiaomi_secret_access_key'] = secret_key
  cluster['xiaomi_access_key_id'] = access_key

  # choose a default ml config
  config_dict = set_default_ml_cluster_config(config_dict)

  write_config_yaml(get_config_file(), config_dict)

  return config_dict


def config_fds(config_dict):

  if 'fds' in config_dict:
    if 'xiaomi_access_key_id' in config_dict['fds']:
      print('Current access key id = %s' % config_dict['fds']['xiaomi_access_key_id'])
    if 'xiaomi_fds_endpoint' in config_dict['fds']:
      print('Current fds endpoint = %s' % config_dict['fds']['xiaomi_fds_endpoint'])

  access_key = compatibility_input("Please input access key: ")
  secret_key = getpass.getpass("Please input secret key (will not be echoed): ")

  if access_key.find("/") >= 0 or secret_key.find("/") >= 0:
    print(color_util.colorize_warning("\nATTENSION:"))
    print(color_util.colorize_warning(
      "The access key and secret key with slashes may cause fault in getting data from fds when use tensorflow framework!\n"))

  print("\n[FDS CONFIG]")
  print("/ *")
  print("  * ----only for Xiaomi inner users----")
  print("  * {}".format(color_util.colorize_warning("Ecology cloud users can skip this steps")))
  print("  * c3 endpoint ({}): cnbj1-fds.api.xiaomi.net".format(color_util.colorize_warning("highly recommend")))
  print("  * c3-inner endpoint: cnbj1-inner-fds.api.xiaomi.net")
  print("  */")
  fds_endpoint = compatibility_input("Please input fds endpoint: ")

  fds = {}
  fds['xiaomi_access_key_id'] = access_key
  fds['xiaomi_secret_access_key'] = secret_key
  fds['xiaomi_fds_endpoint'] = fds_endpoint
  config_dict['fds'] = fds

  write_config_yaml(get_config_file(), config_dict)

  return config_dict


def config_hdfs(config_dict):

  if 'hdfs' in config_dict:
    if 'xiaomi_hdfs_krb_account' in config_dict['hdfs']:
      print('Current xiaomi hdfs krb account = %s' % config_dict['hdfs']['xiaomi_hdfs_krb_account'])
    if 'xiaomi_hdfs_endpoint' in config_dict:
      print('Current hdfs endpoint = %s' % config_dict['hdfs']['xiaomi_hdfs_endpoint'])
  
  print("\n[HDFS CONFIG]")
  print("/*")
  print("  * ----only for Xiaomi inner users----")
  print("  * If you don't want set, you can skip this steps")
  print("  * Please make sure you have quota from https://kdc.d.xiaomi.net/hdfs_quota/hadoop/")
  print("  */")

  hdfs_krb_account = compatibility_input("Please input kerberos account: ")
  hdfs_krb_password = getpass.getpass("Please input kerberos password (will not be echoed): ")
  hdfs_endpoint = compatibility_input("Please input hdfs endpoint: ")

  hdfs = {}
  hdfs['xiaomi_hdfs_krb_account'] = hdfs_krb_account
  hdfs['xiaomi_hdfs_krb_password'] = hdfs_krb_password
  hdfs['xiaomi_hdfs_endpoint'] = hdfs_endpoint

  config_dict['hdfs'] = hdfs

  write_config_yaml(get_config_file(), config_dict)

  return config_dict


def init_all_config(args):
  # for compatibility
  if os.path.exists(os.path.join(
    os.path.expanduser("~"), constant.CONFIG_PATH, constant.CONFIG_FILENAME)):
    return init_config(args)

  # check file exist.
  config_file = get_config_file()

  config = {}
  if os.path.exists(config_file):
    config = read_config_yaml(config_file)

  print("Config steps:")
  print("- config xiaomi cloudml.")
  print("- config fds.")
  print("- config hdfs.")

  # ask if go to config xiaomi cloudml
  if compatibility_input("Step-1: start config xiaomi cloudml? [y/N]: ") in ["y", "Y"]:
    config = config_xiaomi_cloudml(config)

  # ask if go to config fds
  if compatibility_input("Step-2: start config fds? [y/N]: ") in ["y", "Y"]:
    config = config_fds(config)
  
  # ask if go to config hdfs
  if compatibility_input("Step-3: start config hdfs? [y/N]: ") in ["y", "Y"]:
    config = config_hdfs(config)

  print("config end.")



def add_config_parser(main_subparser):
  # Deprecated `cloudml init`, use `cloudml config init` instead
  # init_parser = main_subparser.add_parser("init", help="[DEPRECATED] Init cloudml config")
  # init_parser.set_defaults(func=util.deprecated_init_config)

  # subcommand: config
  config_parser = main_subparser.add_parser("config", help="Commands about cloudml config")

  config_subparser = config_parser.add_subparsers(
      dest="config_command", help="Subcommands of config")

  # subcommand of config: init
  config_init_parser = config_subparser.add_parser("init", help="Init cloudml config")
  config_init_parser.set_defaults(func=init_all_config)

  # subcommand of config: show
  config_show_parser = config_subparser.add_parser("show", help="Show cloudml config")
  config_show_parser.set_defaults(func = show_config_v2)

  # subcommand of config: set default config context
  config_set_parser = config_subparser.add_parser(
      "set_default", help="Set default cloudml config context",
      description="Get potential `config_context_name` from config via: cloudml config show")
  config_set_parser.add_argument("config_context_name", help="The name of the default config context")
  config_set_parser.set_defaults(func = set_default_config_v2)

  # subcommand of config: delete cluster config
  config_delete_parser = config_subparser.add_parser("delete", help="Delete cloudml config context")
  config_delete_parser.add_argument("config_context_names", metavar="config_context_name", nargs="+", help="The name of the config context")
  config_delete_parser.set_defaults(func = delete_config_context_v2)

  # subcommand of config: fds
  config_fds_parser = config_subparser.add_parser("fds", help="Set fds config")
  config_fds_parser.set_defaults(func = fds_config_v2)

  # subcommand of config: hdfs
  config_hdfs_parser = config_subparser.add_parser("hdfs", help="Set hdfs config")
  config_hdfs_parser.set_defaults(func = hdfs_config_v2)