#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# PYTHON_ARGCOMPLETE_OK

# Copyright 2017 Xiaomi, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argcomplete
import argparse
import logging
import pkg_resources
import sys

from cloud_ml_sdk.command.constant import GlobalArgsStore

sys.path.append("../../")

from . import constant
from . import util
from . import secret
from . import train_v2
from . import cloudml_init
from . import quota_v2
from . import ms_deploy

from cloud_ml_sdk.thetis import datasets

from . import command_models
# from cloudml_init import add_config_parser

global_args = {}
def new_parser():
  parser = argparse.ArgumentParser()

  parser.add_argument(
      "-v",
      "--version",
      action="version",
      version=pkg_resources.require(constant.SDK_NAME)[0].version,
      help="Show version")

  parser.add_argument(
      '--debug',
      help="Print debugging logs",
      action="store_const", dest="loglevel", const=logging.DEBUG,
      default=logging.WARNING,
  )

  parser.add_argument(
      "-k", "--context", dest="config_context_name", action=util.SetConfigContextEnv,
      help="The cloudml config context name, available values can be listed by: cloudml config show")

  main_subparser = parser.add_subparsers(dest="command_group", help="Commands")

  cloudml_init.add_config_parser(main_subparser)

  org_id_parser = main_subparser.add_parser(
      "org_id", help="Get org_id by access_key and secret_key")
  org_id_parser.set_defaults(func=util.get_org_id)

  #subcommand :hptuning
  hptuning_parser = main_subparser.add_parser("hp", help="Commands about hptuning")
  hptuning_subparser = hptuning_parser.add_subparsers(
      dest="hp_command", help="Subcommands of hptuning")

  # subcommand of hptunings: submit
  hptuning_submit_parser = hptuning_subparser.add_parser("submit", help="Submit hptuning")
  hptuning_submit_parser.add_argument(
      "-f",
      "--filename",
      dest="filename",
      help="The json file contains the hptuning task msg")
  hptuning_submit_parser.set_defaults(func=util.submit_hptuning)

  # subcommand of hptunings: delete
  hptuning_delete_parser = hptuning_subparser.add_parser(
      "delete", help="Delete the hptuning(s)")
  hptuning_delete_parser.add_argument("hp_names", metavar="hp_name", nargs="+", help="The name of the hptuning")
  hptuning_delete_parser.set_defaults(func=util.delete_hp)

  # subcommand of hptunings: list
  hptuning_list_parser = hptuning_subparser.add_parser("list", help="List hptunings")
  hptuning_list_parser.add_argument("-p", "--priority_class", dest="priority_class",
                                    choices=constant.CLOUDML_PRIORITY_CLASSES,
                                    help="The priority class to list jobs with. If not set, will return all.")
  hptuning_list_parser.add_argument("-pn", "--page_number", type=int, dest="page_number",
                                    help="The page number of the list!")
  hptuning_list_parser.add_argument("-ps", "--page_size", type=int, dest="page_size",
                                    help="The page size of the list!")
  hptuning_list_parser.set_defaults(func=util.list_hp)

  # subcommand of hptunings: describe
  hptuning_describe_parser = hptuning_subparser.add_parser(
      "describe", help="Describe hp job")
  hptuning_describe_parser.add_argument("hp_name", help="The hp job to describe")
  hptuning_describe_parser.set_defaults(func=util.describe_hp_job)

  # subcommand of hptunings: get trials
  hptuning_trials_parser = hptuning_subparser.add_parser("trials", help="get trials")
  hptuning_trials_parser.add_argument("hp_name",
                                      help="the hptuning job the trials belong to")
  hptuning_trials_parser.set_defaults(func=util.get_trials)

  # subcommand of hptunings: get trial log
  hptuning_trial_log_parser = hptuning_subparser.add_parser("logs", help="get log of the trial")

  hptuning_trial_log_parser.add_argument("hp_name", help="the hptuning job the trials belong to")

  hptuning_trial_log_parser.add_argument("-ti",
                                      "--trial_id",
                                      dest="trial_id",
                                      help="the log of the trial(pod)")
  hptuning_trial_log_parser.add_argument("-f", "--follow", action="store_true",
                                help="Follow the log stream of the pod. Defaults to false.")
  hptuning_trial_log_parser.add_argument("-n", "--lines", type=int, help="The last number of lines to show.")
  hptuning_trial_log_parser.set_defaults(func=util.get_trial_log)

  #subcommand of hp_jobs: events
  hptuning_events_parser = hptuning_subparser.add_parser(
      "events", help="Get the events of the train job")
  hptuning_events_parser.add_argument("hp_name", help="The name of the train job")
  hptuning_events_parser.set_defaults(func=util.get_hp_job_events)


  # subcommand of hp jobs: metrics
  hptuning_metrics_parser = hptuning_subparser.add_parser(
      "metrics", help="Get the metrics of the hp job")
  hptuning_metrics_parser.add_argument("hp_name", help="The hp job to get the logs")
  hptuning_metrics_parser.set_defaults(func=util.get_hp_metrics)

  # subcommand of hp jobs: optimal trials
  hptuning_optimal_parser = hptuning_subparser.add_parser(
      "optimal", help="Get the optimal trial")
  hptuning_optimal_parser.add_argument("hp_name", help="The hp job to get the optimal trial info")
  hptuning_optimal_parser.set_defaults(func=util.get_optimal_trial)

  # subcommand: models
  command_models.models_main_command(main_subparser)

  # subcommand: jobs
  jobs_parser = main_subparser.add_parser("jobs", help="Commands about jobs")
  jobs_subparser = jobs_parser.add_subparsers(
      dest="job_command", help="Subcommands of jobs")

  # subcommand of jobs: list
  jobs_list_parser = jobs_subparser.add_parser("list", help="List jobs")
  jobs_list_parser.add_argument("-o", "--output", dest="output", help="The output form : now only support: wide(node the task assigned to)")
  jobs_list_parser.add_argument("-p", "--priority_class", dest="priority_class",
                                choices=constant.CLOUDML_PRIORITY_CLASSES,
                                help="The priority class to list jobs with. If not set, will return all.")
  jobs_list_parser.add_argument("-n", "--job_name", dest="job_name", help="The job will be output.")
  jobs_list_parser.add_argument("-s", "--job_status", dest="job_status", help="The status of the  output job: pending, processing, completed, failed.")
  jobs_list_parser.add_argument("-pn", "--page_number", type=int,  dest="page_number", help="The page number of the list.")
  jobs_list_parser.add_argument("-ps", "--page_size", type=int, dest="page_size", help="The page size of the list.")
  jobs_list_parser.set_defaults(func=util.list_jobs)

  # subcommand of jobs: submit
  jobs_submit_parser = jobs_subparser.add_parser("submit", help="Submit job")
  jobs_submit_parser.add_argument(
      "-f",
      "--filename",
      dest="filename",
      help="The json file contains the job task msg")

  jobs_submit_parser.add_argument(
      "-n", "--job_name", dest="job_name", help="The job name")
  jobs_submit_parser.add_argument(
      "-m", "--module_name", dest="module_name", help="The module name")
  jobs_submit_parser.add_argument(
      "-u", "--trainer_uri", dest="trainer_uri",
      help="The trainer uri, support `fds://` or local tar.gz file path or even local directory`. " +
           "e.g. ./ or /path/to/my/ml_project. " +
           "NOTE: when using local directory, `setup.py` or `requirements.txt` in the top level of the target directory " +
           "will be used to install any extra dependencies.")
  jobs_submit_parser.add_argument(
      "-a", "--job_args", dest="job_args", help="The string of args")
  jobs_submit_parser.add_argument(
      "-fe", "--fds_endpoint", dest="fds_endpoint", help="The fds endpoint use")
  jobs_submit_parser.add_argument(
      "-fb", "--fds_bucket", dest="fds_bucket", help="The fds bucket to mount to /fds")
  jobs_submit_parser.add_argument(
      "-fds", "--fds_mounts", dest="fds_mounts",
      help="Mount fds buckets, value is seperated by comma, e.g. fds://endpoint/bucket,fds://endpoint/bucket")

  #for hdfs
  jobs_submit_parser.add_argument(
      "-hka", "--hdfs_krb_account", type=str,dest="hdfs_krb_account", help="The kerberos account"
  )
  jobs_submit_parser.add_argument(
      "-hkp", "--hdfs_krb_password", type=str,dest="hdfs_krb_password", help="The kerberos password"
  )
  jobs_submit_parser.add_argument(
      "-hkt", "--hdfs_krb_keytab", type=str, dest="hdfs_krb_keytab", help="The kerberos keytab secret name"
  )
  jobs_submit_parser.add_argument(
      "-he", "--hdfs_endpoint", type=str,dest="hdfs_endpoint", help="The hadoop path to mount to /hdfs"
  )
  #for ceph
  jobs_submit_parser.add_argument(
      "-cv", "--ceph_volume", type=str,dest="ceph_volume", help="The ceph volume name"
  )
  jobs_submit_parser.add_argument(
      "-cm", "--ceph_mode", type=str,dest="ceph_mode", help="Ceph access mode, r for ReadOnlyMany, rw for ReadWriteOnce"
  )
  jobs_submit_parser.add_argument(
      "-ceph", "--ceph_mounts", type=str, dest="ceph_mounts",
      help="Mount ceph volumes, value is seperated by comma, VOLUME[:MODE],VOLUME[:MODE]" )
  jobs_submit_parser.add_argument(
      "-c",
      "--cpu_limit",
      dest="cpu_limit",
      help="The CPU limit with unit core")
  jobs_submit_parser.add_argument(
      "-M",
      "--memory_limit",
      dest="memory_limit",
      help="The memory limit with unit K, M or G")
  jobs_submit_parser.add_argument(
      "-g", "--gpu_limit", dest="gpu_limit", help="The number of GPU limit")
  jobs_submit_parser.add_argument(
      "-gm", "--gpu_memory", dest="gpu_memory", help="The required least GPU memory")
  gpu_type_help = "select gpu type from ({})".format("/".join(constant.GPULIST))
  if not constant.ENABLE_GPU_TYPE_SELECT:
    gpu_type_help = "[DEPRECATED] " + gpu_type_help
  jobs_submit_parser.add_argument(
    "-gt",
    "--gpu_type",
    dest="gpu_type",
    help=gpu_type_help
  )
  jobs_submit_parser.add_argument(
      "-p",
      "--ps_count",
      dest="ps_count",
      help="The number of ps for distributed training")
  jobs_submit_parser.add_argument(
      "-w",
      "--worker_count",
      dest="worker_count",
      help="The number of worker for distributed training")
  jobs_submit_parser.add_argument(
      "-D",
      "--distributed",
      action="store_true",
      help="General distributed train")
  jobs_submit_parser.add_argument(
      "-R",
      "--enable_rank",
      dest = "enable_rank",
      action = "store_true",
      help = "Create priority high train"
  )
  jobs_submit_parser.add_argument(
      "-d", "--docker_image", dest="docker_image", help="The docker image")
  jobs_submit_parser.add_argument(
      "-dc",
      "--docker_command",
      dest="docker_command",
      help="The docker command")
  jobs_submit_parser.add_argument(
      "-F",
      "--framework",
      dest="framework",
      help="The framework of machine learning")
  jobs_submit_parser.add_argument(
      "-V",
      "--framework_version",
      dest="framework_version",
      help="The version of the framework")
  jobs_submit_parser.add_argument(
      "-py",
      "--python",
      dest="framework_python",
      choices=["", "py3"],
      default="",
      help="The python version of machine learning train, default is empty for Python2.7.")
  jobs_submit_parser.add_argument(
      "-hdfs",
      "--hdfs",
      dest="framework_hdfs",
      choices=["", "zjyprc-hadoop"],
      default="",
      help="The hdfs cluster of machine learning train, default is empty for zjyprc-hadoop.")
  jobs_submit_parser.add_argument(
      "-vt", "--volume_type", dest="volume_type", help="The volume type")
  jobs_submit_parser.add_argument(
      "-vp", "--volume_path", dest="volume_path", help="The volume path")
  jobs_submit_parser.add_argument(
      "-mp", "--mount_path", dest="mount_path", help="The mount type")
  jobs_submit_parser.add_argument(
      "-mro",
      "--mount_read_only",
      dest="mount_read_only",
      help="Whether mount read only or not")
  jobs_submit_parser.add_argument(
      "-pc",
      "--prepare_command",
      dest="prepare_command",
      help="The prepare command")
  jobs_submit_parser.add_argument(
      "-fc",
      "--finish_command",
      dest="finish_command",
      help="The finish command")
  jobs_submit_parser.add_argument(
      "-W", "--watch", action="store_true", help="Watch the status of job")
  jobs_submit_parser.add_argument(
      "-nsk",
      "--node_selector_key",
      dest="node_selector_key",
      help="The node selector key")
  jobs_submit_parser.add_argument(
      "-nsv",
      "--node_selector_value",
      dest="node_selector_value",
      help="The node selector value")
  jobs_submit_parser.add_argument(
      "-cs",
      "--cluster_spec",
      dest="cluster_spec",
      help='The cluster specific, e.g. --cluster_spec=\'{"ps": {"count": 1, "memory_limit": "1G", "task_args": "--num_gpus=0", "gpu_limit": 0, "cpu_limit": "1"}, "chief": {"count": 1, "memory_limit": "1G", "task_args": "--num_gpus=0", "gpu_limit": 0, "cpu_limit": "1"}, "worker": {"count": 1, "memory_limit": "1G", "task_args": "--num_gpus=0", "gpu_limit": 0, "cpu_limit": "1"}}\'')
  jobs_submit_parser.add_argument(
      "-sv",
      "--save_mode",
      dest="save_mode",
      help="Save job, don't run",
      action='store_true')
  jobs_submit_parser.add_argument(
      "-argo",
      "--argo_mode",
      dest="argo_mode",
      help="mark it a argo job",
      action='store_true')
  jobs_submit_parser.add_argument(
      "-tl",
      "--tensorboard_logdir",
      dest="tensorboard_logdir",
      help="Tensorboard logdir of the job")
  jobs_submit_parser.add_argument(
      "-mn",
      "--model_name",
      dest="model_name",
      help="model name (must be initial status) of the job")
  jobs_submit_parser.add_argument(
    "-mv",
    "--model_version",
    dest="model_version",
    help="model version(must be initial status) of the job")
  jobs_submit_parser.add_argument(
    "-e",
    "--owner_email",
    dest="owner_email",
    help="The value is owner email."
  )
  jobs_submit_parser.add_argument(
    "--priority_class",
    dest="priority_class",
    choices=constant.CLOUDML_PRIORITY_CLASSES,
    default=constant.TJ_DEFAULT_PRIORITY_CLASS,
    help="The value of priority_class."
  )

  jobs_submit_parser.set_defaults(func=util.submit_job)

  # subcommand of jobs: describe
  jobs_describe_parser = jobs_subparser.add_parser(
      "describe", help="Describe job")
  jobs_describe_parser.add_argument("job_name", help="The job to describe")
  jobs_describe_parser.set_defaults(func=util.describe_job)

  # subcommand of jobs: 'kill', set activeDeadlineSeconds to 5
  jobs_kill_parser = jobs_subparser.add_parser(
      "kill", help="Kill job instance. Will release resources but not delete train job.")
  jobs_kill_parser.add_argument("job_name", help="The job to kill")
  jobs_kill_parser.set_defaults(func=util.kill_job)

  # subcommand of jobs: logs
  jobs_logs_parser = jobs_subparser.add_parser(
      "logs", help="Get the logs of the job")
  jobs_logs_parser.add_argument("-f", "--follow", action="store_true",
                                help="Follow the log stream of the pod. Defaults to false.")
  jobs_logs_parser.add_argument("-n", "--lines", type=int, help="The last number of lines to show.")
  jobs_logs_parser.add_argument("job_name", help="The job to get the logs")
  jobs_logs_parser.set_defaults(func=util.get_job_logs)

  # subcommand of jobs: history logs
  jobs_his_logs_parser = jobs_subparser.add_parser(
      "history_logs", help="Get the history logs of the job")
  jobs_his_logs_parser.add_argument("-n", "--lines", type=int, help="The last number of lines to show.")
  jobs_his_logs_parser.add_argument("job_name", help="The job to get the history logs")
  jobs_his_logs_parser.set_defaults(func=util.get_job_history_logs)

  # subcommand of jobs: metrics
  jobs_metrics_parser = jobs_subparser.add_parser(
      "metrics", help="Get the metrics of the job")
  jobs_metrics_parser.add_argument("job_name", help="The job to get the logs")
  jobs_metrics_parser.set_defaults(func=util.get_job_metrics)

  # subcommand of jobs: hp
  jobs_hp_parser = jobs_subparser.add_parser(
      "hp", help="Get the hyperparameters data of the job")
  jobs_hp_parser.add_argument("job_name", help="The job name")
  jobs_hp_parser.set_defaults(func=util.get_job_hyperparameters_data)

  # subcommand of jobs: delete
  jobs_delete_parser = jobs_subparser.add_parser(
      "delete", help="Delete the job(s)")
  jobs_delete_parser.add_argument("-n", "--is_cluster_name", action="store_true",
                                  help="Delete the cluster job by cluster name, Defaults to false.")
  jobs_delete_parser.add_argument("job_names", metavar="job_name", nargs="+", help="The name of the job")
  jobs_delete_parser.set_defaults(func=util.delete_job)

  # subcommand of jobs: events
  jobs_events_parser = jobs_subparser.add_parser(
      "events", help="Get the events of the train job")
  jobs_events_parser.add_argument("job_name", help="The name of the train job")
  jobs_events_parser.set_defaults(func=util.get_train_job_events)

  # subcommand of jobs: start
  jobs_start_parser = jobs_subparser.add_parser(
      "start", help="Start the train job")
  jobs_start_parser.add_argument(
      "job_name", help="The name of the job")
  jobs_start_parser.set_defaults(func=util.start_job)

  # subcommand of jobs: 'rerun', set status to enqueued
  jobs_rerun_parser = jobs_subparser.add_parser(
      "rerun", help="Rerun job task.")
  jobs_rerun_parser.add_argument("job_name", help="The job to rerun")
  jobs_rerun_parser.set_defaults(func=util.rerun_job)

  # subcommand: jobs_v2
  jobs_v2_parser = main_subparser.add_parser("jobs_v2", help="Commands about jobs_v2")
  jobs_v2_subparser = jobs_v2_parser.add_subparsers(
    dest="job_command", help="Subcommands of jobs_v2")

  # subcommand of jobs_v2: list
  jobs_v2_list_parser = jobs_v2_subparser.add_parser("list", help="List jobs_v2")
  jobs_v2_list_parser.set_defaults(func=train_v2.list_jobs)

  # subcommand of jobs_v2: submit
  jobs_v2_submit_parser = jobs_v2_subparser.add_parser("submit", help="Submit job")
  jobs_v2_submit_parser.add_argument(
    "-f",
    "--filename",
    dest="filename",
    help="The json file contains the job task msg")

  jobs_v2_submit_parser.add_argument(
    "-n", "--job_name", dest="job_name", help="The job name")
  jobs_v2_submit_parser.add_argument(
    "-p", "--project_name", dest="project_name", help="The project that current train belongs to.")
  jobs_v2_submit_parser.add_argument(
    "-m", "--module_name", dest="module_name", help="The module name")
  jobs_v2_submit_parser.add_argument(
    "-u", "--trainer_uri", dest="trainer_uri",
    help="The trainer uri, support `fds://` or local tar.gz file path or even local directory`. " +
         "e.g. ./ or /path/to/my/ml_project. " +
         "NOTE: when using local directory, `setup.py` or `requirements.txt` in the top level of the target directory " +
         "will be used to install any extra dependencies.")
  jobs_v2_submit_parser.add_argument(
    "-a", "--job_args", dest="job_args", help="The string of args")
  jobs_v2_submit_parser.add_argument(
    "-fs", "--fds_secret", dest="fds_secret", help="The secret that holds fds configs, including: ep/bucket/ak/sk")
  jobs_v2_submit_parser.add_argument(
    "-fe", "--fds_endpoint", dest="fds_endpoint", help="The fds endpoint use")
  jobs_v2_submit_parser.add_argument(
    "-fb", "--fds_bucket", dest="fds_bucket", help="The fds bucket to mount to /fds")
  jobs_v2_submit_parser.add_argument(
    "-fds", "--fds_mounts", dest="fds_mounts",
    help="Mount fds buckets, value is seperated by comma, e.g. fds://endpoint/bucket,fds://endpoint/bucket")

  # for hdfs
  jobs_v2_submit_parser.add_argument(
    "-hka", "--hdfs_krb_account", type=str, dest="hdfs_krb_account", help="The kerberos account"
  )
  jobs_v2_submit_parser.add_argument(
    "-hkp", "--hdfs_krb_password", type=str, dest="hdfs_krb_password", help="The kerberos password"
  )
  jobs_v2_submit_parser.add_argument(
    "-hkt", "--hdfs_krb_keytab", type=str, dest="hdfs_krb_keytab", help="The kerberos keytab secret name"
  )
  jobs_v2_submit_parser.add_argument(
    "-he", "--hdfs_endpoint", type=str, dest="hdfs_endpoint", help="The hadoop path to mount to /hdfs"
  )
  # for ceph
  jobs_v2_submit_parser.add_argument(
    "-cv", "--ceph_volume", type=str, dest="ceph_volume", help="The ceph volume name"
  )
  jobs_v2_submit_parser.add_argument(
    "-cm", "--ceph_mode", type=str, dest="ceph_mode", help="Ceph access mode, r for ReadOnlyMany, rw for ReadWriteOnce"
  )
  jobs_v2_submit_parser.add_argument(
    "-ceph", "--ceph_mounts", type=str, dest="ceph_mounts",
    help="Mount ceph volumes, value is seperated by comma, VOLUME[:MODE],VOLUME[:MODE]")
  jobs_v2_submit_parser.add_argument(
    "-c",
    "--cpu_limit",
    dest="cpu_limit",
    help="The CPU limit with unit core")
  jobs_v2_submit_parser.add_argument(
    "-M",
    "--memory_limit",
    dest="memory_limit",
    help="The memory limit with unit K, M or G")
  jobs_v2_submit_parser.add_argument(
    "-g", "--gpu_limit", dest="gpu_limit", help="The number of GPU limit")
  jobs_v2_submit_parser.add_argument(
    "-gm", "--gpu_memory", dest="gpu_memory", help="The required least GPU memory")
  gpu_type_help = "select gpu type from ({})".format("/".join(constant.GPULIST))
  if not constant.ENABLE_GPU_TYPE_SELECT:
    gpu_type_help = "[DEPRECATED] " + gpu_type_help
  jobs_v2_submit_parser.add_argument(
      "-gt",
      "--gpu_type",
      dest="gpu_type",
      help=gpu_type_help
  )
  jobs_v2_submit_parser.add_argument(
    "-D",
    "--distributed",
    action="store_true",
    help="General distributed train")
  jobs_v2_submit_parser.add_argument(
    "-d", "--docker_image", dest="docker_image", help="The docker image")
  jobs_v2_submit_parser.add_argument(
    "-dc",
    "--docker_command",
    dest="docker_command",
    help="The docker command")
  jobs_v2_submit_parser.add_argument(
    "-F",
    "--framework",
    dest="framework",
    help="The framework of machine learning")
  jobs_v2_submit_parser.add_argument(
    "-V",
    "--framework_version",
    dest="framework_version",
    help="The version of the framework")
  jobs_v2_submit_parser.add_argument(
    "-py",
    "--python",
    dest="framework_python",
    choices=["", "py3"],
    default="",
    help="The python version of machine learning train, default is empty for Python2.7.")
  jobs_v2_submit_parser.add_argument(
    "-hdfs",
    "--hdfs",
    dest="framework_hdfs",
    choices=["", "zjyprc-hadoop"],
    default="",
    help="The hdfs cluster of machine learning train, default is empty for zjyprc-hadoop.")
  jobs_v2_submit_parser.add_argument(
    "-vt", "--volume_type", dest="volume_type", help="The volume type")
  jobs_v2_submit_parser.add_argument(
    "-vp", "--volume_path", dest="volume_path", help="The volume path")
  jobs_v2_submit_parser.add_argument(
    "-mp", "--mount_path", dest="mount_path", help="The mount type")
  jobs_v2_submit_parser.add_argument(
    "-mro",
    "--mount_read_only",
    dest="mount_read_only",
    help="Whether mount read only or not")
  jobs_v2_submit_parser.add_argument(
    "-pc",
    "--prepare_command",
    dest="prepare_command",
    help="The prepare command")
  jobs_v2_submit_parser.add_argument(
    "-fc",
    "--finish_command",
    dest="finish_command",
    help="The finish command")
  jobs_v2_submit_parser.add_argument(
    "-nsk",
    "--node_selector_key",
    dest="node_selector_key",
    help="The node selector key")
  jobs_v2_submit_parser.add_argument(
    "-nsv",
    "--node_selector_value",
    dest="node_selector_value",
    help="The node selector value")
  jobs_v2_submit_parser.add_argument(
    "-cs",
    "--cluster_spec",
    dest="cluster_spec",
    help='The cluster specific, e.g. --cluster_spec=\'{"ps": {"count": 1, "memory_limit": "1G", "task_args": "--num_gpus=0", "gpu_limit": 0, "cpu_limit": "1"}, "chief": {"count": 1, "memory_limit": "1G", "task_args": "--num_gpus=0", "gpu_limit": 0, "cpu_limit": "1"}, "worker": {"count": 1, "memory_limit": "1G", "task_args": "--num_gpus=0", "gpu_limit": 0, "cpu_limit": "1"}}\'')
  jobs_v2_submit_parser.add_argument(
    "-e",
    "--owner_email",
    dest="owner_email",
    help="The value is owner email."
  )
  jobs_v2_submit_parser.set_defaults(func=train_v2.submit_job)

  # subcommand of jobs_v2: describe
  jobs_v2_describe_parser = jobs_v2_subparser.add_parser(
    "describe", help="Describe job")
  jobs_v2_describe_parser.add_argument("job_name", help="The job to describe")
  jobs_v2_describe_parser.set_defaults(func=train_v2.describe_job)

  # subcommand of jobs_v2: delete
  jobs_v2_delete_parser = jobs_v2_subparser.add_parser(
    "delete", help="Delete the job(s)")
  jobs_v2_delete_parser.add_argument("job_names", metavar="job_name", nargs="+", help="The name of the job")
  jobs_v2_delete_parser.set_defaults(func=train_v2.delete_job)

  # subcommand: deploy
  deploy_parser = main_subparser.add_parser(
    "deploy", help="deploy related cmds and infos about model service"
  )

  deploy_subparser = deploy_parser.add_subparsers(
    dest="deploy_commands", help="subcommands of deploy"
  )

  deploy_list_parser = deploy_subparser.add_parser(
    "list", help="List model deploys"
  )
  deploy_list_parser.set_defaults(func=ms_deploy.deploy_list)

  deploy_create_parser = deploy_subparser.add_parser(
    "create", help="create a deployment of a models service"
  )

  deploy_create_parser.add_argument(
      "-f",
      "--filename",
      dest="filename",
      help="The json data file",
      required=True
  )
  deploy_create_parser.set_defaults(func=ms_deploy.deploy_create)

  deploy_describe_parser = deploy_subparser.add_parser(
    "describe", help="describe a deployment of a models service"
  )
  deploy_describe_parser.add_argument(
    "-n",
    "--deploy_name",
    dest="deploy_name",
    help="The name of the deploy",
    required=True
  )
  deploy_describe_parser.add_argument(
    "-v",
    "--deploy_version",
    dest="deploy_version",
    help="The version of the deploy",
    required=True
  )
  deploy_describe_parser.set_defaults(func=ms_deploy.deploy_describe)

  deploy_delete_parser = deploy_subparser.add_parser(
    "delete", help="delete a deployment of a models service"
  )
  deploy_delete_parser.add_argument(
    "-n",
    "--deploy_name",
    dest="deploy_name",
    help="The name of the deploy",
    required=True
  )
  deploy_delete_parser.add_argument(
    "-v",
    "--deploy_version",
    dest="deploy_version",
    help="The version of the deploy",
    required=True
  )
  deploy_delete_parser.set_defaults(func=ms_deploy.deploy_delete)

  deploy_logs_parser = deploy_subparser.add_parser(
    "logs", help="show the logs when execute a deployment of a models service"
  )
  deploy_logs_parser.add_argument(
    "-n",
    "--deploy_name",
    dest="deploy_name",
    help="The name of the deploy",
    required=True
  )
  deploy_logs_parser.add_argument(
    "-v",
    "--deploy_version",
    dest="deploy_version",
    help="The version of the deploy",
    required=True
  )
  deploy_logs_parser.set_defaults(func=ms_deploy.deploy_logs)

  deploy_cmd_parser = deploy_subparser.add_parser(
    "cmd", help="show current command when execute a deployment of a models service"
  )
  deploy_cmd_parser.add_argument(
    "-n",
    "--deploy_name",
    dest="deploy_name",
    help="The name of the deploy",
    required=True
  )
  deploy_cmd_parser.add_argument(
    "-v",
    "--deploy_version",
    dest="deploy_version",
    help="The version of the deploy",
    required=True
  )
  deploy_cmd_parser.set_defaults(func=ms_deploy.deploy_cmd)

  deploy_setcmd_parser = deploy_subparser.add_parser(
    "setcmd", help="commands to inject when execute a deployment of a models service"
  )

  deploy_status_parser = deploy_subparser.add_parser(
    "status", help="status when execute  a deployment of a models service"
  )
  deploy_status_parser.add_argument(
    "-n",
    "--deploy_name",
    dest="deploy_name",
    help="The name of the deploy",
    required=True
  )
  deploy_status_parser.add_argument(
    "-v",
    "--deploy_version",
    dest="deploy_version",
    help="The version of the deploy",
    required=True
  )
  deploy_status_parser.set_defaults(func=ms_deploy.deploy_status)

  deploy_event_parser = deploy_subparser.add_parser(
    "event", help="event when execute  a deployment of a models service"
  )
  deploy_event_parser.add_argument(
    "-n",
    "--deploy_name",
    dest="deploy_name",
    help="The name of the deploy",
    required=True
  )
  deploy_event_parser.add_argument(
    "-v",
    "--deploy_version",
    dest="deploy_version",
    help="The version of the deploy",
    required=True
  )
  deploy_event_parser.set_defaults(func=ms_deploy.deploy_event)

  deploy_setcmd_subparser = deploy_setcmd_parser.add_subparsers(
    dest="runtime_commands", help="commands during deploying"
  )

  deploy_stop_parser = deploy_setcmd_subparser.add_parser(
    "stop", help="stop when execute  a deployment of a models service, the status will change to stopped"
  )
  deploy_stop_parser.add_argument(
    "-n",
    "--deploy_name",
    dest="deploy_name",
    help="The name of the deploy",
    required=True
  )
  deploy_stop_parser.add_argument(
    "-v",
    "--deploy_version",
    dest="deploy_version",
    help="The version of the deploy",
    required=True
  )
  deploy_stop_parser.set_defaults(func=ms_deploy.deploy_setcmd_stop)

  deploy_rerun_parser = deploy_setcmd_subparser.add_parser(
    "rerun", help="rerun a stopped or finished deployment of a models service, the status will change to enqueued"

  )
  deploy_rerun_parser.add_argument(
    "-n",
    "--deploy_name",
    dest="deploy_name",
    help="The name of the deploy",
    required=True
  )
  deploy_rerun_parser.add_argument(
    "-v",
    "--deploy_version",
    dest="deploy_version",
    help="The version of the deploy",
    required=True
  )
  deploy_rerun_parser.set_defaults(func=ms_deploy.deploy_setcmd_rerun)

  deploy_pause_parser = deploy_setcmd_subparser.add_parser(
    "pause", help="pause  a running deployment of a models service, the status will change to paused"
  )
  deploy_pause_parser.add_argument(
    "-n",
    "--deploy_name",
    dest="deploy_name",
    help="The name of the deploy",
    required=True
  )
  deploy_pause_parser.add_argument(
    "-v",
    "--deploy_version",
    dest="deploy_version",
    help="The version of the deploy",
    required=True
  )
  deploy_pause_parser.set_defaults(func=ms_deploy.deploy_setcmd_pause)

  deploy_resume_parser = deploy_setcmd_subparser.add_parser(
    "resume", help="resume a paused deployment of a models service, the status will change to running"
  )
  deploy_resume_parser.add_argument(
    "-n",
    "--deploy_name",
    dest="deploy_name",
    help="The name of the deploy",
    required=True
  )
  deploy_resume_parser.add_argument(
    "-v",
    "--deploy_version",
    dest="deploy_version",
    help="The version of the deploy",
    required=True
  )
  deploy_resume_parser.set_defaults(func=ms_deploy.deploy_setcmd_resume)

  deploy_abort_parser = deploy_setcmd_subparser.add_parser(
    "abort", help="call this command just when your deploy job occurs error, the status will change to aborted"
  )
  deploy_abort_parser.add_argument(
    "-n",
    "--deploy_name",
    dest="deploy_name",
    help="The name of the deploy",
    required=True
  )
  deploy_abort_parser.add_argument(
    "-v",
    "--deploy_version",
    dest="deploy_version",
    help="The version of the deploy",
    required=True
  )
  deploy_abort_parser.set_defaults(func=ms_deploy.deploy_setcmd_abort)

  deploy_start_parser = deploy_setcmd_subparser.add_parser(
    "start", help="call this command only when you deploy is creating, the status will change to running"
  )
  deploy_start_parser.add_argument(
    "-n",
    "--deploy_name",
    dest="deploy_name",
    help="The name of the deploy",
    required=True
  )
  deploy_start_parser.add_argument(
    "-v",
    "--deploy_version",
    dest="deploy_version",
    help="The version of the deploy",
    required=True
  )
  deploy_start_parser.set_defaults(func=ms_deploy.deploy_setcmd_start)

  deploy_finish_parser = deploy_setcmd_subparser.add_parser(
    "finish", help="call this command only when you deploy job done, the status will change to finished"
  )
  deploy_finish_parser.add_argument(
    "-n",
    "--deploy_name",
    dest="deploy_name",
    help="The name of the deploy",
    required=True
  )
  deploy_finish_parser.add_argument(
    "-v",
    "--deploy_version",
    dest="deploy_version",
    help="The version of the deploy",
    required=True
  )
  deploy_finish_parser.set_defaults(func=ms_deploy.deploy_setcmd_finish)

  # subcommand: autoscaler
  autoscaler_parser = main_subparser.add_parser(
      "autoscaler", help="Commands about model service autoscaler")
  autoscaler_subparser = autoscaler_parser.add_subparsers(
      dest="autoscaler_command", help="Subcommands of autoscaler"
  )
  # subcommand of autoscaler: list
  autoscaler_list_parser = autoscaler_subparser.add_parser(
      "list", help="List autoscaler rule."
  )
  autoscaler_list_parser.set_defaults(func=util.list_autoscaler_rule)
  # subcommand of autoscaler: describe
  autoscaler_describe_parser = autoscaler_subparser.add_parser(
      "describe", help="Describe the autoscaler rule."
  )
  autoscaler_describe_parser.add_argument(
      "model_name", help="The name of model service name."
  )
  autoscaler_describe_parser.add_argument(
      "model_version", help="The version of model service version."
  )
  autoscaler_describe_parser.set_defaults(func=util.describe_autoscaler_rule)
  # subcommand of autoscaler: create
  autoscaler_create_parser = autoscaler_subparser.add_parser(
      "create", help="Create autoscaler rule."
  )
  # autoscaler_create_parser.add_argument(
  #     "-yf",
  #     "--config_yaml_file",
  #     dest="config_yaml_file",
  #     help="Valid yaml config file in current directory, void all other arguments",
  # )
  autoscaler_create_parser.add_argument(
      "model_name",
      help="The value of model service name."
  )
  autoscaler_create_parser.add_argument(
      "model_version",
      help="The value of model service verison."
  )
  autoscaler_create_parser.add_argument(
      "-jf",
      "--config_json_file",
      dest="config_json_file",
      help=('Valid json config file name in current directory, example:\n'
            '{"trigger_type": "qps",\n'
            '"threshold_max_replicas": 5,\n'
            '"up_threshold_qps": 20,\n'
            '"threshold_min_replicas": 1,\n'
            '"down_threshold_qps": 15,\n'
            '"collect_metrics_duration": 1,\n'
            '"instance_up_step": 1,\n'
            '"instance_down_step": 1,\n'
            '"trigger_interval": 5,\n'
            '"up_qps_buffer_percent": 0.2,\n'
            '"cron_configs": [{\n'
            '\t"time_start_cron_str": "01 01 * * *",\n'
            '\t"time_end_cron_str": "10 01 * * *",\n'
            '\t"target_replica": 5\n'
            '\t},\n'
            '\t{\n'
            '\t"time_start_cron_str": "02 02 * * *",\n'
            '\t"time_end_cron_str": "10 02 * * *",\n'
            '\t"target_replica": 5\n'
            '\t}\n'
            '\t],\n'
            '"falcon_qps_counter_name": "counter_sample_1",\n'
            '"falcon_endpoints": [\n'
            '\t"c5-ai-i1-sample01.bj",\n'
            '\t"c5-ai-i1-sample02.bj"\n'
            ']}'),
      required=True
  )
  autoscaler_create_parser.set_defaults(func=util.create_autoscaler_rule)
  # subcommand of autoscaler: update
  autoscaler_update_parser = autoscaler_subparser.add_parser(
      "update", help="Update autoscaler rule."
  )
  autoscaler_update_parser.add_argument(
      "model_name",
      help="The value of model service name."
  )
  autoscaler_update_parser.add_argument(
      "model_version",
      help="The value of model service verison."
  )
  autoscaler_update_parser.add_argument(
    "-jf",
    "--config_json_file",
    dest="config_json_file",
    help=('Valid json config file name in current directory, example:\n'
          '{"trigger_type": "qps",\n'
          '"threshold_max_replicas": 5,\n'
          '"up_threshold_qps": 20,\n'
          '"threshold_min_replicas": 1,\n'
          '"down_threshold_qps": 15,\n'
          '"collect_metrics_duration": 1,\n'
          '"instance_up_step": 1,\n'
          '"instance_down_step": 1,\n'
          '"trigger_interval": 5,\n'
          '"up_qps_buffer_percent": 0.2,\n'
          '"cron_configs": [{\n'
          '\t"time_start_cron_str": "01 01 * * *",\n'
          '\t"time_end_cron_str": "10 01 * * *",\n'
          '\t"target_replica": 5\n'
          '\t},\n'
          '\t{\n'
          '\t"time_start_cron_str": "02 02 * * *",\n'
          '\t"time_end_cron_str": "10 02 * * *",\n'
          '\t"target_replica": 5\n'
          '\t}\n'
          '\t],\n'
          '"falcon_qps_counter_name": "counter_sample_1",\n'
          '"falcon_endpoints": [\n'
          '\t"c5-ai-i1-sample01.bj",\n'
          '\t"c5-ai-i1-sample02.bj"\n'
          ']}'),
    required=True
  )
  autoscaler_update_parser.set_defaults(func=util.update_autoscaler_rule)
  # subcommand of autoscaler: pause
  autoscaler_delete_parser = autoscaler_subparser.add_parser(
      "pause", help="Pause autoscaler rule."
  )
  autoscaler_delete_parser.add_argument(
      "model_name", help="The value of model service name."
  )
  autoscaler_delete_parser.add_argument(
      "model_version", help="The value of model service verison."
  )
  autoscaler_delete_parser.set_defaults(func=util.pause_autoscaler_rule)
  # subcommand of autoscaler: resume
  autoscaler_delete_parser = autoscaler_subparser.add_parser(
      "resume", help="Resume paused autoscaler rule."
  )
  autoscaler_delete_parser.add_argument(
      "model_name", help="The value of model service name."
  )
  autoscaler_delete_parser.add_argument(
      "model_version", help="The value of model service verison."
  )
  autoscaler_delete_parser.set_defaults(func=util.resume_autoscaler_rule)
  # subcommand of autoscaler: delete
  autoscaler_delete_parser = autoscaler_subparser.add_parser(
      "delete", help="Delete autoscaler rule."
  )
  autoscaler_delete_parser.add_argument(
      "model_name", help="The value of model service name."
  )
  autoscaler_delete_parser.add_argument(
      "model_version", help="The value of model service verison."
  )
  autoscaler_delete_parser.set_defaults(func=util.delete_autoscaler_rule)
  # subcommand: tensorboard
  tensorboard_parser = main_subparser.add_parser(
      "tensorboard", help="Commands about tensorboard")
  tensorboard_subparser = tensorboard_parser.add_subparsers(
      dest="tensorboard_command", help="Subcommands of tensorboard")

  # subcommand of tensorboard: list
  tensorboard_list_parser = tensorboard_subparser.add_parser(
      "list", help="List tensorboards")
  tensorboard_list_parser.add_argument("-p", "--priority_class", dest="priority_class",
                                       choices=constant.CLOUDML_PRIORITY_CLASSES,
                                       help="The priority class to list jobs with. If not set, will return all.")
  tensorboard_list_parser.add_argument("-pn", "--page_number", type=int, dest="page_number",
                                       help="The page number of the list.")
  tensorboard_list_parser.add_argument("-ps", "--page_size", type=int, dest="page_size",
                                       help="The page size of the list.")
  tensorboard_list_parser.set_defaults(func=util.list_tensorboard_services)

  # subcommand of tensorboard: create
  tensorboard_create_parser = tensorboard_subparser.add_parser(
      "create", help="Create tensorboard")
  tensorboard_create_parser.add_argument(
      "-n",
      "--tensorboard_name",
      dest="tensorboard_name",
      help="The name of the tensorboard",
      required=True)
  tensorboard_create_parser.add_argument(
      "-l",
      "--logdir",
      dest="logdir",
      help="The directory of tensorboard log",
      required=True)
  tensorboard_create_parser.add_argument(
      "-R",
      "--enable_rank",
      dest="enable_rank",
      action="store_true",
      help="Create priority high tensorboard"
  )
  tensorboard_create_parser.add_argument(
      "-d", "--docker_image", dest="docker_image", help="The docker image")
  tensorboard_create_parser.add_argument(
      "-dc",
      "--docker_command",
      dest="docker_command",
      help="The docker command")
  tensorboard_create_parser.add_argument(
      "-F",
      "--framework",
      dest="framework",
      help="The framework of machine learning")
  tensorboard_create_parser.add_argument(
      "-V",
      "--framework_version",
      dest="framework_version",
      help="The version of the framework")

  tensorboard_create_parser.add_argument(
      "-fe", "--fds_endpoint",dest="fds_endpoint", help="The fds endpoint use")
  tensorboard_create_parser.add_argument(
      "-fb", "--fds_bucket", dest="fds_bucket", help="The fds bucket to mount to /fds")

  tensorboard_create_parser.add_argument(
      "-nsk",
      "--node_selector_key",
      dest="node_selector_key",
      help="The node selector key")
  tensorboard_create_parser.add_argument(
      "-nsv",
      "--node_selector_value",
      dest="node_selector_value",
      help="The node selector value")
  tensorboard_create_parser.add_argument(
    "--priority_class",
    dest="priority_class",
    choices=constant.CLOUDML_PRIORITY_CLASSES,
    default=constant.CLOUDML_DEFAULT_PRIORITY_CLASS,
    help="The value of priority_class."
  )
  tensorboard_create_parser.set_defaults(func=util.create_tensorboard_service)

  # subcommand of tensorboard: describe
  tensorboard_describe_parser = tensorboard_subparser.add_parser(
      "describe", help="Describe the tensorboard")
  tensorboard_describe_parser.add_argument(
      "tensorboard_name", help="The name of the tensorboard")
  tensorboard_describe_parser.set_defaults(
      func=util.describe_tensorboard_service)

  # subcommand of tensorboard: delete
  tensorboard_delete_parser = tensorboard_subparser.add_parser(
      "delete", help="Delete the tensorboard(s)")
  tensorboard_delete_parser.add_argument(
      "tensorboard_names", metavar="tensorboard_name", nargs="+", help="The name of the tensorboard")
  tensorboard_delete_parser.set_defaults(func=util.delete_tensorboard_service)

  # subcommand of tensorboard: events
  tensorboard_events_parser = tensorboard_subparser.add_parser(
      "events", help="Get the events of the tensorboard service")
  tensorboard_events_parser.add_argument(
      "tensorboard_name", help="The name of the tensorboard service")
  tensorboard_events_parser.set_defaults(
      func=util.get_tensorboard_service_events)

  # subcommand: dev
  dev_parser = main_subparser.add_parser("dev", help="Commands about dev")
  dev_subparser = dev_parser.add_subparsers(
      dest="dev_command", help="Subcommands of dev")

  # subcommand of dev: list
  dev_list_parser = dev_subparser.add_parser(
      "list", help="List dev environments")
  dev_list_parser.add_argument("-p", "--priority_class", dest="priority_class",
                               choices=constant.CLOUDML_PRIORITY_CLASSES,
                               help="The priority class to list jobs with. If not set, will return all.")
  dev_list_parser.add_argument("-pn", "--page_number", type=int,  dest="page_number",
                               help="The page number of the list.")
  dev_list_parser.add_argument("-ps", "--page_size", type=int, dest="page_size",
                               help="The page size of the list.")
  dev_list_parser.set_defaults(func=util.list_dev_envs)

  # subcommand of dev: create
  dev_create_parser = dev_subparser.add_parser(
      "create", help="Create dev environment")
  dev_create_parser.add_argument(
      "-n",
      "--dev_name",
      dest="dev_name",
      help="The dev environment name",
      required=True)
  dev_create_parser.add_argument(
      "-p",
      "--password",
      dest="password",
      help="The password of ipython notebook",
      required=True)
  dev_create_parser.add_argument(
      "-fe", "--fds_endpoint",dest="fds_endpoint", help="The fds endpoint use")
  dev_create_parser.add_argument(
      "-fb", "--fds_bucket", dest="fds_bucket", help="The fds bucket to mount to /fds")
  #for hdfs
  dev_create_parser.add_argument(
      "-hka", "--hdfs_krb_account", type=str,dest="hdfs_krb_account", help="The kerberos account"
  )
  dev_create_parser.add_argument(
      "-hkp", "--hdfs_krb_password", type=str,dest="hdfs_krb_password", help="The kerberos password"
  )
  dev_create_parser.add_argument(
      "-hkt", "--hdfs_krb_keytab", type=str, dest="hdfs_krb_keytab", help="The kerberos keytab secret name"
  )
  dev_create_parser.add_argument(
      "-he", "--hdfs_endpoint", type=str,dest="hdfs_endpoint", help="The hadoop path to mount to /hdfs"
  )
  dev_create_parser.add_argument(
      "-cv", "--ceph_volume", type=str,dest="ceph_volume", help="The ceph volume name"
  )
  dev_create_parser.add_argument(
      "-cm", "--ceph_mode", type=str,dest="ceph_mode", help="Ceph access mode, r for ReadOnlyMany, rw for ReadWriteOnce"
  )
  dev_create_parser.add_argument(
      "-ceph", "--ceph_mounts", type=str, dest="ceph_mounts",
      help="Mount ceph volumes, value is seperated by comma, VOLUME[:MODE],VOLUME[:MODE]" )
  dev_create_parser.add_argument(
      "-c",
      "--cpu_limit",
      dest="cpu_limit",
      help="The CPU limit with unit core")
  dev_create_parser.add_argument(
      "-M",
      "--memory_limit",
      dest="memory_limit",
      help="The memory limit with unit K, M or G")
  dev_create_parser.add_argument(
      "-g", "--gpu_limit", dest="gpu_limit", help="The number of GPU limit")
  dev_create_parser.add_argument(
      "-gm", "--gpu_memory", dest="gpu_memory", help="The required least GPU memory")
  gpu_type_help = "select gpu type from ({})".format("/".join(constant.GPULIST))
  if not constant.ENABLE_GPU_TYPE_SELECT:
    gpu_type_help = "[DEPRECATED] " + gpu_type_help
  dev_create_parser.add_argument(
      "-gt",
      "--gpu_type",
      dest="gpu_type",
      help=gpu_type_help
  )
  dev_create_parser.add_argument(
      "-R",
      "--enable_rank",
      dest="enable_rank",
      action="store_true",
      help="Create priority high dev"
  )
  dev_create_parser.add_argument(
      "-d", "--docker_image", dest="docker_image", help="The ")
  dev_create_parser.add_argument(
      "-dc",
      "--docker_command",
      dest="docker_command",
      help="The docker command")
  dev_create_parser.add_argument(
      "-F",
      "--framework",
      dest="framework",
      help="The framework of machine learning")
  dev_create_parser.add_argument(
      "-V",
      "--framework_version",
      dest="framework_version",
      help="The version of the framework")
  dev_create_parser.add_argument(
      "-py",
      "--python",
      dest="framework_python",
      choices=["", "py3"],
      default="",
      help="The python version of machine learning train, default is empty for Python2.7.")
  dev_create_parser.add_argument(
      "-hdfs",
      "--hdfs",
      dest="framework_hdfs",
      choices=["", "zjyprc-hadoop"],
      default="",
      help="The hdfs cluster of machine learning train, default is empty for zjyprc-hadoop.")
  dev_create_parser.add_argument(
      "-nsk",
      "--node_selector_key",
      dest="node_selector_key",
      help="The node selector key")
  dev_create_parser.add_argument(
      "-nsv",
      "--node_selector_value",
      dest="node_selector_value",
      help="The node selector value")
  dev_create_parser.add_argument(
    "-W", "--watch", action="store_true", help="Watch the status of dev_env creation")
  dev_create_parser.add_argument(
    "-net", "--network", action="store_true", help="network configuration")
  dev_create_parser.add_argument(
    "-e",
    "--owner_email",
    dest="owner_email",
    help="The value is owner email."
  )
  dev_create_parser.add_argument(
    "--priority_class",
    dest="priority_class",
    choices=constant.CLOUDML_DEV_PRIORITY_CLASSES,
    default=constant.DEV_DEFAULT_PRIORITY_CLASS,
    help="The value of priority_class."
  )
  dev_create_parser.set_defaults(func=util.create_dev_env)

  # subcommand of dev: describe
  dev_describe_parser = dev_subparser.add_parser(
      "describe", help="Describe the dev environment")
  dev_describe_parser.add_argument(
      "dev_name", help="The name of dev environment")
  dev_describe_parser.set_defaults(func=util.describe_dev_env)

  # subcommand of dev: stop
  dev_stop_parser = dev_subparser.add_parser(
      "stop", help="Stop the dev environment(s)")
  dev_stop_parser.add_argument(
      "dev_names", metavar="dev_name", nargs="+", help="The name of dev environment")
  dev_stop_parser.set_defaults(func=util.stop_dev_env)

  # subcommand of dev: stop_v2
  dev_stop_v2_parser = dev_subparser.add_parser(
      "stop_v2", help="Stop the dev environment(s)")
  dev_stop_v2_parser.add_argument(
      "dev_names", metavar="dev_name", nargs="+", help="The name of dev environment")
  dev_stop_v2_parser.set_defaults(func=util.v2_stop_dev_env)

  # subcommand of dev: save
  dev_save_parser = dev_subparser.add_parser(
      "save", help="Save the dev environment(s)")
  dev_save_parser.add_argument(
      "dev_names", metavar="dev_name", nargs="+", help="The name of dev environment")
  dev_save_parser.set_defaults(func=util.save_dev_env)

  # subcommand of dev: save_v2
  dev_save_v2_parser = dev_subparser.add_parser(
      "save_v2", help="Save the dev environment")
  dev_save_v2_parser.add_argument(
      "dev_name", metavar="dev_name", help="The name of dev dev")
  dev_save_v2_parser.add_argument(
      "env_name", metavar="env_name", help="The name of saved environment")
  dev_save_v2_parser.add_argument(
      "-f", "--force", action="store_true", help="ignore exist environment")
  dev_save_v2_parser.set_defaults(func=util.v2_save_dev_env)

  # subcommand of dev: restart
  dev_restart_parser = dev_subparser.add_parser(
      "restart", help="Restart the dev environment(s)")
  dev_restart_parser.add_argument(
      "dev_names", metavar="dev_name", nargs="+", help="The name of dev environment")
  dev_restart_parser.set_defaults(func=util.restart_dev_env)

  # subcommand of dev: scale
  dev_scale_parser = dev_subparser.add_parser(
      "scale", help="Scale the dev environment")
  dev_scale_parser.add_argument(
      "dev_name", help="The name of dev environment")
  dev_scale_parser.add_argument(
      "-c",
      "--cpu_limit",
      dest="cpu_limit",
      help="The CPU limit with unit core")
  dev_scale_parser.add_argument(
      "-M",
      "--memory_limit",
      dest="memory_limit",
      help="The memory limit with unit K, M or G")
  dev_scale_parser.add_argument(
      "-g", "--gpu_limit", dest="gpu_limit", help="The number of GPU limit")
  dev_scale_parser.add_argument(
      "-gm", "--gpu_memory", dest="gpu_memory", help="The required least GPU memory")
  dev_scale_parser.add_argument(
      "-d", "--docker_image", dest="docker_image", help="The docker images.")
  dev_scale_parser.set_defaults(func=util.scale_dev_env)

  # subcommand of dev: delete
  dev_delete_parser = dev_subparser.add_parser(
      "delete", help="Delete the dev environment(s)")
  dev_delete_parser.add_argument(
      "dev_names", metavar="dev_name", nargs="+", help="The name of dev environment")
  dev_delete_parser.set_defaults(func=util.delete_dev_env)

  # subcommand of dev: events
  dev_events_parser = dev_subparser.add_parser(
      "events", help="Get the events of the dev environment")
  dev_events_parser.add_argument(
      "dev_name", help="The name of dev environment")
  dev_events_parser.set_defaults(func=util.get_dev_env_events)

  # subcommand of dev: metrics
  dev_metrics_parser = dev_subparser.add_parser(
      "metrics", help="Get the metrics of the dev environment")
  dev_metrics_parser.add_argument(
      "dev_name", help="The name of dev environment")
  dev_metrics_parser.set_defaults(func=util.get_dev_env_metrics)

  # subcommand: dev_server
  dev_server_parser = main_subparser.add_parser(
      "dev_server", help="Commands about dev_server")
  dev_server_subparser = dev_server_parser.add_subparsers(
      dest="dev_server_command", help="Subcommands of dev_server")

  # subcommand of dev_server: list
  dev_server_list_parser = dev_server_subparser.add_parser(
      "list", help="List dev servers")
  dev_server_list_parser.set_defaults(func=util.list_dev_servers)

  # subcommand of dev_server: create
  dev_server_create_parser = dev_server_subparser.add_parser(
      "create", help="Create dev server")
  dev_server_create_parser.add_argument(
      "-n",
      "--dev_name",
      dest="dev_name",
      help="The dev environment name",
      required=True)
  dev_server_create_parser.add_argument(
      "-p",
      "--password",
      dest="password",
      help="The password of ipython notebook",
      required=True)
  dev_server_create_parser.add_argument(
      "-d", "--docker_image", dest="docker_image", help="The ")
  dev_server_create_parser.add_argument(
      "-dc",
      "--docker_command",
      dest="docker_command",
      help="The docker command")
  dev_server_create_parser.add_argument(
      "-F",
      "--framework",
      dest="framework",
      help="The framework of machine learning")
  dev_server_create_parser.add_argument(
      "-V",
      "--framework_version",
      dest="framework_version",
      help="The version of the framework")
  dev_server_create_parser.add_argument(
      "-R",
      "--enable_rank",
      dest="enable_rank",
      action="store_true",
      help="Create priority high dev_server"
  )
  dev_server_create_parser.set_defaults(func=util.create_dev_server)

  # subcommand of dev_server: describe
  dev_server_describe_parser = dev_server_subparser.add_parser(
      "describe", help="Describe the dev server")
  dev_server_describe_parser.add_argument(
      "dev_name", help="The name of dev server")
  dev_server_describe_parser.set_defaults(func=util.describe_dev_server)

  # subcommand of dev_server: delete
  dev_server_delete_parser = dev_server_subparser.add_parser(
      "delete", help="Delete the dev server(s)")
  dev_server_delete_parser.add_argument(
      "dev_names", metavar="dev_name", nargs="+", help="The name of dev server")
  dev_server_delete_parser.set_defaults(func=util.delete_dev_server)

  # subcommand of dev_server: events
  dev_server_events_parser = dev_server_subparser.add_parser(
      "events", help="Get the events of the dev server")
  dev_server_events_parser.add_argument(
      "dev_name", help="The name of dev server")
  dev_server_events_parser.set_defaults(func=util.get_dev_server_events)

  # subcommand: quota
  quota_parser = main_subparser.add_parser(
      "quota", help="Commands about quota")
  quota_subparser = quota_parser.add_subparsers(
      dest="quota_command", help="Subcommands of quota")

  # subcommand of quota: list
  quota_list_parser = quota_subparser.add_parser("list", help="List the quota")
  quota_list_parser.set_defaults(func=util.list_quota)

  # subcommand of quota: list
  example = """
  Examples:
      dev apply:         cloudml quota update -t dev -g 1 -c 8 -M 30G -e youmail@xiaomi.com
      jobs apply:        cloudml quota update -t jobs -g 2 -c 1 -M 30G -e youmail@xiaomi.com
      models apply:      cloudml quota update -t models -g 1 -c 8 -M 30G -e youmail@xiaomi.com
      tensorboard apply: cloudml quota update -t tensorboard -T 10  -e youmail@xiaomi.com
  """
  quota_update_parser = quota_subparser.add_parser("update", help="Apply for a quota update", epilog=example ,formatter_class=argparse.RawDescriptionHelpFormatter)
  quota_update_parser.add_argument(
      "-t",
      "--type",
      dest="type",
      help="select service type (dev/jobs/models/tensorboard)",
      required=True
  )
  quota_update_parser.add_argument(
      "-c",
      "--cpu_limit",
      dest="cpu_limit",
      help="The number of CPU limit",
      required=False)
  quota_update_parser.add_argument(
      "-M",
      "--memory_limit",
      dest="memory_limit",
      help="The memory limit, e.g. 2G",
      required=False)
  quota_update_parser.add_argument(
      "-g",
      "--gpu_limit",
      dest="gpu_limit",
      help="The number of GPU limit",
      required=False)
  quota_update_parser.add_argument(
      "-T",
      "--tensorboard",
      dest="tensorboard",
      help="The number of tensorboard limit",
      required=False)
  quota_update_parser.add_argument(
      "-atl",
      "--allow_third_level_leader",
      type=util.str2bool,
      nargs="?",
      const=True,
      dest="allow_third_level_leader",
      help="Whether allow third_level_leader",
      required=False
      )

  quota_update_parser.add_argument(
      "-e",
      "--org_mail",
      dest="org_mail",
      help="The mail of yours",
      required=True)
  quota_update_parser.add_argument(
      "-l",
      "--second_org_leader",
      dest="second_org_leader",
      help="The second org leader email",
      required=False)
  quota_update_parser.set_defaults(func=util.apply_quota)

  # subcommand: quota_v2
  quota_v2_parser = main_subparser.add_parser(
      "quota_v2", help="Commands about quota")
  quota_v2_subparser = quota_v2_parser.add_subparsers(
      dest="quota_command", help="Subcommands of quota")

  # subcommand of quota: list
  quota_v2_list_parser = quota_v2_subparser.add_parser("list", help="List the quota")
  quota_v2_list_parser.set_defaults(func=quota_v2.list_quota_v2)

  # subcommand of quota: list
  example = """
  Examples:
      guaranteed quota apply:         cloudml quota_v2 update -p guaranteed -g 1 -c 8 -M 30G -e youmail@xiaomi.com
      preferred quota apply:         cloudml quota_v2 update -p preferred -g 1 -c 8 -M 30G -e youmail@xiaomi.com
      best-effort quota apply:         cloudml quota_v2 update -p best-effort -g 1 -c 8 -M 30G -e youmail@xiaomi.com
  """
  quota_v2_update_parser = quota_v2_subparser.add_parser("update", help="Apply for a quota update", epilog=example ,formatter_class=argparse.RawDescriptionHelpFormatter)
  quota_v2_update_parser.add_argument(
      "-p",
      "--priority_class",
      dest="priority_class",
      help="select priority class type (guaranteed/preferred/best-effort)",
      required=True
  )
  quota_v2_update_parser.add_argument(
      "-c",
      "--cpu_limit",
      dest="cpu_limit",
      help="The number of CPU limit",
      required=False)
  quota_v2_update_parser.add_argument(
      "-M",
      "--memory_limit",
      dest="memory_limit",
      help="The memory limit, e.g. 2G",
      required=False)
  quota_v2_update_parser.add_argument(
      "-g",
      "--gpu_limit",
      dest="gpu_limit",
      help="The number of GPU limit",
      required=False)
  quota_v2_update_parser.add_argument(
      "-e",
      "--org_mail",
      dest="org_mail",
      help="The mail of yours",
      required=True)
  quota_v2_update_parser.add_argument("-gt", "--gpu-type", dest="gpu_types",
                             action='append', help="The GPU type to add quota.")
  quota_v2_update_parser.add_argument("-gc", "--gpu-count", dest="gpu_counts",
                             action='append', help="The GPU count of current type to add quota.")
  quota_v2_update_parser.set_defaults(func=quota_v2.apply_quota_v2)

  # subcommand: framework
  framework_parser = main_subparser.add_parser(
      "framework", help="Commands about framework")
  framework_subparser = framework_parser.add_subparsers(
      dest="framework_command", help="Subcommands of framework")

  # subcommand of framework: list
  framework_list_parser = framework_subparser.add_parser(
      "list", help="List the frameworks")
  framework_list_parser.set_defaults(func=util.list_framework)


  # subcommand of pipeline
  pipeline_parser = main_subparser.add_parser(
    "pipeline", help="Commands about pipeline")
  pipeline_subparser = pipeline_parser.add_subparsers(
    dest="pipeline_command", help="Subcommands of pipeline")

  # subcommand of pipeline: list
  pipeline_list_parser = pipeline_subparser.add_parser(
    "list", help="List pipelines")
  pipeline_list_parser.add_argument("-p", "--priority_class", dest="priority_class",
                                    choices=constant.CLOUDML_PRIORITY_CLASSES,
                                    help="The priority class to list jobs with. If not set, will return all.")
  pipeline_list_parser.add_argument("-pn", "--page_number", type=int, dest="page_number",
                                    help="The page number of the list.")
  pipeline_list_parser.add_argument("-ps", "--page_size", type=int, dest="page_size",
                                    help="The page size of the list.")
  pipeline_list_parser.set_defaults(func=util.list_pipelines)

  # subcommand of pipeline: list
  pipeline_schedule_parser = pipeline_subparser.add_parser(
    "schedulelist", help="Pipeline schedule")
  pipeline_schedule_parser.add_argument("pipeline_name", help="The pipeline name of the schedule")
  pipeline_schedule_parser.set_defaults(func=util.list_pipeline_schedules)


  # subcommand of pipeline: list
  pipeline_schedule_parser = pipeline_subparser.add_parser(
    "scheduledescribe", help="Describe Pipeline schedule")
  pipeline_schedule_parser.add_argument("schedule_id", help="The id of the schedule")
  pipeline_schedule_parser.set_defaults(func=util.describe_pipeline_schedule)

  # subcommand of pipeline: list
  pipeline_schedule_parser = pipeline_subparser.add_parser(
    "schedule", help="Pipeline schedule")
  pipeline_schedule_parser.add_argument("pipeline_name", help="The pipeline name of the schedule")
  pipeline_schedule_parser.add_argument("-d", "--pipeline_rerun_date", help="The expected run date of the schedule")
  pipeline_schedule_parser.set_defaults(func=util.create_pipeline_schedule)


  # subcommand of pipeline: delete
  pipeline_delete_parser = pipeline_subparser.add_parser(
      "delete", help="Delete the pipeline(s)")
  pipeline_delete_parser.add_argument("pipeline_names",
                                      metavar="pipeline_name",
                                      nargs="+",
                                      help="The name of the pipeline")
  pipeline_delete_parser.set_defaults(func=util.delete_pipeline)

  # subcommand of pipeline: describe
  pipeline_describe_parser = pipeline_subparser.add_parser(
    "describe", help="Describe pipeline")
  pipeline_describe_parser.add_argument("pipeline_name", help="The pipeline to describe")
  pipeline_describe_parser.set_defaults(func=util.describe_pipeline)

  # subcommand of pipeline: logs
  pipeline_logs_parser = pipeline_subparser.add_parser(
      "logs", help="Get the logs of the pipeline pod")
  pipeline_logs_parser.add_argument("pod_name", help="The pod name of the pipeline")
  pipeline_logs_parser.set_defaults(func=util.get_pipeline_logs)

  # subcommand of pipeline: logs
  pipeline_schedule_logs_parser = pipeline_subparser.add_parser(
      "schedulelogs", help="Get the logs of the pipeline schedule pod")
  pipeline_schedule_logs_parser.add_argument("pod_name", help="The pod name of the pipeline")
  pipeline_schedule_logs_parser.set_defaults(func=util.get_pipeline_logs)

  # subcommand of pipeline: start
  pipeline_start_parser = pipeline_subparser.add_parser(
      "start", help="start the pipeline")
  pipeline_start_parser.add_argument("pipeline_name", help="The name of the pipeline")
  pipeline_start_parser.set_defaults(func=util.start_pipeline)

  # subcommand of pipeline: stop
  pipeline_stop_parser = pipeline_subparser.add_parser(
      "stop", help="Stop the pipeline")
  pipeline_stop_parser.add_argument("pipeline_name", help="The name of the pipeline")
  pipeline_stop_parser.set_defaults(func=util.stop_pipeline)

  # subcommand of pipeline: stop
  pipeline_rerun_parser = pipeline_subparser.add_parser(
      "rerun", help="Rerun the pipeline")
  pipeline_rerun_parser.add_argument("pipeline_name", help="The name of the pipeline")
  pipeline_rerun_parser.add_argument(
    "-node", "--single_node", dest="single_node", help="The single node")
  pipeline_rerun_parser.add_argument(
    "-sn", "--start_node", dest="start_node", help="The start_node")
  pipeline_rerun_parser.add_argument(
    "-dr", "--date_range", dest="date_range", help="The date_range")
  pipeline_rerun_parser.set_defaults(func=util.rerun_pipeline)


  # subcommand of pipeline : create
  pipeline_create_parser = pipeline_subparser.add_parser(
    "create", help="Create pipeline")
  pipeline_create_parser.add_argument(
    "-n", "--pipeline_name", dest="pipeline_name", help="The pipeline name")
  pipeline_create_parser.add_argument(
    "--priority_class",
    dest="priority_class",
    choices=constant.CLOUDML_PRIORITY_CLASSES,
    default=constant.CLOUDML_DEFAULT_PRIORITY_CLASS,
    help="The value of priority_class."
  )
  pipeline_update_parser = pipeline_subparser.add_parser(
    "update", help="Update pipeline")
  pipeline_update_parser.add_argument("pipeline_name", help="The name of the pipeline")

  def add_pipeline_create_update_argument():
    for parser in [pipeline_update_parser, pipeline_create_parser]:
      parser.add_argument(
        "-f",
        "--filename",
        dest="filename",
        help="The json file contains the pipeline task msg")
      parser.add_argument(
        "-a", "--pipeline_config", dest="pipeline_config", help="The string of config ")
      parser.add_argument(
        "-ap", "--alarm_phone", dest="alarm_phone", help="The alarm phone number")
      parser.add_argument(
        "-fe", "--fds_endpoint", dest="fds_endpoint", help="The fds endpoint use")
      parser.add_argument(
        "-fb", "--fds_bucket", dest="fds_bucket", help="The fds bucket to mount to /fds")
      # for hdfs
      parser.add_argument(
        "-hka", "--hdfs_krb_account", type=str, dest="hdfs_krb_account", help="The kerberos account"
      )
      parser.add_argument(
        "-hkp", "--hdfs_krb_password", type=str, dest="hdfs_krb_password", help="The kerberos password"
      )
      parser.add_argument(
        "-hkt", "--hdfs_krb_keytab", type=str, dest="hdfs_krb_keytab", help="The kerberos keytab secret name"
      )
      parser.add_argument(
        "-he", "--hdfs_endpoint", type=str, dest="hdfs_endpoint", help="The hadoop path to mount to /hdfs"
      )
      parser.add_argument(
        "-ce", "--cloudml_endpoint", type=str, dest="cloudml_endpoint", help="The cloudml endpoint")
      parser.add_argument(
        "-om", "--org_mail", type=str, dest="org_mail", help="The org mail")
      parser.add_argument(
          "-sv",
          "--save_mode",
          dest="save_mode",
          help="Save pipeline, don't run",
          action='store_true')
      parser.add_argument(
          "-W", "--watch", action="store_true", help="Watch the status of pipeline")

      parser.add_argument(
        "-mt", "--max_wait_time", type=int, default=43200, help="Seconds of max watch time, default 43200 seconds")
  add_pipeline_create_update_argument()
  pipeline_create_parser.set_defaults(func=util.create_pipeline)
  pipeline_update_parser.set_defaults(func=util.update_pipeline)


  # subcommand: ceph
  ceph_parser = main_subparser.add_parser(
      "ceph", help="Commands about ceph volume")
  ceph_subparser = ceph_parser.add_subparsers(
      dest="ceph_command", help="Subcommands of ceph")

  # subcommand of ceph: list
  ceph_list_parser = ceph_subparser.add_parser(
      "list", help="List ceph volumes")
  ceph_list_parser.set_defaults(func=util.list_cephs)

  # subcommand of ceph: create
  ceph_create_parser = ceph_subparser.add_parser(
      "create", help="Create a ceph volume")
  ceph_create_parser.add_argument(
      "-n",
      "--ceph_name",
      dest="ceph_name",
      help="The name of ceph volume",
      required=True)
  ceph_create_parser.add_argument(
      "-c",
      "--capacity",
      dest="capacity",
      help="The capacity of ceph volume, e.g. 2M, 3G, 4T",
      required=True)
  ceph_create_parser.set_defaults(func=util.create_ceph)

  # subcommand of ceph: describe
  ceph_describe_parser = ceph_subparser.add_parser(
      "describe", help="Describe a ceph volume")
  ceph_describe_parser.add_argument(
      "ceph_name", help="The name of the ceph volume")
  ceph_describe_parser.set_defaults(
      func=util.describe_ceph)

  # subcommand of ceph: delete
  ceph_delete_parser = ceph_subparser.add_parser(
      "delete", help="Delete ceph volume(s)")
  ceph_delete_parser.add_argument(
      "ceph_names", metavar="ceph_name", nargs="+", help="The name of the ceph volume")
  ceph_delete_parser.add_argument(
      "-f", "--force", action="store_true", help="ignore ceph status and mount")
  ceph_delete_parser.set_defaults(func=util.delete_ceph)

  # subcommand of ceph: events
  ceph_events_parser = ceph_subparser.add_parser(
      "events", help="Get the events of the ceph volume")
  ceph_events_parser.add_argument(
      "ceph_name", help="The name of the ceph volume")
  ceph_events_parser.set_defaults(
      func=util.get_ceph_events)

  # subcommand of ceph: sync
  ceph_sync_parser = ceph_subparser.add_parser(
      "sync", help="Sync files from fds to  a ceph volume")
  ceph_sync_parser.add_argument(
      "-n",
      "--ceph_name",
      dest="ceph_name",
      help="The name of ceph volume",
      required=True)
  ceph_sync_parser.add_argument(
        "-fe", "--fds_endpoint", dest="fds_endpoint", help="The fds endpoint",required=True)
  ceph_sync_parser.add_argument(
        "-fb", "--fds_bucket", dest="fds_bucket", help="The fds bucket",required=True)
  ceph_sync_parser.set_defaults(func=util.sync_ceph)


  # subcommand: env
  env_parser = main_subparser.add_parser(
      "env", help="Commands about env")
  env_subparser = env_parser.add_subparsers(
      dest="env_command", help="Subcommands of env")

  # subcommand of env: list
  env_list_parser = env_subparser.add_parser(
      "list", help="List envs")
  env_list_parser.set_defaults(func=util.list_envs)
  # subcommand of env: describe
  env_describe_parser = env_subparser.add_parser(
    "describe", help="Describe a env")
  env_describe_parser.add_argument(
    "env_name", help="The name of the env")
  env_describe_parser.set_defaults(
    func=util.describe_env)

  # subcommand of env: delete
  env_delete_parser = env_subparser.add_parser(
    "delete", help="Delete envs")
  env_delete_parser.add_argument(
    "env_names", metavar="env_name", nargs="+", help="The name of the env volume")
  env_delete_parser.set_defaults(func=util.delete_env)

  # subcommand: secret
  secret_parser = main_subparser.add_parser(
      "secret", help="Commands about secret. [DEPRECATED] Please use hdfs_secret instead.")
  secret_subparser = secret_parser.add_subparsers(
      dest="secret_command", help="Subcommands of secret")

  # subcommand of secret: list
  secret_list_parser = secret_subparser.add_parser(
      "list", help="List secrets")
  secret_list_parser.set_defaults(func=secret.list_secret_services)

  # subcommand of secret: create
  secret_create_parser = secret_subparser.add_parser(
      "create", help="Create secret")
  secret_create_parser.add_argument(
      "-n",
      "--secret_name",
      dest="secret_name",
      help="The name of the secret",
      required=True)
  secret_create_parser.add_argument(
      "-f",
      "--file_path",
      dest="file_path",
      help="The file path that used to create secret with.",
      required=True)
  secret_create_parser.set_defaults(func=secret.create_secret_service)

  # subcommand of secret: describe
  secret_describe_parser = secret_subparser.add_parser(
      "describe", help="Describe the secret")
  secret_describe_parser.add_argument(
      "secret_name", help="The name of the secret")
  secret_describe_parser.set_defaults(
      func=secret.describe_secret_service)

  # subcommand of secret: delete
  secret_delete_parser = secret_subparser.add_parser(
      "delete", help="Delete the secret(s)")
  secret_delete_parser.add_argument(
      "secret_names", metavar="secret_name", nargs="+", help="The name of the secret")
  secret_delete_parser.set_defaults(func=secret.delete_secret_service)


  # subcommand: hdfs-secret
  hdfs_secret_parser = main_subparser.add_parser(
      "hdfs-secret", help="Commands about hdfs-secret")
  hdfs_secret_subparser = hdfs_secret_parser.add_subparsers(
      dest="hdfs_secret_command", help="Subcommands of hdfs-secret")

  # subcommand of hdfs secret: list
  hdfs_secret_list_parser = hdfs_secret_subparser.add_parser(
      "list", help="List secrets")
  hdfs_secret_list_parser.set_defaults(func=secret.list_hdfs_secret_services)

  # subcommand of hdfs secret: create
  hdfs_secret_create_parser = hdfs_secret_subparser.add_parser(
      "create", help="Create secret")
  hdfs_secret_create_parser.add_argument(
      "-n",
      "--secret_name",
      dest="secret_name",
      help="The name of the secret",
      required=True)
  hdfs_secret_create_parser.add_argument(
      "-f",
      "--keytab_file_path",
      dest="keytab_file_path",
      help="The keytab file path that used to create secret with.",
  )
  hdfs_secret_create_parser.add_argument(
      "-p",
      "--password",
      dest="password",
      help="The password that need to store as secret.",
  )
  hdfs_secret_create_parser.set_defaults(func=secret.create_hdfs_secret_service)

  # subcommand of hdfs secret: describe
  hdfs_secret_describe_parser = hdfs_secret_subparser.add_parser(
      "describe", help="Describe the secret")
  hdfs_secret_describe_parser.add_argument(
      "secret_name", help="The name of the secret")
  hdfs_secret_describe_parser.set_defaults(
      func=secret.describe_hdfs_secret_service)

  # subcommand of hdfs secret: delete
  hdfs_secret_delete_parser = hdfs_secret_subparser.add_parser(
      "delete", help="Delete the secret(s)")
  hdfs_secret_delete_parser.add_argument(
      "secret_names", metavar="secret_name", nargs="+", help="The name of the secret")
  hdfs_secret_delete_parser.set_defaults(func=secret.delete_hdfs_secret_service)


  # subcommand: fds-secret
  fds_secret_parser = main_subparser.add_parser(
      "fds-secret", help="Commands about fds-secret")
  fds_secret_subparser = fds_secret_parser.add_subparsers(
      dest="fds_secret_command", help="Subcommands of fds-secret")

  # subcommand of fds secret: list
  fds_secret_list_parser = fds_secret_subparser.add_parser(
      "list", help="List secrets")
  fds_secret_list_parser.set_defaults(func=secret.list_fds_secret_services)

  # subcommand of fds secret: create
  fds_secret_create_parser = fds_secret_subparser.add_parser(
      "create", help="Create secret")
  fds_secret_create_parser.add_argument(
      "-n",
      "--secret_name",
      dest="secret_name",
      help="The name of the secret",
      required=True)
  fds_secret_create_parser.add_argument(
      "-ak",
      "--access_key",
      dest="access_key",
      help="The access_key that need to store as secret.",
      required=True)
  fds_secret_create_parser.add_argument(
      "-sk",
      "--secret_key",
      dest="secret_key",
      help="The secret_key that need to store as secret.",
      required=True)
  fds_secret_create_parser.set_defaults(func=secret.create_fds_secret_service)

  # subcommand of fds secret: describe
  fds_secret_describe_parser = fds_secret_subparser.add_parser(
      "describe", help="Describe the secret")
  fds_secret_describe_parser.add_argument(
      "secret_name", help="The name of the secret")
  fds_secret_describe_parser.set_defaults(
      func=secret.describe_fds_secret_service)

  # subcommand of fds secret: delete
  fds_secret_delete_parser = fds_secret_subparser.add_parser(
      "delete", help="Delete the secret(s)")
  fds_secret_delete_parser.add_argument(
      "secret_names", metavar="secret_name", nargs="+", help="The name of the secret")
  fds_secret_delete_parser.set_defaults(func=secret.delete_fds_secret_service)


  # subcommand: git-secret
  git_secret_parser = main_subparser.add_parser(
      "git-secret", help="Commands about git-secret")
  git_secret_subparser = git_secret_parser.add_subparsers(
      dest="git_secret_command", help="Subcommands of git-secret")

  # subcommand of git secret: list
  git_secret_list_parser = git_secret_subparser.add_parser(
      "list", help="List secrets")
  git_secret_list_parser.set_defaults(func=secret.list_git_secret_services)

  # subcommand of git secret: create
  git_secret_create_parser = git_secret_subparser.add_parser(
      "create", help="Create secret")
  git_secret_create_parser.add_argument(
      "-n",
      "--secret_name",
      dest="secret_name",
      help="The name of the secret",
      required=True)
  git_secret_create_parser.add_argument(
      "-ssh",
      "--ssh",
      dest="ssh",
      help="The ssh private key file path that used to create secret with.",
      required=True)
  git_secret_create_parser.add_argument(
      "-kh",
      "--known_hosts",
      dest="known_hosts",
      help="The known_hosts file path that used to create secret with.",
      required=True)
  git_secret_create_parser.set_defaults(func=secret.create_git_secret_service)

  # subcommand of git secret: describe
  git_secret_describe_parser = git_secret_subparser.add_parser(
      "describe", help="Describe the secret")
  git_secret_describe_parser.add_argument(
      "secret_name", help="The name of the secret")
  git_secret_describe_parser.set_defaults(
      func=secret.describe_git_secret_service)

  # subcommand of git secret: delete
  git_secret_delete_parser = fds_secret_subparser.add_parser(
      "delete", help="Delete the secret(s)")
  git_secret_delete_parser.add_argument(
      "secret_names", metavar="secret_name", nargs="+", help="The name of the secret")
  git_secret_delete_parser.set_defaults(func=secret.delete_git_secret_service)


  # subcommand of schedule
  schedule_parser = main_subparser.add_parser(
    "schedule", help="Commands about schedule")
  schedule_subparser = schedule_parser.add_subparsers(
    dest="schedule_command", help="Subcommands of schedule")

  # subcommand of schedule: list
  schedule_list_parser = schedule_subparser.add_parser(
    "list", help="List schedules")
  schedule_list_parser.set_defaults(func=util.list_schedules)

  # subcommand of schedule: delete
  schedule_delete_parser = schedule_subparser.add_parser(
    "delete", help="Delete the schedule(s)")
  schedule_delete_parser.add_argument("schedule_names",
                                      metavar="schedule_name",
                                      nargs="+",
                                      help="The name of the schedule")
  schedule_delete_parser.set_defaults(func=util.delete_schedule)

  # subcommand of schedule: describe
  schedule_describe_parser = schedule_subparser.add_parser(
    "describe", help="Describe schedule")
  schedule_describe_parser.add_argument("schedule_name", help="The schedule to describe")
  schedule_describe_parser.set_defaults(func=util.describe_schedule)

  # subcommand of schedule: start
  schedule_start_parser = schedule_subparser.add_parser(
    "start", help="start the schedule")
  schedule_start_parser.add_argument("schedule_name", help="The name of the schedule")
  schedule_start_parser.set_defaults(func=util.start_schedule)

  # subcommand of schedule: stop
  schedule_stop_parser = schedule_subparser.add_parser(
    "stop", help="Stop the schedule")
  schedule_stop_parser.add_argument("schedule_name", help="The name of the schedule")
  schedule_stop_parser.set_defaults(func=util.stop_schedule)


  # subcommand of schedule: stop
  schedule_logs_parser = schedule_subparser.add_parser(
    "logs", help="Get logs of the schedule")
  schedule_logs_parser.add_argument("schedule_name", help="The name of the schedule")
  schedule_logs_parser.set_defaults(func=util.get_schedule_logs)

  # subcommand of schedule : create
  schedule_create_parser = schedule_subparser.add_parser(
    "create", help="Create schedule")
  schedule_create_parser.add_argument(
    "-n", "--schedule_name", dest="schedule_name", help="The schedule name")
  schedule_update_parser = schedule_subparser.add_parser(
    "update", help="Update schedule")
  schedule_update_parser.add_argument("schedule_name", help="The name of the schedule")

  def add_schedule_create_update_argument():
    for parser in [schedule_update_parser, schedule_create_parser]:
      parser.add_argument(
        "-rn",
        "--resource_name",
        type=str,
        dest="resource_name",
        help="The resource_name of the schedule")
      parser.add_argument(
        "-rt", "--resource_type", type=str, dest="resource_type", help="The type of the schedule resource")
      parser.add_argument(
        "-p", "--cron_param", type=str, dest="cron_param", help="The cron param of the schedule")
      parser.add_argument(
        "-shl", "--success_history_limit", dest="success_history_limit", type=int, help="The success history limit of the schedule")
      # for hdfs
      parser.add_argument(
        "-fhl", "--failed_history_limit", type=int, dest="failed_history_limit", help="The failed history limit of the schedule"
      )
      parser.add_argument(
        "-spd", "--suspend", type=util.str2bool, dest="suspend", help="Whether to suspend the schedule when create"
      )
      parser.add_argument(
        "-cp", "--concurrency_policy", type=str, dest="concurrency_policy", help="The concurrent policy of the schedule: Allow Replace Forbid"
      )
      parser.add_argument(
        "-d", "--image_name", type=str, dest="image_name", help="The image name of the cronjob")

  add_schedule_create_update_argument()
  schedule_create_parser.set_defaults(func=util.create_schedule)
  schedule_update_parser.set_defaults(func=util.update_schedule)

  # subcommand: project
  project_parser = main_subparser.add_parser(
      "project", help="Commands about project")
  project_subparser = project_parser.add_subparsers(
      dest="project_command", help="Subcommands of project")

  # subcommand of project: list
  project_list_parser = project_subparser.add_parser(
      "list", help="List projects")
  project_list_parser.set_defaults(func=train_v2.list_project_services)

  # subcommand of project: create
  project_create_parser = project_subparser.add_parser(
      "create", help="Create project")
  project_create_parser.add_argument(
      "-n",
      "--project_name",
      dest="project_name",
      help="The name of the project",
      required=True)
  project_create_parser.add_argument(
      "-d",
      "--description",
      dest="description",
      help="The description of the project")
  project_create_parser.add_argument(
      "-cs",
      "--code_scheme",
      dest="code_scheme",
      default="git",
      help="The scheme of the project code. Supported: git, http, ftp, ssh. Default: git")
  project_create_parser.add_argument(
      "-cu",
      "--code_uri",
      dest="code_uri",
      help="The uri of the project code")
  project_create_parser.add_argument(
    "-ck",
    "--code_secret",
    dest="code_secret",
    help="The secret used to sync project code")
  project_create_parser.set_defaults(func=train_v2.create_project_service)

  #TODO(xychu): add update support

  # subcommand of project: describe
  project_describe_parser = project_subparser.add_parser(
      "describe", help="Describe the project")
  project_describe_parser.add_argument(
      "project_name", help="The name of the project")
  project_describe_parser.set_defaults(
      func=train_v2.describe_project_service)

  # subcommand of project: delete
  project_delete_parser = project_subparser.add_parser(
      "delete", help="Delete the project(s)")
  project_delete_parser.add_argument(
      "project_names", metavar="project_name", nargs="+", help="The name of the project")
  project_delete_parser.set_defaults(func=train_v2.delete_project_service)

  # subcommand: all
  all_parser = main_subparser.add_parser(
      "all", help="Commands about all")
  all_subparser = all_parser.add_subparsers(
      dest="all_command", help="Subcommands of all")

  # subcommand of all: list
  all_list_parser = all_subparser.add_parser(
      "list", help="List all resources")
  all_list_parser.set_defaults(func=util.list_all)

  # subcommand: cluster resources info
  resources_parser = main_subparser.add_parser(
      "resources", help="Commands about available resources"
  )
  resources_subparser = resources_parser.add_subparsers(
      dest = "resources_command", help= "Subcommands of resources"
  )
  # subcommand of resources: list
  resources_list_parser = resources_subparser.add_parser(
      "list", help="list cluster available resources"
  )
  resources_list_parser.set_defaults(func=util.list_cluster_available_resources)

  # subcommand: dataset
  dataset_parser = main_subparser.add_parser("dataset", help="Commands about dataset")
  dataset_subparser = dataset_parser.add_subparsers(
      dest="dataset_command", help="Subcommands of dataset")

  # subcommand of dataset: list
  dataset_list_parser = dataset_subparser.add_parser("list", help="List dataset")
  dataset_list_parser.add_argument(
      "-a",
      "--access_key",
      dest="access_key")
  dataset_list_parser.set_defaults(func=datasets.list_dataset)

  dataset_exist_parser = dataset_subparser.add_parser("exist", help="Judge dataset is exist")
  dataset_exist_parser.add_argument(
      "-n",
      "--dataset_name",
      dest="dataset_name",
      help="The dataset name",
      required=True)
  dataset_exist_parser.add_argument(
      "-v",
      "--version",
      dest="version",
      help="The dataset version",
      required=True)
  dataset_exist_parser.set_defaults(func=datasets.exist_dataset)

  # subcommand of dataset: create
  dataset_create_parser = dataset_subparser.add_parser(
      "create", help="Create dataset")
  dataset_create_parser.add_argument(
      "-n",
      "--dataset_name",
      dest="dataset_name",
      help="The dataset name",
      required=True)
  dataset_create_parser.add_argument(
      "-t",
      "--type",
      dest="type",
      help="The dataset type, text or photo",
      required=True)
  dataset_create_parser.add_argument(
      "-v",
      "--version",
      dest="version",
      help="The dataset version",
      required=True)
  dataset_create_parser.add_argument(
      "-fp",
      "--filepath",
      dest="filepath",
      help="The dataset path",
      required=True)
  dataset_create_parser.add_argument(
      "-a",
      "--access_key",
      dest="access_key")
  dataset_create_parser.set_defaults(func=datasets.create_dataset)

  dataset_download_parser = dataset_subparser.add_parser(
      "download", help="Download dataset")
  dataset_download_parser.add_argument(
      "-n",
      "--dataset_name",
      dest="dataset_name",
      help="The dataset name",
      required=True)
  dataset_download_parser.add_argument(
      "-v",
      "--version",
      dest="version",
      help="The dataset version",
      required=True)
  dataset_download_parser.add_argument(
      "-t",
      "--type",
      dest="type",
      help="The dataset type",
      required=True)
  dataset_download_parser.add_argument(
      "-fp",
      "--filepath",
      dest="filepath",
      help="The dataset dwonload path",
      required=True)
  dataset_download_parser.add_argument(
      "-l",
      "--label_name",
      dest="label_name",
      help="The dataset label",
      default="")
  dataset_download_parser.set_defaults(func=datasets.download_dataset)

  dataset_label_parser = dataset_subparser.add_parser("label", help="Label dataset")
  dataset_label_parser.add_argument(
      "-n",
      "--label_name",
      dest="label_name",
      help="The label name",
      required=True)
  dataset_label_parser.add_argument(
      "-t",
      "--type",
      dest="type",
      help="The label type: add, remove, delete",
      required=True)
  dataset_label_parser.add_argument(
      "-f",
      "--filepath",
      dest="filepath",
      help="The labeled file's local path.")
  dataset_label_parser.set_defaults(func=datasets.label_dataset)

  dataset_share_parser = dataset_subparser.add_parser("share", help="Share dataset")
  dataset_share_parser.add_argument(
      "-n",
      "--dataset_name",
      dest="dataset_name",
      help="The shared dataset name",
      required=True)
  dataset_share_parser.add_argument(
      "-o",
      "--org_id",
      dest="org_id",
      help="The org_id which you will share to.")
  dataset_share_parser.add_argument(
      "-t",
      "--type",
      dest="type",
      help="The share type: add, remove.",
      required=True)
  
  dataset_share_parser.set_defaults(func=datasets.share_dataset)

  return parser

def main():
  parser = new_parser()
  # For auto-complete
  argcomplete.autocomplete(parser)

  if len(sys.argv) == 1:
    args = parser.parse_args(["-h"])
  else:
    arg_list = sys.argv[1:]
    # work around bug fixing: argparse failed when -a "--model_name", but successful when -a " --model_name"
    try:
      job_args_idx = arg_list.index('-a')
      arg_list[job_args_idx + 1] = ' ' + arg_list[job_args_idx + 1]
    except:
      pass
    args = parser.parse_args(arg_list)

    # store commmand line
    argv = [sys.argv[0]]
    argv.extend(arg_list)
    GlobalArgsStore(argv)

  logging.basicConfig(level=args.loglevel)

  args.func(args)



if __name__ == "__main__":
  main()
