#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright 2017 Xiaomi, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import logging
import sys

from cloud_ml_sdk.command.constant import GlobalArgsStore

sys.path.append("../../")

from . import util
from . import secret
from . import namespaced_user
from . import constant
from . import quota_v2

def main():
  parser = argparse.ArgumentParser()

  parser.add_argument(
      '--debug',
      help="Print debugging logs",
      action="store_const", dest="loglevel", const=logging.DEBUG,
      default=logging.WARNING,
  )

  parser.add_argument(
      "-k", "--context", dest="config_context_name", action=util.SetConfigContextEnv,
      help="The cloudml config context name, available values can be listed by: cloudml config show")

  main_subparser = parser.add_subparsers(dest="command_group", help="Commands")

  # subcommand: hptunings
  hptunings_parser = main_subparser.add_parser("hp", help="Commands about hp jobs")
  hptunings_subparser = hptunings_parser.add_subparsers(
      dest="hp_command", help="Subcommands of hp jobs")

  # subcommand of hptunings: list
  hptunings_list_parser = hptunings_subparser.add_parser("list", help="List hp jobs")
  hptunings_list_parser.add_argument(
      "org_id", help="The org_id to list (with -1 stand for all)")
  hptunings_list_parser.add_argument("-p", "--priority_class", dest="priority_class",
                                     choices=constant.CLOUDML_PRIORITY_CLASSES,
                                     help="The priority class to list jobs with. If not set, will return all.")
  hptunings_list_parser.add_argument("-pn", "--page_number", type=int, dest="page_number",
                                    help="The page number of the list!")
  hptunings_list_parser.add_argument("-ps", "--page_size", type=int, dest="page_size",
                                    help="The page size of the list!")
  hptunings_list_parser.set_defaults(func=util.list_hp)

  # subcommand of jobs: describe
  hptunings_describe_parser = hptunings_subparser.add_parser(
      "describe", help="Describe hp job")
  hptunings_describe_parser.add_argument("hp_name", help="The name of the hp job")
  hptunings_describe_parser.add_argument(
      "org_id", help="The org_id of the hp job's owner")
  hptunings_describe_parser.set_defaults(func=util.describe_hp_job)


  # subcommand of hptunings: get trials
  hptuning_trials_parser = hptunings_subparser.add_parser("trials", help="get trials")
  hptuning_trials_parser.add_argument("hp_name",
                                      help="the hptuning job the trials belong to")
  hptuning_trials_parser.add_argument(
      "org_id", help="The org_id of the hp job's owner")
  hptuning_trials_parser.set_defaults(func=util.get_trials)


  # subcommand of hp jobs: logs
  hptunings_trial_logs_parser = hptunings_subparser.add_parser(
      "logs", help="Get the logs of the hp job trial")
  hptunings_trial_logs_parser.add_argument("hp_name",help="the hptuning job the trials belong to")

  hptunings_trial_logs_parser.add_argument("-ti",
                                         "--trial_id",
                                         dest="trial_id",
                                         help="the log of the trial(pod)")
  hptunings_trial_logs_parser.add_argument("-f", "--follow", action="store_true",
                                help="Follow the log stream of the pod. Defaults to false.")
  hptunings_trial_logs_parser.add_argument("-n", "--lines", type=int, help="The last number of lines to show.")

  hptunings_trial_logs_parser.add_argument(
      "org_id", help="The org_id of the hp job's owner")
  hptunings_trial_logs_parser.set_defaults(func=util.get_trial_log)


  # subcommand of hp jobs: events
  hptunings_events_parser = hptunings_subparser.add_parser(
      "events", help="Get the events of the hp job")
  hptunings_events_parser.add_argument("hp_name", help="The name of the hp job")
  hptunings_events_parser.add_argument(
      "org_id", help="The org_id of the hp job's owner")
  hptunings_events_parser.set_defaults(func=util.get_hp_job_events)

  # subcommand: jobs
  jobs_parser = main_subparser.add_parser("jobs", help="Commands about jobs")
  jobs_subparser = jobs_parser.add_subparsers(
      dest="job_command", help="Subcommands of jobs")

  # subcommand of jobs: list
  jobs_list_parser = jobs_subparser.add_parser("list", help="List jobs")
  jobs_list_parser.add_argument("-n", "--job_name", dest="job_name", help="The job will be output.")
  jobs_list_parser.add_argument("-s", "--job_status", dest="job_status", help="The status of the  output job: pending, processing, completed, failed.")
  jobs_list_parser.add_argument(
      "org_id", help="The org_id to list (with -1 stand for all)")
  jobs_list_parser.add_argument("-p", "--priority_class", dest="priority_class",
                                choices=constant.CLOUDML_PRIORITY_CLASSES,
                                help="The priority class to list jobs with. If not set, will return all.")
  jobs_list_parser.add_argument("-pn", "--page_number", type=int,  dest="page_number",
                                  help="The page number of the list.")
  jobs_list_parser.add_argument("-ps", "--page_size", type=int, dest="page_size",
                                  help="The page size of the list.")
  jobs_list_parser.set_defaults(func=util.list_jobs)

  # subcommand of jobs: describe
  jobs_describe_parser = jobs_subparser.add_parser(
      "describe", help="Describe job")
  jobs_describe_parser.add_argument("job_name", help="The name of the job")
  jobs_describe_parser.add_argument(
      "org_id", help="The org_id of the job's owner")
  jobs_describe_parser.set_defaults(func=util.describe_job)

  # subcommand of jobs: 'kill', set activeDeadlineSeconds to 5
  jobs_kill_parser = jobs_subparser.add_parser(
      "kill", help="Kill job instance. Will release resources but not delete train job.")
  jobs_kill_parser.add_argument("job_name", help="The job to kill")
  jobs_kill_parser.add_argument(
      "org_id", help="The org_id of the job's owner")
  jobs_kill_parser.set_defaults(func=util.kill_job)

  # subcommand of jobs: logs
  jobs_logs_parser = jobs_subparser.add_parser(
      "logs", help="Get the logs of the job")
  jobs_logs_parser.add_argument("-f", "--follow", action="store_true",
                                help="Follow the log stream of the pod. Defaults to false.")
  jobs_logs_parser.add_argument("-n", "--lines", type=int, help="The last number of lines to show.")
  jobs_logs_parser.add_argument("job_name", help="The job to get the logs")
  jobs_logs_parser.add_argument(
      "org_id", help="The org_id of the job's owner")
  jobs_logs_parser.set_defaults(func=util.get_job_logs)

  # subcommand of jobs: events
  jobs_events_parser = jobs_subparser.add_parser(
      "events", help="Get the events of the train job")
  jobs_events_parser.add_argument("job_name", help="The name of the train job")
  jobs_events_parser.add_argument(
      "org_id", help="The org_id of the job's owner")
  jobs_events_parser.set_defaults(func=util.get_train_job_events)

  # subcommand of jobs: delete
  jobs_delete_parser = jobs_subparser.add_parser(
      "delete", help="Delete the job(s)")
  jobs_delete_parser.add_argument("job_names", metavar="job_name", nargs="+", help="The name of the job")
  jobs_delete_parser.add_argument("org_id", help="The org_id of the job's owner")
  jobs_delete_parser.set_defaults(func=util.delete_job)

  # subcommand of jobs: 'rerun', set status to enqueued
  jobs_rerun_parser = jobs_subparser.add_parser(
      "rerun", help="Rerun job task.")
  jobs_rerun_parser.add_argument("job_name", help="The job to rerun")
  jobs_rerun_parser.add_argument(
      "org_id", help="The org_id of the job's owner")
  jobs_rerun_parser.set_defaults(func=util.rerun_job)

  # subcommand: model
  models_parser = main_subparser.add_parser(
      "models", help="Commands about models")
  models_subparser = models_parser.add_subparsers(
      dest="model_command", help="Subcommands of models")

  # subcommand of models: list
  models_list_parser = models_subparser.add_parser("list", help="List models")
  models_list_parser.add_argument(
      "org_id", help="The org_id to list (with -1 stand for all)")
  models_list_parser.add_argument("-p", "--priority_class", dest="priority_class",
                                  choices=constant.CLOUDML_PRIORITY_CLASSES,
                                  help="The priority class to list jobs with. If not set, will return all.")
  models_list_parser.add_argument("-pn", "--page_number", type=int,  dest="page_number",
                                  help="The page number of the list.")
  models_list_parser.add_argument("-ps", "--page_size", type=int, dest="page_size",
                                  help="The page size of the list.")
  models_list_parser.set_defaults(func=util.list_models)

  # subcommand of models: describe
  models_describe_parser = models_subparser.add_parser(
      "describe", help="Describe model")
  models_describe_parser.add_argument(
      "model_name", help="The name of the model")
  models_describe_parser.add_argument(
      "model_version", help="The version of the model")
  models_describe_parser.add_argument(
      "org_id", help="The org_id of the model's owner")
  models_describe_parser.set_defaults(func=util.describe_model)

  # subcommand of models: logs
  models_logs_parser = models_subparser.add_parser(
      "logs", help="Get the logs of the model service")
  models_logs_parser.add_argument("model_name", help="The name of the model")
  models_logs_parser.add_argument(
      "model_version", help="The version of the model")
  models_logs_parser.add_argument(
      "org_id", help="The org_id of the model's owner")
  models_logs_parser.set_defaults(func=util.get_model_logs)

  # subcommand of models: events
  models_events_parser = models_subparser.add_parser(
      "events", help="Get the events of the model service")
  models_events_parser.add_argument(
      "model_name", help="The name of the model service")
  models_events_parser.add_argument(
      "model_version", help="The version of the model service")
  models_events_parser.add_argument(
      "org_id", help="The org_id of the model's owner")
  models_events_parser.set_defaults(func=util.get_model_service_events)

    # subcommand of models: delete
  models_delete_parser = models_subparser.add_parser(
      "delete", help="Delete the model service")
  models_delete_parser.add_argument("model_name", help="The name of the model")
  models_delete_parser.add_argument(
      "model_version", help="The version of the model")
  models_delete_parser.add_argument(
      "org_id", help="The org_id of the model's owner")
  models_delete_parser.set_defaults(func=util.delete_model)

  models_daildown_parser = models_subparser.add_parser(
      "daildown", help="Dail Down the model")
  models_daildown_parser.add_argument(
      "model_name", help="The name of the model service")
  models_daildown_parser.add_argument(
      "model_version")
  models_daildown_parser.add_argument(
      "org_id", help="The org_id of the model's owner")
  models_daildown_parser.set_defaults(func=util.model_daildown)

  models_onboard_parse = models_subparser.add_parser(
      "onboard", help="Formal Onboard model service")
  models_onboard_parse.add_argument(
      "model_name", help="The name of model service")
  models_onboard_parse.add_argument(
      "model_version", help="The version of model service")
  models_onboard_parse.add_argument(
      "org_id", help="The org_id of the model's owner")
  models_onboard_parse.set_defaults(func=util.model_onboard)

  models_restart_parser = models_subparser.add_parser(
      "restart", help="Restart model service instart."
  )
  models_restart_parser.add_argument(
      "model_name", help="The name of model service."
  )
  models_restart_parser.add_argument(
      "model_version", help="The version of model version."
  )
  models_restart_parser.add_argument(
      "org_id", help="The org_id of the model's owner."
  )
  models_restart_parser.set_defaults(func=util.restart_model_service_instance)

  # subcommand: autoscaler
  autoscaler_parser = main_subparser.add_parser(
      "autoscaler", help="Commands about tensorboard")
  autoscaler_subparser = autoscaler_parser.add_subparsers(
      dest="autoscaler_command", help="Subcommands of autoscaler"
  )
  # subcommand of autoscaler: list
  autoscaler_list_parser = autoscaler_subparser.add_parser(
      "list", help="List autoscaler rule."
  )
  autoscaler_list_parser.add_argument(
      "org_id", help="Query target org id."
  )
  autoscaler_list_parser.set_defaults(func=util.list_autoscaler_rule)

  # subcommand of autoscaler: describe
  autoscaler_describe_parser = autoscaler_subparser.add_parser(
      "describe", help="Describe the autoscaler rule."
  )
  autoscaler_describe_parser.add_argument(
      "model_name", help="The name of model service name."
  )
  autoscaler_describe_parser.add_argument(
      "model_version", help="The version of model service version."
  )
  autoscaler_describe_parser.add_argument(
      "org_id", help="Query target org id."
  )
  autoscaler_describe_parser.set_defaults(func=util.describe_autoscaler_rule)

  # subcommand of autoscaler: create
  autoscaler_create_parser = autoscaler_subparser.add_parser(
    "create", help="Create autoscaler rule."
  )
  autoscaler_create_parser.add_argument(
      "model_name",
      help="The value of model service name."
  )
  autoscaler_create_parser.add_argument(
      "model_version",
      help="The value of model service verison."
  )
  autoscaler_create_parser.add_argument(
    "-jf",
    "--config_json_file",
    dest="config_json_file",
    help=('Valid json config file name in current directory, example:\n'
          '{"trigger_type": "qps",\n'
          '"threshold_max_replicas": 5,\n'
          '"up_threshold_qps": 20,\n'
          '"threshold_min_replicas": 1,\n'
          '"down_threshold_qps": 15,\n'
          '"collect_metrics_duration": 1,\n'
          '"instance_up_step": 1,\n'
          '"instance_down_step": 1,\n'
          '"trigger_interval": 5,\n'
          '"up_qps_buffer_percent": 0.2,\n'
          '"cron_configs": [{\n'
          '\t"time_start_cron_str": "01 01 * * *",\n'
          '\t"time_end_cron_str": "10 01 * * *",\n'
          '\t"target_replica": 5\n'
          '\t},\n'
          '\t{\n'
          '\t"time_start_cron_str": "02 02 * * *",\n'
          '\t"time_end_cron_str": "10 02 * * *",\n'
          '\t"target_replica": 5\n'
          '\t}\n'
          '\t],\n'
          '"falcon_qps_counter_name": "counter_sample_1",\n'
          '"falcon_endpoints": [\n'
          '\t"c5-ai-i1-sample01.bj",\n'
          '\t"c5-ai-i1-sample02.bj"\n'
          ']}'),
    required=True
  )
  autoscaler_create_parser.add_argument(
    "org_id", help="org_id")
  # autoscaler_create_parser.add_argument(
  #   "-yf",
  #   "--config_yaml_file",
  #   dest="config_yaml_file",
  #   help="The path to a valid yaml config file, void all other arguments"
  # )
  autoscaler_create_parser.set_defaults(func=util.create_autoscaler_rule)

  # subcommand of autoscaler: update
  autoscaler_update_parser = autoscaler_subparser.add_parser(
    "update", help="Update autoscaler rule."
  )
  autoscaler_update_parser.add_argument(
      "model_name",
      help="The value of model service name."
  )
  autoscaler_update_parser.add_argument(
      "model_version",
      help="The value of model service verison."
  )
  autoscaler_update_parser.add_argument(
    "-jf",
    "--config_json_file",
    dest="config_json_file",
    help=('Valid json config file name in current directory, example:\n'
          '{"trigger_type": "qps",\n'
          '"threshold_max_replicas": 5,\n'
          '"up_threshold_qps": 20,\n'
          '"threshold_min_replicas": 1,\n'
          '"down_threshold_qps": 15,\n'
          '"collect_metrics_duration": 1,\n'
          '"instance_up_step": 1,\n'
          '"instance_down_step": 1,\n'
          '"trigger_interval": 5,\n'
          '"up_qps_buffer_percent": 0.2,\n'
          '"cron_configs": [{\n'
          '\t"time_start_cron_str": "01 01 * * *",\n'
          '\t"time_end_cron_str": "10 01 * * *",\n'
          '\t"target_replica": 5\n'
          '\t},\n'
          '\t{\n'
          '\t"time_start_cron_str": "02 02 * * *",\n'
          '\t"time_end_cron_str": "10 02 * * *",\n'
          '\t"target_replica": 5\n'
          '\t}\n'
          '\t],\n'
          '"falcon_qps_counter_name": "counter_sample_1",\n'
          '"falcon_endpoints": [\n'
          '\t"c5-ai-i1-sample01.bj",\n'
          '\t"c5-ai-i1-sample02.bj"\n'
          ']}'),
    required=True
  )
  autoscaler_update_parser.add_argument(
    "org_id", help="org_id")
  autoscaler_update_parser.set_defaults(func=util.update_autoscaler_rule)

  # subcommand of autoscaler: delete
  autoscaler_delete_parser = autoscaler_subparser.add_parser(
    "delete", help="Delete autoscaler rule."
  )
  autoscaler_delete_parser.add_argument(
    "model_name", help="The value of model service name."
  )
  autoscaler_delete_parser.add_argument(
    "model_version", help="The value of model service verison."
  )
  autoscaler_delete_parser.add_argument(
    "org_id", help="Query target org id."
  )
  autoscaler_delete_parser.set_defaults(func=util.delete_autoscaler_rule)

  # subcommand: dev
  dev_parser = main_subparser.add_parser("dev", help="Commands about dev")
  dev_subparser = dev_parser.add_subparsers(
      dest="dev_command", help="Subcommands of dev")

  # subcommand of dev: list
  dev_list_parser = dev_subparser.add_parser("list", help="List devs")
  dev_list_parser.add_argument(
      "org_id", help="The org_id to list (with -1 stand for all)")
  dev_list_parser.add_argument("-p", "--priority_class", dest="priority_class",
                               choices=constant.CLOUDML_PRIORITY_CLASSES,
                               help="The priority class to list jobs with. If not set, will return all.")
  dev_list_parser.add_argument("-pn", "--page_number", type=int, dest="page_number",
                               help="The page number of the list.")
  dev_list_parser.add_argument("-ps", "--page_size", type=int, dest="page_size",
                               help="The page size of the list.")
  dev_list_parser.set_defaults(func=util.list_dev_envs)

  # subcommand of dev: describe
  dev_describe_parser = dev_subparser.add_parser(
      "describe", help="Describe dev")
  dev_describe_parser.add_argument("dev_name", help="The name of the dev")
  dev_describe_parser.add_argument(
      "org_id", help="The org_id of the dev's owner")
  dev_describe_parser.set_defaults(func=util.describe_dev_env)

  # subcommand: dev_server
  dev_server_parser = main_subparser.add_parser(
      "dev_server", help="Commands about dev_server")
  dev_server_subparser = dev_server_parser.add_subparsers(
      dest="dev_server_command", help="Subcommands of dev_server")

  # subcommand of dev_server: list
  dev_server_list_parser = dev_server_subparser.add_parser(
      "list", help="List dev_servers")
  dev_server_list_parser.add_argument(
      "org_id", help="The org_id to list (with -1 stand for all)")
  dev_server_list_parser.set_defaults(func=util.list_dev_servers)

  # subcommand of dev_server: describe
  dev_server_describe_parser = dev_server_subparser.add_parser(
      "describe", help="Describe dev_server")
  dev_server_describe_parser.add_argument(
      "dev_name", help="The name of the dev_server")
  dev_server_describe_parser.add_argument(
      "org_id", help="The org_id of the dev_server's owner")
  dev_server_describe_parser.set_defaults(func=util.describe_dev_server)

    # subcommand of dev: stop
  dev_stop_parser = dev_subparser.add_parser(
      "stop", help="Stop the dev environment(s)")
  dev_stop_parser.add_argument(
      "dev_names", metavar="dev_name", nargs="+", help="The name of dev environment")
  dev_stop_parser.add_argument("org_id", help="The org_id of the dev's owner")
  dev_stop_parser.set_defaults(func=util.stop_dev_env)

  # subcommand of dev: stop
  dev_stop_v2_parser = dev_subparser.add_parser(
      "stop_v2", help="Stop the dev environment(s)")
  dev_stop_v2_parser.add_argument(
      "dev_names", metavar="dev_name", nargs="+", help="The names of dev environment")
  dev_stop_v2_parser.add_argument("org_id", help="The org_id of the dev's owner")
  dev_stop_v2_parser.set_defaults(func=util.v2_stop_dev_env)

  # subcommand of dev: save
  dev_save_parser = dev_subparser.add_parser(
      "save", help="Save the dev environment(s)")
  dev_save_parser.add_argument(
      "dev_names", metavar="dev_name", nargs="+", help="The name of dev environment")
  dev_save_parser.add_argument("org_id", help="The org_id of the dev's owner")
  dev_save_parser.set_defaults(func=util.save_dev_env)


  # subcommand of dev: save_v2
  dev_save_v2_parser = dev_subparser.add_parser(
      "save_v2", help="Save the dev environment")
  dev_save_v2_parser.add_argument(
      "dev_name", metavar="dev_name", help="The name of dev dev")
  dev_save_v2_parser.add_argument(
      "env_name", metavar="env_name", help="The name of saved environment")
  dev_save_v2_parser.add_argument(
      "-f", "--force", action="store_true", help="ignore exist environment")
  dev_save_v2_parser.add_argument("org_id", help="The org_id of the dev's owner")
  dev_save_v2_parser.set_defaults(func=util.v2_save_dev_env)

  # subcommand of dev: restart
  dev_restart_parser = dev_subparser.add_parser(
      "restart", help="Restart the dev environment(s)")
  dev_restart_parser.add_argument(
      "dev_names", metavar="dev_name", nargs="+", help="The name of dev environment")
  dev_restart_parser.add_argument("org_id", help="The org_id of the dev's owner")
  dev_restart_parser.set_defaults(func=util.restart_dev_env)

  # subcommand of dev: delete
  dev_delete_parser = dev_subparser.add_parser(
      "delete", help="Delete the dev environment(s)")
  dev_delete_parser.add_argument(
      "dev_names", metavar="dev_name", nargs="+", help="The name of dev environment")
  dev_delete_parser.add_argument("org_id", help="The org_id of the dev's owner")
  dev_delete_parser.set_defaults(func=util.delete_dev_env)

  # subcommand: tensorboard
  tensorboard_parser = main_subparser.add_parser(
      "tensorboard", help="Commands about tensorboard")
  tensorboard_subparser = tensorboard_parser.add_subparsers(
      dest="tensorboard_command", help="Subcommands of tensorboard")

  # subcommand of tensorboard: list
  tensorboard_list_parser = tensorboard_subparser.add_parser(
      "list", help="List tensorboard")
  tensorboard_list_parser.add_argument(
      "org_id", help="The org_id to list (with -1 stand for all)")
  tensorboard_list_parser.add_argument("-p", "--priority_class", dest="priority_class",
                                       choices=constant.CLOUDML_PRIORITY_CLASSES,
                                       help="The priority class to list jobs with. If not set, will return all.")
  tensorboard_list_parser.add_argument("-pn", "--page_number", type=int, dest="page_number",
                                       help="The page number of the list.")
  tensorboard_list_parser.add_argument("-ps", "--page_size", type=int, dest="page_size",
                                       help="The page size of the list.")
  tensorboard_list_parser.set_defaults(func=util.list_tensorboard_services)

  # subcommand of tensorboard: describe
  tensorboard_describe_parser = tensorboard_subparser.add_parser(
      "describe", help="Describe tensorboard")
  tensorboard_describe_parser.add_argument(
      "tensorboard_name", help="The name of the tensorboard")
  tensorboard_describe_parser.add_argument(
      "org_id", help="The org_id of the tensorboard's owner")
  tensorboard_describe_parser.set_defaults(
      func=util.describe_tensorboard_service)

    # subcommand of tensorboard: delete
  tensorboard_delete_parser = tensorboard_subparser.add_parser(
      "delete", help="Delete the tensorboard(s)")
  tensorboard_delete_parser.add_argument(
      "tensorboard_names", metavar="tensorboard_name", nargs="+", help="The name of the tensorboard")
  tensorboard_delete_parser.add_argument("org_id", help="The org_id of the tensorboard's owner")
  tensorboard_delete_parser.set_defaults(func=util.delete_tensorboard_service)

  # subcommand: quota
  quota_parser = main_subparser.add_parser(
      "quota", help="Commands about quota")
  quota_subparser = quota_parser.add_subparsers(
      dest="quota_command", help="Subcommands of quota")

  # subcommand of quota: list
  quota_list_parser = quota_subparser.add_parser("list", help="List the quota")
  quota_list_parser.add_argument(
      "org_id", help="Org_id to list (with -1 stand for all)")
  quota_list_parser.set_defaults(func=util.list_quota)

  # subcommand of quota: update
  quota_update_parser = quota_subparser.add_parser(
      "update", help="Update quota")
  quota_update_parser.add_argument("org_id", help="The org_id to set")
  quota_update_parser.add_argument(
      "-n", "--org_name", dest="org_name", help="The org_name to set")
  quota_update_parser.add_argument(
      "-t",
      "--type",
      dest="type",
      help="The job type to update, support for jobs, models, dev, tensorboard, total")
  quota_update_parser.add_argument(
      "-c", "--cpu", dest="cpu", help="The new cpu quota")
  quota_update_parser.add_argument(
      "-m", "--memory", dest="memory", help="The new memory quota")
  quota_update_parser.add_argument(
      "-g", "--gpu", dest="gpu", help="The new gpu quota")
  quota_update_parser.add_argument(
      "-T",
      "--tensorboard",
      dest="tensorboard",
      help="The new tensorboard quota")
  quota_update_parser.add_argument(
      "-C", "--count", dest="count", help="The count quota")
  quota_update_parser.add_argument(
      "-R", "--priority_quota", dest="priority_quota", help="The priority high quota"
  )
  quota_update_parser.add_argument(
      "-P", "--priority",dest="priority", help="Set priority rank"
  )
  quota_update_parser.add_argument(
      "-M", "--org_mail",dest="org_mail", help="Set org mail"
  )
  quota_update_parser.add_argument(
      "-l",
      "--second_org_leader",
      dest="second_org_leader",
      help="The second org leader email")
  quota_update_parser.add_argument(
    "-proj", "--project", dest="project", help="the project name"
  )
  quota_update_parser.add_argument(
      "-qt",
      "--quota_type",
      dest="quota_type",
      help="The quota type")
  quota_update_parser.add_argument(
    "-atl",
    "--allow_third_level_leader",
    type=util.str2bool,
    nargs="?",
    const=True,
    dest="allow_third_level_leader",
    help="Whether allow third_level_leader",
    required=False
    )
  quota_update_parser.set_defaults(func=util.update_quota)

  # subcommand: ceph
  ceph_parser = main_subparser.add_parser(
      "ceph", help="Commands about ceph volume")
  ceph_subparser = ceph_parser.add_subparsers(
      dest="ceph_command", help="Subcommands of ceph")

  # subcommand of ceph: list
  ceph_list_parser = ceph_subparser.add_parser(
      "list", help="List ceph volumes")
  ceph_list_parser.add_argument("org_id", help="The org id of ceph volumes (with -1 stand for all)")
  ceph_list_parser.set_defaults(func=util.list_cephs)

  # subcommand of ceph: describe
  ceph_describe_parser = ceph_subparser.add_parser(
      "describe", help="Describe a ceph volume")
  ceph_describe_parser.add_argument(
      "ceph_name", help="The name of the ceph volume")
  ceph_describe_parser.add_argument("org_id", help="The org id of ceph volume (with -1 stand for all)")
  ceph_describe_parser.set_defaults(
      func=util.describe_ceph)

  # subcommand of ceph: delete
  ceph_delete_parser = ceph_subparser.add_parser(
      "delete", help="Delete ceph volume(s)")
  ceph_delete_parser.add_argument(
      "ceph_names", metavar="ceph_name", nargs="+", help="The name of the ceph volume")
  ceph_delete_parser.add_argument(
      "-f", "--force", action="store_true", help="ignore ceph status and mount")
  ceph_delete_parser.add_argument("org_id", help="The org id of ceph volume (with -1 stand for all)")
  ceph_delete_parser.set_defaults(func=util.delete_ceph)

  # subcommand of ceph: events
  ceph_events_parser = ceph_subparser.add_parser(
      "events", help="Get the events of the ceph volume")
  ceph_events_parser.add_argument(
      "ceph_name", help="The name of the ceph volume")
  ceph_events_parser.add_argument("org_id", help="The org id of ceph volume (with -1 stand for all)")
  ceph_events_parser.set_defaults(
      func=util.get_ceph_events)


  # subcommand: env
  env_parser = main_subparser.add_parser(
      "env", help="Commands about env")
  env_subparser = env_parser.add_subparsers(
      dest="env_command", help="Subcommands of env")

  # subcommand of env: list
  env_list_parser = env_subparser.add_parser(
      "list", help="List env")
  env_list_parser.add_argument("org_id", help="The org id of env (with -1 stand for all)")
  env_list_parser.set_defaults(func=util.list_envs)

  # subcommand of env: describe
  env_describe_parser = env_subparser.add_parser(
      "describe", help="Describe a env")
  env_describe_parser.add_argument(
      "env_name", help="The name of the env")
  env_describe_parser.add_argument("org_id", help="The org id of env (with -1 stand for all)")
  env_describe_parser.set_defaults(
      func=util.describe_env)

  # subcommand of env: delete
  env_delete_parser = env_subparser.add_parser(
      "delete", help="Delete env(s)")
  env_delete_parser.add_argument(
      "env_names", metavar="env_name", nargs="+", help="The name of the env")
  env_delete_parser.add_argument("org_id", help="The org id of env (with -1 stand for all)")
  env_delete_parser.set_defaults(func=util.delete_env)

  # subcommand: secret
  secret_parser = main_subparser.add_parser(
      "secret", help="Commands about secret")
  secret_subparser = secret_parser.add_subparsers(
      dest="secret_command", help="Subcommands of secret")

  # subcommand of secret: list
  secret_list_parser = secret_subparser.add_parser(
      "list", help="List secrets")
  secret_list_parser.add_argument(
      "org_id", help="The org_id of the secret's owner")
  secret_list_parser.set_defaults(func=secret.list_secret_services)

  # subcommand of secret: describe
  secret_describe_parser = secret_subparser.add_parser(
      "describe", help="Describe the secret")
  secret_describe_parser.add_argument(
      "secret_name", help="The name of the secret")
  secret_describe_parser.add_argument(
      "org_id", help="The org_id of the secret's owner")
  secret_describe_parser.set_defaults(
      func=secret.describe_secret_service)

  # subcommand of secret: delete
  secret_delete_parser = secret_subparser.add_parser(
      "delete", help="Delete the secret(s)")
  secret_delete_parser.add_argument(
      "secret_names", metavar="secret_name", nargs="+", help="The name of the secret")
  secret_delete_parser.add_argument(
      "org_id", help="The org_id of the secret's owner")
  secret_delete_parser.set_defaults(func=secret.delete_secret_service)

  # subcommand: namespace
  namespace_parser = main_subparser.add_parser(
      "namespace", help="Commands about namespace")
  namespace_subparser = namespace_parser.add_subparsers(
      dest="namespace_command", help="Subcommands of namespace")

  # subcommand of namespace: list
  namespace_list_parser = namespace_subparser.add_parser(
      "list", help="List namespaces")
  namespace_list_parser.set_defaults(func=namespaced_user.list_namespaces)

  # subcommand of namespace: describe
  namespace_describe_parser = namespace_subparser.add_parser(
      "describe", help="Describe the namespace")
  namespace_describe_parser.add_argument(
      "name", help="The name of the namespace")
  namespace_describe_parser.set_defaults(
      func=namespaced_user.describe_namespace)

  # subcommand of namespace: create
  namespace_create_parser = namespace_subparser.add_parser(
      "create", help="Create the namespace")
  namespace_create_parser.add_argument(
      "-n", "--name", required=True, dest="name", help="The name of the namespace")
  namespace_create_parser.add_argument(
      "-e", "--owner_email", dest="owner_email", help="The email of the namespace owner")
  namespace_create_parser.add_argument(
      "-d", "--description", dest="description", help="The description of the namespace")
  namespace_create_parser.add_argument(
      "-c", "--colocate_node", dest="colocate_node", choices=["true", "false"],
      help="Use namespace as node owner label or not. Defaults to false.")
  namespace_create_parser.set_defaults(
      func=namespaced_user.create_namespace)

  # subcommand of namespace: update
  namespace_update_parser = namespace_subparser.add_parser(
      "update", help="Update the namespace")
  namespace_update_parser.add_argument(
      "name", help="The name of the namespace to update")
  namespace_update_parser.add_argument(
      "-e", "--owner_email", dest="owner_email", help="The email of the namespace owner")
  namespace_update_parser.add_argument(
      "-d", "--description", dest="description", help="The description of the namespace")
  namespace_update_parser.add_argument(
      "-c", "--colocate_node", dest="colocate_node", choices=["true", "false"],
      help="Use namespace as node owner label or not, true or false.")
  namespace_update_parser.set_defaults(
      func=namespaced_user.update_namespace)

  # subcommand of namespace: delete
  namespace_delete_parser = namespace_subparser.add_parser(
      "delete", help="Delete the namespace(s). Warning: This deletes everything under the namespace!")
  namespace_delete_parser.add_argument(
      "names", metavar="name", nargs="+", help="The name of the namespace")
  namespace_delete_parser.set_defaults(func=namespaced_user.delete_namespace)

  # subcommand: priority_class
  priority_class_parser = main_subparser.add_parser(
      "priority_class", help="Commands about priority_class")
  priority_class_subparser = priority_class_parser.add_subparsers(
      dest="priority_class_command", help="Subcommands of priority_class")

  # subcommand of priority_class: list
  priority_class_list_parser = priority_class_subparser.add_parser(
      "list", help="List priority_classes")
  priority_class_list_parser.set_defaults(func=namespaced_user.list_priority_classes)

  # subcommand of priority_class: describe
  priority_class_describe_parser = priority_class_subparser.add_parser(
      "describe", help="Describe the priority_class")
  priority_class_describe_parser.add_argument(
      "name", help="The name of the priority_class")
  priority_class_describe_parser.set_defaults(
      func=namespaced_user.describe_priority_class)

  # subcommand of priority_class: create
  priority_class_create_parser = priority_class_subparser.add_parser(
      "create", help="Create the priority_class")
  priority_class_create_parser.add_argument(
      "-n", "--name", required=True, dest="name", help="The name of the priority_class")
  priority_class_create_parser.add_argument(
      "-v", "--value", required=True, dest="value", type=int, help="The value of the priority_class")
  priority_class_create_parser.add_argument(
      "-d", "--description", dest="description", help="The description of the priority_class")
  priority_class_create_parser.set_defaults(
      func=namespaced_user.create_priority_class)

  # subcommand of priority_class: update
  priority_class_update_parser = priority_class_subparser.add_parser(
      "update", help="Update the priority_class")
  priority_class_update_parser.add_argument(
      "name", help="The name of the namespace to update")
  priority_class_update_parser.add_argument(
      "-v", "--value", dest="value", help="The value of the priority_class")
  priority_class_update_parser.add_argument(
      "-d", "--description", dest="description", help="The description of the priority_class")
  priority_class_update_parser.set_defaults(
      func=namespaced_user.update_priority_class)

  # subcommand of priority_class: delete
  priority_class_delete_parser = priority_class_subparser.add_parser(
      "delete", help="Delete the priority_class(es).")
  priority_class_delete_parser.add_argument(
      "names", metavar="name", nargs="+", help="The name of the priority_class")
  priority_class_delete_parser.set_defaults(func=namespaced_user.delete_priority_class)


  # subcommand: namespace_quota
  namespace_quota_parser = main_subparser.add_parser(
      "namespace_quota", help="Commands about namespace_quota")
  namespace_quota_subparser = namespace_quota_parser.add_subparsers(
      dest="namespace_quota_command", help="Subcommands of namespace_quota")

  # subcommand of namespace_quota: list
  namespace_quota_list_parser = namespace_quota_subparser.add_parser("list", help="List the namespace_quota")
  namespace_quota_list_parser.add_argument("-n", "--namespace", dest="namespace",
                                           help="The namespace to list quota")
  namespace_quota_list_parser.add_argument("-p", "--priority_class", dest="priority_class",
                                           help="The priority_class quota to list")
  namespace_quota_list_parser.set_defaults(func=namespaced_user.list_namespace_quota)

  # subcommand of namespace_quota: create
  namespace_quota_create_parser = namespace_quota_subparser.add_parser(
      "create", help="Create namespace_quota")
  namespace_quota_create_parser.add_argument("namespace", help="The namespace to set quota")
  namespace_quota_create_parser.add_argument(
      "-p", "--priority_class", dest="priority_class", choices=constant.CLOUDML_PRIORITY_CLASSES,
      required=True, help="The priority class of current quota")
  namespace_quota_create_parser.add_argument(
      "-c", "--cpu", dest="cpu", type=int, help="The cpu namespace_quota")
  namespace_quota_create_parser.add_argument(
      "-m", "--memory", dest="memory", type=int, help="The memory namespace_quota, unit Gi")
  namespace_quota_create_parser.add_argument(
      "-g", "--gpu", dest="gpu", type=int, help="The gpu namespace_quota")
  namespace_quota_create_parser.add_argument(
      "-o", "--org_mail", dest="org_mail", type=str, required=True, help="The user org_mail")
  namespace_quota_create_parser.add_argument("-gt", "--gpu-type", dest="gpu_types",
                             action='append', help="The GPU type to add quota.")
  namespace_quota_create_parser.add_argument("-gc", "--gpu-count", dest="gpu_counts",
                             action='append', help="The GPU count of current type to add quota.")
  namespace_quota_create_parser.set_defaults(func=namespaced_user.create_namespace_quota)

  # subcommand of namespace_quota: update
  namespace_quota_update_parser = namespace_quota_subparser.add_parser(
      "update", help="Update namespace_quota")
  namespace_quota_update_parser.add_argument("namespace", help="The namespace to set quota")
  namespace_quota_update_parser.add_argument(
      "-p", "--priority_class", dest="priority_class", choices=constant.CLOUDML_PRIORITY_CLASSES,
      required=True, help="The priority class of current quota")
  namespace_quota_update_parser.add_argument(
      "-c", "--cpu", dest="cpu", type=int, help="The cpu namespace_quota")
  namespace_quota_update_parser.add_argument(
      "-m", "--memory", dest="memory", type=int, help="The memory namespace_quota, unit Gi")
  namespace_quota_update_parser.add_argument(
      "-g", "--gpu", dest="gpu", type=int, help="The gpu namespace_quota")
  namespace_quota_update_parser.add_argument(
      "-o", "--org_mail", dest="org_mail", type=str, help="The user org_mail")
  namespace_quota_update_parser.add_argument("-gt", "--gpu-type", dest="gpu_types",
                             action='append', help="The GPU type to add quota.")
  namespace_quota_update_parser.add_argument("-gc", "--gpu-count", dest="gpu_counts",
                             action='append', help="The GPU count of current type to add quota.")
  namespace_quota_update_parser.set_defaults(func=namespaced_user.update_namespace_quota)

  # subcommand of namespace_quota: describe
  namespace_quota_describe_parser = namespace_quota_subparser.add_parser(
      "describe", help="Describe the namespace_quota")
  namespace_quota_describe_parser.add_argument(
      "namespace", help="The namespace of the namespace_quota")
  namespace_quota_describe_parser.add_argument(
      "-p", "--priority_class", dest="priority_class", choices=constant.CLOUDML_PRIORITY_CLASSES,
      help="The priority class of current quota")
  namespace_quota_describe_parser.set_defaults(
      func=namespaced_user.describe_namespace_quota)

  # subcommand of namespace_quota: delete
  namespace_quota_delete_parser = namespace_quota_subparser.add_parser(
      "delete", help="Delete the namespace_quota.")
  namespace_quota_delete_parser.add_argument(
      "namespace", help="The name of the priority_class")
  namespace_quota_delete_parser.add_argument(
      "-p", "--priority_class", dest="priority_class", choices=constant.CLOUDML_PRIORITY_CLASSES,
      required=True, help="The priority class of current quota")
  namespace_quota_delete_parser.set_defaults(func=namespaced_user.delete_namespace_quota)

  # subcommand: namespace_user
  namespace_user_parser = main_subparser.add_parser(
      "namespace_user", help="Commands about namespace_user")
  namespace_user_subparser = namespace_user_parser.add_subparsers(
      dest="namespace_user_command", help="Subcommands of namespace_user")

  # subcommand of namespace_user: list
  namespace_user_list_parser = namespace_user_subparser.add_parser("list", help="List the namespace_user")
  namespace_user_list_parser.add_argument(
    "-n", "--namespace", dest="namespace", help="The name of the namespace")
  namespace_user_list_parser.set_defaults(func=namespaced_user.list_namespace_user)

  # subcommand of namespace_user: add
  namespace_user_add_parser = namespace_user_subparser.add_parser(
      "add", help="Add namespace_user")
  namespace_user_add_parser.add_argument("namespace", help="The namespace to set")
  namespace_user_add_parser.add_argument(
      "user_ids", metavar="user_id", nargs="+", help="The ids of the user")
  namespace_user_add_parser.set_defaults(func=namespaced_user.add_namespace_user)

  # subcommand of namespace_user: remove
  namespace_user_remove_parser = namespace_user_subparser.add_parser(
      "remove", help="Remove namespace_user")
  namespace_user_remove_parser.add_argument("namespace", help="The namespace to set")
  namespace_user_remove_parser.add_argument(
      "user_ids", metavar="user_id", nargs="+", help="The ids of the user")
  namespace_user_remove_parser.set_defaults(func=namespaced_user.remove_namespace_user)

  # subcommand: quota_v2
  quota_v2_parser = main_subparser.add_parser(
      "quota_v2", help="Commands about quota_v2")
  quota_v2_subparser = quota_v2_parser.add_subparsers(
      dest="quota_command", help="Subcommands of quota_v2")

  # subcommand of quota_v2: list
  quota_v2_list_parser = quota_v2_subparser.add_parser("list", help="List the quota")
  quota_v2_list_parser.add_argument(
      "org_id", help="Org_id to list (with -1 stand for all)")
  quota_v2_list_parser.set_defaults(func=quota_v2.list_quota_v2)

  # subcommand of quota_v2: update
  quota_v2_update_parser = quota_v2_subparser.add_parser(
      "update", help="Update quota of v2")
  quota_v2_update_parser.add_argument("org_id", help="The org_id to set")
  quota_v2_update_parser.add_argument(
      "-p",
      "--priority_class",
      required=True,
      dest="priority_class",
      help="The priority of quota to update, models, guaranteed, preferred, best-effort")
  quota_v2_update_parser.add_argument(
      "-c", "--cpu", dest="cpu", type=int, help="The new cpu quota")
  quota_v2_update_parser.add_argument(
      "-m", "--memory", dest="memory", type=int, help="The new memory quota")
  quota_v2_update_parser.add_argument(
      "-g", "--gpu", dest="gpu", type=int, help="The new gpu quota")
  quota_v2_update_parser.add_argument(
      "-o", "--org_mail", dest="org_mail", help="Set org mail"
  )
  quota_v2_update_parser.add_argument("-gt", "--gpu-type", dest="gpu_types",
                                             action='append', help="The GPU type to add quota.")
  quota_v2_update_parser.add_argument("-gc", "--gpu-count", dest="gpu_counts",
                                             action='append', help="The GPU count of current type to add quota.")
  quota_v2_update_parser.add_argument(
      "-ns",
      "--namespace",
      dest="namespace",
      help="The namespace of the quota")
  quota_v2_update_parser.add_argument(
      "-d",
      "--description",
      dest="description",
      help="The description of the project")
  quota_v2_update_parser.add_argument(
      "-co", "--colocate_node", dest="colocate_node", choices=["true", "false"],
      help="Use namespace as node owner label or not, true or false.")
  quota_v2_update_parser.set_defaults(func=quota_v2.update_quota_v2)

  args = parser.parse_args()

  # store commmand line
  GlobalArgsStore(sys.argv)
  logging.basicConfig(level=args.loglevel)

  args.func(args)


if __name__ == "__main__":
  main()
