# Copyright 2019 Xiaomi, Inc.
# -*- coding: UTF-8 -*-
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import json

from cloud_ml_sdk.client import CloudMlClient
from cloud_ml_sdk.models.priority_class_service import PriorityClassService
from cloud_ml_sdk.models.namespace_service import NamespaceService
from cloud_ml_sdk.models.namespace_quota_service import NamespaceQuotaService
from cloud_ml_sdk.models.namespace_user_service import NamespaceUserService

from . import util


def list_priority_classes(args):
  """List priority_classes."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  response = client.list_priority_classes()

  if not isinstance(response, str):
    print_priority_class_list(response, args)
  else:
    print("response: {}".format(response))


def print_priority_class_list(response_priority_classes, args):
  print("{:32} {:16} {:32}".format(
    "PRIORITY_CLASS", "VALUE", "DESCRIPTION"))
  for obj in response_priority_classes:
    print("{:32} {:<16} {:32}".format(obj["name"],
                                      obj.get("value", 0),
                                      obj.get("description", "")))


def create_priority_class(args):
  """create the priority_class."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  ns = PriorityClassService(args.name,
                            args.value,
                            args.description)

  response = client.create_priority_class(ns)
  if not isinstance(response, str):
    print_priority_class_info(response)
  else:
    print("response: {}".format(response))


def update_priority_class(args):
  """update the priority_class."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  ns_json = {
    "name": args.name
  }
  if args.value:
    ns_json["value"] = args.value
  if args.description:
    ns_json["description"] = args.description
  if not args.value and not args.description:
    print("Nothing to update with.")

  try:
    response = client.update_priority_class(json.dumps(ns_json))
  except Exception as e:
    util.exit_as_error("ERROR: Failed to update priority_class: {}".format(e))
    return
  if not isinstance(response, str):
    print_priority_class_info(response)
  else:
    print("response: {}".format(response))


def print_priority_class_info(ns):
  """Print namespace in customed format.

  Args:
    ns: The dictionary of response namespace data.
  """
  print("{:16} {}".format("Name:", ns["name"]))
  print("{:16} {}".format("Value:", ns["value"]))
  print("{:16} {}".format("Description:", ns.get("description", "")))


def describe_priority_class(args):
  """Describe the priority_class."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  response = client.describe_priority_class(args.name)
  if not isinstance(response, str):
    print_priority_class_info(response)
  else:
    print("response: {}".format(response))


def delete_priority_class(args):
  """Delete the priority_class(es)."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  for ns in args.names:
    response = client.delete_priority_class(ns)
    print(response)


def list_namespaces(args):
  """List namespaces."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  response = client.list_namespaces()

  if not isinstance(response, str):
    print_namespace_list(response, args)
  else:
    print("response: {}".format(response))


def print_namespace_list(response_namespaces, args):
  print("{:32} {:32} {:16} {:32}".format(
    "NAMESPACE", "OWER_EMAIL", "COLOCATE_NODE", "DESCRIPTION"))
  for ns in response_namespaces:
    print("{:32} {:32} {:16} {:32}".format(ns["name"],
                                     ns["owner_email"],
                                     ns.get("colocate_node", "false"),
                                     ns.get("description", "")))


def create_namespace(args):
  """create the namespace."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  colocate_node = "false"
  if args.colocate_node:
    colocate_node = args.colocate_node
  ns = NamespaceService(args.name,
                        args.owner_email,
                        args.description,
                        colocate_node)

  response = client.create_namespace(ns)
  if not isinstance(response, str):
    print_namespace_info(response)
  else:
    print("response: {}".format(response))


def update_namespace(args):
  """update the namespace."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  ns_json = {
    "name": args.name
  }
  if args.owner_email:
    ns_json["owner_email"] = args.owner_email
  if args.description:
    ns_json["description"] = args.description
  if args.colocate_node:
    ns_json["colocate_node"] = args.colocate_node
  if not args.owner_email and not args.description and not args.colocate_node:
    print("Nothing to update with.")

  try:
    response = client.update_namespace(json.dumps(ns_json))
  except Exception as e:
    util.exit_as_error("ERROR: Failed to update namespace: {}".format(e))
    return
  if not isinstance(response, str):
    print_namespace_info(response)
  else:
    print("response: {}".format(response))


def print_namespace_info(ns):
  """Print namespace in customed format.

  Args:
    ns: The dictionary of response namespace data.
  """
  print("{:16} {}".format("Name:", ns["name"]))
  if "owner_email" in ns:
    print("{:16} {}".format("Owner Email:", ns["owner_email"]))
  print("{:16} {}".format("Colocate Node:", ns.get("colocate_node", "false")))
  print("{:16} {}".format("Description:", ns.get("description", "")))


def describe_namespace(args):
  """Describe the namespace."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  response = client.describe_namespace(args.name)
  if not isinstance(response, str):
    print_namespace_info(response)
  else:
    print("response: {}".format(response))


def delete_namespace(args):
  """Delete the namespace(s)."""
  confirm = raw_input("Warning: This deletes everything under the namespace!\nDo you want to continue?[y/n]")
  if confirm == "y" or confirm == "yes":
    try:
      client = CloudMlClient()
    except Exception as e:
      util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
      return

    for ns in args.names:
      response = client.delete_namespace(ns)
      print(response)


def list_namespace_quota(args):
  """List namespace quotas."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  response = client.list_namespace_quotas(args.namespace, args.priority_class)

  if not isinstance(response, str):
    print_namespace_quota_list(response)
  else:
    print("response: {}".format(response))


def print_namespace_quota_list(response_namespace_quotas):
  for quota in response_namespace_quotas:
    print("-------")
    print_namespace_quota_info(quota)


def create_namespace_quota(args):
  """create the namespace quota."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  ns_quota = NamespaceQuotaService(args.namespace,
                                   args.cpu,
                                   args.memory,
                                   args.gpu,
                                   args.priority_class,
                                   args.org_mail,
                                   args.gpu_types,
                                   args.gpu_counts)

  response = client.create_namespace_quota(ns_quota)
  if not isinstance(response, str):
    print_namespace_quota_info(response)
  else:
    print("response: {}".format(response))


def update_namespace_quota(args):
  """update the namespace quota."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  ns_quota = NamespaceQuotaService(args.namespace,
                                   args.cpu,
                                   args.memory,
                                   args.gpu,
                                   args.priority_class,
                                   args.org_mail,
                                   args.gpu_types,
                                   args.gpu_counts)

  response = client.update_namespace_quota(ns_quota)
  if not isinstance(response, str):
    print_namespace_quota_info(response)
  else:
    print("response: {}".format(response))


def print_namespace_quota_info(quota):
  """Print namespace quota in customed format.

  Args:
    ns: The dictionary of response namespace quota data.
  """
  print("{:16} {}".format("Name:", quota["name"]))
  print("{:16} {}".format("Org_mail:", quota.get("org_mail", "")))
  print("Spec:")
  for resource in quota["spec"]:
    print("\t{:32} {}".format(resource, quota["spec"][resource]))
  if "used" in quota:
    print("Used:")
    for resource in quota["used"]:
      print("\t{:32} {}".format(resource, quota["used"][resource]))


def describe_namespace_quota(args):
  """Describe the namespace quota."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  response = client.describe_namespace_quota(args.namespace, args.priority_class)
  if not isinstance(response, str):
    if not args.priority_class:
      print_namespace_quota_list(response)
    else:
      print_namespace_quota_info(response)
  else:
    print("response: {}".format(response))


def delete_namespace_quota(args):
  """Delete the namespace quota."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  response = client.delete_namespace_quota(args.namespace, args.priority_class)
  print(response)


def list_namespace_user(args):
  """List namespace users."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  response = client.list_namespace_users(args.namespace)

  if not isinstance(response, str):
    print_namespace_user_list(response)
  else:
    print("response: {}".format(response))


def print_namespace_user_list(response_namespace_users):
  print("{:32} {:32}".format(
    "NAMESPACE", "USER ID"))
  for ns_user in response_namespace_users:
    print("{:32} {:32}".format(ns_user["namespace"],
                               ns_user["user_id"]))


def add_namespace_user(args):
  """add the namespace user."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  for user_id in args.user_ids:
    ns_user = NamespaceUserService(args.namespace, user_id)

    response = client.add_namespace_user(ns_user)
    if not isinstance(response, str):
      print_namespace_user_info(response)
    else:
      print("response: {}".format(response))


def remove_namespace_user(args):
  """remove the namespace user."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  for user_id in args.user_ids:
    ns_user = NamespaceUserService(args.namespace, user_id)

    response = client.remove_namespace_user(ns_user)
    print("response: {}".format(response))


def print_namespace_user_info(ns):
  """Print namespace user in customed format.

  Args:
    ns: The dictionary of response namespace user data.
  """
  print("{:16} {}".format("Namespace:", ns["namespace"]))
  print("{:16} {}".format("User ID:", ns["user_id"]))
