import json
from . import constant
from cloud_ml_sdk.client import CloudMlClient
from cloud_ml_sdk.models.namespace_service import NamespaceService
from cloud_ml_sdk.models.namespace_quota_service import NamespaceQuotaService
from cloud_ml_sdk.models.namespace_user_service import NamespaceUserService
from cloud_ml_sdk.command.util import exit_as_error
from cloud_ml_sdk.command.namespaced_user import print_namespace_quota_info

def list_quota_v2(args):
  """List the quota."""
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if 'org_id' in args:
    response_quotas = client.get_quota_v2(org_id=args.org_id)
  else:
    response_quotas = client.get_quota_v2()
  if not isinstance(response_quotas, str):
    for quota in response_quotas:
      print("-------")
      print_quota_info_v2(quota)
  else:
    print(response_quotas)

def update_quota_v2(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if args.namespace is not None:
    namespace = args.namespace
  else:
    namespace = constant.NAMESPACE_QUOTA_FREFIX + args.org_id
  if not namespace_exist(namespace):
    if not (args.namespace or args.org_mail):
      exit_as_error("ERROR: namespace and org_mail args cannot be None!")
    colocate_node = "false"
    if args.colocate_node:
      colocate_node = args.colocate_node
    ns = NamespaceService(namespace,
                          args.org_mail,
                          args.description,
                          colocate_node)
    response = client.create_namespace(ns)
    if isinstance(response, str):
      print("response: {}".format(response))
      exit_as_error("ERROR: create namespace error!")
      return
  else:
    if args.description or args.colocate_node:
      ns_json = {
        "name": args.namespace
      }
      if args.owner_email:
        ns_json["owner_email"] = args.owner_email
      ns_json["description"] = args.description
      ns_json["colocate_node"] = args.colocate_node
      try:
        client.update_namespace(json.dumps(ns_json))
      except Exception as e:
        exit_as_error("ERROR: Failed to update namespace: {}".format(e))
        return
  if not namespace_user_exist(namespace, args.org_id):
    ns_user = NamespaceUserService(namespace, args.org_id)
    response = client.add_namespace_user(ns_user)
    if isinstance(response, str):
      print("response: {}".format(response))
      exit_as_error("ERROR: Add namespace user error!")
      return
  if not namespace_quota_exist(namespace, args.priority_class):
    ns_quota = NamespaceQuotaService(namespace,
                                     args.cpu,
                                     args.memory,
                                     args.gpu,
                                     args.priority_class,
                                     args.org_mail,
                                     args.gpu_types,
                                     args.gpu_counts)

    response = client.create_namespace_quota(ns_quota)
    if not isinstance(response, str):
      print_namespace_quota_info(response)
    else:
      print("response: {}".format(response))
      exit_as_error("ERROR: Create namespace quota error!")
      return
  else:
    ns_quota = NamespaceQuotaService(namespace,
                                     args.cpu,
                                     args.memory,
                                     args.gpu,
                                     args.priority_class,
                                     args.org_mail,
                                     args.gpu_types,
                                     args.gpu_counts)

    response = client.update_namespace_quota(ns_quota)
    if not isinstance(response, str):
      print_namespace_quota_info(response)
    else:
      print("response: {}".format(response))
      exit_as_error("ERROR: Update namespace quota error!")
      return


def apply_quota_v2(args):
  """Apply for a quota update."""

  if not (args.priority_class == "guaranteed"
          or args.priority_class == "preferred"
          or args.priority_class == "best-effort"):
    print("ERROR: Unknown  priority class {}.  Supported priority class are 'guaranteed' 'preferred' and 'best-effort'".format(args.priority_class))
  else:
    try:
      client = CloudMlClient()
    except Exception as e:
      exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
      return
    org_mail=''
    second_org_leader=''
    if (not args.org_mail) or ("@xiaomi.com" not in args.org_mail):
      exit_as_error("ERROR: Please specify the org_mail param, e.g. --org_mail=someone@xiaomi.com", )
    else:
      org_mail = args.org_mail
    data = {
      "org_mail": org_mail,
      "region": client.endpoint,
    }
    if not (args.cpu_limit and args.memory_limit and args.gpu_limit):
      exit_as_error(
        "ERROR: Please specify the cpu_limit memory_limit and gpu_limit param for priority class '{}'".format(args.priority_class))
    data["cpu_limit"]=args.cpu_limit
    data["memory_limit"]=args.memory_limit
    data["gpu_limit"]=args.gpu_limit
    data["priority_class"]=args.priority_class
    if args.gpu_types and args.gpu_counts:
      if len(args.gpu_types) != len(args.gpu_counts):
        exit_as_error("ERROR: gpu_counts per type must be same len as gpu_types!")
      data.update(dict(zip(args.gpu_types, args.gpu_counts)))
    response = client.apply_quota_v2(data)
    if not isinstance(response, str) and ("{}".format(response["error"]).lower() == "false"):
      print("Your application has been submitted for approval")
    else:
      print("response: {}".format(response))

def print_quota_info_v2(quota):
  """Print quota in customed format.

  Args:
    quota: The dictionary of response quota data.
  """
  print("{:16} {}".format("Name:", quota["name"]))
  print("{:16} {}".format("Org_mail:", quota.get("org_mail", "")))
  print("Spec:")
  for resource in quota["spec"]:
    print("\t{:32} {}".format(resource, quota["spec"][resource]))
  if "used" in quota:
    print("Used:")
    for resource in quota["used"]:
      print("\t{:32} {}".format(resource, quota["used"][resource]))


def namespace_exist(namespace):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return False
  response = client.list_namespaces()
  if isinstance(response, str):
    print("response: {}".format(response))
    return False
  else:
    for ns in response:
      if namespace == ns['name']:
        return True
  return False


def namespace_user_exist(namespace, user_id):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return False
  response = client.list_namespace_users(namespace)
  if isinstance(response, str):
    print("response: {}".format(response))
    return False
  else:
    for ns_user in response:
      if user_id == ns_user['user_id']:
        return True
  return False


def namespace_quota_exist(namespace, priority_class):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return False
  response = client.list_namespace_quotas(namespace, priority_class)

  if isinstance(response, str) or not response:
    print("response: {}".format(response))
    return False
  return True