# Copyright 2019 Xiaomi, Inc.
# -*- coding: UTF-8 -*-
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import base64

from cloud_ml_sdk.client import CloudMlClient
from cloud_ml_sdk.models.secret_service import (SecretService, HDFSSecretService,
                                                FDSSecretService, GitSecretService)

from . import constant, util


def list_secret_services(args):
  """List secret_services."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    if args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
      response_secret = client.list_secret_services(
          constant.CLOUDML_ALL_ORG_PARAMETER)
    else:
      response_secret = client.list_secret_services(args.org_id)
  else:
    response_secret = client.list_secret_services()

  if not isinstance(response_secret, str):
    print_secret_list(response_secret, args)
  else:
    print("response: {}".format(response_secret))


def print_secret_list(response_secrets, args):
  if "org_id" in args and args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
    print("{:16} {:16} {:32} {:32}".format(
      "ORG ID", "ORG NAME", "SECRET_NAME", "CREATED"))
    for secret in response_secrets:
      print("{:16} {:16} {:32} {:32}".format(secret["org_id"], secret["org_name"],
                                             secret["secret_name"], secret["create_time"]))
  else:
    print("{:32} {:32}".format(
      "SECRET_NAME", "CREATED"))
    for secret in response_secrets:
      print("{:32} {:32}".format(secret["secret_name"], secret["create_time"]))


def print_secret_info(secret):
  """Print secret in customed format.

  Args:
    secret: The dictionary of response secret data.
  """
  print("{:16} {}".format("Org id:", secret["org_id"]))
  print("{:16} {}".format("Org name:", secret["org_name"]))
  print("{:16} {}".format("secret name:", secret["secret_name"]))
  print("{:16} {}".format("Create time:", secret["create_time"]))
  #print("{:16} {}".format("Data:", secret["Data"]))


def create_secret_service(args):
  """create the secret_service."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  with open(args.file_path, 'rb') as f:
    # TODO: Check file format and verify the items
    try:
      data = base64.b64encode(f.read())
    except Exception as e:
      util.exit_as_error("ERROR: secret file({}) is not valid: {}".format(args.file_path, e))
      return

  secret = SecretService(args.secret_name, data)

  response = client.create_secret_service(secret)
  if not isinstance(response, str):
    print_secret_info(response)
  else:
    print("response: {}".format(response))


def describe_secret_service(args):
  """Describe the secret_service."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.describe_secret_service(args.secret_name,
                                              args.org_id)
  else:
    response = client.describe_secret_service(args.secret_name)
  if not isinstance(response, str):
    print_secret_info(response)
  else:
    print("response: {}".format(response))


def delete_secret_service(args):
  """Delete the secret_service(s)."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    for secret_name in args.secret_names:
      response = client.delete_secret_service(secret_name, args.org_id)
      print(response)
  else:
    for secret_name in args.secret_names:
      response = client.delete_secret_service(secret_name)
      print(response)


def list_hdfs_secret_services(args):
  """List secret_services."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    if args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
      response_secret = client.list_hdfs_secret_services(
          constant.CLOUDML_ALL_ORG_PARAMETER)
    else:
      response_secret = client.list_hdfs_secret_services(args.org_id)
  else:
    response_secret = client.list_hdfs_secret_services()

  if not isinstance(response_secret, str):
    print_secret_list(response_secret, args)
  else:
    print("response: {}".format(response_secret))


def create_hdfs_secret_service(args):
  """create the secret_service."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  if args.keytab_file_path:
    with open(args.file_path, 'rb') as f:
      # TODO: Check file format and verify the items
      try:
        keytab = base64.b64encode(f.read())
      except Exception as e:
        util.exit_as_error("ERROR: secret file({}) is not valid: {}".format(args.file_path, e))
        return
  if args.password:
    password = base64.b64encode(args.password)

  secret = HDFSSecretService(args.secret_name, keytab, password)

  response = client.create_hdfs_secret_service(secret)
  if not isinstance(response, str):
    print_secret_info(response)
  else:
    print("response: {}".format(response))


def describe_hdfs_secret_service(args):
  """Describe the secret_service."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.describe_hdfs_secret_service(args.secret_name,
                                                   args.org_id)
  else:
    response = client.describe_hdfs_secret_service(args.secret_name)
  if not isinstance(response, str):
    print_secret_info(response)
  else:
    print("response: {}".format(response))


def delete_hdfs_secret_service(args):
  """Delete the secret_service(s)."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    for secret_name in args.secret_names:
      response = client.delete_hdfs_secret_service(secret_name, args.org_id)
      print(response)
  else:
    for secret_name in args.secret_names:
      response = client.delete_hdfs_secret_service(secret_name)
      print(response)


def list_fds_secret_services(args):
  """List secret_services."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    if args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
      response_secret = client.list_fds_secret_services(
          constant.CLOUDML_ALL_ORG_PARAMETER)
    else:
      response_secret = client.list_fds_secret_services(args.org_id)
  else:
    response_secret = client.list_fds_secret_services()

  if not isinstance(response_secret, str):
    print_secret_list(response_secret, args)
  else:
    print("response: {}".format(response_secret))


def create_fds_secret_service(args):
  """create the secret_service."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  access_key = base64.b64encode(args.access_key)
  secret_key = base64.b64encode(args.secret_key)

  secret = FDSSecretService(args.secret_name, access_key, secret_key)

  response = client.create_fds_secret_service(secret)
  if not isinstance(response, str):
    print_secret_info(response)
  else:
    print("response: {}".format(response))


def describe_fds_secret_service(args):
  """Describe the secret_service."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.describe_fds_secret_service(args.secret_name,
                                                  args.org_id)
  else:
    response = client.describe_fds_secret_service(args.secret_name)
  if not isinstance(response, str):
    print_secret_info(response)
  else:
    print("response: {}".format(response))


def delete_fds_secret_service(args):
  """Delete the secret_service(s)."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    for secret_name in args.secret_names:
      response = client.delete_fds_secret_service(secret_name, args.org_id)
      print(response)
  else:
    for secret_name in args.secret_names:
      response = client.delete_fds_secret_service(secret_name)
      print(response)


def list_git_secret_services(args):
  """List secret_services."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    if args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
      response_secret = client.list_git_secret_services(
          constant.CLOUDML_ALL_ORG_PARAMETER)
    else:
      response_secret = client.list_git_secret_services(args.org_id)
  else:
    response_secret = client.list_git_secret_services()

  if not isinstance(response_secret, str):
    print_secret_list(response_secret, args)
  else:
    print("response: {}".format(response_secret))


def create_git_secret_service(args):
  """create the secret_service."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  with open(args.ssh, 'rb') as f:
    # TODO: Check file format and verify the items
    try:
      ssh = base64.b64encode(f.read())
    except Exception as e:
      util.exit_as_error("ERROR: secret file({}) is not valid: {}".format(args.ssh, e))
      return
  with open(args.known_hosts, 'rb') as f:
    # TODO: Check file format and verify the items
    try:
      known_hosts = base64.b64encode(f.read())
    except Exception as e:
      util.exit_as_error("ERROR: secret file({}) is not valid: {}".format(args.known_hosts, e))
      return

  secret = GitSecretService(args.secret_name, ssh, known_hosts)

  response = client.create_git_secret_service(secret)
  if not isinstance(response, str):
    print_secret_info(response)
  else:
    print("response: {}".format(response))


def describe_git_secret_service(args):
  """Describe the secret_service."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.describe_git_secret_service(args.secret_name,
                                              args.org_id)
  else:
    response = client.describe_git_secret_service(args.secret_name)
  if not isinstance(response, str):
    print_secret_info(response)
  else:
    print("response: {}".format(response))


def delete_git_secret_service(args):
  """Delete the secret_service(s)."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    for secret_name in args.secret_names:
      response = client.delete_git_secret_service(secret_name, args.org_id)
      print(response)
  else:
    for secret_name in args.secret_names:
      response = client.delete_git_secret_service(secret_name)
      print(response)
