# Copyright 2017 Xiaomi, Inc.
# -*- coding: UTF-8 -*-
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import json
import logging
import os
import sys
import time
import yaml

import requests

sys.path.append("../../")

from . import color_util
from . import constant
from . import util
from cloud_ml_sdk.client import CloudMlClient
from cloud_ml_sdk.models.train_job_v2 import TrainJob
from cloud_ml_sdk.models.project_service import ProjectService
from cloud_ml_sdk.models import models_util


logging.basicConfig(level=logging.DEBUG)
logging.getLogger("requests").setLevel(logging.WARNING)


def list_project_services(args):
  """List project_services."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    if args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
      response_project = client.list_project_services(
          constant.CLOUDML_ALL_ORG_PARAMETER)
    else:
      response_project = client.list_project_services(args.org_id)
  else:
    response_project = client.list_project_services()

  if not isinstance(response_project, str):
    print_project_list(response_project, args)
  else:
    print("response: {}".format(response_project))


def print_project_list(response_projects, args):
  if "org_id" in args and args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
    print("{:16} {:16} {:32} {:32}".format(
      "ORG ID", "ORG NAME", "PROJECT_NAME", "CREATED"))
    for project in response_projects:
      print("{:16} {:16} {:32} {:32}".format(project["org_id"], project["org_name"],
                                             project["project_name"], project["create_time"]))
  else:
    print("{:32} {:32} {:32}".format(
      "PROJECT_NAME", "CREATED", "DESCRIPTION"))
    for project in response_projects:
      print("{:32} {:32}  {:32}".format(project["project_name"], project["create_time"],  project["description"]))


def create_project_service(args):
  """create the project_service."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  kwargs = {}
  if args.description:
    kwargs["description"] = args.description
  if args.code_scheme:
    kwargs["code_scheme"] = args.code_scheme
  if args.code_uri:
    kwargs["code_uri"] = args.code_uri
  if args.code_secret:
    kwargs["code_secret"] = args.code_secret
  project = ProjectService(args.project_name, **kwargs)

  response = client.create_project_service(project)
  if not isinstance(response, str):
    print_project_info(response)
  else:
    print("response: {}".format(response))


def describe_project_service(args):
  """Describe the project_service."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.describe_project_service(args.project_name,
                                               args.org_id)
  else:
    response = client.describe_project_service(args.project_name)
  if not isinstance(response, str):
    print_project_info(response)
  else:
    print("response: {}".format(response))


def print_project_info(project):
  """Print project in customed format.

  Args:
    project: The dictionary of response project data.
  """
  print("{:16} {}".format("Org id:", project["org_id"]))
  print("{:16} {}".format("Org name:", project["org_name"]))
  print("{:16} {}".format("Project name:", project["project_name"]))
  print("{:16} {}".format("Create time:", project["create_time"]))
  print("{:16} {}".format("Description:", project["description"]))
  if project["code"]["uri"]:
    print("{:16}".format("Code:"))
    print("\t{:12} {}".format("Scheme:", project["code"]["scheme"]))
    print("\t{:12} {}".format("URI:", project["code"]["uri"]))
    print("\t{:12} {}".format("Secret:", project["code"]["secret"]))


def delete_project_service(args):
  """Delete the project_service(s)."""
  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    for project_name in args.project_names:
      response = client.delete_project_service(project_name, args.org_id)
      print(response)
  else:
    for project_name in args.project_names:
      response = client.delete_project_service(project_name)
      print(response)


def print_train_job_info(train_job):
  """Print train job in customized format.

  Args:
    train_job: The dictionary of response train job data.
  """
  if "message" in train_job:
    print((train_job["message"]))
  print("{:16} {}".format("Org id:", train_job["org_id"]))
  print("{:16} {}".format("Org name:", train_job["org_name"]))
  if "owner_email" in train_job:
    print("{:16} {}".format("Owner email:", train_job["owner_email"]))
  print("{:16} {}".format("Project name:", train_job["project_name"]))
  print("{:16} {}".format("Job name:", train_job["job_name"]))
  train_job_v2 = train_job
  train_job = json.loads(train_job["request_body"])
  print("{:16} {}".format("Framework:", train_job["framework"]))
  print("{:16} {}".format("Framework version:", train_job[
      "framework_version"]))
  print("{:16} {}".format("Module name:", train_job["module_name"]))
  print("{:16} {}".format("Trainer uri:", train_job["trainer_uri"]))
  print("{:16} {}".format("Job args:", train_job["job_args"]))
  print("{:16} {}".format("FDS Endpoint:", train_job["fds_endpoint"]))
  print("{:16} {}".format("Fuse mount bucket:", train_job["fds_bucket"]))
  print("{:16} {}".format("Option Fuse mounts:", train_job["fds_mounts"] if "fds_mounts" in train_job else ""))
  print("{:16} {}".format("Kerberos account:", train_job.get("hdfs_krb_account", "")))
  print("{:16} {}".format("HDFS Endpoint:", train_job.get("hdfs_endpoint", "")))
  print("{:16} {}".format("Ceph volume:", train_job["ceph_volume"] if "ceph_volume" in train_job else ""))
  print("{:16} {}".format("Ceph mode:", train_job["ceph_mode"] if "ceph_mode" in train_job else ""))
  print("{:16} {}".format("Option Ceph mounts:", train_job["ceph_mounts"] if "ceph_mounts" in train_job else ""))
  if "cpu_limit" in train_job and train_job["cpu_limit"]:
    print("{:16} {}".format("CPU limit:", train_job["cpu_limit"]))
  if "memory_limit" in train_job and train_job["memory_limit"]:
    print("{:16} {}".format("Memory limit:", train_job["memory_limit"]))
  if "gpu_limit" in train_job and train_job["gpu_limit"]:
    print("{:16} {}".format("GPU limit:", train_job["gpu_limit"]))
  print("{:16} {}".format("Docker image:", train_job.get("docker_image", "")))
  print("{:16} {}".format("Docker command:", train_job.get("docker_command", "")))
  # print("{:16} {}".format("Create time:", train_job["create_time"]))
  # print("{:16} {}".format("Update time:", train_job["update_time"]))
  # print("{:16} {}".format("Volume type:", train_job["volume_type"]))
  # print("{:16} {}".format("Volume path:", train_job["volume_path"]))
  # print("{:16} {}".format("Mount path:", train_job["mount_path"]))
  # print("{:16} {}".format("Mount read only:", train_job["mount_read_only"]))
  print("{:16} {}".format("Prepare command:", train_job.get("prepare_command", "")))
  print("{:16} {}".format("Finish command:", train_job.get("finish_command")))
  if "node_selector" in train_job and train_job["node_selector"]:
    print("{:16} {}".format("Node selector:", train_job["node_selector"]))
  if "node_name" in train_job and train_job["node_name"]:
    print("{:16} {}".format("Node name:", train_job["node_name"]))
  print("{:16} {}".format("Monitoring url:", train_job["monitoring"] if "monitoring" in train_job else ""))
  print("Spec:")
  for item in train_job_v2["spec"]:
    print("{:16} {}".format("\t"+item+":", train_job_v2["spec"][item]))
  print("Stauts:")
  for item in train_job_v2["status"]:
    if item == "conditions":
      print("\tconditions:")
      for cond in train_job_v2["status"]["conditions"]:
        print("\t\t---")
        for k, v in cond.items():
          print("{:32} {}".format("\t\t"+k+":", v))
    else:
      print("{:16} {}".format("\t"+item+":", train_job_v2["status"][item]))


def list_jobs(args):
  """List train jobs."""

  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    if args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
      response_train_jobs = client.list_train_v2(
          constant.CLOUDML_ALL_ORG_PARAMETER)
    else:
      response_train_jobs = client.list_train_v2(args.org_id)
  else:
    response_train_jobs = client.list_train_v2()
  if not isinstance(response_train_jobs, str):
    print_trainjob_list(response_train_jobs, args)
  else:
    print("response: {}".format(response_train_jobs))


def print_trainjob_list(response_train_jobs, args):
  if "org_id" in args and args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
    print("{:16} {:16} {:32} {:32} {:32} {:32}".format(
       "ORG ID", "ORG NAME", "PROJECT_NAME", "JOB_NAME", "CREATED", "STATUS"))
    for train_job in response_train_jobs:
      status = get_status_from_train_job()
      print("{:<16} {:16} {:32} {:32} {:32} {:32}".format(train_job["org_id"], train_job["org_name"],
                                                          train_job["project_name"], train_job["job_name"],
                                                      train_job["create_time"],
                                                      color_util.colorize_state(status)))
    print("{:16} {:16} {:32} {:32} {:32} {:32}".format("ORG ID", "ORG NAME", "PROJECT_NAME", "JOB_NAME", "CREATED",
                                           "STATUS"))
    for train_job in response_train_jobs:
      status = get_status_from_train_job(train_job)
      print("{:16} {:16} {:32} {:32} {:32} {:32}".format(train_job["org_id"], train_job["org_name"],
                                                         train_job["project_name"], train_job["job_name"],
                                                         train_job["create_time"],
                                                         color_util.colorize_state(status)))
  else:
    print("{:32} {:32} {:32} {:32}".format("PROJECT_NAME", "JOB_NAME", "CREATED",
                                     "STATUS"))
    for train_job in response_train_jobs:
      status = get_status_from_train_job(train_job)
      print("{:32} {:32} {:32} {:32}".format(train_job["project_name"], train_job["job_name"],
                                       train_job["create_time"],
                                      color_util.colorize_state(status)))


def get_status_from_train_job(train_job):
  if "conditions" in train_job["status"]:
    for cond in train_job["status"]["conditions"]:
      if cond.get("type", None) == "Succeeded" and cond.get("status", None) == "True":
        return constant.JOB_STATE_COMPLETED
      if cond.get("type", None) == "Failed" and cond.get("status", None) == "True":
        return constant.JOB_STATE_FAILED
  # TODO(xychu): add more status here
  return "processing"


def submit_job(args):
  """Submit the job."""
  # set environment variable to check args.trainer_uri
  if args.fds_endpoint:
    os.environ["XIAOMI_FDS_ENDPOINT"] = args.fds_endpoint
  if args.fds_bucket:
    os.environ["CLOUDML_DEFAULT_FDS_BUCKET"] = args.fds_bucket

  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  args_dict = vars(args)
  if args.filename:
    with open(args.filename) as f:
      # TODO: Check file format and verify the items
      try:
        json_data = yaml.safe_load(f)
      except Exception as e:
        util.exit_as_error("ERROR: job description json file({}) is not valid: {}".format(args.filename, e))
        return
      for key in json_data:
        if key in args_dict and not args_dict[key]:
          args_dict[key] = json_data[key]
  else:
    d = {}
    except_group = ['command_group', 'func', 'job_command']
    f = open(args.job_name + '.yaml', "w")
    for key in args_dict:
      if args_dict[key] and key not in except_group:
        d[key] = args_dict[key]
    y = yaml.safe_dump(d, default_flow_style=False)
    f.write(y)
    f.close()

  if not (args.job_name and args.module_name and args.trainer_uri and args.project_name):
    util.exit_as_error(
        "ERROR: parameters job_name, project_name, module_name and trainer_uri cannot be None.")
    return

  try:
    train_job = TrainJob(args.job_name, args.module_name, args.trainer_uri, project_name=args.project_name)

    if (args.cpu_limit or args.memory_limit or args.gpu_limit or args.gpu_type) and args.distributed:
      print("When use -D(--distributed) para, don't set resource limit by -c(--cpu_limit), -M(--memory_limit) or -g(--gpu_limit) and -gt(--gpu_type)")
      print("The resource limit of distributed jobs should be provied followed.")
      util.exit_as_error("Exited!")
      return

    if args.cluster_spec:
      if isinstance(args.cluster_spec, str):
        try:
          train_job._cluster_spec = json.loads(args.cluster_spec)
        except Exception as e:
          util.exit_as_error("The cluster spec format should be json. Got error: {}".format(e))
          return
      else:
        train_job._cluster_spec = args.cluster_spec
    elif args.distributed:
      train_job._cluster_spec = util.get_cluster_spec()
    if args.fds_endpoint:
      train_job.fds_endpoint = args.fds_endpoint
    elif client.fds_endpoint:
      train_job.fds_endpoint = client.fds_endpoint
    if args.fds_bucket:
      train_job.fds_bucket = args.fds_bucket
    if args.fds_secret:
      train_job.fds_secret = args.fds_secret
    elif client.fds_bucket:
      train_job.fds_bucket = client.fds_bucket
    if args.fds_mounts:
      train_job.fds_mounts = args.fds_mounts
    # for hdfs
    if args.hdfs_krb_account:
      train_job.hdfs_krb_account = args.hdfs_krb_account
    elif client.hdfs_krb_account:
      train_job.hdfs_krb_account = client.hdfs_krb_account
    if args.hdfs_krb_password:
      train_job.hdfs_krb_password = args.hdfs_krb_password
    elif client.hdfs_krb_password:
      train_job.hdfs_krb_password = client.hdfs_krb_password
    if args.hdfs_endpoint:
      if args.hdfs_endpoint.startswith("hdfs://"):
        train_job.hdfs_endpoint = args.hdfs_endpoint
      else:
        raise ValueError("ERROR: -he[hdfs_endpoint] must start with hdfs://")
    elif client.hdfs_endpoint:
      train_job.hdfs_endpoint = client.hdfs_endpoint
    if args.hdfs_krb_keytab:
      train_job.hdfs_krb_keytab = args.hdfs_krb_keytab
    if train_job.hdfs_krb_account:
      if not train_job.hdfs_krb_password and not train_job.hdfs_krb_keytab:
        raise ValueError("ERROR: either -hkp[hdfs_krb_password] or -hkt[hdfs_krb_keytab] is required")
      if not train_job.hdfs_endpoint:
        raise ValueError("ERROR: -he[hdfs_endpoint] is required")
    elif train_job.hdfs_endpoint:
      raise ValueError("ERROR: -hka[hdfs_krb_account] is required")
    if args.ceph_volume:
      train_job.ceph_volume = args.ceph_volume
      if not args.ceph_mode:
        raise ValueError("ERROR: -cm[ceph_mode] cannot be empty")
      if args.ceph_mode != "r" and args.ceph_mode != "rw":
        raise ValueError("ERROR: invalid value for -cm[ceph_mode]")
      train_job.ceph_mode = args.ceph_mode
    if args.ceph_mounts:
      train_job.ceph_mounts = args.ceph_mounts
    if args.job_args:
      train_job.job_args = args.job_args
    if args.cpu_limit:
      train_job.cpu_limit = args.cpu_limit
    if args.memory_limit:
      train_job.memory_limit = args.memory_limit
    if args.gpu_limit:
      train_job.gpu_limit = args.gpu_limit
    if args.gpu_memory:
      train_job.gpu_memory = args.gpu_memory
    if args.framework:
      train_job.framework = args.framework
    if args.framework_version:
      train_job.framework_version = args.framework_version
    if args.docker_image:
      train_job.docker_image = args.docker_image
    if args.docker_command:
      train_job.docker_command = args.docker_command
    if args.volume_type:
      train_job.volume_type = args.volume_type
    if args.volume_path:
      train_job.volume_path = args.volume_path
    if args.mount_path:
      train_job.mount_path = args.mount_path
    if args.mount_read_only:
      if args.mount_read_only == "true" or args.mount_read_only == "True":
        train_job.mount_read_only = True
      else:
        train_job.mount_read_only = False
    if args.prepare_command:
      train_job.prepare_command = args.prepare_command
    if args.finish_command:
      train_job.finish_command = args.finish_command
    if args.node_selector_key:
      train_job.node_selector_key = args.node_selector_key
    if args.node_selector_value:
      train_job.node_selector_value = args.node_selector_value
    if args.gpu_type:
      if args.gpu_limit:
        train_job.gpu_type = args.gpu_type
      else:
        raise ValueError("ERROR: If you want select gpu type, gpu limit (-g) cannot be None.")
    if args.owner_email:
      train_job.owner_email = args.owner_email
    else:
      train_job.owner_email = client.org_mail
  except ValueError as e:
    util.exit_as_error("Error init TrainJob({}): {}".format(args.job_name, e))
    return

  # try to get framework_version if framework given
  if args.framework:
    train_job.framework_version = util.get_framework_version_or_exit(client,
                                                                framework=args.framework,
                                                                version=args.framework_version,
                                                                python=args.framework_python,
                                                                hdfs=args.framework_hdfs)

  json_data = train_job.get_json_data()
  response = client.create_train_v2(json_data)
  if not isinstance(response, str):
    logging.debug("create train v2 response.content: {}".format(response))
    print_train_job_info(response)
  else:
    print("response: {}".format(response))


def describe_job(args):
  """Describe the job."""

  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.describe_train_v2(args.job_name, args.org_id)
  else:
    response = client.describe_train_v2(args.job_name)
  if not isinstance(response, str):
    print_train_job_info(response)
  else:
    print("response: {}".format(response))


def delete_job(args):
  """Delete the job(s)."""

  try:
    client = CloudMlClient()
  except Exception as e:
    util.exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    for job_name in args.job_names:
      response = client.delete_train_v2(job_name, args.org_id)
      print(response)
  else:
    for job_name in args.job_names:
      response = client.delete_train_v2(job_name)
      print(response)
