# Copyright 2017 Xiaomi, Inc.
# -*- coding: UTF-8 -*-
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from datetime import datetime
import argparse
import fds
import getpass
import json
import logging
import copy
import os
import sys
import time
import optparse
import shlex
import tempfile
import requests
import yaml
import tarfile
import hashlib
import subprocess


sys.path.append("../../")

from . import color_util
from . import constant
from cloud_ml_sdk.utils import log_util
from cloud_ml_sdk.client import CloudMlClient
from cloud_ml_sdk.models.train_job import TrainJob
from cloud_ml_sdk.models.model_service import ModelService
from cloud_ml_sdk.models.dev_env import DevEnv
from cloud_ml_sdk.models.tensorboard_service import TensorboardService
from cloud_ml_sdk.models.quota import Quota
from cloud_ml_sdk.models.dev_server import DevServer
from cloud_ml_sdk.models.pipeline import Pipeline
from cloud_ml_sdk.models.schedule import Schedule
from cloud_ml_sdk.models.ceph_service import CephService
from cloud_ml_sdk.models.autoscaler_rule import AutoScalerRule
from cloud_ml_sdk.models import models_util


class SetConfigContextEnv(argparse.Action):
  def __call__(self, parser, namespace, values, option_string=None):
    # Set cloudml config context env
    os.environ['XIAOMI_CLOUDML_CONFIG_CONTEXT'] = values
    setattr(namespace, self.dest, values)


def compatibility_input(obj):
  if sys.version_info > (3, 0):
    res = input(obj)
  else:
    res = raw_input(obj)
  return res


def str2bool(v):
  if v.lower() in ('yes', 'true', 't', 'y', '1'):
    return True
  elif v.lower() in ('no', 'false', 'f', 'n', '0'):
    return False
  else:
    raise argparse.ArgumentTypeError('Boolean value( yes, no, true, false ....) expected.')


def authentication(access_key, secret_key, endpoint, org_mail):
  client = CloudMlClient(access_key=access_key, secret_key=secret_key, endpoint=endpoint, xiaomi_org_mail=org_mail)
  response = client.authentication()
  if "error" in response:
    return False
  else:
    return True


def get_org_id(args):
  """Get org_id by access_key and secret_key in config file."""

  try:
    client = CloudMlClient()
  except Exception as e:
    print("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  response = client.get_org_id()
  if isinstance(response, str):
    print("response: {}".format(response))
  else:
    print("Your org_id is: {}".format(response.get("org_id")))


def print_hp_job_info(hp_job):
  """Print hp job in customed format.

    Args:
      hp_job: The dictionary of response hptuning job data.
    """
  if "message" in hp_job:
    print((hp_job["message"]))
  print("{:16} {}".format("Org id:", hp_job["org_id"]))
  print("{:16} {}".format("Org name:", hp_job["org_name"]))
  if "priority_class" in hp_job:
    print("{:16} {}".format("Priority Class:", hp_job["priority_class"]))
  print("{:16} {}".format("hp name:", hp_job["hp_name"]))
  print("{:16} {}".format("Module name:", hp_job["module_name"]))
  print("{:16} {}".format("Trainer uri:", hp_job["trainer_uri"]))
  print("{:16} {}".format("Job args:", hp_job["job_args"]))
  print("{:16} {}".format("FDS Endpoint:", hp_job["fds_endpoint"]))
  print("{:16} {}".format("Fuse mount bucket:", hp_job["fds_bucket"]))
  print("{:16} {}".format("Kerberos account:", hp_job["hdfs_krb_account"]))
  print("{:16} {}".format("HDFS Endpoint:", hp_job["hdfs_endpoint"]))
  print("{:16} {}".format("CPU limit:", hp_job["cpu_limit"]))
  print("{:16} {}".format("Memory limit:", hp_job["memory_limit"]))
  print("{:16} {}".format("GPU limit:", hp_job["gpu_limit"]))
  print("{:16} {}".format("Current rank", hp_job.get("current_rank","-")))
  print("{:16} {}".format("Docker image:", hp_job["docker_image"]))
  print("{:16} {}".format("Docker command:", hp_job["docker_command"]))
  print("{:16} {}".format("Framework:", hp_job["framework"]))
  print("{:16} {}".format("Framework version:", hp_job[
      "framework_version"]))
  print("{:16} {}".format("State:", color_util.colorize_state(hp_job["state"])))
  print("{:16} {}".format("Create time:", hp_job["create_time"]))
  print("{:16} {}".format("Update time:", hp_job["update_time"]))

  print("{:16} {}".format("Volume type:", hp_job["volume_type"]))
  print("{:16} {}".format("Volume path:", hp_job["volume_path"]))
  print("{:16} {}".format("Mount path:", hp_job["mount_path"]))
  print("{:16} {}".format("Mount read only:", hp_job["mount_read_only"]))
  print("{:16} {}".format("Prepare command:", hp_job["prepare_command"]))
  print("{:16} {}".format("Finish command:", hp_job["finish_command"]))
  if "node_selector" in hp_job and hp_job["node_selector"]:
    print("{:16} {}".format("Node selector:", hp_job["node_selector"]))
  print("{:16} {}".format("Monitoring url:", hp_job["monitoring"] if "monitoring" in hp_job else ""))
  print("{:16} {}".format("Owner:", hp_job["owner"]))
  print("{:16} {}".format("Optimization type:", hp_job["optimization_type"]))
  print("{:16} {}".format("Objectivevalue name:", hp_job["objectivevalue_name"]))
  print("{:16} {}".format("Metrics names:", hp_job["metrics_names"]))
  print("{:16} {}".format("Parameter configs:", hp_job["parametertype_configs"]))
  print("{:16} {}".format("Suggestion algorithm:", hp_job["suggestion_algorithm"]))
  print("{:16} {}".format("Parallel number:", hp_job["parallel_number"]))
  print("{:16} {}".format("Max number:", hp_job["max_number"]))
  print("{:16} {}".format("Max fail number:", hp_job["max_fail_number"]))


def print_train_job_info(train_job):
  """Print train job in customed format.

  Args:
    train_job: The dictionary of response train job data.
  """
  if "message" in train_job:
    print((train_job["message"]))
  print("{:16} {}".format("Org id:", train_job["org_id"]))
  print("{:16} {}".format("Org name:", train_job["org_name"]))
  if "owner_email" in train_job:
    print("{:16} {}".format("Owner email:",train_job["owner_email"]))
  if "priority_class" in train_job:
    print("{:16} {}".format("Priority Class:", train_job["priority_class"]))
  print("{:16} {}".format("Job name:", train_job["job_name"]))
  print("{:16} {}".format("Module name:", train_job["module_name"]))
  print("{:16} {}".format("Trainer uri:", train_job["trainer_uri"]))
  print("{:16} {}".format("Job args:", train_job["job_args"]))
  print("{:16} {}".format("FDS Endpoint:", train_job["fds_endpoint"]))
  print("{:16} {}".format("Fuse mount bucket:", train_job["fds_bucket"]))
  print("{:16} {}".format("Option Fuse mounts:", train_job["fds_mounts"] if "fds_mounts" in train_job else ""))
  print("{:16} {}".format("Kerberos account:", train_job["hdfs_krb_account"]))
  print("{:16} {}".format("HDFS Endpoint:", train_job["hdfs_endpoint"]))
  print("{:16} {}".format("Ceph volume:", train_job["ceph_volume"] if "ceph_volume" in train_job else ""))
  print("{:16} {}".format("Ceph mode:", train_job["ceph_mode"] if "ceph_mode" in train_job else ""))
  print("{:16} {}".format("Option Ceph mounts:", train_job["ceph_mounts"] if "ceph_mounts" in train_job else ""))
  print("{:16} {}".format("CPU limit:", train_job["cpu_limit"]))
  print("{:16} {}".format("Memory limit:", train_job["memory_limit"]))
  print("{:16} {}".format("GPU limit:", train_job["gpu_limit"]))
  print("{:16} {}".format("Current rank", train_job.get("current_rank","-")))
  print("{:16} {}".format("Docker image:", train_job["docker_image"]))
  print("{:16} {}".format("Docker command:", train_job["docker_command"]))
  print("{:16} {}".format("Framework:", train_job["framework"]))
  print("{:16} {}".format("Framework version:", train_job[
      "framework_version"]))
  print("{:16} {}".format("State:", color_util.colorize_state(train_job["state"])))
  print("{:16} {}".format("Create time:", train_job["create_time"]))
  print("{:16} {}".format("Update time:", train_job["update_time"]))
  if train_job["cluster_name"]:
    print("{:16} {}".format("Cluster name:", train_job["cluster_name"]))
  if train_job["service_port"]:
    print("{:16} {}".format("Service port:", train_job["service_port"]))
  if train_job["cluster_env_var"]:
    print("{:16} {}".format("Cluter environment variables:", train_job[
        "cluster_env_var"]))
  if train_job["hyperparameters_name"]:
    print("{:16} {}".format("Hyperparameters name:", train_job[
        "hyperparameters_name"]))
  if train_job["hyperparameters_goal"]:
    print("{:16} {}".format("Hyperparameters goal:", train_job[
        "hyperparameters_goal"]))
  if train_job["hyperparameters_params"]:
    print("{:16} {}".format("Hyperparameters params:", train_job[
        "hyperparameters_params"]))
  print("{:16} {}".format("Volume type:", train_job["volume_type"]))
  print("{:16} {}".format("Volume path:", train_job["volume_path"]))
  print("{:16} {}".format("Mount path:", train_job["mount_path"]))
  print("{:16} {}".format("Mount read only:", train_job["mount_read_only"]))
  print("{:16} {}".format("Prepare command:", train_job["prepare_command"]))
  print("{:16} {}".format("Finish command:", train_job["finish_command"]))
  if "node_selector" in train_job and train_job["node_selector"]:
    print("{:16} {}".format("Node selector:", train_job["node_selector"]))
  if "node_name" in train_job and train_job["node_name"]:
    print("{:16} {}".format("Node name:", train_job["node_name"]))
  print("{:16} {}".format("Monitoring url:", train_job["monitoring"] if "monitoring" in train_job else ""))


def print_model_service_info(model):
  """Print model service in customed format.

  Args:
    model: The dictionary of response model service data.
  """
  for key, value in model.items():
    str_key = " ".join(key.split("_")).capitalize() + ":"
    print("{:16} {}".format(str_key, value))

def print_dev_env_info(dev_env):
  """Print dev_env in customed format.

  Args:
    dev_env: The dictionary of response dev env data.
  """
  print("{:16} {}".format("Org id:", dev_env["org_id"]))
  print("{:16} {}".format("Org name:", dev_env["org_name"]))
  print("{:16} {}".format("Dev name:", dev_env["dev_name"]))
  print("{:16} {}".format("Password:", dev_env["password"]))
  if "owner_email" in dev_env:
    print("{:16} {}".format("Owner email:", dev_env["owner_email"]))
  if "priority_class" in dev_env:
    print("{:16} {}".format("Priority Class:", dev_env["priority_class"]))
  print("{:16} {}".format("Notebook port:", dev_env["notebook_port"]))
  print("{:16} {}".format("Ssh port:", dev_env["ssh_port"]))
  print("{:16} {}".format("FDS Endpoint:", dev_env["fds_endpoint"]))
  print("{:16} {}".format("Fuse mount bucket:", dev_env["fds_bucket"]))
  print("{:16} {}".format("Kerberos account:", dev_env["hdfs_krb_account"]))
  print("{:16} {}".format("HDFS Endpoint:", dev_env["hdfs_endpoint"]))
  print("{:16} {}".format("Ceph volume:", dev_env["ceph_volume"] if "ceph_volume" in dev_env else ""))
  print("{:16} {}".format("Ceph mode:", dev_env["ceph_mode"] if "ceph_mode" in dev_env else ""))
  print("{:16} {}".format("Option Ceph mounts:", dev_env["ceph_mounts"] if "ceph_mounts" in dev_env else ""))
  print("{:16} {}".format("CPU limit:", dev_env["cpu_limit"]))
  print("{:16} {}".format("Memory limit:", dev_env["memory_limit"]))
  print("{:16} {}".format("GPU limit:", dev_env["gpu_limit"]))
  print("{:16} {}".format("Current rank:",dev_env.get("current_rank","-")))
  print("{:16} {}".format("Docker image:", dev_env["docker_image"]))
  print("{:16} {}".format("Docker command:", dev_env["docker_command"]))
  print("{:16} {}".format("Framework:", dev_env["framework"]))
  print("{:16} {}".format("Framework version:", dev_env["framework_version"]))
  print("{:16} {}".format("State:", color_util.colorize_state(dev_env["state"])))
  print("{:16} {}".format("Create time:", dev_env["create_time"]))
  print("{:16} {}".format("Update time:", dev_env["update_time"]))
  print("{:16} {}".format("Jupyter Address:", dev_env["address"]))
  print("{:16} {}".format("Ssh address:", dev_env["ssh_address"]))
  print("{:16} {}".format("Proxy whitelist:", dev_env["proxy_whitelist"]))
  if "node_selector" in dev_env and dev_env["node_selector"]:
    print("{:16} {}".format("Node selector:", dev_env["node_selector"]))
  print("{:16} {}".format("Monitoring url:", dev_env["monitoring"] if "monitoring" in dev_env else ""))
  print("{:16}".format("Networks:"))
  if "network" in dev_env:
      networks = dev_env["network"]
      for network in networks:
          print("{:16} {} {}/{}:{}".format("",
              network["address"],
              network["protocol"],
              network["port"],
              network["whitelist"]))


def print_dev_server_info(dev_server):
  """Print dev_server in customed format.

  Args:
    dev_server: The dictionary of response dev server data.
  """
  print("{:16} {}".format("Org id:", dev_server["org_id"]))
  print("{:16} {}".format("Org name:", dev_server["org_name"]))
  print("{:16} {}".format("Dev name:", dev_server["dev_name"]))
  print("{:16} {}".format("Password:", dev_server["password"]))
  print("{:16} {}".format("Notebook port:", dev_server["notebook_port"]))
  print("{:16} {}".format("Ssh port:", dev_server["ssh_port"]))
  print("{:16} {}".format("CPU limit:", dev_server["cpu_limit"]))
  print("{:16} {}".format("Memory limit:", dev_server["memory_limit"]))
  print("{:16} {}".format("GPU limit:", dev_server["gpu_limit"]))
  print("{:16} {}".format("Current rank:", dev_server.get("current_rank","-")))
  print("{:16} {}".format("Docker image:", dev_server["docker_image"]))
  print("{:16} {}".format("Docker command:", dev_server["docker_command"]))
  print("{:16} {}".format("Framework:", dev_server["framework"]))
  print("{:16} {}".format("Framework version:", dev_server[
      "framework_version"]))
  print("{:16} {}".format("State:", color_util.colorize_state(dev_server["state"])))
  print("{:16} {}".format("Create time:", dev_server["create_time"]))
  print("{:16} {}".format("Update time:", dev_server["update_time"]))
  print("{:16} {}".format("Address:", dev_server["address"]))
  print("{:16} {}".format("Ssh address:", dev_server["ssh_address"]))


def print_tensorboard_info(tensorboard):
  """Print tensorboard in customed format.

  Args:
    tensorboard: The dictionary of response tensorboard data.
  """
  print("{:16} {}".format("Org id:", tensorboard["org_id"]))
  print("{:16} {}".format("Org name:", tensorboard["org_name"]))
  if "priority_class" in tensorboard:
    print("{:16} {}".format("Priority Class:", tensorboard["priority_class"]))
  print("{:16} {}".format("Tensorboard name:", tensorboard["tensorboard_name"]))
  print("{:16} {}".format("Logdir:", tensorboard["logdir"]))
  print("{:16} {}".format("Current rank:", tensorboard.get("current_rank", "-")))
  print("{:16} {}".format("Service port:", tensorboard["service_port"]))
  print("{:16} {}".format("Docker image:", tensorboard["docker_image"]))
  print("{:16} {}".format("Docker command:", tensorboard["docker_command"]))
  print("{:16} {}".format("Framework:", tensorboard["framework"]))
  print("{:16} {}".format("Framework version:", tensorboard["framework_version"]))
  print("{:16} {}".format("State:", color_util.colorize_state(tensorboard["state"])))
  print("{:16} {}".format("Create time:", tensorboard["create_time"]))
  print("{:16} {}".format("Update time:", tensorboard["update_time"]))
  print("{:16} {}".format("Address:", tensorboard["address"]))
  if "node_selector" in tensorboard and tensorboard["node_selector"]:
    print("{:16} {}".format("Node selector:", tensorboard["node_selector"]))
  print("{:16} {}".format("FDS Endpoint:", tensorboard["fds_endpoint"]))
  print("{:16} {}".format("Fuse mount bucket:", tensorboard["fds_bucket"]))
  print("{:16} {}".format("Monitoring url:", tensorboard["monitoring"] if "monitoring" in tensorboard else ""))


def print_quota_info(quota):
  """Print quota in customed format.

  Args:
    quota: The dictionary of response quota data.
  """
  total_memory_quota = "INF" if quota[
      "total_memory_quota"] == constant.INF_TOTAL_MEMORY_QUOTA else quota[
          "total_memory_quota"]
  total_cpu_quota = "INF" if quota[
      "total_cpu_quota"] == constant.INF_TOTAL_CPU_QUOTA else quota[
          "total_cpu_quota"]
  total_gpu_quota = "INF" if quota[
      "total_gpu_quota"] == constant.INF_TOTAL_GPU_QUOTA else quota[
          "total_gpu_quota"]
  train_count_quota = "INF" if quota[
      "train_count_quota"] == constant.INF_JOB_COUNT else quota[
          "train_count_quota"]
  model_count_quota = "INF" if quota[
      "model_count_quota"] == constant.INF_JOB_COUNT else quota[
          "model_count_quota"]
  dev_count_quota = "INF" if quota[
      "dev_count_quota"] == constant.INF_JOB_COUNT else quota[
          "dev_count_quota"]

  print("{:32} {}".format("Org id:", quota["org_id"]))
  if "org_mail" in quota:
    print("{:32} {}".format("Org mail address:", quota["org_mail"]))
  if "second_org_leader_mail" in quota:
    print("{:32} {}".format("Second level org leader:", quota["second_org_leader_mail"]))
  print("{:16} {:26} {:26} {:26} {:26} {:26} {:26} {:26}".format(
      "",
      "Memory / Used",
      "CPU / Used",
      "GPU / Used",
      "Tensorboard / Used",
      "Current rank",
      "Priority / Used",
      "Count / Used"
      ))
  print("{:16} {:26} {:26} {:26} {:26} {:26} {:26} {:26}".format(
      "Train job",
      quota["train_memory_quota"] + " / " + quota["train_memory_used"],
      quota["train_cpu_quota"] + " / " + quota["train_cpu_used"],
      quota["train_gpu_quota"] + " / " + quota["train_gpu_used"],
      "- / -",
      quota.get("train_priority_rank","-"),
      quota.get("train_priority_quota", "-") + " / " + quota.get("train_priority_used", "-"),
      train_count_quota + " / " + quota["train_count_used"]
      ))
  print("{:16} {:26} {:26} {:26} {:26} {:26} {:26} {:26}".format(
      "Model service",
      quota["model_memory_quota"] + " / " + quota["model_memory_used"],
      quota["model_cpu_quota"] + " / " + quota["model_cpu_used"],
      quota["model_gpu_quota"] + " / " + quota["model_gpu_used"],
      "- / -",
      quota.get("model_priority_rank","-"),
      quota.get("model_priority_quota","-") + " / " + quota.get("model_priority_used","-"),
      model_count_quota + " / " + quota["model_count_used"]
      ))
  print("{:16} {:26} {:26} {:26} {:26} {:26} {:26} {:26}".format(
      "Dev environment",
      quota["dev_memory_quota"] + " / " + quota["dev_memory_used"],
      quota["dev_cpu_quota"] + " / " + quota["dev_cpu_used"],
      quota["dev_gpu_quota"] + " / " + quota["dev_gpu_used"],
      "- / -",
      quota.get("dev_priority_rank", "-"),
      quota.get("dev_priority_quota", "-") + " / " + quota.get("dev_priority_used", "-"),
      dev_count_quota + " / " + quota["dev_count_used"]
      ))
  print("{:16} {:26} {:26} {:26} {:26} {:26} {:26} {:26}".format(
      "Tensorboard",
      "- / -",
      "- / -",
      "- / -",
      quota["tensorboard_quota"] + " / " + quota["tensorboard_used"],
      quota.get("tensorboard_priority_rank", "-"),
      quota.get("tensorboard_priority_quota", "-") + " / " + quota.get("tensorboard_priority_used", "-"),
      "- / -"
      ))

  print("{:16} {:26} {:26} {:26} {:26} {:26} {:26} {:26}".format(
      "Total quota",
      total_memory_quota + " / " + quota["total_memory_used"],
      total_cpu_quota + " / " + quota["total_cpu_used"],
      total_gpu_quota + " / " + quota["total_gpu_used"],
      "- / -",
      "-",
      "- / -",
      "- / -"
      ))


def print_frameworks_info(framework):
  print("{:16} {}".format("FRAMEWORK", "FRAMEWORK VERSION"))
  for framework_name in framework:
    for index in range(len(framework[framework_name])):
      if index == 0:
        print("{:16} {}".format(framework_name, framework[framework_name][
            index]))
      else:
        print("{:16} {}".format("", framework[framework_name][index]))


def print_pipeline_info(response):
  """Print pipeline customed format.

  Args:
    pipeline: The dictionary of response train job data.
  """

  pipeline = response['data']
  if "message" in pipeline:
    print((pipeline["message"]))
  print("{:16} {}".format("Org id:", pipeline["org_id"]))
  print("{:16} {}".format("Org name:", pipeline["org_name"]))
  if "priority_class" in pipeline:
    print("{:16} {}".format("Priority Class:", pipeline["priority_class"]))
  print("{:16} {}".format("Pipeline name:", pipeline["pipeline_name"]))
  print("{:16} {}".format("Pipeline config:", pipeline["pipeline_config"]))

  #print("{:16}{}".format("Pipeline argo yaml:\n", pipeline["argo_yaml"]))
  print("{:16} {}".format("FDS Endpoint:", pipeline["fds_endpoint"]))
  print("{:16} {}".format("Fuse mount bucket:", pipeline["fds_bucket"]))
  print("{:16} {}".format("Create time:", pipeline["create_time"]))
  print("{:16} {}".format("Update time:", pipeline["update_time"]))
  if "argo_get_success" in pipeline:
    print("#"*48)
    if pipeline["argo_get_success"]:
      print("ARGO INFO AS FOLLOWS")
      print("#"*48)
      print("{:16} {}".format("Argo UI:", pipeline["argo_ui"]))
    else:
      print("ARGO GET FAILED")
      print("#"*48)
    if sys.version_info > (3, 0):
      print(pipeline["argo_get_info"])
    else:
      print(pipeline["argo_get_info"].encode("utf-8"))

  if 'error' in response and len(response["error"]) > 0:

    print("\nERROR MESSAGE")
    for pipeline_type in response['error']:
      print("{} error: {}".format(pipeline_type, response['error'][pipeline_type]))


def print_pipeline_schedule_info(response):
  pipeline_schedule = response['data']
  if "message" in pipeline_schedule:
    print((pipeline_schedule["message"]))
  print("{:16} {}".format("Org id:", pipeline_schedule["org_id"]))
  print("{:16} {}".format("Pipeline name:", pipeline_schedule["pipeline_name"]))
  print("{:16} {}".format("State:", pipeline_schedule["state"]))
  print("{:16} {}".format("Create time:", pipeline_schedule["create_time"]))
  print("{:16} {}".format("Update time:", pipeline_schedule["update_time"]))
  if "argo_get_success" in pipeline_schedule:
    print("#"*48)
    if pipeline_schedule["argo_get_success"]:
      print("ARGO INFO AS FOLLOWS")
      print("#"*48)
      print("{:16} {}".format("Argo UI:", pipeline_schedule["argo_ui"]))
    else:
      print("ARGO GET FAILED")
      print("#"*48)
    if sys.version_info > (3, 0):
      print(pipeline_schedule["argo_get_info"])
    else:
      print(pipeline_schedule["argo_get_info"].encode("utf-8"))

  if 'error' in response and len(response["error"]) > 0:

    print("\nERROR MESSAGE")
    for pipeline_type in response['error']:
      print("{} error: {}".format(pipeline_type, response['error'][pipeline_type]))


def print_predict_result(result):
  """Print the predict result in customed format.

  Args:
    result: The predict result. Example: {u"predict": [30.00011444091797, 70.00050354003906], u"keys": [10, 20]}
  """
  print("Predictions:")
  for i in range(len(list(result.values())[0])):
    print_divider = True
    for k, v in list(result.items()):
      if print_divider:
        print("{} {}: {}".format("-", k, v[i]))
        print_divider = False
      else:
        print("{} {}: {}".format(" ", k, v[i]))


def print_hyperparameter_data_result(result):
  """Print the hyperparameter result in customed format.

  Args:
    result: The result. Example: {"best_trials": {}, "goal": "MINIMIZE", "trialCount": 4, "trial": [{}]}
                                 Each trial contains {"metric": 0.08, "params": "--foo=bar", "step": 100}
  """
  print("Goal: {}".format(result["goal"]))
  print("Trial count: {}".format(result["trialCount"]))
  print("Best trial:")
  print("{:8} Metrics: {}".format("", result["best_trials"]["metric"]))
  print("{:8} Params: {}".format("", result["best_trials"]["params"]))
  print("{:8} Step: {}".format("", result["best_trials"]["step"]))
  print("{:8} State: {}".format("", result["best_trials"]["state"]))

  trials = result["trials"]
  for i in range(len(trials)):
    print("Trial {}:".format(i))
    print("{:8} Metrics: {}".format("", trials[i]["metric"]))
    print("{:8} Params: {}".format("", trials[i]["params"]))
    print("{:8} Step: {}".format("", trials[i]["step"]))
    print("{:8} State: {}".format("", trials[i]["state"]))


def print_kubernetes_events(events):
  """Print the events logs from Kubernetes API.

  Args:
    events: The list of event. Example: {"count": 1, "firstTimestamp": "2017-02-20T06:16:39Z"}
  """
  if isinstance(events,dict):
    print("engine events: ")
    print("{:20} {:20} {:8} {:32} {:8} {:8} {}".format(
        "FirstTimestamp", "LastTimestamp", "Count", "Name", "Type", "Reason",
        "Message"))
    if "engine" in events and len(events["engine"]) > 0:
      for event in events["engine"][0]:
        print("{:20} {:20} {:8} {:32} {:8} {:8} {}".format(event[
            "firstTimestamp"], event["lastTimestamp"], event["count"], event[
                "involvedObject"]["name"], event["type"], event["reason"], event[
                    "message"]))
    print("model events: ")
    print("{:20} {:20} {:8} {:32} {:8} {:8} {}".format(
        "FirstTimestamp", "LastTimestamp", "Count", "Name", "Type", "Reason",
        "Message"))
    if "model" in events and len(events["model"]) > 0:
      for event in events["model"][0]:
        print("{:20} {:20} {:8} {:32} {:8} {:8} {}".format(event[
            "firstTimestamp"], event["lastTimestamp"], event["count"], event[
                "involvedObject"]["name"], event["type"], event["reason"], event[
                    "message"]))

  else:
    print("{:20} {:20} {:8} {:32} {:8} {:8} {}".format(
        "FirstTimestamp", "LastTimestamp", "Count", "Name", "Type", "Reason",
        "Message"))
    for event in events:
      print("{:20} {:20} {:8} {:32} {:8} {:8} {}".format(event[
          "firstTimestamp"], event["lastTimestamp"], event["count"], event[
              "involvedObject"]["name"], event["type"], event["reason"], event[
                  "message"]))


def print_recommend_resource(resource):
  print("recommend resources configure:")
  print("{:32} {:32} {:32}".format("Available cpu", "Available memory", "Available gpu"))
  for item in resource:
    mem = round(float(item[1]) / 1000**3,3)
    print("{:32} {:32} {:32}".format(str(item[0]), str(mem)+"G", str(item[2])))


def print_metrics_result(metrics):
  """Print the metrics result.

  Args:
    metrics: The metrics.
  """
  print("{:20} {} %".format("CPU:", metrics["cpu"]))
  print("{:20} {} Bytes".format("Memory:", metrics["memory"]))
  print("{:20} {} Bytes".format("Network receive:", metrics["network_receive"]))
  print("{:20} {} Bytes".format("Network transmit:", metrics["network_transmit"]))


def list_jobs(args):
  """List train jobs."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    if args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
      response_train_jobs = client.list_train_jobs(
          org_id=constant.CLOUDML_ALL_ORG_PARAMETER, priority_class=args.priority_class, job_name=args.job_name, job_status=args.job_status, page_number=args.page_number, page_size=args.page_size)
    else:
      response_train_jobs = client.list_train_jobs(org_id=args.org_id, priority_class=args.priority_class, job_name=args.job_name, job_status=args.job_status, page_number=args.page_number, page_size=args.page_size)
  else:
    response_train_jobs = client.list_train_jobs(priority_class=args.priority_class, job_name=args.job_name, job_status=args.job_status, page_number=args.page_number, page_size=args.page_size)
  if not isinstance(response_train_jobs, str):
    print_trainjob_list(response_train_jobs, args)
  else:
    print("response: {}".format(response_train_jobs))


def print_trainjob_list(response_train_jobs, args):
  if not args.priority_class:  # NOTE(xychu): will show all priority classes if no priority is given
    prioritized = {}
    unprioritized = []
    for priority in constant.CLOUDML_PRIORITY_CLASSES:
      prioritized[priority] = []
    for item in response_train_jobs:
      if "priority_class" in item and item["priority_class"]:
        prioritized[item["priority_class"]].append(item)
      else:
        unprioritized.append(item)
    for priority in prioritized:
      if len(prioritized[priority]) > 0:
        print("Priority class {}:".format(priority))
        do_print_trainjob_list(prioritized[priority], args)
    if len(unprioritized) > 0:
      print("Un-prioritized:")
      do_print_trainjob_list(unprioritized, args)
  else:
    do_print_trainjob_list(response_train_jobs, args)


def do_print_trainjob_list(train_jobs, args):
  if "org_id" in args and args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
    if "output" in args and args.output == "wide":
      print("{:16} {:16} {:32} {:16} {:32} {:32} {:32}".format(
        "ORG ID", "ORG NAME", "JOB_NAME", "STATE", "CREATED", "UPDATED", "NODE NAME"))
      for train_job in train_jobs:
        print("{:<16} {:16} {:32} {:16} {:32} {:32} {:32}".format(train_job[
                                                          "org_id"], train_job["org_name"], train_job["job_name"],
                                                        color_util.colorize_state(train_job["state"]),
                                                        train_job["create_time"], train_job["update_time"],
                                                        train_job["node_name"]))
    else:
      print("{:16} {:16} {:32} {:16} {:32} {:32}".format(
        "ORG ID", "ORG NAME", "JOB_NAME", "STATE", "CREATED", "UPDATED"))
      for train_job in train_jobs:
        print("{:<16} {:16} {:32} {:16} {:32} {:32}".format(train_job[
                                                          "org_id"], train_job["org_name"], train_job["job_name"],
                                                        color_util.colorize_state(train_job["state"]),
                                                        train_job["create_time"], train_job["update_time"]))
  else:
    if "output" in args and args.output == "wide":
      print("{:32} {:16} {:32} {:32} {:32}".format("JOB_NAME", "STATE", "CREATED", "UPDATED", "NODE NAME"))
      for train_job in train_jobs:
        print("{:32} {:16} {:32} {:32} {:32}".format(train_job[
                                                         "job_name"], color_util.colorize_state(train_job["state"]),
                                                     train_job["create_time"], train_job["update_time"],
                                                     train_job["node_name"]))
    else:
      print("{:32} {:16} {:32} {:32}".format("JOB_NAME", "STATE", "CREATED",
                                             "UPDATED"))
      for train_job in train_jobs:
        print("{:32} {:16} {:32} {:32}".format(train_job[
                                                 "job_name"], color_util.colorize_state(train_job["state"]),
                                               train_job["create_time"], train_job["update_time"]))

def setNetwork():
  task_port_types = compatibility_input("Please input port, separated with a space. [Default is: {}]: ".format(
    " ".join(
      ["{}(for {})".format(x[0],x[1]) for x in list(constant.DEV_PROXY_PROTOCOL.items())]
      )
    )
  ) or list(constant.DEV_PROXY_PROTOCOL.keys())
  if not isinstance(task_port_types,list):
    port_list = task_port_types.strip().split(" ")
    port_list = [models_util.check_network_port(port) for port in port_list]
  else:
    port_list = task_port_types
  networkList = []
  jupyter_num = 0
  for i in port_list:
    netDict = {}
    netDict["port"] = i
    protocol = compatibility_input("\nPlease input the protocol of port:'{}'{}: ".
                                   format(i,". [Default is ]"+constant.DEV_PROXY_PROTOCOL[i] if constant.DEV_PROXY_PROTOCOL.get(i) else "")) \
               or constant.DEV_PROXY_PROTOCOL.get(i)
    if protocol not in list(constant.DEV_PROXY_PROTOCOL.values()):
      raise ValueError("Network protocol must be in {}".format(str(list(constant.DEV_PROXY_PROTOCOL.values()))))
    if protocol == "JUPYTER":
      jupyter_num += 1
    if jupyter_num > 1:
      raise ValueError("Multiple JUPYTER protocol is not allowed")
    netDict["protocol"] = protocol
    whitelist = compatibility_input(
      "Please input the white list of IP, separatedwith a space. [Default is: 0.0.0.0/0]: ") or "0.0.0.0/0"
    whitelist = models_util.check_network_whitelist(whitelist)
    netDict["whitelist"] = whitelist
    networkList.append(netDict)
  return networkList


def get_cluster_spec():
  task_types_str = compatibility_input("Please input distributed task type, separated with a space. [Default is: ps worker]: ") or 'ps worker'
  task_types = task_types_str.split(' ')

  cluster_spec = {}
  for task_type in task_types:
    count = compatibility_input("\nPlease input the count of '{}'. [Default is 1]: ".format(task_type)) or "1"
    count = models_util.check_distributed_count(count)
    cpu_limit = compatibility_input("Please input the cpu limit of '{}'. [Default is 1]: ".format(task_type)) or '1'
    cpu_limit = models_util.check_cpu_value(cpu_limit)
    memory_limit = compatibility_input("Please input the memory limit of '{}'. [Default is 1G]: ".format(task_type)) or '1G'
    memory_limit = models_util.check_memory_value(memory_limit)
    gpu_limit = compatibility_input("Please input the gpu limit of '{}'. [Defalut is 0]: ".format(task_type)) or "0"
    gpu_limit = models_util.check_gpu_value(gpu_limit)
    if gpu_limit > 0:
      gpu_type = compatibility_input("Please input the gpu type of '{}'. [Default is None]: ".format(task_type)) or None
      if isinstance(gpu_type, str) and gpu_type.lower() in constant.GPULIST:
        gpu_type = gpu_type.lower()
      elif gpu_type is not None:
        raise ValueError("Only three types of GPUs ({})".format("/".join(constant.GPULIST)))
    else:
      gpu_type = None
    task_args = compatibility_input("Please input the specific args of '{}'. [Default is '']: ".format(task_type)) or ""
    cluster_spec[task_type] = {
      'count': count,
      'cpu_limit': cpu_limit,
      'memory_limit': memory_limit,
      'gpu_limit': gpu_limit,
      'gpu_type': gpu_type,
      'task_args': task_args,
    }
  print("--------------------------\n")

  return cluster_spec


#For hptuning
def submit_hptuning(args):
  """Submit the hptuning"""
  # set environment variable to check args.trainer_uri

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  if args.filename:
    with open(args.filename) as f:
      try:
        hp_obj = json.load(f)
      except Exception as e:
        exit_as_error("ERROR: hptuning description json file({}) is not valid: {}".format(args.filename, e))
        return
      if not (("hp_name" in hp_obj) and ("module_name" in hp_obj) and
              ("trainer_uri" in hp_obj)):
        exit_as_error("ERROR: parameters hp_name, module_name and trainer_uri are necessary.")
        return
      # get the real and valid trainer_uri
      hp_obj["trainer_uri"] = models_util.validate_and_get_trainer_uri(str(hp_obj["trainer_uri"]))
      json_data = json.dumps(hp_obj)
  else:
    #TODO
    json_data=None
  response = client.submit_hptuning(json_data)
  if not isinstance(response, str):
    print_hp_job_info(response)
  else:
    print("response: {}".format(response))


def describe_hp_job(args):
  """Describe the hp job."""
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.describe_hp_job(args.hp_name, args.org_id)
  else:
    response = client.describe_hp_job(args.hp_name)
  if not isinstance(response, str):
    print_hp_job_info(response)
  else:
    print("response: {}".format(response))


def delete_hp(args):
  """Delete the job(s)."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    for hp_name in args.hp_names:
      response = client.delete_train_job(hp_name, args.org_id)
      print(response)
  else:
    for hp_name in args.hp_names:
      response = client.delete_hp_job(hp_name)
      print(response)


def list_hp(args):
    """List train jobs."""
    try:
      client = CloudMlClient()
    except Exception as e:
      exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
      return
    if "org_id" in args:
      if args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
        response_hp_jobs = client.list_hp_jobs(
          constant.CLOUDML_ALL_ORG_PARAMETER, priority_class=args.priority_class, page_number=args.page_number, page_size=args.page_size)
      else:
        response_hp_jobs = client.list_hp_jobs(args.org_id, priority_class=args.priority_class, page_number=args.page_number, page_size=args.page_size)
    else:
      response_hp_jobs = client.list_hp_jobs(priority_class=args.priority_class, page_number=args.page_number, page_size=args.page_size)
    if not isinstance(response_hp_jobs, str):
      print_hp_list(response_hp_jobs, args)
    else:
      print("response: {}".format(response_hp_jobs))


def print_hp_list(response_hp_jobs, args):
  if not args.priority_class:  # NOTE(xychu): will show all priority classes if no priority is given
    prioritized = {}
    unprioritized = []
    for priority in constant.CLOUDML_PRIORITY_CLASSES:
      prioritized[priority] = []
    for item in response_hp_jobs:
      if "priority_class" in item and item["priority_class"]:
        prioritized[item["priority_class"]].append(item)
      else:
        unprioritized.append(item)
    for priority in prioritized:
      if len(prioritized[priority]) > 0:
        print("Priority class {}:".format(priority))
        do_print_hp_list(prioritized[priority], args)
    if len(unprioritized) > 0:
      print("Un-prioritized:")
      do_print_hp_list(unprioritized, args)
  else:
    do_print_hp_list(response_hp_jobs, args)


def do_print_hp_list(response_hp_jobs, args):
    if "org_id" in args and args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
      print("{:16} {:16} {:32} {:16} {:32} {:32}".format(
        "ORG ID", "ORG NAME", "HP_NAME", "STATE", "CREATED", "UPDATED"))
      for hp_job in response_hp_jobs:
        print("{:<16} {:16} {:32} {:16} {:32} {:32}".format(hp_job[
                                                              "org_id"], hp_job["org_name"], hp_job["hp_name"],
                                                            color_util.colorize_state(hp_job["state"]),
                                                            hp_job["create_time"], hp_job["update_time"]))
    else:
      print("{:32} {:16} {:32} {:32}".format("JOB_NAME", "STATE", "CREATED",
                                             "UPDATED"))
      for hp_job in response_hp_jobs:
        print("{:32} {:16} {:32} {:32}".format(hp_job[
                                                 "hp_name"], color_util.colorize_state(hp_job["state"]),
                                               hp_job["create_time"], hp_job["update_time"]))


def get_trials(args):
  """list the trials of certain hp job"""
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if not args.hp_name:
    exit_as_error ("ERROR: parameter hp_name cannot be None.")
    return
  response=client.list_hp_trials(args.hp_name)
  if not isinstance(response,str):
    print("response: {}".format(yaml.safe_dump(response,default_flow_style=False)))
  else:
    print(response)


def get_trial_log(args):
  """log of the trial"""
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  extra_args = dict()
  if "follow" in args:
    extra_args = {"follow": args.follow}
  if "lines" in args and args.lines:
    extra_args["lines"] = args.lines
  if "org_id" in args:
    extra_args["org_id"] = args.org_id

  if not (args.hp_name and args.trial_id):
    exit_as_error("ERROR: parameters hp_name and trial_id cannot be None.")
    return
  response = client.get_hp_trial_logs(args.hp_name,args.trial_id, **extra_args)

  if "follow" in args and args.follow:
    try:
      for line in response.iter_lines():
        print(line)
    except KeyboardInterrupt:
      # exit silently
      return
    except requests.exceptions.ChunkedEncodingError as e:
      if "IncompleteRead(0 bytes read)" in str(e):
        # exit duo to no data read, maybe timeout
        exit_as_error("INFO: 0 bytes read, maybe closed by server due to timeout(5 mins).")
        return
      else:
        exit_as_error("ERROR: read log got ChunkedEncodingError: {}".format(e))
        return
    except Exception as e:
      exit_as_error("ERROR: read log got Error: {}".format(e))
      return
  else:
    # NOTE(xychu): compatible with old version client needed response format.
    # this format will be removed from the server side after no user using
    # client older than 0.2.21
    if not isinstance(response, str):
      response = response.json()
      if "error" in response and response["error"]:
        print(response)
      else:
        if sys.version_info > (3, 0):
          print(response["logs"])
        else:
          print(response["logs"].encode('utf-8'))


def get_hp_job_events(args):
  """Get events of the hp job."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.get_hp_job_events(args.hp_name, args.org_id)
  else:
    response = client.get_hp_job_events(args.hp_name)
  if not isinstance(response, str):
    print_kubernetes_events(response["events"])
  else:
    print("response: {}".format(response))


def get_hp_metrics(args):
  """Get the metrics of the hp job."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.get_hp_job_metrics(args.hp_name, args.org_id)
  else:
    response = client.get_hp_job_metrics(args.hp_name)
  if not isinstance(response, str):
    print_metrics_result(response)
  else:
    print("response: {}".format(response))


def get_optimal_trial(args):
  """Get the optimal trial with it's hyper paramters"""
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.get_optimal_trial(args.hp_name, args.org_id)
  else:
    response = client.get_optimal_trial(args.hp_name)
  print(response)


def submit_job(args):
  """Submit the job."""
  # set environment variable to check args.trainer_uri
  if args.fds_endpoint:
    os.environ["XIAOMI_FDS_ENDPOINT"] = args.fds_endpoint
  if args.fds_bucket:
    os.environ["CLOUDML_DEFAULT_FDS_BUCKET"] = args.fds_bucket

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  args_dict = vars(args)
  if args.filename:
    with open(args.filename) as f:
      # TODO: Check file format and verify the items
      try:
        json_data = yaml.safe_load(f)
      except Exception as e:
        exit_as_error("ERROR: job description json file({}) is not valid: {}".format(args.filename, e))
        return
      for key in json_data:
        if key in args_dict and not args_dict[key]:
          args_dict[key] = json_data[key]
  else:
    d = {}
    except_group = ['command_group', 'func', 'job_command']
    if args.job_name:
      f = open(args.job_name + '.yaml', "w")
      for key in args_dict:
        if args_dict[key] and key not in except_group:
          d[key] = args_dict[key]
      y = yaml.safe_dump(d, default_flow_style=False)
      f.write(y)
      f.close()

  if not (args.job_name and args.module_name and args.trainer_uri):
    exit_as_error(
        "ERROR: parameters job_name, module_name and trainer_uri cannot be None.")
    return

  try:
    if args.argo_mode:
      args.job_name = args.job_name + '-' + get_cur_time()
    train_job = TrainJob(args.job_name, args.module_name, args.trainer_uri)

    if (args.cpu_limit or args.memory_limit or args.gpu_limit or args.gpu_type) and args.distributed:
      print("When use -D(--distributed) para, don't set resource limit by -c(--cpu_limit), -M(--memory_limit) or -g(--gpu_limit) and -gt(--gpu_type)")
      print("The resource limit of distributed jobs should be provied followed.")
      exit_as_error("Exited!")
      return

    if args.cluster_spec:
      if isinstance(args.cluster_spec, str):
        try:
          train_job._cluster_spec = json.loads(args.cluster_spec)
        except Exception as e:

          exit_as_error("The cluster spec format should be json")
          return
      else:
        train_job._cluster_spec = args.cluster_spec
    elif args.distributed:
      train_job._cluster_spec = get_cluster_spec()
    if args.ps_count:
      train_job.ps_count = int(args.ps_count)
    if args.worker_count:
      train_job.worker_count = int(args.worker_count)
    if args.fds_endpoint:
      train_job.fds_endpoint = args.fds_endpoint
    elif client.fds_endpoint:
      train_job.fds_endpoint = client.fds_endpoint
    if args.fds_bucket:
      train_job.fds_bucket = args.fds_bucket
    elif client.fds_bucket:
      train_job.fds_bucket = client.fds_bucket
    if args.fds_mounts:
      train_job.fds_mounts = args.fds_mounts
    # for hdfs
    if args.hdfs_krb_account:
      train_job.hdfs_krb_account = args.hdfs_krb_account
    elif client.hdfs_krb_account:
      train_job.hdfs_krb_account = client.hdfs_krb_account
    if args.hdfs_krb_password:
      train_job.hdfs_krb_password = args.hdfs_krb_password
    elif client.hdfs_krb_password:
      train_job.hdfs_krb_password = client.hdfs_krb_password
    if args.hdfs_endpoint:
      if args.hdfs_endpoint.startswith("hdfs://"):
        train_job.hdfs_endpoint = args.hdfs_endpoint
      else:
        raise ValueError("ERROR: -he[hdfs_endpoint] must start with hdfs://")
    elif client.hdfs_endpoint:
      train_job.hdfs_endpoint = client.hdfs_endpoint
    if args.hdfs_krb_keytab:
      train_job.hdfs_krb_keytab = args.hdfs_krb_keytab
    if train_job.hdfs_krb_account:
      if not train_job.hdfs_krb_password and not train_job.hdfs_krb_keytab:
        raise ValueError("ERROR: either -hkp[hdfs_krb_password] or -hkt[hdfs_krb_keytab] is required")
      if not train_job.hdfs_endpoint:
        raise ValueError("ERROR: -he[hdfs_endpoint] is required")
    elif train_job.hdfs_endpoint:
      raise ValueError("ERROR: -hka[hdfs_krb_account] is required")
    if args.ceph_volume:
      train_job.ceph_volume = args.ceph_volume
      if not args.ceph_mode:
        raise ValueError("ERROR: -cm[ceph_mode] cannot be empty")
      if args.ceph_mode != "r" and args.ceph_mode != "rw":
        raise ValueError("ERROR: invalid value for -cm[ceph_mode]")
      train_job.ceph_mode = args.ceph_mode
    if args.ceph_mounts:
      train_job.ceph_mounts = args.ceph_mounts
    if args.job_args:
      train_job.job_args = args.job_args
    if args.cpu_limit:
      train_job.cpu_limit = args.cpu_limit
    if args.memory_limit:
      train_job.memory_limit = args.memory_limit
    if args.gpu_limit:
      train_job.gpu_limit = args.gpu_limit
    if args.gpu_memory:
      train_job.gpu_memory = args.gpu_memory
    if args.framework:
      train_job.framework = args.framework
    if args.framework_version:
      train_job.framework_version = args.framework_version
    if args.docker_image:
      train_job.docker_image = args.docker_image
    if args.docker_command:
      train_job.docker_command = args.docker_command
    if args.volume_type:
      train_job.volume_type = args.volume_type
    if args.volume_path:
      train_job.volume_path = args.volume_path
    if args.mount_path:
      train_job.mount_path = args.mount_path
    if args.mount_read_only:
      if args.mount_read_only == "true" or args.mount_read_only == "True":
        train_job.mount_read_only = True
      else:
        train_job.mount_read_only = False
    if args.prepare_command:
      train_job.prepare_command = args.prepare_command
    if args.finish_command:
      train_job.finish_command = args.finish_command
    if args.node_selector_key:
      train_job.node_selector_key = args.node_selector_key
    if args.node_selector_value:
      train_job.node_selector_value = args.node_selector_value
    if args.gpu_type:
      if args.gpu_limit:
        train_job.gpu_type = args.gpu_type
      else:
        raise ValueError("ERROR: If you want select gpu type, gpu limit (-g) cannot be None.")
    if args.enable_rank:
      train_job.enable_rank = 1
    else:
      train_job.enable_rank = 0

    if args.save_mode:
      train_job.save_mode = args.save_mode

    if args.argo_mode:
      train_job.argo_mode = args.argo_mode

    if args.tensorboard_logdir:
      train_job.tensorboard_logdir = args.tensorboard_logdir

    if args.model_name:
      train_job.model_name = args.model_name
    if args.model_version:
      train_job.model_version = args.model_version
    if args.owner_email:
      train_job.owner_email = args.owner_email
    else:
      train_job.owner_email = client.org_mail
    if args.priority_class:
      train_job.priority_class = args.priority_class
  except ValueError as e:
    exit_as_error("Error init TrainJob({}): {}".format(args.job_name, e))
    return

  # try to get framework_version if framework given
  if args.framework:
    train_job.framework_version = get_framework_version_or_exit(client,
                                                                framework=args.framework,
                                                                version=args.framework_version,
                                                                python=args.framework_python,
                                                                hdfs=args.framework_hdfs)

  json_data = train_job.get_json_data()

  if args.watch:
    job_name = args.job_name

  response = client.submit_train_job(json_data)
  if args.argo_mode:
    while True:
      response_state = client.describe_train_job(args.job_name)
      response_state = response_state["state"]
      if response_state in [constant.JOB_STATE_COMPLETED, constant.JOB_STATE_MODELSERVED]:
        break
      elif response_state == constant.JOB_STATE_FAILED:
        exit_as_error("Train job failed.")
      time.sleep(constant.ARGO_COMMAND_WAIT)
    response = client.get_train_job_logs(args.job_name)
  if not isinstance(response, str):
    if args.argo_mode:
      response = response.json()
      if "error" in response and response["error"]:
        exit_as_error(response)
      else:
        if sys.version_info > (3, 0):
          print(response["logs"])
        else:
          print(response["logs"].encode('utf-8'))
    else:
      print_train_job_info(response)
    if args.watch:
      print("\nThe job has submitted, feel free to Ctrl+C to stop watching\n")
      print("{:32} {:16} {:32} {:32}".format("JOB_NAME", "STATE", "CREATED",
                                             "UPDATED"))
      while True:
        watch_response = client.describe_train_job(job_name)
        if not isinstance(watch_response, str):
          print("{:32} {:16} {:32} {:32}".format(watch_response[
                "job_name"], color_util.colorize_state(watch_response["state"]),
                watch_response["create_time"], watch_response["update_time"]))
          if watch_response["state"] == constant.JOB_STATE_COMPLETED:
            return
          try:
            time.sleep(constant.JOB_WATCH_INTERVAL)
          except KeyboardInterrupt:
            return
        else:
          return
  else:
    print("response: {}".format(response))


def describe_job(args):
  """Describe the job."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.describe_train_job(args.job_name, args.org_id)
  else:
    response = client.describe_train_job(args.job_name)
  if not isinstance(response, str):
    print_train_job_info(response)
  else:
    print("response: {}".format(response))


def kill_job(args):
  """Kill the job."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.kill_train_job(args.job_name, args.org_id)
  else:
    response = client.kill_train_job(args.job_name)
  print("response: {}".format(response))


def get_job_logs(args):
  """Get the logs of the job."""
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  extra_args = dict()
  if "follow" in args:
    extra_args = {"follow": args.follow}
  if "lines" in args and args.lines:
    extra_args["lines"] = args.lines
  if "org_id" in args:
    extra_args["org_id"] = args.org_id

  response = client.get_train_job_logs(args.job_name, **extra_args)

  if "follow" in args and args.follow:
    try:
      for line in response.iter_lines():
        print(line)
    except KeyboardInterrupt:
      # exit silently
      return
    except requests.exceptions.ChunkedEncodingError as e:
      if "IncompleteRead(0 bytes read)" in str(e):
        # exit duo to no data read, maybe timeout
        exit_as_error("INFO: 0 bytes read, maybe closed by server due to timeout(5 mins).")
        return
      else:
        exit_as_error("ERROR: read log got ChunkedEncodingError: {}".format(e))
        return
    except Exception as e:
      exit_as_error("ERROR: read log got Error: {}".format(e))
      return
  else:
    # NOTE(xychu): compatible with old version client needed response format.
    # this format will be removed from the server side after no user using
    # client older than 0.2.21
    if not isinstance(response, str):
      response = response.json()
      if "error" in response and response["error"]:
        print(response)
      else:
        if sys.version_info > (3, 0):
          print(response["logs"])
        else:
          print(response["logs"].encode('utf-8'))


def get_job_history_logs(args):
  """Get the logs of the job."""
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  extra_args = dict()
  if "org_id" in args:
    extra_args["org_id"] = args.org_id

  pod_name = client.get_train_job_pod_name(args.job_name, **extra_args)
  print("Job pod name is {}, trying to get logs now.".format(pod_name))
  lines = args.lines if "lines" in args and args.lines else None
  print(log_util.get_log_from_es(pod_name, lines))


def get_job_metrics(args):
  """Get the metrics of the job."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.get_train_job_metrics(args.job_name, args.org_id)
  else:
    response = client.get_train_job_metrics(args.job_name)
  if not isinstance(response, str):
    print_metrics_result(response)
  else:
    print("response: {}".format(response))


def get_job_hyperparameters_data(args):
  """Get hyperparameters data of the job."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  response = client.get_train_job_hyperparameters_data(args.job_name)
  if not isinstance(response, str):
    print_hyperparameter_data_result(response)
  else:
    print("response: {}".format(response))


def delete_job(args):
  """Delete the job(s)."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  extra_args = dict()
  if "is_cluster_name" in args:
    extra_args = {"is_cluster_name": args.is_cluster_name}
  if "org_id" in args:
    for job_name in args.job_names:
      response = client.delete_train_job(job_name, args.org_id, **extra_args)
      print(response)
  else:
    for job_name in args.job_names:
      response = client.delete_train_job(job_name, **extra_args)
      print(response)


def get_train_job_events(args):
  """Get events of the train job."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.get_train_job_events(args.job_name, args.org_id)
  else:
    response = client.get_train_job_events(args.job_name)
  if not isinstance(response, str):
    print_kubernetes_events(response["events"])
    if response["recommend"]:
      print_recommend_resource(response["recommend"])
  else:
    print("response: {}".format(response))


def start_job(args):
  """Start the job."""
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  response = client.start_train_job(args.job_name)
  if not isinstance(response, str):
    print_train_job_info(response)
  else:
    print("response: {}".format(response))


def rerun_job(args):
  """Rerun the job."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.rerun_train_job(args.job_name, args.org_id)
  else:
    response = client.rerun_train_job(args.job_name)
  print("response: {}".format(response))


def list_models(args, silence=False):
  """List model services."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return False
  if "org_id" in args:
    if args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
      response_models = client.list_model_services(
          constant.CLOUDML_ALL_ORG_PARAMETER, priority_class=args.priority_class, page_number=args.page_number, page_size=args.page_size)
    else:
      response_models = client.list_model_services(args.org_id, priority_class=args.priority_class, page_number=args.page_number, page_size=args.page_size)
  else:
    response_models = client.list_model_services(priority_class=args.priority_class, page_number=args.page_number, page_size=args.page_size)
  if silence:
    return response_models
  if not isinstance(response_models, str):
    print_model_list(response_models, args)
  else:
    print("response: {}".format(response_models))


def print_model_list(response_models, args):
  if not args.priority_class:  # NOTE(xychu): will show all priority classes if no priority is given
    prioritized_models = {}
    unprioritized_models = []
    for priority in constant.CLOUDML_PRIORITY_CLASSES:
      prioritized_models[priority] = []
    for item in response_models:
      if "priority_class" in item and item["priority_class"]:
        prioritized_models[item["priority_class"]].append(item)
      else:
        unprioritized_models.append(item)
    for priority in prioritized_models:
      if len(prioritized_models[priority]) > 0:
        print("Priority class {}:".format(priority))
        do_print_model_list(prioritized_models[priority], args)
    if len(unprioritized_models) > 0:
      print("Un-prioritized:")
      do_print_model_list(unprioritized_models, args)
  else:
    do_print_model_list(response_models, args)


def do_print_model_list(response_models, args):
  total_model = 0
  for model in response_models:
    total_model += 1
  print("TOTAL MODEL NUM: {}".format(total_model))
  if "org_id" in args and args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
    print("{:16} {:16} {:32} {:16} {:16} {:32} {:32} {:32}{:6}{:6}".format(
      "ORG ID", "ORG NAME", "MODEL_NAME", "MODEL_VERSION",
      "STATE", "CREATED", "UPDATED", "MULTI", "MAX_WT", "CUR_WT"))
    for model in response_models:
      try:
        max_weight = model["max_weight"]
        max_weight = "%.2f" % float(max_weight)
      except:
        max_weight = "NaN"
      try:
        cur_weight = model["cur_weight"]
        cur_weight = "%.2f" % float(cur_weight)
      except:
        cur_weight = "NaN"
      multi_serving_name = model.get("multi_serving_name", "")
      print("{:16} {:16} {:32} {:16} {:16} {:32} {:32} {:32}{:6}{:6}"
            .format(model["org_id"], model["org_name"],model["model_name"], model["model_version"],
            color_util.colorize_state(model["state"]), model["create_time"], model["update_time"],
            multi_serving_name,max_weight,cur_weight))
  else:
    print("{:32} {:16} {:32} {:16} {:16} {:16} {:6}"
      .format("MODEL_NAME", "MODEL_VERSION", "REQ/AVA REPLICAS" ,"STATE", "DEPLOY", "MULTI", "MAX_WT/CUR_WT"))
    models_list = []
    for model in response_models:
      models_list.append(model)
    models_list.sort(key=lambda i: i["model_name"])
    gpu_share_models = {}
    if "org_id" not in args:
      for model in models_list:
        if model["model_name"].endswith("-gpushare") and model["state"] != "historical":
          gpu_share_replica = {"available_replicas": model.get("available_replicas", None),
                               "replicas": model.get("replicas", None)}
          gpu_share_models[model["model_name"]] = gpu_share_replica
    for model in models_list:
      if "org_id" not in args:
        if model["model_name"].endswith("-gpushare"):
          continue
        gpu_share_available_replicas = 0
        gpu_share_replicas = 0
        if model["model_name"] + "-gpushare" in gpu_share_models:
          gpu_share_available_replicas = gpu_share_models[model["model_name"] + "-gpushare"]["available_replicas"]
          gpu_share_replicas = gpu_share_models[model["model_name"] + "-gpushare"]["replicas"]
      try:
        max_weight = model["max_weight"]
        max_weight = "%.2f" % float(max_weight)
      except:
        max_weight = "NaN"
      try:
        cur_weight = model["cur_weight"]
        cur_weight = "%.2f" % float(cur_weight)
      except:
        cur_weight = "NaN"
      if model["deploy_type"].startswith("pod"):
        max_weight = "NaN"
        cur_weight = "NaN"
      mc_w = "{}/{}".format(max_weight,cur_weight)
      multi_serving_name = model.get("multi_serving_name", "None")
      available_replicas_str = str(model.get("available_replicas", None))
      request_replicas_str = str(model.get("replicas", None))
      if "org_id" not in args:
        if available_replicas_str == "None" or request_replicas_str == "None":
          available_replicas = available_replicas_str
          request_replicas = request_replicas_str
        else:
          available_replicas = str(int(available_replicas_str) + gpu_share_available_replicas)
          request_replicas = str(int(request_replicas_str) + gpu_share_replicas)
      else:
        available_replicas = available_replicas_str
        request_replicas = request_replicas_str
      req_ava_rep = request_replicas + "/" + available_replicas
      print("{:32} {:16} {:32} {:16} {:16} {:16} {:6}"
            .format(model["model_name"],
                    model["model_version"],
                    req_ava_rep,
                    model["state"],
                    model["deploy_type"],
                    multi_serving_name,
                    mc_w))


def _is_local_uri(uri):
  if not (uri.startswith("fds://") or uri.startswith("/fds/")):
    return True
  return False


def _check_bucket_and_convert_uri(args, client):
  bucket = client.fds_bucket
  fds_client = client.get_fds_client()
  uri = args.model_uri
  if uri.startswith("fds://"):
    bucket = uri.split('/', 3)[2]
    if not fds_client.does_bucket_exist(bucket):
      raise ValueError("Can't find " + bucket + " bucket in fds server! Please check again!")

  if uri.startswith("/fds/"):
    if args.fds_bucket:
      bucket = args.fds_bucket
    if not fds_client.does_bucket_exist(bucket):
      raise ValueError("Can't find " + bucket + " bucket in fds server! Please check again!")
    uri = "fds://" + bucket + "/" + uri.split('/', 2)[2]
  return uri


def _has_config_file_in_path(uri, client):
  fds_client = client.get_fds_client()
  bucket = uri.split('/', 3)[2]
  obj_path = os.path.join(uri.split('/', 3)[-1], constant.MULTI_MODEL_CONFIG_FILE)
  obj_path = obj_path.replace('\\', '/')
  try:
    if fds_client.does_object_exists(bucket, obj_path):
      return True
  except Exception as e:
    print("fds error: %s" % str(e))

  return False


def _get_org_id(client):
  response = client.get_org_id()
  if isinstance(response, str):
    print("response: {}".format(response))
    exit_as_error("ERROR: can't get your org_id!")
    return None
  else:
    return response.get("org_id")


def _gen_targz_fds_path(args, client):
  org_id = "default"
  real_org_id = _get_org_id(client)
  if real_org_id:
    org_id = str(real_org_id)
  model_name = args.model_name
  model_version = args.model_version
  fds_path = [constant.CLOUDML_USER_MODELS_DIR, "targz",
    org_id, model_name, model_version, constant.MODELS_TAR_GZ
  ]
  return '/'.join(fds_path)


def _gen_raw_fds_path(args, client):
  org_id = "default"
  real_org_id = _get_org_id(client)
  if real_org_id:
    org_id = str(real_org_id)
  model_name = args.model_name
  model_version = args.model_version
  fds_path = [constant.CLOUDML_USER_MODELS_DIR, "raw",
    org_id, model_name, model_version, model_name
  ]
  return '/'.join(fds_path)


def _upload_models_to_fds(source_path, target_path, client):
  print("source_path:"+source_path)
  print("target_path:"+target_path)
  bucket = constant.CLOUDML_SHARED_BUCKET
  fds_client = client.get_fds_client()
  try:
    with open(source_path, 'rb') as f:
      data = f.read()
      res = fds_client.put_object(bucket, target_path, data)
      print("res:"+str(res))
      return 'fds://' + bucket + "/" + target_path
  except Exception as e:
    raise ValueError("Upload models got error: ", e)
  finally:
    models_util.remove_file(source_path)


def _copy_raw_to_fds(client, args):
  bucket = constant.CLOUDML_SHARED_BUCKET
  prefix = _gen_raw_fds_path(args, client)
  fds_client = client.get_fds_client()
  base_path = args.local_model_path
  if base_path[-1] != '/':
    base_path = base_path + '/'
  try:
    for curs, dirs, files in os.walk(base_path, followlinks=True):
      prefix_dir = curs[len(base_path):]
      if len(files) <= 0 and len(dirs) <= 0:
        null_file = "__XM_CLOUDML_NULL__"
        open(os.path.join(curs, null_file), 'a').close()
        files.append(null_file)
      for name in files:
        obj_name = os.path.join(prefix, prefix_dir, name)
        data_path = os.path.join(curs, name)
        with open(data_path, 'rb') as f:
          res = fds_client.put_object(bucket, obj_name, f.read())
          print("upload file: {} as {}".format(data_path, obj_name))
  except Exception as e:
    raise ValueError("Upload raw model files got error: ", e)
  print("copy dirs from fds  to local  done. ")


def _download_fds_file_to_path(client, bucket, objname, abspath):
  fds_client = client.get_fds_client()
  if not os.path.exists(os.path.dirname(abspath)):
    os.makedirs(os.path.dirname(abspath))
  ok = False
  while not ok:
    try:
      fds_client.download_object(bucket, objname, abspath)
      ok = True
    except Exception as e:
      raise ValueError("download raw model files got error: ", e)
      time.sleep(2)
  print("download file successful! {} ".format(abspath))
  return ok


def _extract_model_files_from_targz(tar_path):
  tar = tarfile.open(tar_path)
  tar.extractall(os.path.dirname(tar_path))
  file_list = tar.getnames()
  for f in file_list:
    print("extract file: {}".format(f))
  tar.close()
  os.remove(tar_path)
  print("extract {} done.".format(constant.MODELS_TAR_GZ))


def _download_model_to_local(client, args):
  bucket = constant.CLOUDML_SHARED_BUCKET
  base_path = args.local_model_path
  model_name = args.model_name
  if args.archive_type == "targz":
    objname = _gen_targz_fds_path(args, client)
    abspath = os.path.join(base_path, objname)
    _download_fds_file_to_path(client, bucket, objname, abspath)
    _extract_model_files_from_targz(abspath)
  else:
    prefix = _gen_raw_fds_path(args, client)
    if prefix[-1] != '/':
      prefix = prefix + '/'
    fds_client = client.get_fds_client()
    results = fds_client.list_all_objects(bucket, prefix=prefix, delimiter="")
    for obj in results:
      objname = obj.object_name
      abspath = os.path.join(base_path, model_name, objname[len(prefix):])
      if objname[-1] != '/':
        if not os.path.exists(os.path.dirname(abspath)):
          os.makedirs(os.path.dirname(abspath))
        _download_fds_file_to_path(client, bucket, objname, abspath)
      elif not os.path.exists(abspath):
        os.makedirs(abspath)
        print("make dir: {}".format(abspath))
      else:
        pass
  print("copy dirs from fds  to local  done. ")
  return


def _tar_and_upload_models(args, client, local_model_dir):
  if local_model_dir == None:
    exit_as_error("model_uri or local_model_path should be given!")
    return
  basename = os.path.basename(local_model_dir)
  source_path = os.path.join(tempfile.gettempdir(), basename + ".models.tar.gz")
  models_util.compression_dir(local_model_dir, source_path)
  target_path = _gen_targz_fds_path(args, client)
  fds_path = _upload_models_to_fds(source_path, target_path, client)
  return fds_path


def _validate_local_dir_for_multi_sm(path, args):
  if not os.path.isdir(path):
    exit_as_error("local model path should be a dir!")

  config_file = os.path.join(path, constant.MULTI_MODEL_CONFIG_FILE)
  if os.path.exists(config_file):
    args.model_format = "multi-sm"
  elif args.model_format == "multi-sm":
    exit_as_error("model_format: multi-sm  needs models.config in path")


def validate_and_get_model_uri(args, client):
  if not isinstance(args.model_uri, str):
    exit_as_error("model_uri must be a string!")

  if not _is_local_uri(args.model_uri):
    uri = _check_bucket_and_convert_uri(args, client)
    has_config_file = _has_config_file_in_path(uri, client)
    if has_config_file:
      args.model_format = "multi-sm"
    elif args.model_format == "multi-sm":
        exit_as_error("model_format: multi-sm  needs models.config in path")
    return uri

  _validate_local_dir_for_multi_sm(args.model_uri, args)

  local_model_dir = os.path.abspath(args.model_uri)
  to_return = _tar_and_upload_models(args, client, local_model_dir)
  return to_return


def validate_model_version(args):
  try:
    str(int(args.model_version))
  except:
    exit_as_error("ERROR: type of model version is not int!!")
  return


def create_model(args):
  """Create the model service."""
  validate_model_version(args)
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  if args.argo_mode:
      args.model_name = args.model_name + '-' + get_cur_time()
  args.model_uri = validate_and_get_model_uri(args, client)
  model = ModelService(args.model_name, args.model_version, args.model_uri)
  if args.service_type:
    model.model_type = args.service_type
  if args.fds_endpoint:
    model.fds_endpoint = args.fds_endpoint
  elif client.fds_endpoint:
    model.fds_endpoint = client.fds_endpoint
  if args.fds_bucket:
    model.fds_bucket = args.fds_bucket
  elif client.fds_bucket:
    model.fds_bucket = client.fds_bucket
  if args.model_args:
    model.model_args = args.model_args
  if args.cpu_limit:
    model.cpu_limit = args.cpu_limit
  if args.memory_limit:
    model.memory_limit = args.memory_limit
  if args.gpu_limit:
    model.gpu_limit = args.gpu_limit
  if args.gpu_memory:
    model.gpu_memory = args.gpu_memory
  if args.framework:
    model.framework = args.framework
  if args.framework_version:
    model.framework_version = args.framework_version
  if args.docker_image:
    model.docker_image = args.docker_image
  if args.docker_command:
    model.docker_command = args.docker_command
  if args.replicas:
    model.replicas = int(args.replicas)
  if args.prepare_command:
    model.prepare_command = args.prepare_command
  if args.finish_command:
    model.finish_command = args.finish_command
  if args.node_selector_key:
    model.node_selector_key = args.node_selector_key
  if args.node_selector_value:
    model.node_selector_value = args.node_selector_value
  if args.gpu_type:
    if args.gpu_limit:
      model.gpu_type = args.gpu_type
    else:
      raise ValueError("ERROR: If you want select gpu type, gpu limit (-g) cannot be None.")
  if args.enable_rank:
    model.enable_rank = 1
  else:
    model.enable_rank = 0
  if args.debug:
    model.debug = 1
  else:
    model.debug = 0
  if args.watch:
    model_name = args.model_name
    model_version = args.model_version
  if args.save_mode:
    model.save_mode = args.save_mode
  if args.use_seldon:
    model.use_seldon = args.use_seldon
  if args.engine_cpu:
    model.engine_cpu = args.engine_cpu
  if args.engine_mem:
    model.engine_mem = args.engine_mem
  if args.run_class_name:
    model.run_class_name = args.run_class_name
  if args.service_schema:
    model.service_schema = args.service_schema
  if args.initial_delay_sec:
    model.initial_delay_sec = args.initial_delay_sec
  if args.pod_replicas:
    model.pod_replicas = args.pod_replicas
  model.multi_serving_name = args.multi_serving_name
  if args.per_process_gpu_memory_fraction:
    if not args.multi_serving_name:
      raise ValueError("ERROR: If you want share GPU with multi-serving, must be set multi_serving_name")
    model.per_process_gpu_memory_fraction = args.per_process_gpu_memory_fraction
  if args.describe:
    model.describe = args.describe
  if args.telephone:
    model.telephone = args.telephone
  if args.version_policy:
    model.version_policy = args.version_policy
  if args.model_format:
    model.model_format = args.model_format
  if args.owner_email:
    model.owner_email = args.owner_email
  else:
    model.owner_email = client.org_mail
  if args.deploy_type:
    model.deploy_type = args.deploy_type
  if args.priority_class:
    model.priority_class = args.priority_class
  if args.grace_period_seconds:
    model.grace_period_seconds = args.grace_period_seconds
  if args.readiness_command:
    model.readiness_command = args.readiness_command

  if args.meta_info:
    model.meta_info = args.meta_info

  response = client.create_model_service(model)
  if not isinstance(response, str):
    print_model_service_info(response)
    if args.watch:
      print("\nThe model is creating, feel free to Ctrl+C to stop watching\n")
      print("{:32} {:4} {:16} {:32} {:32}".format(
              "Model_NAME",
              "VERSION",
              "STATE",
              "CREATED",
              "UPDATED"))
      while True:
        watch_response = client.describe_model_service(model_name, model_version)
        if not isinstance(watch_response, str):
          print("{:32} {:4} {:16} {:32} {:32}".format(
                watch_response["model_name"],
                watch_response["model_version"],
                color_util.colorize_state(watch_response["state"]),
                watch_response["create_time"],
                watch_response["update_time"]))
          if watch_response["state"] == constant.MODEL_STATE_RUNNING:
            return
          try:
            time.sleep(constant.JOB_WATCH_INTERVAL)
          except KeyboardInterrupt:
            return
        else:
          return
  else:
    print("response: {}".format(response))


def describe_model(args):
  """Describe the model service."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.describe_model_service(args.model_name,
                                             args.model_version, args.org_id)
  else:
    response = client.describe_model_service(args.model_name,
                                             args.model_version)
  if not isinstance(response, str):
    print_model_service_info(response)
  else:
    print("response: {}".format(response))


def start_model(args):
  """Start the model service."""
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  response = client.start_model_service(args.model_name, args.model_version)
  if not isinstance(response, str):
    print_model_service_info(response)
  else:
    print("response: {}".format(response))


def _is_args_allowed_in_update_model(args):
  if args.model_uri:
    print("updating model_uri is not allowed currently.")
    return False
  if args.cpu_limit:
    print("updating cpu is not allowed currently.")
    return False
  if args.memory_limit:
    print("updating memory is not allowed currently.")
    return False
  if args.gpu_limit:
    print("updating gpu is not allowed currently.")
    return False
  if args.framework:
    print("updating framework is not allowed currently.")
    return False
  if args.engine_cpu:
    print("updating engine_cpu is not allowed currently.")
    return False
  if args.engine_mem:
    print("updating engine_mem is not allowed currently.")
    return False
  if args.port:
    print("updating port is not allowed currently.")
    return False
  if args.replicas:
    return True
  if args.weight:
    return True
  if args.meta_info:
    return True
  print(" models update: need arguments -w or -r or --meta_info")
  return False


def update_model(args, silence=False):
  """Update the model service."""
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return False
  if not  _is_args_allowed_in_update_model(args):
    exit_as_error("ERROR parameters")
    return False

  update_json = {}
  if args.model_uri:
    update_json["model_uri"] = args.model_uri
  if args.replicas:
    update_json["replicas"] = int(args.replicas)
  if args.weight:
    update_json["weight"] = args.weight
  if args.cpu_limit:
    update_json["cpu_limit"] = args.cpu_limit
  if args.memory_limit:
    update_json["memory_limit"] = args.memory_limit
  if args.gpu_limit:
    update_json["gpu_limit"] = args.gpu_limit
  if args.framework:
    update_json["framework"] = args.framework
  if args.engine_cpu:
    update_json["engine_cpu"] = args.engine_cpu
  if args.engine_mem:
    update_json["engine_mem"] = args.engine_mem
  if args.port and args.port.isdigit():
    update_json["port"] = args.port

  if args.meta_info:
    update_json["meta_info"] = args.meta_info

  if "org_id" in args:
    response = client.update_model_service(args.model_name,
                                           args.model_version,
                                           update_json,
                                           args.org_id)
  else:
    response = client.update_model_service(args.model_name,
                                           args.model_version,
                                           update_json)
  if silence:
    return True
  if not isinstance(response, str):
    print_model_service_info(response)
  else:
    print("response: {}".format(response))


def _get_model_deploy_type(cmd_parser, model_name, model_version):
  from cloud_ml_sdk.command.command import new_parser
  cmd_parser = new_parser()
  arg_list = ["models", "list"]
  print(" ".join(arg_list))
  list_args = cmd_parser.parse_args(arg_list)
  response_models = list_models(list_args, silence=True)
  if isinstance(response_models, str):
    exit_as_error("ERROR: models list failed")
    return None
  for item in response_models:
    if item["model_name"] == model_name and item["model_version"] == model_version:
      return item["deploy_type"]
  return None


def _get_model_replicas(cmd_parser, model_name, model_version):
  from cloud_ml_sdk.command.command import new_parser
  cmd_parser = new_parser()
  arg_list = ["models", "list"]
  print(" ".join(arg_list))
  list_args = cmd_parser.parse_args(arg_list)
  response_models = list_models(list_args, silence=True)
  if isinstance(response_models, str):
    exit_as_error("ERROR: models list failed")
    return 99999
  for item in response_models:
    if item["model_name"] == model_name and item["model_version"] == model_version:
      return item["replicas"]
  return 99999


def _get_model_available_replicas(cmd_parser, model_name, model_version):
  from cloud_ml_sdk.command.command import new_parser
  cmd_parser = new_parser()
  arg_list = ["models", "list"]
  print(" ".join(arg_list))
  list_args = cmd_parser.parse_args(arg_list)
  response_models = list_models(list_args, silence=True)
  if isinstance(response_models, str):
    exit_as_error("ERROR: models list failed")
    return 0
  for item in response_models:
    if item["model_name"] == model_name and item["model_version"] == model_version:
      if item.get("available_replicas") is None:
        item["available_replicas"] = 0
      return item["available_replicas"]
  return 0


def _update_model_replicas(cmd_parser,  model_name, model_version, replicas):
  from cloud_ml_sdk.command.command import new_parser
  cmd_parser = new_parser()
  arg_list = ["models", "update", model_name, model_version, "-r", str(replicas)]
  print(" ".join(arg_list))
  update_args = cmd_parser.parse_args(arg_list)
  update_model(update_args, silence=True)


def _update_model_weight(cmd_parser, model_name, model_version, weight):
  from cloud_ml_sdk.command.command import new_parser
  cmd_parser = new_parser()
  arg_list = ["models", "update", model_name, model_version, "-w", weight]
  print(" ".join(arg_list))
  update_args = cmd_parser.parse_args(arg_list)
  update_model(update_args, silence=True)


def _delete_model(cmd_parser, model_name, model_version):
  from cloud_ml_sdk.command.command import new_parser
  cmd_parser = new_parser()
  arg_list = ["models", "delete", model_name, model_version]
  print(" ".join(arg_list))
  delete_args = cmd_parser.parse_args(arg_list)
  delete_model(delete_args)
  return


def _is_model_ready(cmd_parser, model_name, model_version):
  from cloud_ml_sdk.command.command import new_parser
  cmd_parser = new_parser()
  arg_list = ["models", "list"]
  print(" ".join(arg_list))
  list_args = cmd_parser.parse_args(arg_list)
  response_models = list_models(list_args, silence=True)
  if isinstance(response_models, str):
    exit_as_error("ERROR: models list failed")
    return 0
  for item in response_models:
    if item["model_name"] == model_name and item["model_version"] == model_version:
      if item.get("available_replicas") is None:
        item["available_replicas"] = 0
      if item["replicas"] > 0 and item["available_replicas"] > 0 and \
        item["available_replicas"] == item["replicas"] and item["state"] == "running": 
        return True
  return False


def ready_model(args):
  if args.model_name is None:
    exit_as_error("ERROR: model name is none")
    return
  if args.model_version is None:
    exit_as_error("ERROR: from version is none")
    return
  from cloud_ml_sdk.command.command import new_parser
  cmd_parser = new_parser()
  if _is_model_ready(cmd_parser, args.model_name, args.model_version):
    print("model is ready.")
  else:
    exit_as_error("ERROR: model is not ready")


def roll_model(args):
  """rolling update the model service."""
  if args.model_name is None:
    exit_as_error("ERROR: model name is none")
    return
  if args.from_version is None:
    exit_as_error("ERROR: from version is none")
    return
  if args.to_version is None:
    exit_as_error("ERROR: to version is none")
    return
  if args.update_timeout is None:
    args.update_timeout = "10"

  timeout = int(args.update_timeout)
  from cloud_ml_sdk.command.command import new_parser
  cmd_parser = new_parser()
  deploy_type = _get_model_deploy_type(cmd_parser, args.model_name, args.from_version)

  # get model replicas
  from_replicas = _get_model_replicas(cmd_parser, args.model_name, args.from_version)
  to_replicas = _get_model_replicas(cmd_parser, args.model_name, args.to_version)
  if from_replicas <= 0:
    exit_as_error("ERROR: the replicas of from version is 0")
    return
  if  to_replicas <= 0:
    exit_as_error("ERROR: the replicas of to version is 0")
    return
  total_replicas = from_replicas + to_replicas

  # rolling
  while from_replicas > 0:
    from_replicas = from_replicas - 1
    if from_replicas >= 1:
      _update_model_replicas(cmd_parser, args.model_name, args.from_version, from_replicas)
      time.sleep(timeout)
      real_from_replicas = _get_model_available_replicas(cmd_parser, args.model_name, args.from_version)
      while real_from_replicas > from_replicas:
        print("not ready, sleep for {}s".format(timeout))
        time.sleep(timeout)
        real_from_replicas = _get_model_available_replicas(cmd_parser, args.model_name, args.from_version)
    else: # from_replicas <= 0
      _delete_model(cmd_parser, args.model_name, args.from_version)
      time.sleep(timeout)
    to_replicas = to_replicas + 1
    _update_model_replicas(cmd_parser, args.model_name, args.to_version, to_replicas)
    time.sleep(timeout)
    real_to_replicas = _get_model_available_replicas(cmd_parser, args.model_name, args.to_version)
    while real_to_replicas < to_replicas:
      print("not ready, sleep for {}s".format(timeout))
      time.sleep(timeout)
      real_to_replicas = _get_model_available_replicas(cmd_parser, args.model_name, args.to_version)
    if deploy_type == "ingress":
      w = int((real_to_replicas/total_replicas)*100)
      _update_model_weight(cmd_parser, args.model_name, args.to_version, str(w))
  return


def upload_model_data(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if args.local_model_path is None:
    exit_as_error("ERROR: local_model_path is none")
    return
  if args.model_name is None:
    key = constant.DEPLOY_PREFIX + "DEPLOY_NAME"
    value = os.getenv(key)
    if value:
      args.model_name = value
    else:
      exit_as_error("ERROR: model_name is none")
      return
  if args.model_version is None:
    key = constant.DEPLOY_PREFIX + "DEPLOY_VERSION"
    value = os.getenv(key)
    if value:
      args.model_version = value
    else:
      exit_as_error("ERROR: model_name is none")
      return
  if args.archive_type == "targz":
    local_model_dir = args.local_model_path
    _tar_and_upload_models(args, client, local_model_dir)
  else:
    _copy_raw_to_fds(client, args)


def download_model_data(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if args.local_model_path is None:
    exit_as_error("ERROR: local_model_path is none")
    return
  if args.model_name is None:
    exit_as_error("ERROR: model_name is none")
    return
  if args.model_version is None:
    exit_as_error("ERROR: model_version is none")
    return
  _download_model_to_local(client, args)


def get_model_logs(args):
  """Get logs of the model service."""
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    if "replica_index" in args and args.replica_index:
      response = client.get_model_service_logs(model_name = args.model_name,
                                               model_version = args.model_version,
                                               org_id = args.org_id,
                                               replica_index = args.replica_index)
    elif "proxy_logs" in args and args.proxy_logs:
      response = client.get_model_service_logs(model_name = args.model_name,
                                               model_version = args.model_version,
                                               org_id = args.org_id,
                                               proxy_logs = args.proxy_logs)
    else:
      response = client.get_model_service_logs(args.model_name, args.model_version, args.org_id)
  else:
    if "replica_index" in args and args.replica_index:
      response = client.get_model_service_logs(args.model_name, args.model_version, replica_index=args.replica_index)
    elif "proxy_logs" in args and args.proxy_logs:
      response = client.get_model_service_logs(model_name=args.model_name,
                                               model_version=args.model_version,
                                               proxy_logs=args.proxy_logs)
    else:
      response = client.get_model_service_logs(args.model_name, args.model_version)
  if not isinstance(response, str):
    if "error" in response:
      print(response['message'])
    else:
      if sys.version_info > (3, 0):
        print(response["logs"])
      else:
        print(response["logs"].encode('utf-8'))
  else:
    print("response: {}".format(response))


def get_model_metrics(args):
  """Get the metrics of the model service."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.get_model_service_metrics(args.model_name, args.model_version, args.org_id)
  else:
    response = client.get_model_service_metrics(args.model_name, args.model_version)
  if not isinstance(response, str):
    print_metrics_result(response)
  else:
    print("response: {}".format(response))


def delete_model(args):
  """Delete the model service."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.delete_model_service(args.model_name, args.model_version, args.history, args.org_id)
  else:
    response = client.delete_model_service(args.model_name, args.model_version, args.history)
  print(response)


def get_model_service_events(args):
  """Get events of the model service."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.get_model_service_events(args.model_name,
                                               args.model_version, args.org_id)
  else:
    response = client.get_model_service_events(args.model_name,
                                               args.model_version)
  if not isinstance(response, str):
    print_kubernetes_events(response["events"])
    if response["recommend"]:
      print_recommend_resource(response["recommend"])
  else:
    print("response: {}".format(response))


def model_review(args):
  """ Review of the model service."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  data = {
    "model_name": args.model_name,
    "model_version": args.model_version,
    "review_leader": args.review_leader
  }
  response = client.model_review(data)
  print("response: {}".format(response))


def model_daildown(args):
  """
  Get
  """
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  response = client.model_daildown(args.model_name, args.model_version)
  print("response: {}".format(response))


def model_onboard(args):
  """
  Get
  """
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  response = client.model_onboard(args.model_name, args.model_version)
  print("response: {}".format(response))


def restart_model_service_instance(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  org_id = None
  if "org_id" in args:
    org_id = args.org_id
  response = client.restart_model_service_instance(args.model_name, args.model_version, org_id)
  print("response: {}".format(response))


def _create_a_model(cmd_parser, client, cur_json):
  from cloud_ml_sdk.command.command import new_parser
  cmd_parser = new_parser()
  arg_list = ["models", "create"]
  if cur_json.get("model_name", None):
    arg_list.extend(["-n", cur_json["model_name"]])
  if cur_json.get("model_version", None):
    arg_list.extend(["-v", cur_json["model_version"]])
  if cur_json.get("model_uri", None):
    arg_list.extend(["-u", cur_json["model_uri"]])
  if cur_json.get("service_type", None):
    arg_list.extend(["-t", cur_json["service_type"]])
  if cur_json.get("deploy_type", None):
    arg_list.extend(["-dt", cur_json["deploy_type"]])
  if cur_json.get("model_format", None):
    arg_list.extend(["-mf", cur_json["model_format"]])
  if cur_json.get("readiness_command", None):
    arg_list.extend(["-rc", cur_json["readiness_command"]])
  if cur_json.get("grace_period_seconds", None):
    arg_list.extend(["-gps", cur_json["grace_period_seconds"]])
  if cur_json.get("docker_image", None):
    arg_list.extend(["-d", cur_json["docker_image"]])
  if cur_json.get("docker_command", None):
    arg_list.extend(["-dc", cur_json["docker_command"]])
  if cur_json.get("framework", None):
    arg_list.extend(["-F", cur_json["framework"]])
  if cur_json.get("framework_version", None):
    arg_list.extend(["-V", cur_json["framework_version"]])
  if cur_json.get("owner_email", None):
    arg_list.extend(["-e", cur_json["owner_email"]])
  if cur_json.get("cpu_limit", None):
    arg_list.extend(["-c", cur_json["cpu_limit"]])
  if cur_json.get("memory_limit", None):
    arg_list.extend(["-M", cur_json["memory_limit"]])
  if cur_json.get("gpu_limit", None):
    arg_list.extend(["-g", cur_json["gpu_limit"]])
  if cur_json.get("gpu_type", None):
    arg_list.extend(["-gt", cur_json["gpu_type"]])
  arg_list.extend(["-r", "1"])
  print(" ".join(arg_list))
  tmp_args = cmd_parser.parse_args(arg_list)
  print(tmp_args)
  create_model(tmp_args)
  return


def _running_model_list(name):
  from cloud_ml_sdk.command.command import new_parser
  cmd_parser = new_parser()
  arg_list = ["models", "list"]
  print(" ".join(arg_list))
  list_args = cmd_parser.parse_args(arg_list)
  response_models = list_models(list_args, silence=True)
  if isinstance(response_models, str):
    exit_as_error("ERROR: models list failed")
    return None
  ret = []
  for item in response_models:
    if item["model_name"] == name and (item["state"] == "running" or \
      item["state"] == "enqueued" or item["state"] == "creating"):
      ret.append(item)
  return ret


def _has_deploy_info(client, name, version):
  response = client.deploy_info("describe", name, version)
  if isinstance(response, str):
    return False
  else:
    return True


def _gen_model_create_info(client, cur_json):
  if cur_json.get("cpu_limit", None) is None:
    exit_as_error("cpu_limit should not be None in your config file!")
  if cur_json.get("memory_limit", None) is None:
    exit_as_error("memory_limit should not be None in your config file!")
  if cur_json.get("replicas", None) is None:
    exit_as_error("replicas should not be None in your config file!")

  if cur_json.get("org_id", None) is None:
    cur_json["org_id"] = client.org_id

  if cur_json.get("model_name", None) is None:
    key = constant.DEPLOY_PREFIX + "DEPLOY_NAME"
    value = os.getenv(key)
    if value:
      cur_json["model_name"] = value
    else:
      exit_as_error("model_name should not be None in your config file!")

  if cur_json.get("model_version", None) is None:
    key = constant.DEPLOY_PREFIX + "DEPLOY_VERSION"
    value = os.getenv(key)
    if value:
      cur_json["model_version"] = value
    else:
      exit_as_error("model_version should not be None in your config file!")

  if cur_json.get("service_type", None) is None:
    key = constant.DEPLOY_PREFIX + "ENV"
    value = os.getenv(key)
    if value:
      cur_json["service_type"] = value
    else:
      exit_as_error("service_type should not be None in your config file!")

  if cur_json.get("owner_email", None) is None:
    key = constant.DEPLOY_PREFIX + "SUBMITTER_EMAIL"
    value = os.getenv(key)
    if value:
      cur_json["owner_email"] = value
    else:
      exit_as_error("owner_email should not be None in your config file!")

  model_name = cur_json["model_name"]
  model_version = cur_json["model_version"]
  if cur_json.get("model_uri", None) is None:
    ulist = [
      "fds:/",
      constant.CLOUDML_SHARED_BUCKET,
      constant.CLOUDML_USER_MODELS_DIR,
      "raw",
      str(client.org_id),
      str(model_name),
      str(model_version)
    ]
    m_uri = '/'.join(ulist)
    print(m_uri)
    cur_json["model_uri"] = str(m_uri)
  return cur_json


def _cmd_pause_wait_loop(cmd_parser, client, cur_json):
  from cloud_ml_sdk.command.command import new_parser
  cmd_parser = new_parser()
  cmd = "pause"
  while cmd == "pause":
    print("user paused, sleep for {}s".format(10))
    time.sleep(10)
    response = client.deploy_info("cmd", cur_json["model_name"], cur_json["model_version"])
    if isinstance(response, str):
      print("get cmd failed! response: {}".format(response))
      break
    cmd = response["cmd"]
  return


def _get_deploy_cmd_for_first_deploy(cmd_parser, client, cur_json, with_deploy_info):
  from cloud_ml_sdk.command.command import new_parser
  cmd_parser = new_parser()
  if not with_deploy_info:
    return
  response = client.deploy_info("cmd", cur_json["model_name"], cur_json["model_version"])
  if isinstance(response, str):
    print("get cmd failed! response: {}".format(response))
    return
  cmd = response["cmd"]
  print("current cmd is: {}".format(cmd))
  if cmd == "stop":
    print("user stop the deploy!! delete the new deployed model service")
    arg_list = ["models", "delete", cur_json["model_name"], cur_json["model_version"]]
    print(" ".join(arg_list))
    args = cmd_parser.parse_args(arg_list)
    args.func(args)
    exit_as_error("delete finished.")
    return
  elif cmd == "abort":
    exit_as_error(" occurs error!!  please start a new deploy to retry.")
    return 
  elif cmd == "pause":
    _cmd_pause_wait_loop(cmd_parser, client, cur_json)
    _get_deploy_cmd_for_first_deploy(cmd_parser, client, cur_json, with_deploy_info)
    return
  elif cmd == "resume" or "rerun" or "start":
    return
  else:
    print("cmd not support!")
  return


def _get_deploy_cmd_for_rolling(cmd_parser, client, name, version, from_json, cur_json, with_deploy_info):
  from cloud_ml_sdk.command.command import new_parser
  cmd_parser = new_parser()
  if not with_deploy_info:
    return
  response = client.deploy_info("cmd", name, version)
  if isinstance(response, str):
    print("get cmd failed! response: {}".format(response))
    return
  cmd = response["cmd"]
  print("current cmd is: {}".format(cmd))
  if cmd == "stop":
    print("user stop the deploy!! start to rollback...")
    # rollback
    arg_list = ["models", "roll", model_name, "-fv", version, "-tv", from_json["model_version"], "-ut", 10]
    print(" ".join(arg_list))
    args = cmd_parser.parse_args(arg_list)
    args.func(args)
    # add auto scaler rule
    if cur_json["config_json_file"]:
      arg_list = ["autoscaler", "create", model_name, from_json["model_version"], "-jf", cur_json["config_json_file"]]
      print(" ".join(arg_list))
      args = cmd_parser.parse_args(arg_list)
      args.func(args)
    exit_as_error("user stopped!! rollback done. ")
    return
  elif cmd == "abort":
    exit_as_error(" occurs error!!  please start a new deploy to retry.")
    return 
  elif cmd == "pause":
    _cmd_pause_wait_loop(cmd_parser, client, cur_json)
    _get_deploy_cmd_for_rolling(cmd_parser, client, name, version, from_json, cur_json, with_deploy_info)
  elif cmd == "resume" or "rerun" or "start":
    return
  else:
    print("cmd not support!")
  return


def _first_deploy(client, cur_json, with_deploy_info):
  from cloud_ml_sdk.command.command import new_parser
  cmd_parser = new_parser()
  timeout = 10
  model_version = cur_json["model_version"]
  model_name = cur_json["model_name"]
  replicas = str(cur_json["replicas"])

  # start
  if with_deploy_info:
    client.deploy_cmd("start", model_name, model_version)
    print("deploy start...")

  #create new version
  print("begin to create a new version ...")
  _create_a_model(cmd_parser, client, cur_json)
  while not _is_model_ready(cmd_parser, cur_json["model_name"], cur_json["model_version"]):
    print("not ready, sleep for {}s".format(timeout))
    time.sleep(timeout)

  # receive cmd
  if with_deploy_info:
    print("begin to receive cmd...")
    _get_deploy_cmd_for_first_deploy(cmd_parser, client, cur_json, with_deploy_info)

  #update replicas
  print("begin to update replicas...")
  _update_model_replicas(cmd_parser,  model_name, model_version, replicas)
  while not _is_model_ready(cmd_parser, model_name, model_version):
    print("not ready, sleep for {}s".format(timeout))
    time.sleep(timeout)

  # add auto scaler rule
  if cur_json.get("config_json_file", None):
    print("begin to add scaler rule...")
    arg_list = ["autoscaler", "create", model_name, cur_json["model_version"], "-jf", cur_json["config_json_file"]]
    print(" ".join(arg_list))
    args = cmd_parser.parse_args(arg_list)
    args.func(args)

  # finished
  if with_deploy_info:
    client.deploy_cmd("finish", model_name, model_version)
  print("deploy finished!")
  return


def _rolling_update_deploy(client, from_json, cur_json, with_deploy_info):
  timeout = 10
  from_version = from_json["model_version"]
  to_version = cur_json["model_version"]
  model_name = cur_json["model_name"]
  replicas = int(cur_json["replicas"])

  from cloud_ml_sdk.command.command import new_parser
  cmd_parser = new_parser()

  # start
  if with_deploy_info:
    client.deploy_cmd("start", model_name, to_version)
    print("deploy start...")

  #delete auto scaler
  print("begin to delete auto scaler ...")
  arg_list = ["autoscaler", "delete", model_name, from_version]
  print(" ".join(arg_list))
  args = cmd_parser.parse_args(arg_list)
  args.func(args)

  #update old version to replicas - 1
  print("begin to update old version to replicas - 1 ...")
  _update_model_replicas(cmd_parser,  model_name, from_version, replicas - 1)
  while not _is_model_ready(cmd_parser, model_name, from_version):
    print("not ready, sleep for {}s".format(timeout))
    time.sleep(timeout)

  #create new version
  print("begin to create only one new version ...")
  _create_a_model(cmd_parser, client, cur_json)
  while not _is_model_ready(cmd_parser, model_name, to_version):
    print("not ready, sleep for {}s".format(timeout))
    time.sleep(timeout)
  
  # receive deploy cmd
  if with_deploy_info:
    print("begin to receive deploy cmd ...")
    _get_deploy_cmd_for_rolling(cmd_parser, client, model_name, to_version, from_json, cur_json, with_deploy_info)

  # rolling
  from_replicas = replicas - 1
  to_replicas = 1
  print("begin to rolling ...")
  while from_replicas > 0:
    from_replicas = from_replicas - 1
    to_replicas = to_replicas + 1

    if cur_json["deploy_type"] == "ingress":
      print("begin to adjust weight ...")
      w = int((to_replicas/replicas)*100)
      _update_model_weight(cmd_parser, model_name, to_version, str(w))
      print("not ready, sleep for {}s".format(timeout))
      time.sleep(timeout)

    if from_replicas >= 1:
      _update_model_replicas(cmd_parser, model_name, from_version, from_replicas)
      while not _is_model_ready(cmd_parser, model_name, from_version):
        print("not ready, sleep for {}s".format(timeout))
        time.sleep(timeout)
    else: # from_replicas <= 0
      _delete_model(cmd_parser, model_name, from_version)
      time.sleep(timeout)

    _update_model_replicas(cmd_parser, model_name, to_version, to_replicas)
    while not _is_model_ready(cmd_parser, model_name, to_version):
      print("not ready, sleep for {}s".format(timeout))
      time.sleep(timeout)

    # receive deploy cmd
    if with_deploy_info:
      _get_deploy_cmd_for_rolling(cmd_parser, client, model_name, to_version, from_json, cur_json, with_deploy_info)

  # add auto scaler rule
  if cur_json.get("config_json_file", None):
    print("begin to add scaler rule...")
    arg_list = ["autoscaler", "create", model_name, cur_json["model_version"], "-jf", cur_json["config_json_file"]]
    print(" ".join(arg_list))
    args = cmd_parser.parse_args(arg_list)
    args.func(args)
  
  # finished
  if with_deploy_info:
    client.deploy_cmd("finish", model_name, to_version)
  print("deploy finished!")
  return


def _deal_with_auto_scarler_config(path, cur_json):
  dir_path = os.path.dirname(path)
  if cur_json.get("config_json_file", None):
    pass
  else:
    scaler_path = os.path.join(dir_path, "scaler.json")
    if os.path.exists(scaler_path):
      cur_json["config_json_file"] = scaler_path
  return cur_json


def deploy_models(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return False

  try:
    with open(args.deploy_config, "r") as f:
      json_config = yaml.safe_load(f)
  except Exception as e:
    exit_as_error("ERROR: Failed to load config file: {}".format(e))
    return

  json_config = _deal_with_auto_scarler_config(args.deploy_config, json_config)
  create_info = _gen_model_create_info(client, json_config)
  has_deploy_info = _has_deploy_info(client, create_info["model_name"], create_info["model_version"])
  ret_list = _running_model_list(create_info["model_name"])

  if len(ret_list) <= 0:
    _first_deploy(client, create_info, has_deploy_info)
  elif len(ret_list) == 1:
    _rolling_update_deploy(client, ret_list[0], create_info, has_deploy_info)
  else:
    exit_as_error("ERROR: There's more than 1 running model! please keep only 1 before deploy.")


def list_autoscaler_rule(args):
  client = get_client()
  is_admin = False
  if "org_id" in args:
    is_admin = True
  response_autoscaler = client.list_autoscaler_rule(args)
  print_autoscaler_list(response_autoscaler, is_admin)


def print_autoscaler_list(response, is_admin=False):
  if is_admin:
    print("Current operator user: admin")
  format_list = ["{:32}", "{:16}", "{:16}", "{:16}"]
  key_list = ["model_name", "model_version", "trigger_type", "create_time"]
  if is_admin:
    format_list.insert(0, "{:16}")
    key_list.insert(0, "org_id")
  format_str = " ".join(format_list)
  title = tuple(map(lambda k: " ".join(k.capitalize().split("_")), key_list))
  print(format_str.format(*title))
  for item in response:
    d = tuple(map(lambda k: str(item[k]), key_list))
    print(format_str.format(*d))


def describe_autoscaler_rule(args):
  client = get_client()
  is_admin = False
  if "org_id" in args:
    is_admin = True
  response_autoscaler = client.describe_autoscaler_rule(args, is_admin)
  print_autoscaler_info(response_autoscaler, is_admin)


def print_autoscaler_info(response, is_admin=False):
  if is_admin:
    print("Current operator user: admin")
  for key, value in response.items():
    if not key.startswith("_"):
      echo_key = " ".join(key.capitalize().split("_"))
      print("{:30} {}".format(echo_key + ":", value))


def create_autoscaler_rule(args):
  client = get_client()
  is_admin = False
  if "org_id" in args:
    is_admin = True
  json_config = {}
  f = open(args.config_json_file, "r")
  json_file = f.read()
  json_config = json.loads(json_file)
  f.close()
  autoscaler_rule = AutoScalerRule(args.model_name,
                                   args.model_version,
                                   str(json_config["trigger_type"]))
  if is_admin:
    autoscaler_rule.org_id = args.org_id
  autoscaler_rule.threshold_max_replicas = json_config["threshold_max_replicas"]
  autoscaler_rule.threshold_min_replicas = json_config["threshold_min_replicas"]
  autoscaler_rule.instance_up_step = json_config["instance_up_step"]
  autoscaler_rule.instance_down_step = json_config["instance_down_step"]
  autoscaler_rule.collect_metrics_duration = json_config["collect_metrics_duration"]
  autoscaler_rule.trigger_interval = json_config["trigger_interval"]
  autoscaler_rule.up_threshold_qps = float(json_config["up_threshold_qps"])
  autoscaler_rule.down_threshold_qps = float(json_config["down_threshold_qps"])
  autoscaler_rule.up_qps_buffer_percent = float(json_config["up_qps_buffer_percent"])
  autoscaler_rule.falcon_qps_counter_name = json_config["falcon_qps_counter_name"]
  autoscaler_rule.falcon_endpoints = " ".join(json_config["falcon_endpoints"])
  autoscaler_rule.cron_configs = json_config["cron_configs"]
  autoscaler_rule.pre_eco_time_second = json_config["pre_eco_time_second"]
  autoscaler_rule.eco_mode_threshold_qps = json_config["eco_mode_threshold_qps"]
  autoscaler_rule.falcon_fetcher_url_format = json_config["falcon_fetcher_url_format"]
  autoscaler_rule.falcon_fetcher_multi_model_tf_method_name = json_config["falcon_fetcher_multi_model_tf_method_name"]
  autoscaler_rule.falcon_fetcher_multi_model_tf_submodel_name = json_config["falcon_fetcher_multi_model_tf_submodel_name"]
  response = client.create_autoscaler_rule(autoscaler_rule, is_admin)
  print_autoscaler_info(response, is_admin)


def update_autoscaler_rule(args):
  is_admin = False
  client = get_client()
  f = open(args.config_json_file, "r")
  json_file = f.read()
  json_config = json.loads(json_file)
  f.close()
  data = {"model_name": args.model_name, "model_version": args.model_version}
  if "org_id" in args:
    is_admin = True
    data["org_id"] = args.org_id
  data["threshold_max_replicas"] = json_config["threshold_max_replicas"]
  data["threshold_min_replicas"] = json_config["threshold_min_replicas"]
  data["instance_up_step"] = json_config["instance_up_step"]
  data["instance_down_step"] = json_config["instance_down_step"]
  data["collect_metrics_duration"] = json_config["collect_metrics_duration"]
  data["trigger_interval"] = json_config["trigger_interval"]
  data["up_threshold_qps"] = float(json_config["up_threshold_qps"])
  data["down_threshold_qps"] = float(json_config["down_threshold_qps"])
  data["up_qps_buffer_percent"] = float(json_config["up_qps_buffer_percent"])
  data["falcon_qps_counter_name"] = json_config["falcon_qps_counter_name"]
  data["falcon_endpoints"] = " ".join(json_config["falcon_endpoints"])
  data["cron_configs"] = json_config["cron_configs"]
  data["pre_eco_time_second"] = json_config["pre_eco_time_second"]
  data["eco_mode_threshold_qps"] = json_config["eco_mode_threshold_qps"]
  data["falcon_fetcher_url_format"] = json_config["falcon_fetcher_url_format"]
  data["falcon_fetcher_multi_model_tf_method_name"] = json_config["falcon_fetcher_multi_model_tf_method_name"]
  data["falcon_fetcher_multi_model_tf_submodel_name"] = json_config["falcon_fetcher_multi_model_tf_submodel_name"]
  response = client.update_autoscaler_rule(data, is_admin)
  print_autoscaler_info(response, is_admin=is_admin)


def pause_autoscaler_rule(args):
  client = get_client()
  is_admin = False
  data = {"model_name": args.model_name,
          "model_version": args.model_version,
          "run_status_update": "pause"}
  if "org_id" in args:
    is_admin = True
    data["org_id"] = args.org_id
  response = client.update_autoscaler_rule_run_status(data, is_admin)
  if is_admin:
    print("Current operator user: admin")
  print(response)


def resume_autoscaler_rule(args):
  client = get_client()
  is_admin = False
  data = {"model_name": args.model_name,
          "model_version": args.model_version,
          "run_status_update": "run"}
  if "org_id" in args:
    is_admin = True
    data["org_id"] = args.org_id
  response = client.update_autoscaler_rule_run_status(data, is_admin)
  if is_admin:
    print("Current operator user: admin")
  print(response)


def delete_autoscaler_rule(args):
  client = get_client()
  is_admin = False
  if "org_id" in args:
    is_admin = True
  response = client.delete_autoscaler_rule(args, is_admin)
  if is_admin:
    print("Current operator user: admin")
  print(response)


def list_tensorboard_services(args):
  """List tensorboard_services."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    if args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
      response_tensorboard = client.list_tensorboard_services(
          constant.CLOUDML_ALL_ORG_PARAMETER, priority_class=args.priority_class, page_number=args.page_number, page_size=args.page_size)
    else:
      response_tensorboard = client.list_tensorboard_services(args.org_id, priority_class=args.priority_class, page_number=args.page_number, page_size=args.page_size)
  else:
    response_tensorboard = client.list_tensorboard_services(priority_class=args.priority_class, page_number=args.page_number, page_size=args.page_size)

  if not isinstance(response_tensorboard, str):
    print_tensorboard_list(response_tensorboard, args)
  else:
    print("response: {}".format(response_tensorboard))


def print_tensorboard_list(response_tensorboard, args):
  if not args.priority_class:  # NOTE(xychu): will show all priority classes if no priority is given
    prioritized = {}
    unprioritized = []
    for priority in constant.CLOUDML_PRIORITY_CLASSES:
      prioritized[priority] = []
    for item in response_tensorboard:
      if "priority_class" in item and item["priority_class"]:
        prioritized[item["priority_class"]].append(item)
      else:
        unprioritized.append(item)
    for priority in prioritized:
      if len(prioritized[priority]) > 0:
        print("Priority class {}:".format(priority))
        do_print_tensorboard_list(prioritized[priority], args)
    if len(unprioritized) > 0:
      print("Un-prioritized:")
      do_print_tensorboard_list(unprioritized, args)
  else:
    do_print_tensorboard_list(response_tensorboard, args)


def do_print_tensorboard_list(response_tensorboard, args):
  if "org_id" in args and args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
    print("{:16} {:16} {:32} {:60} {:10} {:32} {:32}".format(
      "ORG ID", "ORG NAME", "TENSORBOARD_NAME", "ADDRESS", "STATE",
      "CREATED", "UPDATED"))
    for tensorboard in response_tensorboard:
      print("{:16} {:16} {:32} {:60} {:10} {:32} {:32}".format(tensorboard[
                                                                 "org_id"], tensorboard["org_name"], tensorboard[
                                                                 "tensorboard_name"], tensorboard["address"],
                                                               color_util.colorize_state(tensorboard[
                                                                                           "state"]),
                                                               tensorboard["create_time"], tensorboard[
                                                                 "update_time"]))
  else:
    print("{:32} {:60} {:10} {:32} {:32}".format(
      "TENSORBOARD_NAME", "ADDRESS", "STATE", "CREATED", "UPDATED"))
    for tensorboard in response_tensorboard:
      print("{:32} {:60} {:10} {:32} {:32}".format(tensorboard[
                                                     "tensorboard_name"], tensorboard["address"],
                                                   color_util.colorize_state(tensorboard["state"]),
                                                   tensorboard["create_time"], tensorboard["update_time"]))


def create_tensorboard_service(args):
  """Create the tensorboard_service."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  tensorboard = TensorboardService(args.tensorboard_name, args.logdir)

  if args.framework:
    tensorboard.framework = args.framework
  if args.framework_version:
    tensorboard.framework_version = args.framework_version
  if args.docker_image:
    tensorboard.docker_image = args.docker_image
  if args.docker_command:
    tensorboard.docker_command = args.docker_command
  if args.fds_endpoint:
    tensorboard.fds_endpoint = args.fds_endpoint
  elif client.fds_endpoint:
    tensorboard.fds_endpoint = client.fds_endpoint
  if args.fds_bucket:
    tensorboard.fds_bucket = args.fds_bucket
  elif client.fds_bucket:
    tensorboard.fds_bucket = client.fds_bucket
  if args.node_selector_key:
    tensorboard.node_selector_key = args.node_selector_key
  if args.node_selector_value:
    tensorboard.node_selector_value = args.node_selector_value
  if args.enable_rank:
    tensorboard.enable_rank = 1
  else:
    tensorboard.enable_rank = 0
  if args.priority_class:
    tensorboard.priority_class = args.priority_class

  response = client.create_tensorboard_service(tensorboard)
  if not isinstance(response, str):
    print_tensorboard_info(response)
  else:
    print("response: {}".format(response))


def describe_tensorboard_service(args):
  """Describe the tensorboard_service."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.describe_tensorboard_service(args.tensorboard_name,
                                                   args.org_id)
  else:
    response = client.describe_tensorboard_service(args.tensorboard_name)
  if not isinstance(response, str):
    print_tensorboard_info(response)
  else:
    print("response: {}".format(response))


def delete_tensorboard_service(args):
  """Delete the tensorboard_service(s)."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    for tensorboard_name in args.tensorboard_names:
      response = client.delete_tensorboard_service(tensorboard_name, args.org_id)
      print(response)
  else:
    for tensorboard_name in args.tensorboard_names:
      response = client.delete_tensorboard_service(tensorboard_name)
      print(response)


def get_tensorboard_service_events(args):
  """Get events of the tensorboard service."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  response = client.get_tensorboard_service_events(args.tensorboard_name)
  if not isinstance(response, str):
    print_kubernetes_events(response["events"])
    if response["recommend"]:
      print_recommend_resource(response["recommend"])
  else:
    print("response: {}".format(response))


def list_dev_envs(args):
  """List dev environments."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    if args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
      response_dev_envs = client.list_dev_envs(
          constant.CLOUDML_ALL_ORG_PARAMETER, priority_class=args.priority_class, page_number=args.page_number, page_size=args.page_size)
    else:
      response_dev_envs = client.list_dev_envs(args.org_id, priority_class=args.priority_class, page_number=args.page_number, page_size=args.page_size)
  else:
    response_dev_envs = client.list_dev_envs(priority_class=args.priority_class, page_number=args.page_number, page_size=args.page_size)
  if not isinstance(response_dev_envs, str):
    print_dev_env_list(response_dev_envs, args)
  else:
    print("response: {}".format(response_dev_envs))


def print_dev_env_list(response_dev_envs, args):
  if not args.priority_class:  # NOTE(xychu): will show all priority classes if no priority is given
    prioritized = {}
    unprioritized = []
    for priority in constant.CLOUDML_PRIORITY_CLASSES:
      prioritized[priority] = []
    for item in response_dev_envs:
      if "priority_class" in item and item["priority_class"]:
        prioritized[item["priority_class"]].append(item)
      else:
        unprioritized.append(item)
    for priority in prioritized:
      if len(prioritized[priority]) > 0:
        print("Priority class {}:".format(priority))
        do_print_dev_env_list(prioritized[priority], args)
    if len(unprioritized) > 0:
      print("Un-prioritized:")
      do_print_dev_env_list(unprioritized, args)
  else:
    do_print_dev_env_list(response_dev_envs, args)


def do_print_dev_env_list(response_dev_envs, args):
  if not isinstance(response_dev_envs, str):
    if "org_id" in args and args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
      print("{:16} {:16} {:32} {:16} {:32} {:32}".format(
          "ORG ID", "ORG NAME", "DEV_NAME", "STATE", "CREATED",
          "UPDATED"))
      for dev_env in response_dev_envs:
        print("{:16} {:16} {:32} {:16} {:32} {:32}".format(dev_env[
            "org_id"], dev_env["org_name"], dev_env["dev_name"],
            color_util.colorize_state(dev_env["state"]), dev_env[
              "create_time"], dev_env["update_time"]))
    else:
      print("{:32} {:16} {:32} {:32}".format(
          "DEV_NAME", "STATE", "CREATED", "UPDATED"))
      for dev_env in response_dev_envs:
        print("{:32} {:16} {:32} {:32}".format(dev_env[
            "dev_name"], color_util.colorize_state(dev_env[
              "state"]), dev_env["create_time"], dev_env["update_time"]))
  else:
    print("response: {}".format(response_dev_envs))


def create_dev_env(args):
  """Create dev env."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  dev_env = DevEnv(args.dev_name, args.password)

  if args.fds_endpoint:
    dev_env.fds_endpoint = args.fds_endpoint
  elif client.fds_endpoint:
    dev_env.fds_endpoint = client.fds_endpoint
  if args.fds_bucket:
    dev_env.fds_bucket = args.fds_bucket
  elif client.fds_bucket:
    dev_env.fds_bucket = client.fds_bucket
  # for hdfs
  if args.hdfs_krb_account:
    dev_env.hdfs_krb_account = args.hdfs_krb_account
  elif client.hdfs_krb_account:
    dev_env.hdfs_krb_account = client.hdfs_krb_account
  if args.hdfs_krb_password:
    dev_env.hdfs_krb_password = args.hdfs_krb_password
  elif client.hdfs_krb_password:
    dev_env.hdfs_krb_password = client.hdfs_krb_password
  if args.hdfs_endpoint:
    if args.hdfs_endpoint.startswith("hdfs://"):
      dev_env.hdfs_endpoint = args.hdfs_endpoint
    else:
      raise ValueError("ERROR: -he[hdfs_endpoint] must start with hdfs://")
  elif client.hdfs_endpoint:
    dev_env.hdfs_endpoint = client.hdfs_endpoint
  if args.hdfs_krb_keytab:
    dev_env.hdfs_krb_keytab = args.hdfs_krb_keytab
  if dev_env.hdfs_krb_account:
    if not dev_env.hdfs_krb_password and not dev_env.hdfs_krb_keytab:
      raise ValueError("ERROR: either -hkp[hdfs_krb_password] or -hkt[hdfs_krb_keytab] is required")
    if not dev_env.hdfs_endpoint:
      raise ValueError("ERROR: -he[hdfs_endpoint] is required")
  elif dev_env.hdfs_endpoint:
    raise ValueError("ERROR: -hka[hdfs_krb_account] is required")

  if args.ceph_volume:
    dev_env.ceph_volume = args.ceph_volume
    if not args.ceph_mode:
      raise ValueError("ERROR: -cm[ceph_mode] cannot be empty")
    if args.ceph_mode != "r" and args.ceph_mode != "rw":
      raise ValueError("ERROR: invalid value for -cm[ceph_mode]")
    dev_env.ceph_mode = args.ceph_mode
  if args.ceph_mounts:
    dev_env.ceph_mounts = args.ceph_mounts
  if args.cpu_limit:
    dev_env.cpu_limit = args.cpu_limit
  if args.memory_limit:
    dev_env.memory_limit = args.memory_limit
  if args.gpu_limit:
    dev_env.gpu_limit = args.gpu_limit
  if args.gpu_memory:
    dev_env.gpu_memory = args.gpu_memory
  if args.framework:
    dev_env.framework = args.framework
  if args.framework_version:
    dev_env.framework_version = args.framework_version
  if args.docker_image:
    dev_env.docker_image = args.docker_image
  if args.docker_command:
    dev_env.docker_command = args.docker_command
  if args.node_selector_key:
    dev_env.node_selector_key = args.node_selector_key
  if args.node_selector_value:
    dev_env.node_selector_value = args.node_selector_value
  if args.gpu_type:
    if args.gpu_limit:
      dev_env.gpu_type = args.gpu_type
    else:
      raise ValueError("ERROR: If you want select gpu type, gpu limit (-g) cannot be None.")
  if args.watch:
    dev_name = args.dev_name
  if args.network:
    dev_env.network = setNetwork()
  if args.enable_rank:
    dev_env.enable_rank = 1
  else:
    dev_env.enable_rank = 0
  if args.owner_email:
    dev_env.owner_email = args.owner_email
  else:
    dev_env.owner_email = client.org_mail
  if args.priority_class:
    dev_env.priority_class = args.priority_class
  # try to get framework_version if framework given
  if args.framework:
    dev_env.framework_version = get_framework_version_or_exit(client,
                                                              framework=args.framework,
                                                              version=args.framework_version,
                                                              python=args.framework_python,
                                                              hdfs=args.framework_hdfs)

  response_dev_env = client.create_dev_env(dev_env.get_json_data())
  if not isinstance(response_dev_env, str):
    print_dev_env_info(response_dev_env)
    if args.watch:
      print("\nThe dev_env is creating, feel free to Ctrl+C to stop watching\n")
      print("{:32} {:16} {:32} {:32}".format(
              "DEV_NAME",
              "STATE",
              "CREATED",
              "UPDATED"))
      while True:
        watch_response = client.describe_dev_env(dev_name)
        if not isinstance(watch_response, str):
          print("{:32} {:16} {:32} {:32}".format(
                watch_response["dev_name"],
                color_util.colorize_state(watch_response["state"]),
                watch_response["create_time"],
                watch_response["update_time"]))
          if watch_response["state"] == constant.DEV_ENV_STATE_RUNNING:
            return
          try:

            time.sleep(constant.JOB_WATCH_INTERVAL)
          except KeyboardInterrupt:
            return
        else:
          return
  else:
    print("response: {}".format(response_dev_env))


def describe_dev_env(args):
  """Describe the dev environment."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.describe_dev_env(args.dev_name, args.org_id)
  else:
    response = client.describe_dev_env(args.dev_name)
  if not isinstance(response, str):
    print_dev_env_info(response)
  else:
    print("response: {}".format(response))


def stop_dev_env(args):
  """Stop the dev environment(s)."""
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    for dev_name in args.dev_names:
      response = client.stop_dev_env(dev_name, args.org_id)
      print(response)
  else:
    for dev_name in args.dev_names:
      response = client.stop_dev_env(dev_name)
      print(response)


def v2_stop_dev_env(args):
  """Stop the dev environment(s)."""
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    for dev_name in args.dev_names:
      response = client.v2_stop_dev_env(dev_name, args.org_id)
      print(response)
  else:
    for dev_name in args.dev_names:
      response = client.v2_stop_dev_env(dev_name)
      print(response)


def save_dev_env(args):
  """Save the dev environment(s)."""
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    for dev_name in args.dev_names:
      response = client.save_dev_env(dev_name, args.org_id)
      print(response)
  else:
    for dev_name in args.dev_names:
      response = client.save_dev_env(dev_name)
      print(response)


def v2_save_dev_env(args):
  """Save the dev environment(s)."""
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.v2_save_dev_env(args.dev_name, args.env_name, args.force, args.org_id)
    print(response)
  else:
    response = client.v2_save_dev_env(args.dev_name, args.env_name, args.force)
    print(response)


def restart_dev_env(args):
  """Restart the dev environment(s)."""
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    for dev_name in args.dev_names:
      response = client.restart_dev_env(dev_name, args.org_id)
      print(response)
  else:
    for dev_name in args.dev_names:
      response = client.restart_dev_env(dev_name)
      print(response)


def scale_dev_env(args):
  """Scale the dev environment(s)."""
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  data_dict = {}

  if args.cpu_limit:
    data_dict["cpu_limit"] = args.cpu_limit
  if args.memory_limit:
    data_dict["memory_limit"] = args.memory_limit
  if args.gpu_limit:
    data_dict["gpu_limit"] = args.gpu_limit
  if args.gpu_memory:
    data_dict["gpu_memory"] = args.gpu_memory
  if args.docker_image:
    data_dict["docker_image"] = args.docker_image
  if "org_id" in args:
    response = client.scale_dev_env(args.dev_name, json.dumps(data_dict), args.org_id)
  else:
    response = client.scale_dev_env(args.dev_name, json.dumps(data_dict))
  print(response)


def delete_dev_env(args):
  """Delete the dev environment(s)."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    for dev_name in args.dev_names:
      response = client.delete_dev_env(dev_name, args.org_id)
      print(response)
  else:
    for dev_name in args.dev_names:
      response = client.delete_dev_env(dev_name)
      print(response)


def get_dev_env_events(args):
  """Get events of the dev environment."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  response = client.get_dev_env_events(args.dev_name)
  if not isinstance(response, str):
    print_kubernetes_events(response["events"])
    if response["recommend"]:
      print_recommend_resource(response["recommend"])
  else:
    print("response: {}".format(response))


def get_dev_env_metrics(args):
  """Get the metrics of the dev environment."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  response = client.get_dev_env_metrics(args.dev_name)
  if not isinstance(response, str):
    print_metrics_result(response)
  else:
    print("response: {}".format(response))


def list_dev_servers(args):
  """List dev servers."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    if args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
      response_dev_servers = client.list_dev_servers(
          constant.CLOUDML_ALL_ORG_PARAMETER)
    else:
      response_dev_servers = client.list_dev_servers(args.org_id)
  else:
    response_dev_servers = client.list_dev_servers()
  if not isinstance(response_dev_servers, str):
    if "org_id" in args and args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
      print("{:16} {:16} {:32} {:32} {:16} {:32} {:32}".format(
          "ORG ID", "ORG NAME", "DEV_NAME", "ADDRESS", "STATE", "CREATED",
          "UPDATED"))
      for dev_server in response_dev_servers:
        print("{:16} {:16} {:32} {:32} {:16} {:32} {:32}".format(
            dev_server["org_id"], dev_server["org_name"], dev_server[
                "dev_name"], dev_server["address"],
            color_util.colorize_state(dev_server["state"]),
            dev_server["create_time"], dev_server["update_time"]))
    else:
      print("{:32} {:32} {:16} {:32} {:32}".format(
          "DEV_NAME", "ADDRESS", "STATE", "CREATED", "UPDATED"))
      for dev_server in response_dev_servers:
        print("{:32} {:32} {:16} {:32} {:32}".format(
            dev_server["dev_name"], dev_server["address"],
            color_util.colorize_state(dev_server["state"]),
            dev_server["create_time"], dev_server["update_time"]))
  else:
    print("response: {}".format(response_dev_servers))


def create_dev_server(args):
  """Create dev server."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  dev_server = DevServer(args.dev_name, args.password)

  if args.framework:
    dev_server.framework = args.framework
  if args.framework_version:
    dev_server.framework_version = args.framework_version
  if args.docker_image:
    dev_server.docker_image = args.docker_image
  if args.docker_command:
    dev_server.docker_command = args.docker_command
  if args.enable_rank:
    dev_server.enable_rank = 1
  else:
    dev_server.enable_rank = 0
  response_dev_server = client.create_dev_server(dev_server)
  if not isinstance(response_dev_server, str):
    print_dev_server_info(response_dev_server)
  else:
    print("response: {}".format(response_dev_server))


def describe_dev_server(args):
  """Describe the dev server."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.describe_dev_server(args.dev_name, args.org_id)
  else:
    response = client.describe_dev_server(args.dev_name)
  if not isinstance(response, str):
    print_dev_server_info(response)
  else:
    print("response: {}".format(response))


def delete_dev_server(args):
  """Delete the dev server(s)."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  for dev_name in args.dev_names:
    response = client.delete_dev_server(dev_name)
    print(response)


def get_dev_server_events(args):
  """Get events of the dev server."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  response = client.get_dev_server_events(args.dev_name)
  if not isinstance(response, str):
    print_kubernetes_events(response["events"])
  else:
    print("response: {}".format(response))


def list_quota(args):
  """List the quota."""
  try:
      client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
      response_quotas = client.get_quota(args.org_id)
  else:
    response_quotas = client.get_quota()

  if not isinstance(response_quotas, str):
    if "org_id" in args and args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
      for quota in response_quotas:
        print("\nThe quota of user {}-{}:".format(quota["org_id"], quota[
            "org_name"]))
        print_quota_info(quota)
    else:
      for quota in response_quotas:
        print_quota_info(quota)
  else:
    print(response_quotas)


def apply_quota(args):
  """Apply for a quota update."""

  if not (args.type == "dev"
        or args.type == "jobs"
        or args.type == "models"
        or args.type == "tensorboard"):
    print("ERROR: Unknown  type {}.  Supported type are 'jobs' 'dev' 'models' and 'tensorboard'".format(args.type))
  else:
    try:
      client = CloudMlClient()
    except Exception as e:
      exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
      return
    org_mail=''
    second_org_leader=''
    if (not args.org_mail) or ("@xiaomi.com" not in args.org_mail):
      exit_as_error("ERROR: Please specify the org_mail param, e.g. --org_mail=someone@xiaomi.com", )
    else:
      org_mail = args.org_mail
    if (not args.second_org_leader) or ("@xiaomi.com" not in args.second_org_leader):
      pass
    else:
      print(color_util.colorize_warning("second_org_leader parameter has been disprecated, dont use this param."))
      second_org_leader=args.second_org_leader
    if args.allow_third_level_leader:
      print(color_util.colorize_warning("allow_third_level_leader parameter has been disprecated, dont use this param."))
    data = {
      "org_mail": org_mail,
      "region": client.endpoint,
      "service": args.type,
    }

    if args.type in ["dev", "jobs", "models"]:
      if not (args.cpu_limit and args.memory_limit and args.gpu_limit):
        exit_as_error(
          "ERROR: Please specify the cpu_limit memory_limit and gpu_limit param for type '{}'".format(args.type))
      data["cpu_limit"]=args.cpu_limit
      data["memory_limit"]=args.memory_limit
      data["gpu_limit"]=args.gpu_limit
    elif args.type == "tensorboard":
      if not args.tensorboard:
        exit_as_error(
          "ERROR: Please specify the tensorboard param for type tensorboard:e.g. --tensorboard=5 ")
      data["tensorboard"] = args.tensorboard

    response = client.apply_quota(data)

    if not isinstance(response, str) and ("{}".format(response["error"]).lower() == "false"):
      print("Your application has been submitted for approval")
    else:
      print("response: {}".format(response))


def do_predict(args):
  """Do predict."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  # TODO: Remove duplicated code
  if args.timeout:
    if args.model_version:
      response = client.do_predict(args.model_name, args.model_version,
                                   args.filename, float(args.timeout))
    elif args.server:
      response = client.do_predict_server(args.server, args.model_name,
                                          args.filename, float(args.timeout))
    else:
      exit_as_error("Need to set either model_version or server, exit now.")
      return 1
  else:
    if args.model_version:
      response = client.do_predict(args.model_name, args.model_version,
                                   args.filename)
    elif args.server:
      response = client.do_predict_server(args.server, args.model_name,
                                          args.filename)
    else:
      exit_as_error("Need to set either model_version or server, exit now.")
      return 1
  if isinstance(response, str):
    print(response)
  else:
    print_predict_result(response)


def update_job_quota(args, quota):
  if args.cpu:
    quota.train_cpu_quota = args.cpu
  if args.memory:
    quota.train_memory_quota = args.memory
  if args.gpu:
    quota.train_gpu_quota = int(args.gpu)
  if args.count:
    quota.train_count_quota = int(args.count)
  if args.priority_quota:
    quota.train_priority_quota = args.priority_quota
  if args.priority:
    quota.train_priority_rank = args.priority
  return quota


def update_model_quota(args, quota):
  if args.cpu:
    quota.model_cpu_quota = args.cpu
  if args.memory:
    quota.model_memory_quota = args.memory
  if args.gpu:
    quota.model_gpu_quota = int(args.gpu)
  if args.count:
    quota.model_count_quota = int(args.count)
  if args.priority_quota:
    quota.model_priority_quota = args.priority_quota
  if args.priority:
    quota.model_priority_rank = args.priority
  return quota


def update_dev_quota(args, quota):
  if args.cpu:
    quota.dev_cpu_quota = args.cpu
  if args.memory:
    quota.dev_memory_quota = args.memory
  if args.gpu:
    quota.dev_gpu_quota = int(args.gpu)
  if args.count:
    quota.dev_count_quota = int(args.count)
  if args.priority_quota:
    quota.dev_priority_quota = args.priority_quota
  if args.priority:
    quota.dev_priority_rank = args.priority
  return quota


def update_tensorboard_quota(args, quota):
  if args.tensorboard:
    quota.tensorboard_quota = int(args.tensorboard)
  if args.priority_quota:
    quota.tensorboard_priority_quota = args.priority_quota
  if args.priority:
    quota.tensorboard_priority_rank = args.priority
  return quota


def update_total_quota(args, quota):
  if args.cpu:
    quota.total_cpu_quota = args.cpu
  if args.memory:
    quota.total_memory_quota = args.memory
  if args.gpu:
    quota.total_gpu_quota = int(args.gpu)
  return quota


def update_quota(args):
  if not (args.cpu or args.memory or args.gpu or args.tensorboard or
          args.count or args.priority_quota or args.priority or args.org_mail or args.project) \
    and not args.second_org_leader and not args.quota_type:
    print("ERROR: Please give the resource to change or give the second_org_leader to change or give the quota_type to change")
  else:
    quota = Quota(args.org_id, args.org_name)

    if args.type == "jobs":
      quota = update_job_quota(args, quota)
    elif args.type == "models":
      quota = update_model_quota(args, quota)
    elif args.type == "dev":
      quota = update_dev_quota(args, quota)
    elif args.type == "tensorboard":
      quota = update_tensorboard_quota(args, quota)
    elif args.type == "total":
      quota = update_total_quota(args, quota)

    # update the other fields, they can be update separately or along with type parameter
    if args.org_mail:
      quota.org_mail = args.org_mail

    if args.second_org_leader:
      quota.second_org_leader = args.second_org_leader

    if args.project:
      quota.project = args.project

    if args.quota_type:
      if args.quota_type in constant.QUOTA_TYPE_LIST:
        quota.quota_type = args.quota_type
      else:
        exit_as_error('Invalid quota type, quota type shoud in ({})'.format(','.join(constant.QUOTA_TYPE_LIST)))

    try:
      client = CloudMlClient()
    except Exception as e:
      exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
      return
    response_quota = client.update_quota(quota, args.allow_third_level_leader)
    if not isinstance(response_quota, str):
      print_quota_info(response_quota)
    else:
      print("response: {}".format(response_quota))

    print("updated")


def list_framework(args):
  """List the framework."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  response = client.get_frameworks()

  if not isinstance(response, str):
    print_frameworks_info(response)
  else:
    print("response: {}".format(response))


def get_framework_version_or_exit(client, framework, version, python, hdfs):
  """Get the framework version or exit as error."""

  ### added by xiangbin, because default hadoop is zjyprc after 2019-09.
  if hdfs == 'zjyprc-hadoop':
    hdfs = ""
  frameworks_response = client.get_frameworks()
  if not isinstance(frameworks_response, str):
    if framework not in frameworks_response:
      exit_as_error("Framework {} is not supported.".format(framework))
    candidates = []
    for framework_version in frameworks_response[framework]:
      is_candidate = 0
      is_version_exact_match = False
      if version:
        if version == framework_version:
          # exact match found
          is_version_exact_match = True
        if version in framework_version:
          is_candidate += 1
      if python:
        if python in framework_version:
          is_candidate += 1
      else:
        # without python arg, version exact match, then it's candidate
        if is_version_exact_match:
          is_candidate += 1
        elif "-py" not in framework_version:
          is_candidate += 1
      if hdfs:
        if hdfs in framework_version:
          is_candidate += 1
      else:
        # without hdfs arg, version exact match, then it's candidate
        if is_version_exact_match:
          is_candidate += 1
        elif "-hadoop" not in framework_version:
          is_candidate += 1
      if is_candidate == 3:
        if is_version_exact_match:  # directly return if exact match and all three candidate tests passed
          return str(framework_version)
        candidates.append(framework_version)
    if len(candidates) == 0:
      exit_as_error("No Framework Version is not found for framework: {}, version: {}, python: {}, hdfs: {}.".format(
        framework, version, python, hdfs))
    elif len(candidates) == 1:
      return str(candidates[0])
    else:
      exit_as_error("Multi Framework Versions {} found for framework: {}, version: {}, python: {}, hdfs: {}.".format(
        candidates, framework, version,
        python, hdfs))
  else:
    exit_as_error(frameworks_response)


def list_pipelines(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args and args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
    response = client.list_pipelines(constant.CLOUDML_ALL_ORG_PARAMETER, priority_class=args.priority_class, page_number=args.page_number, page_size=args.page_size)
  else:
    response = client.list_pipelines(priority_class=args.priority_class, page_number=args.page_number, page_size=args.page_size)
  if not isinstance(response, str):
    print_pipeline_list(response, args)
  else:
    print("response: {}".format(response))


def list_pipeline_schedules(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  if "org_id" in args and args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
    response = client.list_pipeline_schedules(args.pipeline_name, constant.CLOUDML_ALL_ORG_PARAMETER)
  else:
    response = client.list_pipeline_schedules(args.pipeline_name)
  if not isinstance(response, str):
    print_pipeline_schedule_list(response, args)
  else:
    print("response: {}".format(response))


def create_pipeline_schedule(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  if not args.pipeline_rerun_date:
    schedule_param = {
      "pipeline_rerun_date": datetime.now().strftime("%Y%m%d")
    }
  else:
    schedule_param = {
      "pipeline_rerun_date": args.pipeline_rerun_date
    }
  request_data = {
    "schedule_param": schedule_param
  }
  response = client.create_pipeline_schedule(json.dumps(request_data), args.pipeline_name)
  print("response: {}".format(response))


def describe_pipeline_schedule(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  if "org_id" in args and args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
    response = client.describe_pipeline_schedule(args.schedule_id, constant.CLOUDML_ALL_ORG_PARAMETER)
  else:
    response = client.describe_pipeline_schedule(args.schedule_id)

  if not isinstance(response, str):

    print_pipeline_schedule_info(response)
  else:
    print("response: {}".format(response))


def print_pipeline_list(response, args):
  if not args.priority_class:  # NOTE(xychu): will show all priority classes if no priority is given
    prioritized = {}
    unprioritized = []
    for priority in constant.CLOUDML_PRIORITY_CLASSES:
      prioritized[priority] = []
    for item in response:
      if "priority_class" in item and item["priority_class"]:
        prioritized[item["priority_class"]].append(item)
      else:
        unprioritized.append(item)
    for priority in prioritized:
      if len(prioritized[priority]) > 0:
        print("Priority class {}:".format(priority))
        do_print_pipeline_list(prioritized[priority], args)
    if len(unprioritized) > 0:
      print("Un-prioritized:")
      do_print_pipeline_list(unprioritized, args)
  else:
    do_print_pipeline_list(response, args)


def do_print_pipeline_list(response, args):
  if "org_id" in args and args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
    print("{:16} {:16} {:32} {:32} {:32} {:32}".format(
      "ORG ID", "ORG NAME", "PIPELINE_NAME",
      "STATE", "CREATED", "UPDATED"))
    for pipeline in response:
      print("{:<16} {:16} {:32} {:16} {:32} {:32}".format(pipeline["org_id"], pipeline["org_name"],
                                                          pipeline["pipeline_name"], pipeline["state"],
                                                          pipeline["create_time"], pipeline["update_time"]))
  else:
    print("{:32} {:32} {:32} {:32}".format("PIPELINE_NAME",
                                                       "STATE", "CREATED",
                                                       "UPDATED"))

    for pipeline in response:
      print("{:32} {:32} {:32} {:32}".format(pipeline["pipeline_name"], pipeline["state"],
                                                         pipeline["create_time"], pipeline["update_time"]))


def print_pipeline_schedule_list(response, args):
  if "org_id" in args and args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
    print("{:16} {:16} {:16} {:32} {:32} {:32} {:32}".format(
      "ORG ID", "ORG NAME","SCHEDULE_ID", "PIPELINE_NAME",
      "STATE", "CREATED", "UPDATED"))
    for pipeline in response:
      print("{:<16} {:16} {:16}  {:32} {:16} {:32} {:32}".format(pipeline["org_id"], pipeline["org_name"],pipeline["id"],
                                                          pipeline["pipeline_name"], pipeline["state"],
                                                          pipeline["create_time"], pipeline["update_time"]))
  else:
    print(" {:16} {:32} {:32} {:32} {:32}".format("SCHEDULE_ID", "PIPELINE_NAME",
                                                       "STATE", "CREATED",
                                                       "UPDATED"))

    for pipeline in response:
      print(" {:16} {:32} {:32} {:32} {:32}".format(pipeline["id"], pipeline["pipeline_name"], pipeline["state"],
                                                         pipeline["create_time"], pipeline["update_time"]))


def delete_pipeline(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  for pipeline_name in args.pipeline_names:
    response = client.delete_pipeline(pipeline_name)
    print(response)


def describe_pipeline(args):
  """Describe the pipeline."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.describe_pipeline(args.pipeline_name, args.org_id)
  else:
    response = client.describe_pipeline(args.pipeline_name)

  if not isinstance(response, str):
  #  print("{:32} {:32} {:32} {:32} {:32} {:32}".format(pipeline[
  #                                               "pipeline_name"],pipeline["waiting_nodes"],
  #                                             pipeline["processing_nodes"],pipeline['finished_nodes'],
  #                                             pipeline["create_time"], pipeline["update_time"]))
    print_pipeline_info(response)

  else:
    print("response: {}".format(response))


def get_pipeline_logs(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.get_pipeline_logs(args.pod_name, args.org_id)
  else:
    response = client.get_pipeline_logs(args.pod_name)
  if not isinstance(response, str):
    logs = response["data"]['argo_log_info']
    print("###########################################")
    print("LOGS OF POD {}".format(args.pod_name))
    print("###########################################")
    if sys.version_info > (3, 0):
      print(logs)
    else:
      print(logs.encode('utf-8'))

  else:
    print("response:{}".format(response))


def start_pipeline(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.start_pipeline(args.pipeline_name, args.org_id)
  else:
    response = client.start_pipeline(args.pipeline_name)
  print(response)


def stop_pipeline(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.stop_pipeline(args.pipeline_name, args.org_id)
  else:
    response = client.stop_pipeline(args.pipeline_name)
  print(response)


def rerun_pipeline(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    print("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  data = {}
  data['single_node'] = args.single_node
  data['start_node'] = args.start_node
  data['date_range'] = args.date_range
  json_data = json.dumps(data)
  response = client.rerun_pipeline(args.pipeline_name, json_data)
  print(response)


def _split_cmd_string(cmd):
  cmd_list = []
  str_list = []
  for l in cmd.split():
    if len(str_list) <= 0:
      if l == '\"':
        str_list.append(l + ' ')
      elif l.startswith('\"') and not l.endswith('\"'):
        str_list.append(l)
      else:
        cmd_list.append(l)
    else:
      if l.endswith('\"'):
        str_list.append(l)
        cmd_list.append(' '.join(str_list))
        str_list = []
      else:
        str_list.append(l)
  return cmd_list


def _replace_trainer_uri(clist, nuri):
  prefix = '--trainer_uri'
  for i in range(len(clist)):
    c = clist[i]
    if c == '-u' or c == prefix:
      clist[i+1] = nuri
      return clist
    elif c.startswith(prefix):
      clist[i] = prefix + '=' + nuri
    else:
      pass
  return clist


# rewrite only when cmd is 'cloudml jobs submit xx' and cur_dir include pipeline file
def _rewrite_command_in_pipeline_config(cmd):
  from cloud_ml_sdk.command.command import new_parser
  from cloud_ml_sdk.models.models_util import validate_and_get_trainer_uri
  cmd_parser = new_parser()
  cmd_list = _split_cmd_string(cmd)
  first_cmd = cmd_list[0]
  if first_cmd != 'cloudml':
    return cmd
  cmd_args = cmd_parser.parse_args(cmd_list[1:])
  if cmd_args.command_group == 'jobs' \
      and cmd_args.job_command == 'submit' \
      and cmd_args.trainer_uri != None \
      and not cmd_args.trainer_uri.startswith("fds://"):
    new_uri = validate_and_get_trainer_uri(str(cmd_args.trainer_uri))
    cmd_list = _replace_trainer_uri(cmd_list, new_uri)
    new_cmd = ' '.join(cmd_list)
    print(cmd + '  in pipeline config has been rewrite to:  ')
    print(new_cmd)
    cmd = new_cmd
  return cmd


def _preprocessing_pipeline_config(config_json):
  json_data = yaml.safe_load(config_json)
  for v in json_data['nodes'].values():
    v['command'] = _rewrite_command_in_pipeline_config(v['command'])
  new_config = json.dumps(json_data)
  print(' pipe_config after preprocessing:')
  print(new_config)
  return new_config


def create_pipeline(args):
  return create_or_update_pipeline(args, True)


def update_pipeline(args):
  return create_or_update_pipeline(args, False)


def create_or_update_pipeline(args, is_create=True):
  """Submit the job."""

  # set environment variable to check args
  if args.fds_endpoint:
    os.environ["XIAOMI_FDS_ENDPOINT"] = args.fds_endpoint
  if args.fds_bucket:
    os.environ["CLOUDML_DEFAULT_FDS_BUCKET"] = args.fds_bucket

  args_dict = vars(args)
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if args.filename:
    with open(args.filename) as f:
      # TODO: Check file format and verify the items
      json_data = yaml.safe_load(f)
      for key in json_data :
        if key in args_dict and not args_dict[key]:
          args_dict[key] = json_data[key]

  if not args.pipeline_name:
    exit_as_error("ERROR: parameters pipeline_name is necessary.")
    return

  pipeline = Pipeline(args.pipeline_name)

  if 'pipeline_config' not in args or args.pipeline_config is None:
    raise ValueError('ERROR: pipeline_config is necessary')

  if isinstance(args.pipeline_config, dict):
    # json file load to the args.pipeline_config
    args.pipeline_config = json.dumps(args.pipeline_config)
  pipeline.pipeline_config = args.pipeline_config

  if args.fds_endpoint:
    pipeline.fds_endpoint = args.fds_endpoint
  elif client.fds_endpoint:
    pipeline.fds_endpoint = client.fds_endpoint
  if args.fds_bucket:
    pipeline.fds_bucket = args.fds_bucket
  elif client.fds_bucket:
    pipeline.fds_bucket = client.fds_bucket
  # for hdfs
  if args.hdfs_krb_account:
    pipeline.hdfs_krb_account = args.hdfs_krb_account
  elif client.hdfs_krb_account:
    pipeline.hdfs_krb_account = client.hdfs_krb_account
  if args.hdfs_krb_password:
    pipeline.hdfs_krb_password = args.hdfs_krb_password
  elif client.hdfs_krb_password:
    pipeline.hdfs_krb_password = client.hdfs_krb_password
  if args.hdfs_endpoint:
    if args.hdfs_endpoint.startswith("hdfs://"):
      pipeline.hdfs_endpoint = args.hdfs_endpoint
    else:
      raise ValueError("ERROR: -he[hdfs_endpoint] must start with hdfs://")
  elif client.hdfs_endpoint:
    pipeline.hdfs_endpoint = client.hdfs_endpoint

  if args.hdfs_krb_keytab:
    pipeline.hdfs_krb_keytab = args.hdfs_krb_keytab

  if pipeline.hdfs_krb_account:
    if not pipeline.hdfs_krb_password and not pipeline.hdfs_krb_keytab:
      raise ValueError("ERROR: either -hkp[hdfs_krb_password] or -hkt[hdfs_krb_keytab] is required")
    if not pipeline.hdfs_endpoint:
      raise ValueError("ERROR: -he[hdfs_endpoint] is required")
  elif pipeline.hdfs_endpoint:
    raise ValueError("ERROR: -hka[hdfs_krb_account] is required")

  if args.cloudml_endpoint:
    if args.cloudml_endpoint.startswith("http"):
      pipeline.cloudml_endpoint = args.cloudml_endpoint
    else:
      raise ValueError("ERROR: -he[cloudml_endpoint] must start with http")
  else:
    pipeline.cloudml_endpoint = client.endpoint

  if args.save_mode:
    pipeline.save_mode = args.save_mode

  if args.alarm_phone:
    pipeline.alarm_phone = args.alarm_phone

  if args.org_mail:
    if '@' in args.org_mail:
      pipeline.org_mail = args.org_mail
    else:
      raise ValueError("ERROR: org_mail must contains '@'")
  else:
    pipeline.org_mail = args.org_mail

  if args.priority_class:
    pipeline.priority_class = args.priority_class

  pipeline.pipeline_config = _preprocessing_pipeline_config(pipeline.pipeline_config)
  json_data = pipeline.get_json_data()
  if is_create:
    response = client.create_pipeline(json_data)
  else:
    response = client.update_pipeline(json_data, args.pipeline_name)

  if not isinstance(response, str):
    print_pipeline_info(response)
    if args.watch:
      print("\nThe pipeline has created, feel free to Ctrl+C to stop watching\n")
      print("{:32} {:16} {:32} {:32}".format("PIPELINE_NAME", "STATE", "CREATED",
                                             "UPDATED"))
      begin_time = datetime.now()
      while True:
        watch_response = client.describe_pipeline(pipeline.pipeline_name)
        if not isinstance(watch_response, str):
          print_pipeline_info(watch_response)
          if watch_response["data"]["state"] in (constant.PIPELINE_STATE_SUCCEEDED):
            print("Pipline wait to finish, status is succeed now, exit.")
            return
          elif watch_response["data"]["state"] in \
            (constant.PIPELINE_STATE_FAILED, constant.PIPELINE_STATE_ERROR, constant.PIPELINE_STATE_STOPPED):
            exit_as_error("Error. Pipline wait to finish, status is {} now, exit."
                          .format(watch_response["data"]["state"].lower()))
          try:
            cur_time = datetime.now()
            if (cur_time-begin_time).total_seconds() > args.max_wait_time:
              exit_as_error("Error. Pipline wait to finish timeout, max wait time is {} seconds".format(args.max_wait_time))
            time.sleep(constant.PIPELINE_WATCH_INTERVAL)
          except KeyboardInterrupt:
            exit_as_error("Error. User canceled watch")
        else:
          exit_as_error(watch_response)
  else:
    print("response: {}".format(response))


def print_schedule_list(response, args):
  if "org_id" in args and args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
    print("{:16} {:16} {:32} {:32} {:16} {:16} {:16} {:32}".format(
      "ORG ID", "ORG NAME", "SCHEDULE_NAME", "RESOURCE_NAME",
                        "RESOURCE_TYPE", "CRON_PARAM", "SUSPEND", "LAST_CREATE_TIME"))
    for schedule in response:
      suspend = "True" if schedule["suspend"] else "False"
      print("{:<16} {:16} {:32} {:32} {:16} {:16} {:16} {:32}".format(schedule["org_id"], schedule["org_name"],
                                                          schedule["schedule_name"], schedule["resource_name"],
                                                          schedule["resource_type"], schedule["cron_param"],
                                                                      suspend, schedule["last_create_time"]))
  else:
    print("{:32} {:32} {:16} {:16} {:16} {:32}".format("SCHEDULE_NAME",
                                                       "RESOURCE_NAME", "RESOURCE_TYPE", "CRON_PARAM","SUSPEND",  "LAST_CREATE_TIME"))
    for schedule in response:
      suspend = "True" if schedule["is_suspend"] else "False"
      print("{:32} {:32} {:16} {:16} {:16} {:32}".format(schedule["schedule_name"], schedule["resource_name"],
                                                         schedule["resource_type"], schedule["cron_param"],
                                                         suspend, schedule["last_create_time"]))


def print_schedule_info(response):
  """Print train job in customed format.

  Args:
    schedule: The dictionary of response train job data.
  """

  schedule = response['data']
  print("{:16} {}".format("Org id:", schedule["org_id"]))
  print("{:16} {}".format("Org name:", schedule["org_name"]))
  print("{:16} {}".format("schedule name:", schedule["schedule_name"]))
  print("{:16} {}".format("resource_name:", schedule["resource_name"]))
  print("{:16} {}".format("resource_type:", schedule["resource_type"]))
  print("{:16} {}".format("cron_param:", schedule["cron_param"]))
  print("{:16} {}".format("concurrency_policy:", schedule["concurrency_policy"]))
  print("{:16} {}".format("image_name:", schedule["image_name"]))
  print("{:16} {}".format("is_active:", schedule["is_active"]))
  print("{:16} {}".format("last_schedule:", schedule["last_schedule"]))
  print("{:16} {}".format("last_create_time:", schedule["last_create_time"]))
  print("{:16} {}".format("is_suspend:", schedule["is_suspend"]))


def list_schedules(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args and args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
    response = client.list_schedules(constant.CLOUDML_ALL_ORG_PARAMETER)
  else:
    response = client.list_schedules()
  if not isinstance(response, str):
    print_schedule_list(response, args)
  else:
    print("response: {}".format(response))


def delete_schedule(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  for schedule_name in args.schedule_names:
    response = client.delete_schedule(schedule_name)
    print(response)


def describe_schedule(args):
  """Describe the schedule."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.describe_schedule(args.schedule_name, args.org_id)
  else:
    response = client.describe_schedule(args.schedule_name)

  if not isinstance(response, str):
    print_schedule_info(response)
  else:
    print("response: {}".format(response))


def start_schedule(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.start_schedule(args.schedule_name, args.org_id)
  else:
    response = client.start_schedule(args.schedule_name)
  print(response)


def stop_schedule(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.stop_schedule(args.schedule_name, args.org_id)
  else:
    response = client.stop_schedule(args.schedule_name)
  print(response)


def get_schedule_logs(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.get_schedule_logs(args.schedule_name, args.org_id)
  else:
    response = client.get_schedule_logs(args.schedule_name)
  if not isinstance(response, str):
    if "error" in response and response["error"]:
      print("response: {}".format(response))
    else:
      if sys.version_info > (3, 0):
        print(response["logs"])
      else:
        print(response["logs"].encode('utf-8'))
  else:
    print("response: {}".format(response))


def create_schedule(args):
  return create_or_update_schedule(args, True)


def update_schedule(args):
  return create_or_update_schedule(args, False)


def create_or_update_schedule(args, is_create=True):
  """Submit the job."""

  args_dict = vars(args)
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return

  if not args.schedule_name:
    exit_as_error("ERROR: parameters schedule_name is necessary.")
    return

  schedule = Schedule(args.schedule_name)

  if 'resource_name' not in args:
    raise ValueError('ERROR: resource_name is necessary')

  if args.resource_type:
    schedule.resource_type = args.resource_type

  if args.resource_name:
    schedule.resource_name = args.resource_name
  elif is_create:
    raise ValueError('ERROR: resource_name is necessary')

  if args.cron_param:
    schedule.cron_param = args.cron_param
  elif is_create:
    raise ValueError('ERROR: cron_param is necessary')


  if args.success_history_limit:
    schedule.success_history_limit = args.success_history_limit
  # for hdfs
  if args.failed_history_limit:
    schedule.failed_history_limit = args.failed_history_limit
  if args.suspend:
    schedule.suspend = args.suspend
  if args.concurrency_policy:
    schedule.concurrency_policy = args.concurrency_policy
  if args.image_name:
    schedule.image_name = args.image_name

  json_data = schedule.get_json_data()
  if is_create:
    response = client.create_schedule(json_data)
  else:
    response = client.update_schedule(json_data, args.schedule_name)

  if not isinstance(response, str):
    print_schedule_info(response)
  else:
    print("response: {}".format(response))


def print_ceph_list(response, args):
  if "org_id" in args and args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
    print("{:16} {:16} {:16} {:16} {:16} {:32}".format(
      "ORG ID", "ORG NAME", "CEPH_NAME", "CAPACITY", "STATUS", "CREATED"))
    for ceph in response:
      print("{:16} {:16} {:16} {:16} {:16} {:32}".format(ceph["org_id"],
                                                         ceph["org_name"],
                                                         ceph["ceph_name"],
                                                         ceph["capacity"],
                                                         color_util.colorize_state(ceph["status"]),
                                                         ceph["create_time"]))
  else:
    print("{:16} {:16} {:16} {:32}".format(
      "CEPH_NAME", "CAPACITY", "STATUS", "CREATED"))
    for ceph in response:
      print("{:16} {:16} {:16} {:32}".format(ceph["ceph_name"],
                                            ceph["capacity"],
                                            color_util.colorize_state(ceph["status"]),
                                            ceph["create_time"]))


def print_ceph_info(ceph):
  """Print ceph in customized format.

  Args:
    ceph: The dictionary of response ceph data.
  """
  print("{:16} {}".format("Org id:", ceph["org_id"]))
  print("{:16} {}".format("Org name:", ceph["org_name"]))
  print("{:16} {}".format("Ceph name:", ceph["ceph_name"]))
  print("{:16} {}".format("Capacity:", ceph["capacity"]))
  print("{:16} {}".format("Status:", ceph.get("status")))
  print("{:16} {}".format("Create time:", ceph["create_time"]))
  print("{:16}".format("Events:"))
  if "events" in ceph:
    events = ceph["events"].split(";")
    for event in events:
      print(event)

def print_env_list(response, args):
  if "org_id" in args and args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
    print("{:16} {:16} {:16} {:16} {:16} {:16} {:32} {:64}".format(
      "ORG ID", "ORG NAME", "ENV_NAME", "SOURCE", "DEV_NAME", "STATUS", "CREATED", "IMAGE_NAME"))
    for env in response:
      print("{:16} {:16} {:16} {:16} {:16} {:16} {:32} {:64}".format(env["org_id"],
                                                         env["org_name"],
                                                         env["env_name"],
                                                         env["source"],
                                                         env["dev_name"],
                                                         env["state"],
                                                         env["create_time"],
                                                         env['image_name']))
  else:
    print("{:16} {:16} {:16} {:16} {:32} {:64}".format(
      "ENV_NAME","SOURCE", "DEV_NAME", "STATUS", "CREATED", "IMAGE_NAME"))
    for env in response:
      print("{:16} {:16} {:16} {:16} {:32} {:64}".format(env["env_name"],
                                             env["source"],
                                             env["dev_name"],
                                             env["state"],
                                             env["create_time"],
                                             env['image_name']))

def print_env_info(env):
  """Print env in customized format.

  Args:
    env: The dictionary of response env data.
  """
  print("{:16} {}".format("Org id:", env["org_id"]))
  print("{:16} {}".format("Org name:", env["org_name"]))
  print("{:16} {}".format("env name:", env["env_name"]))
  print("{:16} {}".format("dev name:", env["dev_name"]))
  print("{:16} {}".format("image name:", env["image_name"]))
  print("{:16} {}".format("status:", env.get("state")))
  print("{:16} {}".format("source:", env["source"]))
  print("{:16}".format("messages:"))
  if "message" in env and env["message"]:
    messages = env["message"].split(";")
    for mess in messages:
      print(mess)

def create_ceph(args):
  """Create a ceph volume."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  ceph = CephService(args.ceph_name, args.capacity)

  response = client.create_ceph_service(ceph)
  if not isinstance(response, str):
    print_ceph_info(response)
  else:
    print("response: {}".format(response))


def describe_ceph(args):
  """Describe the ceph volume."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.describe_ceph_service(args.ceph_name,
                                            args.org_id)
  else:
    response = client.describe_ceph_service(args.ceph_name)
  if not isinstance(response, str):
    print_ceph_info(response)
  else:
    print("response: {}".format(response))


def delete_ceph(args):
  """Delete the ceph volume(s)."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    for ceph_name in args.ceph_names:
      response = client.delete_ceph_service(ceph_name=ceph_name, org_id=args.org_id, force=args.force)
      print(response)
  else:
    for ceph_name in args.ceph_names:
      response = client.delete_ceph_service(ceph_name=ceph_name, org_id=None, force=args.force)
      print(response)


def get_ceph_events(args):
  """Get events of the ceph volume."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  response = client.get_ceph_service_events(args.ceph_name)
  # TODO format ceph events
  print("response: {}".format(response))


def list_cephs(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args and args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
    response = client.list_ceph_services(constant.CLOUDML_ALL_ORG_PARAMETER)
  else:
    response = client.list_ceph_services()
  if not isinstance(response, str):
    print_ceph_list(response, args)
  else:
    print("response: {}".format(response))


def sync_ceph(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  response = client.sync_ceph_from_fds(args.ceph_name, args.fds_endpoint, args.fds_bucket)
  print("response: {}".format(response))


def list_envs(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args and args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
    response = client.list_envs(constant.CLOUDML_ALL_ORG_PARAMETER)
  else:
    response = client.list_envs()
  if not isinstance(response, str):
    print_env_list(response, args)
  else:
    print("response: {}".format(response))

def describe_env(args):
  """Describe the env."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    response = client.describe_env(args.env_name,
                                            args.org_id)
  else:
    response = client.describe_env(args.env_name)
  if not isinstance(response, str):
    print_env_info(response)
  else:
    print("response: {}".format(response))

def delete_env(args):
  """Delete the env(s)."""

  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args:
    for env_name in args.env_names:
      response = client.delete_env(env_name=env_name, org_id=args.org_id)
      print(response)
  else:
    for env_name in args.env_names:
      response = client.delete_env(env_name=env_name)
      print(response)

def list_all(args):
  """List all resources."""

  print("List all train jobs:")
  list_jobs(args)
  print("\nList all model services:")
  list_models(args)
  print("\nList all dev environments:")
  list_dev_envs(args)
  print("\nList all tensorboard services:")
  list_tensorboard_services(args)
  print("\nList all pipelines:")
  list_pipelines(args)
  print("\nList all cephs:")
  list_cephs(args)


def exit_as_error(message, error_code=-1, is_os_exit=False):
  print(message)
  if is_os_exit:
    # os exit can not be catched
    os._exit(error_code)
  else:
    sys.exit(error_code)


def list_cluster_available_resources(args):
  try:
    client = CloudMlClient()
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))
    return
  if "org_id" in args and args.org_id == constant.CLOUDML_ALL_ORG_PARAMETER:
    response = client.list_cluster_resources(constant.CLOUDML_ALL_ORG_PARAMETER)
  else:
    response = client.list_cluster_resources()
  if not isinstance(response, str):
    print_resources_list(response)
  else:
    print("response: {}".format(response))


def print_resources_list(data):
  print("{:32} {:32} {:32}".format("Available cpu", "Available memory", "Available gpu"))
  for item in data:
    print("{:32} {:32} {:32}".format(str(item["available_cpu"]), item["available_mem"], str(item["available_gpu"])))


def get_cur_time():
  localtime = time.localtime(time.time())
  return str(localtime.tm_mon).zfill(2) + str(localtime.tm_mday).zfill(2) + str(localtime.tm_hour).zfill(2) + str(localtime.tm_min).zfill(2)


def get_client():
  try:
    client = CloudMlClient()
    return client
  except Exception as e:
    exit_as_error("ERROR: Failed to init CloudML Client: {}".format(e))

